@@ -75,6 +75,7 @@
 #include "style.h"
 #include "icons.h"
 #include "gnome.h"
+#include "ewmh.h"
 #include "focus.h"
 #include "placement.h"
 #include "geometry.h"
@@ -95,22 +96,6 @@ void FetchWmProtocols(FvwmWindow *);
 void GetWindowSizeHints(FvwmWindow *);
 /***********************************************************************/
 
-static void delete_client_context(FvwmWindow *fw)
-{
-  FvwmWindow *cw;
-
-  /* We can not simply delete the context.  X might have reused the
-   * window structure so we would delete the context that was established
-   * by another FvwmWindow structure in the mean time. */
-  if (XFindContext(
-	dpy, fw->w, FvwmContext, (caddr_t *)&cw) != XCNOENT && cw == fw)
-  {
-    XDeleteContext(dpy, fw->w, FvwmContext);
-  }
-
-  return;
-}
-
 Bool setup_window_structure(
   FvwmWindow **ptmp_win, Window w, FvwmWindow *ReuseWin)
 {
@@ -152,6 +137,28 @@ Bool setup_window_structure(
     SET_SHADED(*ptmp_win, IS_SHADED(savewin));
     SET_PLACED_WB3(*ptmp_win,IS_PLACED_WB3(savewin));
     SET_PLACED_BY_FVWM(*ptmp_win, IS_PLACED_BY_FVWM(savewin));
+    SET_HAS_EWMH_WM_ICON_HINT(*ptmp_win, HAS_EWMH_WM_ICON_HINT(savewin));
+    (*ptmp_win)->ewmh_mini_icon_width = savewin->ewmh_mini_icon_width;
+    (*ptmp_win)->ewmh_mini_icon_height = savewin->ewmh_mini_icon_height;
+    (*ptmp_win)->ewmh_icon_width = savewin->ewmh_icon_width;
+    (*ptmp_win)->ewmh_icon_height = savewin->ewmh_icon_height;
+    (*ptmp_win)->ewmh_hint_desktop = savewin->ewmh_hint_desktop;
+    /* restore ewmh state */
+    EWMH_SetWMState(savewin, True);
+    SET_HAS_EWMH_INIT_WM_DESKTOP(
+      *ptmp_win, HAS_EWMH_INIT_WM_DESKTOP(savewin));
+    SET_HAS_EWMH_INIT_FULLSCREEN_STATE(
+      *ptmp_win, HAS_EWMH_INIT_FULLSCREEN_STATE(savewin));
+    SET_HAS_EWMH_INIT_HIDDEN_STATE(
+      *ptmp_win, HAS_EWMH_INIT_HIDDEN_STATE(savewin));
+    SET_HAS_EWMH_INIT_MAXHORIZ_STATE(
+      *ptmp_win, HAS_EWMH_INIT_MAXHORIZ_STATE(savewin));
+    SET_HAS_EWMH_INIT_MAXVERT_STATE(
+      *ptmp_win, HAS_EWMH_INIT_MAXVERT_STATE(savewin));
+    SET_HAS_EWMH_INIT_SHADED_STATE(
+      *ptmp_win, HAS_EWMH_INIT_SHADED_STATE(savewin));
+    SET_HAS_EWMH_INIT_STICKY_STATE(
+      *ptmp_win, HAS_EWMH_INIT_STICKY_STATE(savewin));
   }
 
   (*ptmp_win)->cmap_windows = (Window *)NULL;
@@ -223,70 +230,8 @@ static void get_name_property(
 
   return;
 }
-#else /* I18N_MB */
+#else
 #define GET_NAME_PROPERTY(a, b, c, d) get_name_property(a, b, c)
-#ifdef COMPOUND_TEXT
-static void get_name_property(
-  Status (func)(Display *, Window, XTextProperty *), Window w, char **ret_name)
-{
-  XTextProperty text_prop;
-  int num;
-  char **list;
-
-  /* Note: we dump the name because we cannot keep the list in memory
-   * without a lot of changes */
-  if (func(dpy, w, &text_prop) != 0)
-  {
-    if (text_prop.value)
-    {
-      if (text_prop.encoding == XA_STRING)
-      {
-        /* STRING encoding, use this as it is */
-	CopyString(&*ret_name, (char *)text_prop.value);
-	XFree(text_prop.value);
-      }
-      else
-      {
-        /* not STRING encoding, try to convert */
-        if (XmbTextPropertyToTextList(dpy, &text_prop, &list, &num) == Success
-            && num > 0 && *list)
-	{
-          /* XXX: does not consider the conversion is REALLY succeeded */
-          XFree(text_prop.value); /* return of XGetWM(Icon)Name() */
-	  CopyString(&*ret_name, *list);
-	  if (list)
-	    XFreeStringList(list);
-        }
-	else
-	{
-          if (list)
-            XFreeStringList(list);
-          XFree(text_prop.value); /* return of XGet(Icon)WMName() */
-	  if (func(dpy, w, &text_prop))
-	  {
-	    CopyString(&*ret_name, (char *)text_prop.value);
-	    XFree(text_prop.value);
-	  }
-	  else
-	  {
-	    *ret_name = NoName;
-	  }
-        }
-      }
-    }
-    else
-    {
-      *ret_name = NoName;
-    }
-  }
-  else
-  {
-    *ret_name = NoName;
-  }
-
-  return;
-}
-#else /* COMPOUND_TEXT */
 static void get_name_property(
   Status (func)(Display *, Window, XTextProperty *), Window w, char **ret_name)
 {
@@ -299,18 +244,149 @@ static void get_name_property(
 
   return;
 }
-#endif /* COMPOUND_TEXT */
-#endif /* I18N_MB */
+#endif
+
+void setup_window_name_count(FvwmWindow *tmp_win)
+{
+  FvwmWindow *t;
+  int count = 0;
+  Bool done = False;
+
+  if (!tmp_win->name)
+    done = True;
+
+  if (tmp_win->icon_name && strcmp(tmp_win->name, tmp_win->icon_name) == 0)
+    count = tmp_win->icon_name_count;
+
+  while (!done)
+  {
+    done = True;
+    for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+    {
+      if (t == tmp_win)
+	continue;
+      if ((t->name && strcmp(t->name, tmp_win->name) == 0 &&
+	  t->name_count == count) ||
+	  (t->icon_name && strcmp(t->icon_name, tmp_win->name) == 0 &&
+	   t->icon_name_count == count))
+      {
+	count++;
+	done = False;
+      }
+    }
+  }
+  tmp_win->name_count = count;
+}
+
+static void setup_icon_name_count(FvwmWindow *tmp_win)
+{
+  FvwmWindow *t;
+  int count = 0;
+  Bool done = False;
+
+  if (!tmp_win->icon_name)
+    done = True;
+
+  if (tmp_win->name && strcmp(tmp_win->name, tmp_win->icon_name) == 0)
+    count = tmp_win->name_count;
+
+  while (!done)
+  {
+    done = True;
+    for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+    {
+      if (t == tmp_win)
+	continue;
+      if ((t->icon_name && strcmp(t->icon_name, tmp_win->icon_name) == 0 &&
+	  t->icon_name_count == count) ||
+	  (t->name && strcmp(t->name, tmp_win->icon_name) == 0 &&
+	   t->name_count == count))
+      {
+	count++;
+	done = False;
+      }
+    }
+  }
+  tmp_win->icon_name_count = count;
+}
+
+void setup_visible_name(FvwmWindow *tmp_win, Bool is_icon)
+{
+  char *ext_name;
+  char *name;
+  int count;
+  int len;
+
+  if (tmp_win == NULL)
+    return; /* should never happen */
+
+  if (is_icon)
+  {
+    if (tmp_win->visible_icon_name != NULL &&
+	tmp_win->visible_icon_name != tmp_win->icon_name &&
+	tmp_win->visible_icon_name != tmp_win->name &&
+	tmp_win->visible_icon_name != NoName)
+    {
+      free(tmp_win->visible_icon_name);
+      tmp_win->visible_icon_name = NULL;
+    }
+    name = tmp_win->icon_name;
+    setup_icon_name_count(tmp_win);
+    count = tmp_win->icon_name_count;
+  }
+  else
+  {
+    if (tmp_win->visible_name != NULL &&
+	tmp_win->visible_name != tmp_win->name &&
+	tmp_win->visible_name != NoName)
+    {
+      free(tmp_win->visible_name);
+      tmp_win->visible_name = NULL;
+    }
+    name = tmp_win->name;
+    setup_window_name_count(tmp_win);
+    count = tmp_win->name_count;
+  }
+
+  if (name == NULL)
+  {
+    return; /* should never happen */
+  }
+
+  if (count > 998)
+    count = 998;
+
+  if (count != 0 &&
+      ((is_icon && USE_INDEXED_ICON_NAME(tmp_win)) ||
+       (!is_icon && USE_INDEXED_WINDOW_NAME(tmp_win))))
+  {
+    len = strlen(name);
+    count++;
+    ext_name = (char *)safemalloc(len + 6);
+    sprintf(ext_name,"%s (%d)", name, count);
+  }
+  else
+    ext_name = name;
+
+  if (is_icon)
+    tmp_win->visible_icon_name = ext_name;
+  else
+    tmp_win->visible_name = ext_name;
+}
 
 void setup_window_name(FvwmWindow *tmp_win)
 {
-  GET_NAME_PROPERTY(XGetWMName, tmp_win->w, &(tmp_win->name),
-		    &(tmp_win->name_list));
+  if (!EWMH_WMName(tmp_win, NULL, NULL, 0))
+  {
+      GET_NAME_PROPERTY(XGetWMName, tmp_win->w, &(tmp_win->name),
+			&(tmp_win->name_list));
+  }
+
   if (debugging)
     fvwm_msg(DBG,"setup_window_name","Assigned name %s'",tmp_win->name);
 }
 
-void setup_class_and_resource(FvwmWindow *tmp_win)
+static void setup_class_and_resource(FvwmWindow *tmp_win)
 {
   /* removing NoClass change for now... */
   tmp_win->class.res_name = NoResource;
@@ -385,15 +461,18 @@ void setup_window_font(
     if (tmp_win->title_font.y < tmp_win->title_font.font->ascent)
       tmp_win->title_font.y = tmp_win->title_font.font->ascent;
     tmp_win->title_g.height = height;
+    tmp_win->corner_width =
+      height + tmp_win->boundary_width;
   }
   else
   {
     height = tmp_win->title_font.height;
     tmp_win->title_font.y = tmp_win->title_font.font->ascent;
+    tmp_win->corner_width =
+      height + EXTRA_TITLE_FONT_HEIGHT + tmp_win->boundary_width;
     tmp_win->title_g.height =
       tmp_win->title_font.height + EXTRA_TITLE_FONT_HEIGHT;
   }
-    tmp_win->corner_width = tmp_win->title_g.height + tmp_win->boundary_width;
   if (!HAS_TITLE(tmp_win))
   {
     tmp_win->title_g.height = 0;
@@ -457,7 +536,7 @@ void setup_icon_font(
   /* adjust y position of existing icons */
   if (height)
   {
-    tmp_win->icon_g.y += height - tmp_win->icon_font.height;
+    resize_icon_title_height(tmp_win, height - tmp_win->icon_font.height);
     /* this repositions the icon even if the window is not iconified */
     DrawIconWindow(tmp_win);
   }
@@ -542,6 +621,14 @@ void setup_style_and_decor(
 
   /****** window shading ******/
   tmp_win->shade_anim_steps = pstyle->shade_anim_steps;
+
+  /****** GNOME style hints ******/
+  if (!SDO_IGNORE_GNOME_HINTS(pstyle->flags))
+  {
+    GNOME_GetStyle(tmp_win, pstyle);
+  }
+
+  return;
 }
 
 void setup_icon_boxes(FvwmWindow *tmp_win, window_style *pstyle)
@@ -710,6 +797,37 @@ Bool setup_window_placement(FvwmWindow *tmp_win, window_style *pstyle)
       SGET_START_SCREEN(*pstyle), PLACE_INITIAL);
 }
 
+void setup_placement_penalty(FvwmWindow *tmp_win, window_style *pstyle)
+{
+  int i;
+
+  if (!SHAS_PLACEMENT_PENALTY(&pstyle->flags))
+  {
+    SSET_NORMAL_PLACEMENT_PENALTY(*pstyle, 1);
+    SSET_ONTOP_PLACEMENT_PENALTY(*pstyle, PLACEMENT_AVOID_ONTOP);
+    SSET_ICON_PLACEMENT_PENALTY(*pstyle, PLACEMENT_AVOID_ICON);
+    SSET_STICKY_PLACEMENT_PENALTY(*pstyle, PLACEMENT_AVOID_STICKY);
+    SSET_BELOW_PLACEMENT_PENALTY(*pstyle, PLACEMENT_AVOID_BELOW);
+    SSET_EWMH_STRUT_PLACEMENT_PENALTY(*pstyle, PLACEMENT_AVOID_EWMH_STRUT);
+  }
+  if (!SHAS_PLACEMENT_PERCENTAGE_PENALTY(&pstyle->flags))
+  {
+    SSET_99_PLACEMENT_PERCENTAGE_PENALTY(*pstyle, PLACEMENT_AVOID_COVER_99);
+    SSET_95_PLACEMENT_PERCENTAGE_PENALTY(*pstyle, PLACEMENT_AVOID_COVER_95);
+    SSET_85_PLACEMENT_PERCENTAGE_PENALTY(*pstyle, PLACEMENT_AVOID_COVER_85);
+    SSET_75_PLACEMENT_PERCENTAGE_PENALTY(*pstyle, PLACEMENT_AVOID_COVER_75);
+  }
+  for(i = 0; i < 6; i++)
+  {
+    tmp_win->placement_penalty[i] = (*pstyle).placement_penalty[i];
+  }
+  for(i = 0; i < 4; i++)
+  {
+    tmp_win->placement_percentage_penalty[i] =
+      (*pstyle).placement_percentage_penalty[i];
+  }
+}
+
 void get_default_window_attributes(
   FvwmWindow *tmp_win, unsigned long *pvaluemask,
   XSetWindowAttributes *pattributes)
@@ -978,39 +1096,12 @@ void destroy_resize_handle_windows(
   }
 }
 
-static void resize_resize_handle_windows(FvwmWindow *tmp_win)
-{
-        int i;
-
-        tmp_win->corner_width =
-                tmp_win->title_g.height + tmp_win->boundary_width;
-
-        if (HAS_BORDER(tmp_win))
-        {
-                /* Just dump the windows any old place and let SetupFrame take
-                 * care of the mess */
-                for (i = 0; i < 4; i++)
-                {
-                        XResizeWindow(
-                                dpy, tmp_win->corners[i],
-                                tmp_win->corner_width, tmp_win->corner_width);
-                }
-        }
-
-        return;
-}
-
 void change_resize_handle_windows(FvwmWindow *tmp_win)
 {
   if (HAS_BORDER(tmp_win) && tmp_win->sides[0] == None)
     setup_resize_handle_windows(tmp_win);
   else if (!HAS_BORDER(tmp_win) && tmp_win->sides[0] != None)
     destroy_resize_handle_windows(tmp_win, False);
-  else if (tmp_win->corner_width !=
-           tmp_win->title_g.height + tmp_win->boundary_width)
-  {
-    resize_resize_handle_windows(tmp_win);
-  }
 }
 
 void setup_frame_stacking(FvwmWindow *tmp_win)
@@ -1079,6 +1170,10 @@ void setup_frame_attributes(
 	  break;
   }
   XChangeWindowAttributes(dpy, tmp_win->w, CWBackingStore, &xswa);
+  if (pstyle->flags.use_backing_store == BACKINGSTORE_OFF)
+  {
+    xswa.backing_store = NotUseful;
+  }
   XChangeWindowAttributes(dpy, tmp_win->decor_w, CWBackingStore, &xswa);
   if (HAS_TITLE(tmp_win))
   {
@@ -1090,17 +1185,17 @@ void setup_frame_attributes(
 				&xswa);
     }
   }
+
   /* parent_relative is applied to the frame and the parent */
   xswa.background_pixmap = pstyle->flags.use_parent_relative
 			   ? ParentRelative : None;
+  XChangeWindowAttributes(dpy, tmp_win->frame, CWBackPixmap, &xswa);
+  XChangeWindowAttributes(dpy, tmp_win->Parent, CWBackPixmap, &xswa);
+
   /* Save_under is only useful on the frame */
   xswa.save_under = pstyle->flags.do_save_under
 		    ? Scr.flags.do_save_under : NotUseful;
-  XChangeWindowAttributes(
-	  dpy, tmp_win->Parent, CWBackPixmap | CWBackingStore, &xswa);
-  XChangeWindowAttributes(
-	  dpy, tmp_win->frame, CWBackPixmap | CWBackingStore | CWSaveUnder,
-	  &xswa);
+  XChangeWindowAttributes(dpy, tmp_win->frame, CWSaveUnder, &xswa);
 }
 
 void destroy_auxiliary_windows(FvwmWindow *tmp_win,
@@ -1112,7 +1207,7 @@ void destroy_auxiliary_windows(FvwmWindow *tmp_win,
     XDeleteContext(dpy, tmp_win->frame, FvwmContext);
     XDeleteContext(dpy, tmp_win->decor_w, FvwmContext);
     XDeleteContext(dpy, tmp_win->Parent, FvwmContext);
-    delete_client_context(tmp_win);
+    XDeleteContext(dpy, tmp_win->w, FvwmContext);
   }
 
   if (HAS_TITLE(tmp_win))
@@ -1204,21 +1299,24 @@ ICON_DBG((stderr,"si: using default '%s'\n", tmp_win->name));
   }
 
   /* icon name */
-  GET_NAME_PROPERTY(XGetWMIconName, tmp_win->w, &(tmp_win->icon_name),
-		    &(tmp_win->icon_name_list));
+  if (!EWMH_WMIconName(tmp_win, NULL, NULL, 0))
+      GET_NAME_PROPERTY(XGetWMIconName, tmp_win->w, &(tmp_win->icon_name),
+			&(tmp_win->icon_name_list));
   if (tmp_win->icon_name == NoName)
   {
     tmp_win->icon_name = tmp_win->name;
     SET_WAS_ICON_NAME_PROVIDED(tmp_win, 0);
   }
+  setup_visible_name(tmp_win, True);
+
 
   /* wait until the window is iconified and the icon window is mapped
    * before creating the icon window
    */
-  tmp_win->icon_w = None;
+  tmp_win->icon_title_w = None;
 
-  BroadcastName(M_ICON_NAME,tmp_win->w,tmp_win->frame,
-		(unsigned long)tmp_win,tmp_win->icon_name);
+  EWMH_SetVisibleName(tmp_win, True);
+  BroadcastWindowIconNames(tmp_win, False, True);
   if (tmp_win->icon_bitmap_file != NULL &&
       tmp_win->icon_bitmap_file != Scr.DefaultIcon)
     BroadcastName(M_ICON_FILE,tmp_win->w,tmp_win->frame,
@@ -1228,7 +1326,7 @@ ICON_DBG((stderr,"si: using default '%s'\n", tmp_win->name));
 static void destroy_icon(FvwmWindow *tmp_win)
 {
   free_window_names(tmp_win, False, True);
-  if (tmp_win->icon_w)
+  if (tmp_win->icon_title_w)
   {
     if (IS_PIXMAP_OURS(tmp_win))
     {
@@ -1238,29 +1336,16 @@ static void destroy_icon(FvwmWindow *tmp_win)
 	XFreePixmap(dpy, tmp_win->icon_maskPixmap);
       }
     }
-    XDestroyWindow(dpy, tmp_win->icon_w);
-    XDeleteContext(dpy, tmp_win->icon_w, FvwmContext);
+    XDestroyWindow(dpy, tmp_win->icon_title_w);
+    XDeleteContext(dpy, tmp_win->icon_title_w, FvwmContext);
   }
-  if (tmp_win->icon_pixmap_w != None)
+  if((IS_ICON_OURS(tmp_win))&&(tmp_win->icon_pixmap_w != None))
+    XDestroyWindow(dpy, tmp_win->icon_pixmap_w);
+  if(tmp_win->icon_pixmap_w != None)
   {
-    if (IS_ICON_OURS(tmp_win))
-    {
-      XDestroyWindow(dpy, tmp_win->icon_pixmap_w);
-    }
-    else
-    {
-      XUnmapWindow(dpy, tmp_win->icon_pixmap_w);
-    }
     XDeleteContext(dpy, tmp_win->icon_pixmap_w, FvwmContext);
   }
-  tmp_win->icon_pixmap_w = None;
-  tmp_win->iconPixmap = None;
-  tmp_win->icon_maskPixmap = None;
-  tmp_win->icon_g.width = 0;
-  tmp_win->icon_g.height = 0;
-  tmp_win->icon_t_width = 0;
-  tmp_win->icon_p_width = 0;
-  tmp_win->icon_p_height = 0;
+  clear_icon(tmp_win);
 }
 
 void change_icon(FvwmWindow *tmp_win, window_style *pstyle)
@@ -1449,6 +1534,10 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
   setup_window_font(tmp_win, &style, False);
   setup_icon_font(tmp_win, &style, False);
 
+  /***** visible window name ****/
+  setup_visible_name(tmp_win, False);
+  EWMH_SetVisibleName(tmp_win, False);
+
   /****** state setup ******/
   setup_icon_boxes(tmp_win, &style);
   SET_ICONIFIED(tmp_win, 0);
@@ -1489,6 +1578,8 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
   tmp_win->old_bw = tmp_win->attr.border_width;
   XSetWindowBorderWidth(dpy, tmp_win->w, 0);
 
+  /***** placement penalities *****/
+  setup_placement_penalty(tmp_win, &style);
   /*
    * MatchWinToSM changes tmp_win->attr and the stacking order.
    * Thus it is important have this call *after* PlaceWindow and the
@@ -1587,14 +1678,6 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
   /****** grab keys and buttons ******/
   setup_key_and_button_grabs(tmp_win);
 
-  /****** now we can sefely ungrab the server ******/
-  MyXUngrabServer(dpy);
-
-  /****** inform modules of new window ******/
-  BroadcastConfig(M_ADD_WINDOW,tmp_win);
-  BroadcastName(M_WINDOW_NAME,tmp_win->w,tmp_win->frame,
-		(unsigned long)tmp_win,tmp_win->name);
-
   /****** place the window in the stack ring ******/
   if (!position_new_window_in_stack_ring(tmp_win, SDO_START_LOWERED(sflags)))
   {
@@ -1604,6 +1687,13 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
     XConfigureWindow(dpy, tmp_win->frame, CWSibling|CWStackMode, &xwc);
   }
 
+  /****** now we can sefely ungrab the server ******/
+  MyXUngrabServer(dpy);
+
+  /****** inform modules of new window ******/
+  BroadcastConfig(M_ADD_WINDOW,tmp_win);
+  BroadcastWindowIconNames(tmp_win, True, False);
+
   /* these are sent and broadcast before res_{class,name} for the benefit
    * of FvwmIconBox which can't handle M_ICON_FILE after M_RES_NAME */
   /****** icon and mini icon ******/
@@ -1655,6 +1745,9 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
   /****** window colormap ******/
   InstallWindowColormaps(colormap_win);
 
+  /****** ewmh setup *******/
+  EWMH_WindowInit(tmp_win);
+
   /****** gnome setup ******/
   /* set GNOME hints on the window from flags set on tmp_win */
   GNOME_SetHints(tmp_win);
@@ -1675,15 +1768,6 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
       tmp_win, tmp_win->frame_g.x, tmp_win->frame_g.y,
       tmp_win->frame_g.width, tmp_win->frame_g.height, False);
   }
-  if (!IS_SHADED(tmp_win) && !IS_ICONIFIED(tmp_win))
-  {
-    /* TK always wants some special treatment: If the window is simply mapped,
-     * the tk menus come up at funny Y coordinates.  Tell it its geometry
-     * *again* to work around this problem. */
-    SendConfigureNotify(
-      tmp_win, tmp_win->frame_g.x, tmp_win->frame_g.y, tmp_win->frame_g.width,
-      tmp_win->frame_g.height, 0, False);
-  }
   if (!XGetGeometry(dpy, tmp_win->w, &JunkRoot, &JunkX, &JunkY, &JunkWidth,
 		    &JunkHeight, &JunkBW,  &JunkDepth))
   {
@@ -1968,15 +2052,7 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 	broken_cause = "aspect ratio";
       tmp->hints.flags &= ~PAspect;
       fvwm_msg (WARN, "GetWindowSizeHints",
-		"The applicaton window \"%s\"\n"
-		"(window id %#lx)"
-		" has broken aspect ratio: %d/%d - %d/%d\n"
-		"Fvwm is ignoring the aspect ratio.  "
-		"If you are having a problem\n"
-		"with the application, send this message to"
-		" the application owner.\n"
-		"There is no need to notify "
-		"fvwm-workers@fvwm.org.\n",
+		"%s window %#lx has broken aspect ratio: %d/%d - %d/%d\n",
 		tmp->name, tmp->w, minAspectX, minAspectY, maxAspectX,
 	  	maxAspectY);
     }
@@ -2017,13 +2093,9 @@ void GetWindowSizeHints(FvwmWindow *tmp)
   if (*broken_cause != 0)
   {
     fvwm_msg(WARN, "GetWindowSizeHints",
-	     "The application window \"%s\"\n(window id %#lx) "
-	     "has broken size hints (%s).\n"
-	     "Fvwm is ignoring those hints.  "
-	     "If you are having a problem\n"
-	     "with the application, send this message to"
-	     " the application owner.\n"
-	     "There is no need to notify fvwm-workers@fvwm.org.\n"
+	     "%s window %#lx has broken (%s) size hints\n"
+	     "Please send a bug report to the application owner,\n"
+	     "\tyou may use fvwm-workers@fvwm.org as a reference.\n"
 	     "hint override = %d\n"
 	     "flags = %lx\n"
 	     "min_width = %d, min_height = %d, "
@@ -2066,11 +2138,7 @@ static void free_text_property(char **ptext, char ***ptext_list)
 #define FREE_TEXT_PROPERTY(a, b) free_text_property(a)
 static void free_text_property(char **ptext)
 {
-#ifdef COMPOUND_TEXT
-  free(*ptext);
-#else
   XFree(*ptext);
-#endif
   *ptext = NULL;
 }
 #endif
@@ -2085,15 +2153,53 @@ void free_window_names(FvwmWindow *tmp, Bool nukename, Bool nukeicon)
   if (!tmp)
     return;
 
+  if (nukename && tmp->visible_name &&
+      tmp->visible_name != tmp->name &&
+      tmp->visible_name != NoName)
+  {
+    free(tmp->visible_name);
+    tmp->visible_name = NULL;
+  }
+  if (nukeicon && tmp->visible_icon_name &&
+      tmp->visible_icon_name != tmp->name &&
+      tmp->visible_icon_name != tmp->icon_name &&
+      tmp->visible_icon_name != NoName)
+  {
+    free(tmp->visible_icon_name);
+    tmp->visible_icon_name = NULL;
+  }
   if (nukename && tmp->name)
   {
+#ifdef CODE_WITH_LEAK_I_THINK
     if (tmp->name != tmp->icon_name && tmp->name != NoName)
       FREE_TEXT_PROPERTY(&(tmp->name), &(tmp->name_list));
+#else
+    if (tmp->icon_name == tmp->name)
+      tmp->icon_name = NoName;
+    if (tmp->visible_icon_name == tmp->name)
+      tmp->visible_icon_name = tmp->icon_name;
+    if (tmp->name != NoName)
+    {
+      FREE_TEXT_PROPERTY(&(tmp->name), &(tmp->name_list));
+      tmp->visible_name = NULL;
+    }
+#endif
   }
   if (nukeicon && tmp->icon_name)
   {
+#ifdef CODE_WITH_LEAK_I_THINK
     if ((tmp->name != tmp->icon_name || nukename) && tmp->icon_name != NoName)
+    {
       FREE_TEXT_PROPERTY(&(tmp->icon_name), &(tmp->icon_name_list));
+      tmp->visible_icon_name = NULL;
+    }
+#else
+    if ((tmp->name != tmp->icon_name) && tmp->icon_name != NoName)
+    {
+      FREE_TEXT_PROPERTY(&(tmp->icon_name), &(tmp->icon_name_list));
+      tmp->visible_icon_name = NULL;
+    }
+#endif
   }
 
   return;
@@ -2134,20 +2240,10 @@ void destroy_window(FvwmWindow *tmp_win)
 
   /****** remove from window list ******/
 
-  if (!IS_SCHEDULED_FOR_DESTROY(tmp_win))
-  {
-    /* first of all, remove the window from the list of all windows! */
-    if (tmp_win->prev != NULL)
-    {
-      tmp_win->prev->next = tmp_win->next;
-    }
-    if (tmp_win->next != NULL)
-    {
-      tmp_win->next->prev = tmp_win->prev;
-    }
-    tmp_win->prev = NULL;
-    tmp_win->next = NULL;
-  }
+  /* first of all, remove the window from the list of all windows! */
+  tmp_win->prev->next = tmp_win->next;
+  if (tmp_win->next != NULL)
+    tmp_win->next->prev = tmp_win->prev;
 
   /****** also remove it from the stack ring ******/
 
@@ -2162,21 +2258,17 @@ void destroy_window(FvwmWindow *tmp_win)
 
   if (Scr.flags.is_executing_complex_function && !DO_REUSE_DESTROYED(tmp_win))
   {
-    if (IS_SCHEDULED_FOR_DESTROY(tmp_win))
-    {
-      return;
-    }
     /* mark window for destruction */
     SET_SCHEDULED_FOR_DESTROY(tmp_win, 1);
     Scr.flags.is_window_scheduled_for_destroy = 1;
     /* this is necessary in case the application destroys the client window and
      * a new window is created with the saem window id */
-    delete_client_context(tmp_win);
+    XDeleteContext(dpy, tmp_win->w, FvwmContext);
     /* unmap the the window to fake that it was already removed */
     if (IS_ICONIFIED(tmp_win))
     {
-      if (tmp_win->icon_w)
-	XUnmapWindow(dpy, tmp_win->icon_w);
+      if (tmp_win->icon_title_w)
+	XUnmapWindow(dpy, tmp_win->icon_title_w);
       if(tmp_win->icon_pixmap_w != None)
 	XUnmapWindow(dpy, tmp_win->icon_pixmap_w);
     }
@@ -2187,7 +2279,7 @@ void destroy_window(FvwmWindow *tmp_win)
     adjust_fvwm_internal_windows(tmp_win);
     BroadcastPacket(M_DESTROY_WINDOW, 3,
 		    tmp_win->w, tmp_win->frame, (unsigned long)tmp_win);
-
+    EWMH_DestroyWindow(tmp_win);
     return;
   }
 
@@ -2201,7 +2293,6 @@ void destroy_window(FvwmWindow *tmp_win)
   /* already done above? */
   if (!IS_SCHEDULED_FOR_DESTROY(tmp_win))
   {
-    SET_SCHEDULED_FOR_DESTROY(tmp_win, 1);
     /****** adjust fvwm internal windows and the focus ******/
 
     adjust_fvwm_internal_windows(tmp_win);
@@ -2210,6 +2301,7 @@ void destroy_window(FvwmWindow *tmp_win)
 
     BroadcastPacket(M_DESTROY_WINDOW, 3,
 		    tmp_win->w, tmp_win->frame, (unsigned long)tmp_win);
+    EWMH_DestroyWindow(tmp_win);
   }
 
   /****** adjust fvwm internal windows II ******/
@@ -2315,10 +2407,6 @@ void RestoreWithdrawnLocation(
   if(!tmp)
     return;
 
-  /* always get the latest size hints in case the application changed the
-   * gravity and we do not yet know about it */
-  XSync(dpy, 0);
-  GetWindowSizeHints(tmp);
   get_unshaded_geometry(tmp, &unshaded_g);
   gravity_get_naked_geometry(
     tmp->hints.win_gravity, tmp, &naked_g, &unshaded_g);
@@ -2384,8 +2472,8 @@ void RestoreWithdrawnLocation(
 
   if (IS_ICONIFIED(tmp) && !IS_ICON_SUPPRESSED(tmp))
   {
-    if (tmp->icon_w)
-      XUnmapWindow(dpy, tmp->icon_w);
+    if (tmp->icon_title_w)
+      XUnmapWindow(dpy, tmp->icon_title_w);
     if (tmp->icon_pixmap_w)
       XUnmapWindow(dpy, tmp->icon_pixmap_w);
   }
@@ -2498,6 +2586,8 @@ void CaptureOneWindow(
     XChangeProperty (dpy, fw->w, _XA_WM_DESKTOP, _XA_WM_DESKTOP, 32,
                      PropModeReplace, (unsigned char *) data, 1);
 
+    /* are all these really needed ? */
+    /* EWMH_SetWMDesktop(fw); */
     GNOME_SetHints(fw);
     GNOME_SetDesk(fw);
     GNOME_SetLayer(fw);
@@ -2511,16 +2601,10 @@ void CaptureOneWindow(
     destroy_window(fw);
     Event.xmaprequest.window = w;
     HandleMapRequestKeepRaised(keep_on_top_win, fw);
-    /* HandleMapRequestKeepRaised may have destroyed the fw if the window
-     * vanished while in AddWindow(), so don't access fw anymore before
-     * checking if it is a valid window. */
-    if (check_if_fvwm_window_exists(fw))
+    if (!fFvwmInStartup)
     {
-      if (!fFvwmInStartup)
-      {
-	SET_MAP_PENDING(fw, 0);
-	SET_MAPPED(fw, is_mapped);
-      }
+      SET_MAP_PENDING(fw, 0);
+      SET_MAPPED(fw, is_mapped);
     }
     /* Clean out isIconicState here, otherwise all new windos may start
      * iconified. */
@@ -2730,7 +2814,7 @@ void CaptureAllWindows(Bool is_recapture)
 	;
       if (t)
       {
-	SetFocusWindow(t, 0);
+	SetFocusWindow(t, False, True);
       }
     }
   }
