@@ -20,7 +20,9 @@ extern char NoName[];
 extern char NoClass[];
 extern char NoResource[];
 
+void setup_visible_name(FvwmWindow *tmp_win, Bool is_icon);
 void setup_wm_hints(FvwmWindow *tmp_win);
+void setup_placement_penalty(FvwmWindow *tmp_win, window_style *pstyle);
 void setup_focus_policy(FvwmWindow *tmp_win);
 void setup_window_font(
   FvwmWindow *tmp_win, window_style *pstyle, Bool do_destroy);
