@@ -24,6 +24,7 @@
 #include "functions.h"
 #include "bindings.h"
 #include "defaults.h"
+#include "module_interface.h"
 #include "misc.h"
 #include "screen.h"
 #include "focus.h"
@@ -34,12 +35,11 @@
 static void activate_binding(
   Binding *binding, BindingType type, Bool do_grab, Bool do_grab_root);
 
-#define MODS_UNUSED_DEFAULT LockMask
-static int mods_unused = MODS_UNUSED_DEFAULT;
+static int mods_unused = DEFAULT_MODS_UNUSED;
 
 
 static void update_nr_buttons(
-  int contexts, int *nr_left_buttons, int *nr_right_buttons, Bool do_set)
+  int contexts, int *nr_left_buttons, int *nr_right_buttons)
 {
   int i;
   int l = (nr_left_buttons) ? *nr_left_buttons : 0;
@@ -50,9 +50,10 @@ static void update_nr_buttons(
     /* check for nr_left_buttons */
     for (i = 0; i < NUMBER_OF_BUTTONS; i += 2)
     {
-      if ((contexts & (C_L1 << i)) &&
-	  (do_set || *nr_left_buttons <= i / 2))
+      if ((contexts & (C_L1 << i)) && *nr_left_buttons <= i / 2)
       {
+	Scr.flags.do_need_window_update = 1;
+	Scr.flags.has_nr_buttons_changed = 1;
 	*nr_left_buttons = i / 2 + 1;
       }
     }
@@ -62,9 +63,10 @@ static void update_nr_buttons(
     /* check for nr_right_buttons */
     for (i = 1; i < NUMBER_OF_BUTTONS; i += 2)
     {
-      if ((contexts & (C_L1 << i)) &&
-	  (do_set || *nr_right_buttons <= i / 2))
+      if ((contexts & (C_L1 << i)) && *nr_right_buttons <= i / 2)
       {
+	Scr.flags.do_need_window_update = 1;
+	Scr.flags.has_nr_buttons_changed = 1;
 	*nr_right_buttons = i / 2 + 1;
       }
     }
@@ -88,7 +90,7 @@ int ParseBinding(
   Bool do_ungrab_root, Bool is_silent)
 {
   char *action, context_string[20], modifier_string[20], *ptr, *token;
-  char key_string[201] = "";
+  char key_string[20] = "";
   int button = 0;
   STROKE_CODE(char stroke[STROKE_MAX_SEQUENCE + 1] = "";)
   int n1=0,n2=0,n3=0;
@@ -105,7 +107,7 @@ int ParseBinding(
   if (token != NULL)
   {
     if (type == KEY_BINDING)
-      n1 = sscanf(token,"%200s", key_string); /* see len of key_string above */
+      n1 = sscanf(token,"%19s", key_string);
 #ifdef HAVE_STROKE
     else if (type == STROKE_BINDING)
     {
@@ -256,7 +258,8 @@ int ParseBinding(
 	*buttons_grabbed = 0;
       for (b = *pblist; b != NULL; b = b->NextBinding)
       {
-	if(b->type == MOUSE_BINDING && (b->Context & C_WINDOW) &&
+	if(b->type == MOUSE_BINDING &&
+	   (b->Context & (C_WINDOW|C_EWMH_DESKTOP)) &&
 	   (b->Modifier == 0 || b->Modifier == AnyModifier) &&
 	   buttons_grabbed != NULL)
 	{
@@ -278,7 +281,11 @@ int ParseBinding(
 	  }
 	}
       }
-      update_nr_buttons(bcontext, nr_left_buttons, nr_right_buttons, True);
+      if (nr_left_buttons)
+	*nr_left_buttons = 0;
+      if (nr_right_buttons)
+	*nr_right_buttons = 0;
+      update_nr_buttons(bcontext, nr_left_buttons, nr_right_buttons);
     }
   }
 
@@ -286,7 +293,7 @@ int ParseBinding(
   if (is_unbind_request)
     return 0;
 
-  update_nr_buttons(context, nr_left_buttons, nr_right_buttons, False);
+  update_nr_buttons(context, nr_left_buttons, nr_right_buttons);
 
   if((modifier & AnyModifier)&&(modifier&(~AnyModifier)))
   {
@@ -295,7 +302,8 @@ int ParseBinding(
     modifier = AnyModifier;
   }
 
-  if ((type == MOUSE_BINDING)&&(context & C_WINDOW)&&
+  if ((type == MOUSE_BINDING)&&
+      (context & (C_WINDOW|C_EWMH_DESKTOP))&&
      (((modifier==0)||modifier == AnyModifier)) && (buttons_grabbed != NULL))
   {
     if (button == 0)
@@ -326,7 +334,8 @@ static void activate_binding(
   {
     /* necessary for key bindings that work over unfocused windows */
     GrabWindowKeyOrButton(
-      dpy, Scr.Root, binding, C_WINDOW|C_DECOR|C_ROOT|C_ICON,
+      dpy, Scr.Root, binding,
+      C_WINDOW|C_TITLE|C_RALL|C_LALL|C_SIDEBAR|C_ROOT|C_ICON|C_EWMH_DESKTOP,
       GetUnusedModifiers(), None, do_grab);
   }
   if (fFvwmInStartup == True)
@@ -335,7 +344,7 @@ static void activate_binding(
   /* grab keys immediately */
   for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
   {
-    if ((binding->Context & C_WINDOW) &&
+    if (!IS_EWMH_DESKTOP(t->w) && (binding->Context & C_WINDOW) &&
 	(binding->type == MOUSE_BINDING
 	 STROKE_CODE(|| binding->type == STROKE_BINDING)))
     {
@@ -344,18 +353,22 @@ static void activate_binding(
 	C_WINDOW,
 	GetUnusedModifiers(), None, do_grab);
     }
-    if ((binding->Context & (C_WINDOW|C_DECOR)) && binding->type == KEY_BINDING)
+    if (!IS_EWMH_DESKTOP(t->w) &&
+	(binding->Context & (C_WINDOW|C_TITLE|C_RALL|C_LALL|C_SIDEBAR))
+	&& binding->type == KEY_BINDING)
     {
       GrabWindowKey(
-	dpy, t->frame, binding, C_WINDOW|C_DECOR, GetUnusedModifiers(),
-	do_grab);
+	dpy, t->frame, binding,
+	C_WINDOW|C_TITLE|C_RALL|C_LALL|C_SIDEBAR,
+	GetUnusedModifiers(), do_grab);
     }
     if (binding->Context & C_ICON)
     {
-      if(t->icon_w != None)
+      if(t->icon_title_w != None)
       {
         GrabWindowKeyOrButton(
-	  dpy, t->icon_w, binding, C_ICON, GetUnusedModifiers(), None, do_grab);
+	  dpy, t->icon_title_w, binding, C_ICON, GetUnusedModifiers(), None,
+	  do_grab);
       }
       if(t->icon_pixmap_w != None)
       {
@@ -364,6 +377,13 @@ static void activate_binding(
 	  None, do_grab);
       }
     }
+    if (IS_EWMH_DESKTOP(t->w) && (binding->Context & C_EWMH_DESKTOP))
+    {
+      GrabWindowKeyOrButton(
+	dpy, t->Parent, binding,
+	C_EWMH_DESKTOP,
+	GetUnusedModifiers(), None, do_grab);
+    }
   }
 
   return;
@@ -445,22 +465,26 @@ unsigned int GetUnusedModifiers(void)
 void CMD_IgnoreModifiers(F_CMD_ARGS)
 {
   char *token;
+  int mods_unused_old = mods_unused;
 
-  mods_unused = 0;
-  GetNextToken(action, &token);
+  token = PeekToken(action, &action);
   if (!token)
-    return;
-
-  if (StrEquals(token, "default"))
   {
-    free(token);
-    mods_unused = MODS_UNUSED_DEFAULT;
-    return;
+    mods_unused = 0;
+  }
+  else if (StrEquals(token, "default"))
+  {
+    mods_unused = DEFAULT_MODS_UNUSED;
+  }
+  else if (ParseModifiers(token, &mods_unused))
+  {
+    fvwm_msg(ERR, "ignore_modifiers", "illegal modifier in line %s\n", action);
+  }
+  if (mods_unused != mods_unused_old)
+  {
+    /* broadcast config to modules */
+    broadcast_ignore_modifiers();
   }
 
-  if (ParseModifiers(token, &mods_unused))
-    fvwm_msg(ERR, "ignore_modifiers", "illegal modifier in line %s\n", action);
-
-  free(token);
   return;
 }
