@@ -51,6 +51,7 @@
 #include "icons.h"
 #include "module_interface.h"
 #include "libs/Colorset.h"
+#include "add_window.h"
 
 typedef struct
 {
@@ -223,7 +224,8 @@ static void clip_button_pixmap(
 static void DrawButton(
   FvwmWindow *t, Window win, int w, int h, DecorFace *df, GC ReliefGC,
   GC ShadowGC, Pixel fore_color, Pixel back_color, Bool is_lowest,
-  mwm_flags stateflags, int left1right0, XRectangle *rclip,
+  mwm_flags stateflags, int set_layer, int layer,
+  int left1right0, XRectangle *rclip,
   Pixmap *pbutton_background_pixmap)
 {
   register DecorFaceType type = DFS_FACE_TYPE(df->style);
@@ -255,7 +257,8 @@ static void DrawButton(
     if(HAS_MWM_BUTTONS(t) &&
        ((stateflags & MWM_DECOR_MAXIMIZE && IS_MAXIMIZED(t)) ||
 	(stateflags & MWM_DECOR_SHADE && IS_SHADED(t)) ||
-	(stateflags & MWM_DECOR_STICK && IS_STICKY(t))))
+	(stateflags & MWM_DECOR_STICK && IS_STICKY(t)) ||
+        (set_layer && (t->layer == layer))))
     {
       DrawLinePattern(win, ShadowGC, ReliefGC, fore_color, back_color,
         &df->u.vector, w, h);
@@ -271,10 +274,29 @@ static void DrawButton(
   case MiniIconButton:
   case PixmapButton:
   case TiledPixmapButton:
+#ifdef FANCY_TITLEBARS
+  case MultiPixmap:       /* in case of UseTitleStyle */
+#endif
     if (type == PixmapButton || type == TiledPixmapButton)
     {
       p = df->u.p;
     }
+#ifdef FANCY_TITLEBARS
+    else if (type == MultiPixmap) {
+      if (left1right0 && df->u.multi_pixmaps[TBP_LEFT_BUTTONS])
+        p = df->u.multi_pixmaps[TBP_LEFT_BUTTONS];
+      else if (!left1right0 && df->u.multi_pixmaps[TBP_RIGHT_BUTTONS])
+        p = df->u.multi_pixmaps[TBP_RIGHT_BUTTONS];
+      else if (df->u.multi_pixmaps[TBP_BUTTONS])
+        p = df->u.multi_pixmaps[TBP_BUTTONS];
+      else if (left1right0 && df->u.multi_pixmaps[TBP_LEFT_MAIN])
+        p = df->u.multi_pixmaps[TBP_LEFT_MAIN];
+      else if (!left1right0 && df->u.multi_pixmaps[TBP_RIGHT_MAIN])
+        p = df->u.multi_pixmaps[TBP_RIGHT_MAIN];
+      else
+        p = df->u.multi_pixmaps[TBP_MAIN];
+    }
+#endif
     else
     {
       if (!t->mini_icon)
@@ -298,6 +320,9 @@ static void DrawButton(
     if (is_lowest && !p->mask && pbutton_background_pixmap)
     {
       if (type == TiledPixmapButton ||
+#ifdef FANCY_TITLEBARS
+          type == MultiPixmap ||
+#endif
 	  (p && p->width >= width && p->height >= height))
       {
 	/* better performance: set a background pixmap if possible, i.e. if the
@@ -326,7 +351,11 @@ static void DrawButton(
 	XClearWindow(dpy, win);
       }
     }
-    if (type != TiledPixmapButton)
+    if (type != TiledPixmapButton
+#ifdef FANCY_TITLEBARS
+        && type != MultiPixmap
+#endif
+       )
     {
       if (DFS_H_JUSTIFICATION(df->style) == JUST_RIGHT)
 	x += (int)(width - p->width);
@@ -354,7 +383,11 @@ static void DrawButton(
 
     XSetClipMask(dpy, Scr.TransMaskGC, p->mask);
     XSetClipOrigin(dpy, Scr.TransMaskGC, x, y);
-    if (type != TiledPixmapButton)
+    if (type != TiledPixmapButton
+#ifdef FANCY_TITLEBARS
+        && type != MultiPixmap
+#endif
+       )
     {
       int cx = x;
       int cy = y;
@@ -448,6 +481,159 @@ static void DrawButton(
   return;
 }
 
+#ifdef FANCY_TITLEBARS
+
+/****************************************************************************
+ *
+ *  Tile or stretch src into dest, starting at the given location and
+ *  continuing for the given width and height. This is a utility function used
+ *  by DrawMultiPixmapTitlebar. (tril@igs.net)
+ *
+ ****************************************************************************/
+static void RenderIntoWindow(GC gc, Picture *src, Window dest, int x_start,
+                             int y_start, int width, int height, Bool stretch)
+{
+  Pixmap pm;
+  if (stretch)
+    pm = CreateStretchPixmap(dpy, src->picture, src->width, src->height,
+                             src->depth, width, height, gc);
+  else
+    pm = CreateTiledPixmap(dpy, src->picture, src->width, src->height,
+                           width, height, src->depth, gc);
+  XCopyArea(dpy, pm, dest, gc, 0, 0, width, height, x_start, y_start);
+  XFreePixmap(dpy, pm);
+}
+
+/****************************************************************************
+ *
+ *  Redraws multi-pixmap titlebar (tril@igs.net)
+ *
+ ****************************************************************************/
+static void DrawMultiPixmapTitlebar(FvwmWindow *window, DecorFace *df)
+{
+  Window       title_win;
+  GC           gc;
+  char        *title;
+  Picture    **pm;
+  short        stretch_flags;
+  int          title_width, title_height, text_width, text_offset, text_y_pos;
+  int          left_space, right_space, under_offset, under_width;
+
+  pm = df->u.multi_pixmaps;
+  stretch_flags = df->u.multi_stretch_flags;
+  gc = Scr.TitleGC;
+  XSetClipMask(dpy, gc, None);
+  title_win = window->title_w;
+  title = window->visible_name;
+  title_width = window->title_g.width;
+  title_height = window->title_g.height;
+
+  if (pm[TBP_MAIN])
+    RenderIntoWindow(gc, pm[TBP_MAIN], title_win, 0, 0, title_width,
+                     title_height, (stretch_flags & (1 << TBP_MAIN)));
+  else if (!title)
+    RenderIntoWindow(gc, pm[TBP_LEFT_MAIN], title_win, 0, 0, title_width,
+                     title_height, (stretch_flags & (1 << TBP_LEFT_MAIN)));
+
+  if (title) {
+#ifdef I18N_MB
+    text_width = XmbTextEscapement(window->title_font.fontset, title,
+                                   strlen(title));
+#else
+    text_width = XTextWidth(window->title_font.font, title, strlen(title));
+#endif
+    if (text_width > title_width)
+      text_width = title_width;
+    switch (TB_JUSTIFICATION(GetDecor(window, titlebar))) {
+    case JUST_LEFT:
+      text_offset = TITLE_PADDING;
+      if (pm[TBP_LEFT_OF_TEXT])
+        text_offset += pm[TBP_LEFT_OF_TEXT]->width;
+      if (pm[TBP_LEFT_END])
+        text_offset += pm[TBP_LEFT_END]->width;
+      if (text_offset > title_width - text_width)
+        text_offset = title_width - text_width;
+      break;
+    case JUST_RIGHT:
+      text_offset = title_width - text_width - TITLE_PADDING;
+      if (pm[TBP_RIGHT_OF_TEXT])
+        text_offset -= pm[TBP_RIGHT_OF_TEXT]->width;
+      if (pm[TBP_RIGHT_END])
+        text_offset -= pm[TBP_RIGHT_END]->width;
+      if (text_offset < 0)
+        text_offset = 0;
+      break;
+    default:
+      text_offset = (title_width - text_width) / 2;
+      break;
+    }
+    under_offset = text_offset;
+    under_width = text_width;
+    /* If there's title padding, put it *inside* the undertext area: */
+    if (under_offset >= TITLE_PADDING) {
+      under_offset -= TITLE_PADDING;
+      under_width += TITLE_PADDING;
+    }
+    if (under_offset + under_width + TITLE_PADDING <= title_width)
+      under_width += TITLE_PADDING;
+    left_space = under_offset;
+    right_space = title_width - left_space - under_width;
+
+    if (pm[TBP_LEFT_MAIN] && left_space > 0)
+      RenderIntoWindow(gc, pm[TBP_LEFT_MAIN], title_win, 0, 0, left_space,
+                       title_height, (stretch_flags & (1 << TBP_LEFT_MAIN)));
+    if (pm[TBP_RIGHT_MAIN] && right_space > 0)
+      RenderIntoWindow(gc, pm[TBP_RIGHT_MAIN], title_win,
+                       under_offset + under_width, 0, right_space,
+                       title_height, (stretch_flags & (1 << TBP_RIGHT_MAIN)));
+    if (pm[TBP_UNDER_TEXT] && under_width > 0)
+      RenderIntoWindow(gc, pm[TBP_UNDER_TEXT], title_win, under_offset, 0,
+                       under_width, title_height,
+                       (stretch_flags & (1 << TBP_UNDER_TEXT)));
+    if (pm[TBP_LEFT_OF_TEXT] &&
+        pm[TBP_LEFT_OF_TEXT]->width <= left_space) {
+      XCopyArea(dpy, pm[TBP_LEFT_OF_TEXT]->picture, title_win, gc, 0, 0,
+                pm[TBP_LEFT_OF_TEXT]->width, title_height,
+                under_offset - pm[TBP_LEFT_OF_TEXT]->width, 0);
+      left_space -= pm[TBP_LEFT_OF_TEXT]->width;
+    }
+    if (pm[TBP_RIGHT_OF_TEXT] &&
+        pm[TBP_RIGHT_OF_TEXT]->width <= right_space) {
+      XCopyArea(dpy, pm[TBP_RIGHT_OF_TEXT]->picture, title_win, gc, 0, 0,
+                pm[TBP_RIGHT_OF_TEXT]->width, title_height,
+                under_offset + under_width, 0);
+      right_space -= pm[TBP_RIGHT_OF_TEXT]->width;
+    }
+
+    text_y_pos = title_height
+                 - (title_height - window->title_font.font->ascent) / 2
+                 - 1;
+    if (title_height > 19)
+      text_y_pos--;
+    if (text_y_pos < 0)
+      text_y_pos = 0;
+#ifdef I18N_MB
+    XmbDrawString(dpy, title_win, window->title_font.fontset,
+                  gc, text_offset, text_y_pos, title, strlen(title));
+#else
+    XDrawString(dpy, title_win, gc, text_offset, text_y_pos, title,
+                strlen(title));
+#endif
+  }
+  else
+    left_space = right_space = title_width;
+
+  if (pm[TBP_LEFT_END] && pm[TBP_LEFT_END]->width <= left_space)
+    XCopyArea(dpy, pm[TBP_LEFT_END]->picture, title_win, gc, 0, 0,
+              pm[TBP_LEFT_END]->width, title_height, 0, 0);
+  if (pm[TBP_RIGHT_END] && pm[TBP_RIGHT_END]->width <= right_space)
+    XCopyArea(dpy, pm[TBP_RIGHT_END]->picture, title_win, gc, 0, 0,
+              pm[TBP_RIGHT_END]->width, title_height,
+              title_width - pm[TBP_RIGHT_END]->width, 0);
+}
+
+#endif /* FANCY_TITLEBARS */
+
 /* rules to get button state */
 static enum ButtonState get_button_state(
   Bool has_focus, Bool toggled, Window w)
@@ -1029,6 +1215,7 @@ static void RedrawButtons(
   Window expose_win, XRectangle *rclip)
 {
   int i;
+  int left1right0;
   Bool is_lowest = True;
   Pixmap *pass_bg_pixmap;
 
@@ -1042,8 +1229,10 @@ static void RedrawButtons(
 
   for (i = 0; i < NUMBER_OF_BUTTONS; i++)
   {
-    if (t->button_w[i] != None && (((i & 1) && i / 2 < Scr.nr_right_buttons) ||
-				   (!(i & 1) && i / 2 < Scr.nr_left_buttons)))
+    left1right0 = !(i&1);
+    if (t->button_w[i] != None &&
+        ((!left1right0 && i/2 < Scr.nr_right_buttons) ||
+         (left1right0  && i/2 < Scr.nr_left_buttons)))
     {
       mwm_flags stateflags =
 	TB_MWM_DECOR_FLAGS(GetDecor(t, buttons[i]));
@@ -1090,28 +1279,28 @@ static void RedrawButtons(
 	  DecorFace *tsdf = &TB_STATE(GetDecor(t, titlebar))[bs];
 
 	  is_lowest = True;
-#ifdef MULTISTYLE
 	  for (; tsdf; tsdf = tsdf->next)
-#endif
 	  {
 	    DrawButton(t, t->button_w[i], t->title_g.height, t->title_g.height,
 		       tsdf, cd->relief_gc, cd->shadow_gc,
 		       cd->fore_color, cd->back_color, is_lowest,
-		       TB_MWM_DECOR_FLAGS(GetDecor(t, buttons[i])), 1, NULL,
+ 		       TB_MWM_DECOR_FLAGS(GetDecor(t, buttons[i])),
+		       TB_FLAGS(t->decor->buttons[i]).has_layer,
+		       TB_LAYER(t->decor->buttons[i]), left1right0, NULL,
 		       pass_bg_pixmap);
 	    is_lowest = False;
 	  }
 	}
 	is_lowest = True;
-#ifdef MULTISTYLE
 	for (; df; df = df->next)
-#endif
 	{
 	  DrawButton(t, t->button_w[i], t->title_g.height, t->title_g.height,
-		     df, cd->relief_gc, cd->shadow_gc,
-		     cd->fore_color, cd->back_color, is_lowest,
-		     TB_MWM_DECOR_FLAGS(GetDecor(t, buttons[i])), 1, NULL,
-		     pass_bg_pixmap);
+  		     df, cd->relief_gc, cd->shadow_gc,
+  		     cd->fore_color, cd->back_color, is_lowest,
+ 		     TB_MWM_DECOR_FLAGS(GetDecor(t, buttons[i])),
+		     TB_FLAGS(t->decor->buttons[i]).has_layer,
+		     TB_LAYER(t->decor->buttons[i]),
+		     left1right0, NULL, pass_bg_pixmap);
 	  is_lowest = False;
 	}
 
@@ -1176,12 +1365,13 @@ static void RedrawTitle(
   flush_expose(t->title_w);
 #endif
 
-  if (t->name != (char *)NULL)
+  if (t->visible_name != (char *)NULL)
   {
 #ifdef I18N_MB /* cannot use macro here, rewriting... */
-    w = XmbTextEscapement(t->title_font.fontset,t->name, strlen(t->name));
+    w = XmbTextEscapement(t->title_font.fontset, t->visible_name,
+			  strlen(t->visible_name));
 #else
-    w = XTextWidth(t->title_font.font, t->name, strlen(t->name));
+    w = XTextWidth(t->title_font.font, t->visible_name, strlen(t->visible_name));
 #endif
     if (w > t->title_g.width - 12)
       w = t->title_g.width - 4;
@@ -1246,6 +1436,19 @@ static void RedrawTitle(
 
   if (Pdepth < 2)
   {
+    XFillRectangle(
+      dpy, t->title_w, ((PressedW == t->title_w) ? sgc : rgc), hor_off - 2, 0,
+      w+4,t->title_g.height);
+    if(t->visible_name != (char *)NULL)
+#ifdef I18N_MB
+      XmbDrawString(dpy, t->title_w, t->title_font.fontset,
+		    Scr.TitleGC, hor_off,
+		    t->title_font.y + 1,
+		    t->visible_name, strlen(t->visible_name));
+#else
+      XDrawString(dpy, t->title_w, Scr.TitleGC, hor_off,
+                  t->title_font.y + 1, t->visible_name, strlen(t->visible_name));
+#endif
     /* for mono, we clear an area in the title bar where the window
      * title goes, so that its more legible. For color, no need */
     RelieveRectangle(
@@ -1254,21 +1457,9 @@ static void RedrawTitle(
     RelieveRectangle(
       dpy, t->title_w, hor_off + w + 2, 0, t->title_g.width - w - hor_off - 3,
       t->title_g.height - 1, rgc, sgc, cd->relief_width);
-    XFillRectangle(
-      dpy, t->title_w, ((PressedW == t->title_w) ? sgc : rgc), hor_off - 2, 0,
-      w+4,t->title_g.height);
     XDrawLine(
       dpy, t->title_w, sgc, hor_off + w + 1, 0, hor_off + w + 1,
       t->title_g.height);
-    if(t->name != (char *)NULL)
-#ifdef I18N_MB
-      XmbDrawString(dpy, t->title_w, t->title_font.fontset,
-		    Scr.TitleGC, hor_off,
-		    t->title_font.y + 1, t->name, strlen(t->name));
-#else
-      XDrawString(dpy, t->title_w, Scr.TitleGC, hor_off,
-                  t->title_font.y + 1, t->name, strlen(t->name));
-#endif
   }
   else
   {
@@ -1276,32 +1467,56 @@ static void RedrawTitle(
     /* draw compound titlebar (veliaa@rpi.edu) */
     Bool is_lowest = True;
 
+#ifdef FANCY_TITLEBARS
+    if (df->style.face_type == MultiPixmap)
+      DrawMultiPixmapTitlebar(t, df);
+    else
+#endif
     if (PressedW == t->title_w)
     {
-#ifdef MULTISTYLE
       for (; df; df = df->next)
-#endif
       {
 	DrawButton(
 	  t, t->title_w, t->title_g.width, t->title_g.height, df, sgc,
-	  rgc, cd->fore_color, cd->back_color, is_lowest, 0, 1, rclip,
+	  rgc, cd->fore_color, cd->back_color, is_lowest, 0,
+          0, 0,
+          1, rclip,
           pass_bg_pixmap);
 	is_lowest = False;
       }
     }
     else
     {
-#ifdef MULTISTYLE
       for (; df; df = df->next)
-#endif
       {
 	DrawButton(
 	  t, t->title_w, t->title_g.width, t->title_g.height, df, rgc,
-	  sgc, cd->fore_color, cd->back_color, is_lowest, 0, 1, rclip,
+	  sgc, cd->fore_color, cd->back_color, is_lowest, 0,
+          0, 0,
+          1, rclip,
           pass_bg_pixmap);
 	is_lowest = False;
       }
     }
+
+#ifdef FANCY_TITLEBARS
+    if (TB_STATE(GetDecor(t, titlebar))[title_state].style.face_type
+        != MultiPixmap)
+#endif
+    {
+#ifdef I18N_MB
+    if(t->visible_name != (char *)NULL)
+      XmbDrawString(dpy, t->title_w, t->title_font.fontset,
+		    Scr.TitleGC, hor_off,
+		    t->title_font.y + 1,
+		    t->visible_name, strlen(t->visible_name));
+#else
+    if(t->visible_name != (char *)NULL)
+      XDrawString(dpy, t->title_w, Scr.TitleGC, hor_off,
+		  t->title_font.y + 1, t->visible_name, strlen(t->visible_name));
+#endif
+    }
+
     /*
      * draw title relief
      */
@@ -1318,17 +1533,6 @@ static void RedrawTitle(
       /* flat */
       break;
     }
-
-#ifdef I18N_MB
-    if(t->name != (char *)NULL)
-      XmbDrawString(dpy, t->title_w, t->title_font.fontset,
-		    Scr.TitleGC, hor_off,
-		    t->title_font.y + 1, t->name, strlen(t->name));
-#else
-    if(t->name != (char *)NULL)
-      XDrawString(dpy, t->title_w, Scr.TitleGC, hor_off,
-		  t->title_font.y + 1, t->name, strlen(t->name));
-#endif
   }
 
   /*
@@ -1581,7 +1785,7 @@ static void get_common_decorations(
 
 void draw_clipped_decorations(
   FvwmWindow *t, draw_window_parts draw_parts, Bool has_focus, int force,
-  Window expose_win, XRectangle *rclip)
+  Window expose_win, XRectangle *rclip, clear_window_parts clear_parts)
 {
   common_decorations_type cd;
   Bool is_frame_redraw_allowed = False;
@@ -1628,7 +1832,7 @@ void draw_clipped_decorations(
       {
 	/* make sure that the previously highlighted window got
 	 * unhighlighted */
-	DrawDecorations(Scr.Hilite, DRAW_ALL, False, True, None);
+	DrawDecorations(Scr.Hilite, DRAW_ALL, False, True, None, CLEAR_ALL);
       }
       Scr.Hilite = t;
     }
@@ -1694,7 +1898,7 @@ void draw_clipped_decorations(
   {
     get_common_decorations(
       &cd, t, draw_parts, has_focus, force, expose_win, True, True);
-    if (!rclip || !IS_WINDOW_BORDER_DRAWN(t))
+    if ((clear_parts & CLEAR_FRAME) && (!rclip || !IS_WINDOW_BORDER_DRAWN(t)))
     {
       change_window_background(t->decor_w, cd.valuemask, &cd.attributes);
       SET_WINDOW_BORDER_DRAWN(t, 1);
@@ -1709,9 +1913,10 @@ void draw_clipped_decorations(
 
 void DrawDecorations(
   FvwmWindow *t, draw_window_parts draw_parts, Bool has_focus, int force,
-  Window expose_win)
+  Window expose_win, clear_window_parts clear_parts)
 {
-  draw_clipped_decorations(t, draw_parts, has_focus, force, expose_win, NULL);
+  draw_clipped_decorations(
+    t, draw_parts, has_focus, force, expose_win, NULL, clear_parts);
 
   return;
 }
@@ -2187,7 +2392,7 @@ void CMD_ButtonState(F_CMD_ARGS)
     else if (StrEquals("inactive", token))
     {
       Scr.gs.use_inactive_buttons =
-	ParseToggleArgument(action, &action, DEFAULT_USE_INACTIVE_BUTTONS,
+	ParseToggleArgument(action, &action, DEFAULT_USE_ACTIVE_DOWN_BUTTONS,
 			    True);
     }
     else
@@ -2229,9 +2434,7 @@ void CMD_BorderStyle(F_CMD_ARGS)
       DecorFace tmpdf, *df;
       memset(&tmpdf.style, 0, sizeof(tmpdf.style));
       DFS_FACE_TYPE(tmpdf.style) = SimpleButton;
-#ifdef MULTISTYLE
       tmpdf.next = NULL;
-#endif
 #ifdef MINI_ICONS
       tmpdf.u.p = NULL;
 #endif
@@ -2287,9 +2490,7 @@ void CMD_BorderStyle(F_CMD_ARGS)
       DecorFace tmpdf;
       memset(&tmpdf.style, 0, sizeof(tmpdf.style));
       DFS_FACE_TYPE(tmpdf.style) = SimpleButton;
-#ifdef MULTISTYLE
       tmpdf.next = NULL;
-#endif
 #ifdef MINI_ICONS
       tmpdf.u.p = NULL;
 #endif
@@ -2305,3 +2506,22 @@ void CMD_BorderStyle(F_CMD_ARGS)
     }
   }
 }
+
+/****************************************************************************
+ *
+ *  redraw the decoration when style change
+ *
+ ****************************************************************************/
+void RedrawDecorations(FvwmWindow *tmp_win)
+{
+  FvwmWindow *u = Scr.Hilite;
+
+  if (IS_SHADED(tmp_win))
+    XRaiseWindow(dpy, tmp_win->decor_w);
+  /* domivogt (6-Jun-2000): Don't check if the window is visible here.  If we
+   * do, some updates are not applied and when the window becomes visible
+   * again, the X Server may not redraw the window. */
+  DrawDecorations(
+    tmp_win, DRAW_ALL, (Scr.Hilite == tmp_win), 2, None, CLEAR_ALL);
+  Scr.Hilite = u;
+}
