@@ -24,10 +24,6 @@
 #include "config.h"
 
 #include <stdio.h>
-#include <unistd.h>		/* for STDIN_FILENO */
-#include <sys/types.h>		/* for open */
-#include <sys/stat.h>		/* for open */
-#include <fcntl.h>		/* for open */
 #include <signal.h>
 #include <errno.h>
 #include <X11/keysym.h>
@@ -51,6 +47,7 @@
 #include "geometry.h"
 #include "events.h"
 #include "gnome.h"
+#include "ewmh.h"
 #include "icons.h"
 #include "menus.h"
 #include "virtual.h"
@@ -67,6 +64,9 @@
 
 static char *ReadTitleButton(
     char *s, TitleButton *tb, Boolean append, int button);
+#ifdef FANCY_TITLEBARS
+static char *ReadMultiPixmapDecor(char *s, DecorFace *df);
+#endif
 static void DestroyFvwmDecor(FvwmDecor *decor);
 
 extern float rgpctMovementDefault[32];
@@ -245,18 +245,18 @@ void CMD_WindowShade(F_CMD_ARGS)
       /* make the decor window the full sze before the animation unveils it */
       XMoveResizeWindow(dpy, tmp_win->decor_w, 0, HAS_BOTTOM_TITLE(tmp_win)
         ? frame_g.height - big_g.height : 0, big_g.width, big_g.height);
+      tmp_win->frame_g.width = big_g.width;
+      tmp_win->frame_g.height = big_g.height;
       /* draw the border decoration iff backing store is on */
       if (Scr.use_backing_store != NotUseful)
       { /* eek! fvwm doesn't know the backing_store setting for windows */
         XWindowAttributes xwa;
 
         if (XGetWindowAttributes(dpy, tmp_win->decor_w, &xwa)
-            && xwa.backing_store != NotUseful)
+	    && xwa.backing_store != NotUseful)
         {
-          tmp_win->frame_g.width = big_g.width;
-          tmp_win->frame_g.height = big_g.height;
           DrawDecorations(
-	    tmp_win, DRAW_FRAME, sf == tmp_win, True, None);
+	    tmp_win, DRAW_FRAME, sf == tmp_win, True, None, CLEAR_ALL);
         }
       }
       while (frame_g.height + diff.height < big_g.height)
@@ -315,6 +315,8 @@ void CMD_WindowShade(F_CMD_ARGS)
 	    dpy, tmp_win->frame, FShapeBounding, 0, 0, shape_w, FShapeBounding,
 	    FShapeSet);
 	}
+	DrawDecorations(
+	  tmp_win, DRAW_FRAME, sf == tmp_win, True, None, CLEAR_NONE);
         FlushAllMessageQueues();
         XSync(dpy, 0);
       }
@@ -498,10 +500,12 @@ void CMD_WindowShade(F_CMD_ARGS)
       M_WINDOWSHADE, 3, tmp_win->w, tmp_win->frame, (unsigned long)tmp_win);
   }
   DrawDecorations(
-    tmp_win, DRAW_FRAME | DRAW_BUTTONS, (Scr.Hilite == tmp_win), True, None);
+    tmp_win, DRAW_FRAME | DRAW_BUTTONS, (Scr.Hilite == tmp_win),
+    True, None, CLEAR_NONE);
   FlushAllMessageQueues();
   XSync(dpy, 0);
 
+  EWMH_SetWMState(tmp_win, False);
   GNOME_SetHints(tmp_win);
   GNOME_SetWinArea(tmp_win);
 }
@@ -594,8 +598,12 @@ void CMD_CursorMove(F_CMD_ARGS)
     return;
   }
 
-  XQueryPointer( dpy, Scr.Root, &JunkRoot, &JunkChild,
-                 &x, &y, &JunkX, &JunkY, &JunkMask);
+  if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
+		    &x, &y, &JunkX, &JunkY, &JunkMask) == False)
+  {
+    /* pointer is on a different screen */
+    return;
+  }
 
   x = x + val1 * val1_unit / 100;
   y = y + val2 * val2_unit / 100;
@@ -660,7 +668,7 @@ void CMD_Destroy(F_CMD_ARGS)
   if (DeferExecution(eventp,&w,&tmp_win,&context, CRS_DESTROY, ButtonRelease))
     return;
 
-  if(check_if_function_allowed(F_DESTROY,tmp_win,True,NULL) == 0)
+  if(!is_function_allowed(F_DESTROY, NULL, tmp_win, True, True))
   {
     XBell(dpy, 0);
     return;
@@ -681,7 +689,7 @@ void CMD_Delete(F_CMD_ARGS)
   if (DeferExecution(eventp,&w,&tmp_win,&context, CRS_DESTROY,ButtonRelease))
     return;
 
-  if(check_if_function_allowed(F_DELETE,tmp_win,True,NULL) == 0)
+  if (!is_function_allowed(F_DELETE, NULL, tmp_win, True, True))
   {
     XBell(dpy, 0);
     return;
@@ -704,7 +712,7 @@ void CMD_Close(F_CMD_ARGS)
   if (DeferExecution(eventp,&w,&tmp_win,&context, CRS_DESTROY,ButtonRelease))
     return;
 
-  if(check_if_function_allowed(F_CLOSE,tmp_win,True,NULL) == 0)
+  if (!is_function_allowed(F_CLOSE, NULL, tmp_win, True, True))
   {
     XBell(dpy, 0);
     return;
@@ -786,10 +794,6 @@ void CMD_Exec(F_CMD_ARGS)
   /* Not everyone has vfork! */
   if (!(fork())) /* child process */
   {
-    /* This is for fixing a problem with rox filer */
-    int fd;
-    fd = open("/dev/null", O_RDONLY, 0);
-    dup2(fd,STDIN_FILENO);
     if (execl(exec_shell_name, exec_shell_name, "-c", cmd, NULL)==-1)
     {
       fvwm_msg(ERR,"exec_function","execl failed (%s)",strerror(errno));
@@ -842,7 +846,7 @@ void CMD_RefreshWindow(F_CMD_ARGS)
 {
   if (DeferExecution(eventp,&w,&tmp_win,&context,CRS_SELECT,ButtonRelease))
     return;
-  refresh_window((context == C_ICON)? tmp_win->icon_w : tmp_win->frame);
+  refresh_window((context == C_ICON)? tmp_win->icon_title_w : tmp_win->frame);
 }
 
 
@@ -1071,7 +1075,7 @@ void CMD_HilightColor(F_CMD_ARGS)
   if (Scr.cur_decor && Scr.cur_decor != &Scr.DefaultDecor)
   {
     fvwm_msg(
-      ERR, "CMD_HilightColor",
+      ERR, "SetHiColor",
       "Decors do not support the HilightColor command anymore."
       " Please use 'Style <stylename> HilightFore <forecolor>' and"
       " 'Style <stylename> HilightBack <backcolor>' instead."
@@ -1102,7 +1106,7 @@ void CMD_HilightColorset(F_CMD_ARGS)
   if (Scr.cur_decor && Scr.cur_decor != &Scr.DefaultDecor)
   {
     fvwm_msg(
-      ERR, "CMD_HilightColorset",
+      ERR, "SetHiColorset",
       "Decors do not support the HilightColorset command anymore."
       " Please use 'Style <stylename> HilightColorset <colorset>' instead."
       " Sorry for the inconvenience.");
@@ -1184,7 +1188,6 @@ void CMD_TitleStyle(F_CMD_ARGS)
   do_title_style(F_PASS_ARGS, False);
 } /* SetTitleStyle */
 
-#if defined(MULTISTYLE)
 /*****************************************************************************
  *
  * Appends a titlestyle (veliaa@rpi.edu)
@@ -1194,7 +1197,6 @@ void CMD_AddTitleStyle(F_CMD_ARGS)
 {
   do_title_style(F_PASS_ARGS, True);
 }
-#endif /* MULTISTYLE */
 
 void ApplyDefaultFontAndColors(void)
 {
@@ -1252,21 +1254,6 @@ void CMD_Colorset(F_CMD_ARGS)
   int n = -1, ret;
   char *token;
 
-  /* migo (19/May/2002): this block also fixes a crash with new configs */
-  char *ptr;
-  for (ptr = action; *ptr; ptr++)
-  {
-    if (*ptr != ' ' && !isxdigit(*ptr))
-    {
-      /* suppose it is a new Colorset command and pass it to FvwmTheme */
-      char *old_action = action;
-      action = CatString2("FvwmTheme Colorset ", action);
-      CMD_SendToModule(F_PASS_ARGS);
-      action = old_action;
-      return;
-    }
-  }
-
   token = PeekToken(action, NULL);
   if (token == NULL)
     return;
@@ -1275,9 +1262,7 @@ void CMD_Colorset(F_CMD_ARGS)
   if ((ret == 0) || (n < 0))
     return;
 
-  /* add a check to avoid crash in intentionally-incorect-config */
-  if (LoadColorset(action) < 0)
-    return;
+  LoadColorset(action);
   BroadcastColorset(n);
 
   if (n == Scr.DefaultColorset)
@@ -1426,6 +1411,10 @@ void CMD_WindowFont(F_CMD_ARGS)
 
 void FreeDecorFace(Display *dpy, DecorFace *df)
 {
+#ifdef FANCY_TITLEBARS
+  int i;
+#endif
+
   switch (DFS_FACE_TYPE(df->style))
   {
   case GradientButton:
@@ -1443,6 +1432,18 @@ void FreeDecorFace(Display *dpy, DecorFace *df)
       DestroyPicture(dpy, df->u.p);
     break;
 
+#ifdef FANCY_TITLEBARS
+  case MultiPixmap:
+    if (df->u.multi_pixmaps) {
+      for (i=0; i < NUM_TB_PIXMAPS; i++) {
+        if (df->u.multi_pixmaps[i])
+          DestroyPicture(dpy, df->u.multi_pixmaps[i]);
+      }
+      free(df->u.multi_pixmaps);
+    }
+    break;
+#endif
+
   case VectorButton:
   case DefaultVectorButton:
     if (df->u.vector.x)
@@ -1453,7 +1454,6 @@ void FreeDecorFace(Display *dpy, DecorFace *df)
     /* see below */
     break;
   }
-#ifdef MULTISTYLE
   /* delete any compound styles */
   if (df->next)
   {
@@ -1461,7 +1461,6 @@ void FreeDecorFace(Display *dpy, DecorFace *df)
     free(df->next);
   }
   df->next = NULL;
-#endif
   memset(&df->style, 0, sizeof(df->style));
   memset(&df->u, 0, sizeof(df->u));
   DFS_FACE_TYPE(df->style) = SimpleButton;
@@ -1480,7 +1479,7 @@ Bool ReadDecorFace(char *s, DecorFace *df, int button, int verbose)
 
   /* some variants of scanf do not increase the assign count when %n is used,
    * so a return value of 1 is no error. */
-  if (sscanf(s, "%255s%n", style, &offset) < 1)
+  if (sscanf(s, "%256s%n", style, &offset) < 1)
   {
     if (verbose)
     {
@@ -1674,6 +1673,19 @@ Bool ReadDecorFace(char *s, DecorFace *df, int button, int verbose)
       else
 	DFS_FACE_TYPE(df->style) = PixmapButton;
     }
+#ifdef FANCY_TITLEBARS
+    else if (strncasecmp(style,"MultiPixmap",11)==0) {
+      if (button != -1) {
+        if (verbose)
+          fvwm_msg(ERR, "ReadDecorFace",
+                   "MultiPixmap is only valid for TitleStyle");
+        return False;
+      }
+      s = ReadMultiPixmapDecor(s, df);
+      if (!s)
+        return False;
+    }
+#endif
 #ifdef MINI_ICONS
     else if (strncasecmp (style, "MiniIcon", 8) == 0)
     {
@@ -1885,7 +1897,7 @@ static char *ReadTitleButton(
     {
       for (i = 0; i < BS_MaxButtonState; ++i)
       {
-	ReadDecorFace(spec, &TB_STATE(*tb)[i], button, !i);
+	ReadDecorFace(spec, &TB_STATE(*tb)[i], BS_All, !i);
       }
     }
     else
@@ -1896,7 +1908,6 @@ static char *ReadTitleButton(
   else if (ReadDecorFace(spec, &tmpdf, button, True))
   {
     int b = (do_set_all) ? 0 : bs;
-#ifdef MULTISTYLE
     if (append)
     {
       DecorFace *head = &TB_STATE(*tb)[b];
@@ -1958,7 +1969,6 @@ static char *ReadTitleButton(
       }
     }
     else
-#endif
     {
       FreeDecorFace(dpy, &TB_STATE(*tb)[b]);
       memcpy(&(TB_STATE(*tb)[b]), &tmpdf, sizeof(DecorFace));
@@ -1981,6 +1991,78 @@ static char *ReadTitleButton(
   return end;
 }
 
+#ifdef FANCY_TITLEBARS
+/*****************************************************************************
+ *
+ * Reads a multi-pixmap titlebar config. (tril@igs.net)
+ *
+ ****************************************************************************/
+static char *ReadMultiPixmapDecor(char *s, DecorFace *df)
+{
+  static char *pm_names[NUM_TB_PIXMAPS+1] = {
+    "Main",
+    "LeftMain",
+    "RightMain",
+    "UnderText",
+    "LeftOfText",
+    "RightOfText",
+    "LeftEnd",
+    "RightEnd",
+    "Buttons",
+    "LeftButtons",
+    "RightButtons",
+    NULL
+  };
+  Picture **pm;
+  char *token;
+  Bool stretched;
+  int pm_id, i = 0;
+
+  df->style.face_type = MultiPixmap;
+  df->u.multi_pixmaps = pm =
+    (Picture**)safecalloc(NUM_TB_PIXMAPS, sizeof(Picture*));
+
+  s = GetNextTokenIndex(s, pm_names, 0, &pm_id);
+  while (pm_id >= 0) {
+    s = DoPeekToken(s, &token, ",()", NULL, NULL);
+    stretched = False;
+    if (StrEquals(token, "stretched")) {
+      stretched = True;
+      s = DoPeekToken(s, &token, ",", NULL, NULL);
+    }
+    else if (StrEquals(token, "tiled"))
+      s = DoPeekToken(s, &token, ",", NULL, NULL);
+    if (!token)
+      break;
+    if (pm[pm_id]) {
+      fvwm_msg(WARN, "ReadMultiPixmapDecor",
+               "Ignoring already-specified %s pixmap", pm_names[i]);
+      continue;
+    }
+    if (stretched)
+      df->u.multi_stretch_flags |= (1 << pm_id);
+    pm[pm_id] = CachePicture(dpy, Scr.NoFocusWin, NULL, token, Scr.ColorLimit);
+    if (!pm[pm_id])
+      fvwm_msg(ERR, "ReadMultiPixmapDecor", "Pixmap '%s' could not be loaded",
+               token);
+    s = GetNextTokenIndex(s, pm_names, 0, &pm_id);
+  }
+
+  if (!pm[TBP_MAIN] && !(pm[TBP_LEFT_MAIN] && pm[TBP_RIGHT_MAIN])) {
+    fvwm_msg(ERR, "ReadMultiPixmapDecor",
+             "No Main pixmap found for TitleStyle MultiPixmap "
+             "(you must specify either Main, or both LeftMain and RightMain)");
+    for (i=0; i < NUM_TB_PIXMAPS; i++) {
+      if (pm[i])
+        DestroyPicture(dpy, pm[i]);
+    }
+    free(pm);
+    return NULL;
+  }
+
+  return s;
+}
+#endif /* FANCY_TITLEBARS */
 
 #ifdef USEDECOR
 /*****************************************************************************
@@ -2045,7 +2127,8 @@ void CMD_ChangeDecor(F_CMD_ARGS)
   ForceSetupFrame(
     tmp_win, tmp_win->frame_g.x, tmp_win->frame_g.y, tmp_win->frame_g.width,
     tmp_win->frame_g.height - extra_height, True);
-  DrawDecorations(tmp_win, DRAW_ALL, (Scr.Hilite == tmp_win), 2, None);
+  DrawDecorations(
+    tmp_win, DRAW_ALL, (Scr.Hilite == tmp_win), 2, None, CLEAR_ALL);
 }
 
 /*****************************************************************************
@@ -2294,18 +2377,18 @@ void CMD_UpdateDecor(F_CMD_ARGS)
     {
       if (fw->decor == found)
       {
-	DrawDecorations(fw, DRAW_ALL, True, True, None);
-	DrawDecorations(fw, DRAW_ALL, False, True, None);
+	DrawDecorations(fw, DRAW_ALL, True, True, None, CLEAR_ALL);
+	DrawDecorations(fw, DRAW_ALL, False, True, None, CLEAR_ALL);
       }
     }
     else
 #endif
     {
-      DrawDecorations(fw, DRAW_ALL, True, True, None);
-      DrawDecorations(fw, DRAW_ALL, False, True, None);
+      DrawDecorations(fw, DRAW_ALL, True, True, None, CLEAR_ALL);
+      DrawDecorations(fw, DRAW_ALL, False, True, None, CLEAR_ALL);
     }
   }
-  DrawDecorations(hilight, DRAW_ALL, True, True, None);
+  DrawDecorations(hilight, DRAW_ALL, True, True, None, CLEAR_ALL);
 }
 
 void reset_decor_changes(void)
@@ -2324,6 +2407,48 @@ void reset_decor_changes(void)
 #endif
 }
 
+static void SetLayerButtonFlag(
+  int layer, int multi, int set, FvwmDecor *decor, TitleButton *tb)
+{
+  int i;
+  int start = 0;
+  int add = 2;
+
+  if (multi)
+  {
+    if (multi == 2)
+      start = 1;
+    else if (multi == 3)
+      add = 1;
+    for (i = start; i < NUMBER_OF_BUTTONS; i += add)
+    {
+      if (set)
+      {
+        TB_FLAGS(decor->buttons[i]).has_layer = 1;
+        TB_LAYER(decor->buttons[i]) = layer;
+      }
+      else
+      {
+        TB_FLAGS(decor->buttons[i]).has_layer = 0;
+      }
+    }
+  }
+  else
+  {
+    if (set)
+    {
+      TB_FLAGS(*tb).has_layer = 1;
+      TB_LAYER(*tb) = layer;
+    }
+    else
+    {
+      TB_FLAGS(*tb).has_layer = 0;
+    }
+  }
+
+  return;
+}
+
 /*****************************************************************************
  *
  * Changes a button decoration style (changes by veliaa@rpi.edu)
@@ -2493,6 +2618,32 @@ static void do_button_style(F_CMD_ARGS, Bool do_add)
 	{
 	  SetMWMButtonFlag(MWM_DECOR_STICK, multi, set, decor, tb);
 	}
+        else if (StrEquals(tok, "MwmDecorLayer"))
+        {
+          int layer, got_number;
+          char *ltok;
+          text = GetNextToken(text, &ltok);
+          if (ltok)
+          {
+            got_number = (sscanf(ltok, "%d", &layer) == 1);
+            free (ltok);
+          }
+          else
+          {
+            got_number = 0;
+          }
+          if (!ltok || !got_number)
+          {
+            fvwm_msg(ERR, "ButtonStyle",
+                     "could not read integer value for layer -- line: %s",
+                     text);
+          }
+          else
+          {
+            SetLayerButtonFlag(layer, multi, set, decor, tb);
+          }
+
+        }
 	else
 	{
 	  fvwm_msg(ERR, "ButtonStyle",
@@ -2533,7 +2684,6 @@ void CMD_ButtonStyle(F_CMD_ARGS)
   do_button_style(F_PASS_ARGS, False);
 }
 
-#ifdef MULTISTYLE
 /*****************************************************************************
  *
  * Appends a button decoration style (veliaa@rpi.edu)
@@ -2543,7 +2693,6 @@ void CMD_AddButtonStyle(F_CMD_ARGS)
 {
   do_button_style(F_PASS_ARGS, True);
 }
-#endif /* MULTISTYLE */
 
 
 /* add_to_env_list
@@ -2603,19 +2752,20 @@ void CMD_SetEnv(F_CMD_ARGS)
   char *szValue = NULL;
   char *szPutenv = NULL;
 
-  action = GetNextToken(action, &szVar);
+  action = GetNextToken(action,&szVar);
   if (!szVar)
     return;
-  action = GetNextToken(action, &szValue);
+  action = GetNextToken(action,&szValue);
   if (!szValue)
   {
-    /* no value, ignore */
+    /* no value, treat as unset */
+    CMD_UnsetEnv(F_PASS_ARGS);
     free(szVar);
     return;
   }
 
-  szPutenv = safemalloc(strlen(szVar) + strlen(szValue) + 2);
-  sprintf(szPutenv,"%s=%s", szVar, szValue);
+  szPutenv = safemalloc(strlen(szVar)+strlen(szValue)+2);
+  sprintf(szPutenv,"%s=%s",szVar,szValue);
   putenv(szPutenv);
   add_to_env_list(szVar, szPutenv);
   /* szVar is stored in the env list. do not free it */
@@ -2627,7 +2777,7 @@ void CMD_UnsetEnv(F_CMD_ARGS)
   char *szVar = NULL;
   char *szPutenv = NULL;
 
-  action = GetNextToken(action, &szVar);
+  action = GetNextToken(action,&szVar);
   if (!szVar)
     return;
 
@@ -2871,6 +3021,10 @@ void CMD_BugOpts(F_CMD_ARGS)
 	break;
       }
     }
+    else if (EWMH_BugOpts(opt, toggle))
+    {
+      /* work is done in EWMH_BugOpts */
+    }
     else
     {
       fvwm_msg(ERR, "SetBugOptions", "Unknown Bug Option '%s'", opt);
@@ -3024,8 +3178,12 @@ void CMD_FakeClick(F_CMD_ARGS)
   int maxdepth = 0;
 
   /* get the mask of pressed/released buttons */
-  XQueryPointer(
-    dpy, Scr.Root, &root, &JunkRoot, &JunkX, &JunkY, &JunkX, &JunkY, &mask);
+  if (XQueryPointer(
+	dpy, Scr.Root, &root, &JunkRoot, &JunkX, &JunkY, &JunkX, &JunkY,
+	&mask) == False)
+  {
+    /* pointer is on a different screen - that's okay here */
+  }
   token = PeekToken(action, &action);
   while (token && action)
   {
@@ -3067,7 +3225,11 @@ void CMD_FakeClick(F_CMD_ARGS)
 	     depth++)
 	{
 	  w = child_w;
-	  XQueryPointer(dpy, w, &root, &child_w, &rx, &ry, &x, &y, &JunkMask);
+	  if (XQueryPointer(
+		dpy, w, &root, &child_w, &rx, &ry, &x, &y, &JunkMask) == False)
+	  {
+	    /* pointer is on a different screen - that's okay here */
+	  }
 	}
 	if (do_unset)
 	{
@@ -3111,9 +3273,12 @@ void CMD_FakeClick(F_CMD_ARGS)
       if (val > 0 && val <= 1000000)
       {
 	usleep(1000 * val);
-	XQueryPointer(
-	  dpy, Scr.Root, &root, &JunkRoot, &JunkX, &JunkY, &JunkX, &JunkY,
-	  &mask);
+	if (XQueryPointer(
+	      dpy, Scr.Root, &root, &JunkRoot, &JunkX, &JunkY, &JunkX, &JunkY,
+	      &mask) == False)
+	{
+	  /* pointer is on a different screen - that's okay here */
+	}
       }
       else
       {
@@ -3265,8 +3430,13 @@ void CMD_StrokeFunc(F_CMD_ARGS)
   if (draw_motion)
   {
     MyXGrabServer(dpy);
-    XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild, &x[0], &y[0],
-		&JunkX, &JunkY, &JunkMask);
+    if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild, &x[0], &y[0],
+		      &JunkX, &JunkY, &JunkMask) == False)
+    {
+      /* pointer is on a different screen */
+      x[0] = 0;
+      y[0] = 0;
+    }
     XSetLineAttributes(dpy,Scr.XorGC,stroke_width,LineSolid,CapButt,JoinMiter);
   }
 
@@ -3281,14 +3451,15 @@ void CMD_StrokeFunc(F_CMD_ARGS)
     switch (eventp->type)
     {
     case MotionNotify:
-      if (eventp->xmotion.same_screen == False)
-      {
-	continue;
-      }
       if (eventp->xany.window != Scr.Root)
       {
-	XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild, &tmpx, &tmpy,
-		&JunkX, &JunkY, &JunkMask);
+	if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild, &tmpx, &tmpy,
+			  &JunkX, &JunkY, &JunkMask) == False)
+	{
+	  /* pointer is on a different screen */
+	  tmpx = 0;
+	  tmpy = 0;
+	}
       }
       else
       {
