@@ -18,10 +18,8 @@
 
 /* This file contains all command prototypes. */
 void CMD_Plus(F_CMD_ARGS);
-#ifdef MULTISTYLE
 void CMD_AddButtonStyle(F_CMD_ARGS);
 void CMD_AddTitleStyle(F_CMD_ARGS);
-#endif /* MULTISTYLE */
 #ifdef USEDECOR
 void CMD_AddToDecor(F_CMD_ARGS);
 #endif /* USEDECOR */
@@ -29,6 +27,7 @@ void CMD_AddToFunc(F_CMD_ARGS);
 void CMD_AddToMenu(F_CMD_ARGS);
 void CMD_All(F_CMD_ARGS);
 void CMD_AnimatedMove(F_CMD_ARGS);
+void CMD_Any(F_CMD_ARGS);
 void CMD_Beep(F_CMD_ARGS);
 void CMD_BorderStyle(F_CMD_ARGS);
 void CMD_BugOpts(F_CMD_ARGS);
@@ -55,6 +54,7 @@ void CMD_DefaultIcon(F_CMD_ARGS);
 void CMD_DefaultLayers(F_CMD_ARGS);
 void CMD_Delete(F_CMD_ARGS);
 void CMD_Desk(F_CMD_ARGS);
+void CMD_DesktopName(F_CMD_ARGS);
 void CMD_DesktopSize(F_CMD_ARGS);
 void CMD_Destroy(F_CMD_ARGS);
 #ifdef USEDECOR
@@ -120,6 +120,7 @@ void CMD_PipeRead(F_CMD_ARGS);
 void CMD_PixmapPath(F_CMD_ARGS);
 void CMD_PlaceAgain(F_CMD_ARGS);
 void CMD_PointerKey(F_CMD_ARGS);
+void CMD_PointerWindow(F_CMD_ARGS);
 void CMD_Popup(F_CMD_ARGS);
 void CMD_Prev(F_CMD_ARGS);
 void CMD_Quit(F_CMD_ARGS);
@@ -134,7 +135,9 @@ void CMD_Refresh(F_CMD_ARGS);
 void CMD_RefreshWindow(F_CMD_ARGS);
 void CMD_Repeat(F_CMD_ARGS);
 void CMD_Resize(F_CMD_ARGS);
+void CMD_ResizeMaximize(F_CMD_ARGS);
 void CMD_ResizeMove(F_CMD_ARGS);
+void CMD_ResizeMoveMaximize(F_CMD_ARGS);
 void CMD_Restart(F_CMD_ARGS);
 void CMD_SaveQuitSession(F_CMD_ARGS);
 void CMD_SaveSession(F_CMD_ARGS);
@@ -156,7 +159,6 @@ void CMD_Stroke(F_CMD_ARGS);
 void CMD_StrokeFunc(F_CMD_ARGS);
 #endif /* HAVE_STROKE */
 void CMD_Style(F_CMD_ARGS);
-void CMD_ThisWindow(F_CMD_ARGS);
 void CMD_Title(F_CMD_ARGS);
 void CMD_TitleStyle(F_CMD_ARGS);
 void CMD_UnsetEnv(F_CMD_ARGS);
