@@ -37,6 +37,7 @@
 #include "update.h"
 #include "style.h"
 #include "focus.h"
+#include "geometry.h"
 
 
 /**********************************************************************
@@ -69,17 +70,14 @@ char *CreateFlagString(char *string, char **restptr)
     start = c;
     length = 0;
     while (*c != closeopt) {
-      char *d;
       if (*c == 0) {
 	fvwm_msg(ERR, "CreateFlagString",
 		 "Conditionals require closing parenthesis");
 	*restptr = NULL;
 	return NULL;
       }
-      /* skip quoted string */
-      d = SkipQuote(c, NULL, NULL, NULL);
-      length += d - c;
-      c = d;
+      c++;
+      length++;
     }
 
     /* We must allocate a new string because we null terminate the string
@@ -152,6 +150,22 @@ void CreateConditionMask(char *flags, WindowConditionMask *mask)
       mask->my_flags.do_accept_focus = 0;
       mask->my_flags.use_do_accept_focus = 1;
     }
+    else if (StrEquals(condition,"Focused"))
+    {
+      mask->my_flags.needs_focus = NEEDS_TRUE;
+    }
+    else if (StrEquals(condition,"!Focused"))
+    {
+      mask->my_flags.needs_focus = NEEDS_FALSE;
+    }
+    else if (StrEquals(condition,"HasPointer"))
+    {
+      mask->my_flags.needs_pointer = NEEDS_TRUE;
+    }
+    else if (StrEquals(condition,"!HasPointer"))
+    {
+      mask->my_flags.needs_pointer = NEEDS_FALSE;
+    }
     else if (StrEquals(condition,"Iconic"))
     {
       SET_ICONIFIED(mask, 1);
@@ -257,7 +271,7 @@ void CreateConditionMask(char *flags, WindowConditionMask *mask)
     else if (StrEquals(condition,"CurrentPageAnyDesk") ||
 	    StrEquals(condition,"CurrentScreen"))
       mask->my_flags.needs_current_page = 1;
-    else if (StrEquals(condition,"CurrentGlobalPageAnyDesk"))
+    else if (StrEquals(condition,"CurrentGlobbalPageAnyDesk"))
       mask->my_flags.needs_current_global_page = 1;
     else if (StrEquals(condition,"CirculateHit"))
       mask->my_flags.use_circulate_hit = 1;
@@ -319,49 +333,59 @@ Bool MatchesConditionMask(FvwmWindow *fw, WindowConditionMask *mask)
 
   if (!blockcmpmask((char *)&(fw->flags), (char *)&(mask->flags),
                     (char *)&(mask->flag_mask), sizeof(fw->flags)))
+  {
     return 0;
-
-  /*
-  if ((mask->onFlags & fw->flags) != mask->onFlags)
-    return 0;
-
-  if ((mask->offFlags & fw->flags) != 0)
-    return 0;
-  */
+  }
 
   if (!mask->my_flags.use_circulate_hit && DO_SKIP_CIRCULATE(fw))
+  {
     return 0;
+  }
 
   if (!mask->my_flags.use_circulate_hit_icon && IS_ICONIFIED(fw) &&
       DO_SKIP_ICON_CIRCULATE(fw))
+  {
     return 0;
+  }
 
   if (!mask->my_flags.use_circulate_hit_shaded && IS_SHADED(fw) &&
       DO_SKIP_SHADED_CIRCULATE(fw))
+  {
     return 0;
+  }
 
   if (IS_ICONIFIED(fw) && IS_TRANSIENT(fw) && IS_ICONIFIED_BY_PARENT(fw))
+  {
     return 0;
+  }
 
   if (mask->my_flags.needs_current_desk && fw->Desk != Scr.CurrentDesk)
+  {
     return 0;
+  }
   if (mask->my_flags.needs_current_page)
   {
     if (FScreenIsEnabled())
     {
       if (!FScreenIsRectangleOnScreen(NULL, FSCREEN_CURRENT, &(fw->frame_g)))
+      {
 	return 0;
+      }
     }
     else
     {
       if (!IsRectangleOnThisPage(&(fw->frame_g), Scr.CurrentDesk))
+      {
 	return 0;
+      }
     }
   }
   else if (mask->my_flags.needs_current_global_page)
   {
     if (!IsRectangleOnThisPage(&(fw->frame_g), Scr.CurrentDesk))
+    {
       return 0;
+    }
   }
 
   /* Yes, I know this could be shorter, but it's hard to understand then */
@@ -375,16 +399,24 @@ Bool MatchesConditionMask(FvwmWindow *fw, WindowConditionMask *mask)
 	      fMatchesResource);
 
   if (mask->my_flags.needs_name && !fMatches)
+  {
     return 0;
+  }
 
   if (mask->my_flags.needs_not_name && fMatches)
+  {
     return 0;
+  }
 
   if ((mask->layer == -1) && (sf) && (fw->layer != sf->layer))
+  {
     return 0;
+  }
 
   if ((mask->layer >= 0) && (fw->layer != mask->layer))
+  {
     return 0;
+  }
 
   if (mask->my_flags.use_do_accept_focus)
   {
@@ -404,6 +436,43 @@ Bool MatchesConditionMask(FvwmWindow *fw, WindowConditionMask *mask)
       return 0;
     }
   }
+  if (mask->my_flags.needs_focus != NEEDS_ANY)
+  {
+    int is_focused;
+
+    is_focused = (fw == get_focus_window());
+    if (!is_focused && mask->my_flags.needs_focus == NEEDS_TRUE)
+    {
+      return 0;
+    }
+    else if (is_focused && mask->my_flags.needs_focus == NEEDS_FALSE)
+    {
+      return 0;
+    }
+  }
+  if (mask->my_flags.needs_pointer != NEEDS_ANY)
+  {
+    int has_pointer;
+    FvwmWindow *t;
+
+    t = get_pointer_fvwm_window();
+    if (t != NULL && t == fw)
+    {
+	    has_pointer = 1;
+    }
+    else
+    {
+	    has_pointer = 0;
+    }
+    if (!has_pointer && mask->my_flags.needs_pointer == NEEDS_TRUE)
+    {
+      return 0;
+    }
+    else if (has_pointer && mask->my_flags.needs_pointer == NEEDS_FALSE)
+    {
+      return 0;
+    }
+  }
 
   return 1;
 }
@@ -430,7 +499,6 @@ static FvwmWindow *Circulate(char *action, int Direction, char **restofline)
   { /* override for Current [] */
     mask.my_flags.use_circulate_hit = 1;
     mask.my_flags.use_circulate_hit_icon = 1;
-    mask.my_flags.use_circulate_hit_shaded = 1;
   }
   CreateConditionMask(flags, &mask);
   if (flags)
@@ -446,13 +514,7 @@ static FvwmWindow *Circulate(char *action, int Direction, char **restofline)
       fw = sf;
   }
   else
-  {
-    if (Direction == 0)
-    {
-      return NULL;
-    }
     fw = NULL;
-  }
 
   for (pass = 0; pass < 3 && !found; pass++)
   {
@@ -542,6 +604,19 @@ void CMD_None(F_CMD_ARGS)
   }
 }
 
+void CMD_Any(F_CMD_ARGS)
+{
+  FvwmWindow *found;
+  char *restofline;
+
+  found = Circulate(action, 1, &restofline);
+  if(found && restofline)
+  {
+    old_execute_function(
+      restofline, NULL, eventp, C_ROOT, *Module, 0, NULL);
+  }
+}
+
 void CMD_Current(F_CMD_ARGS)
 {
   FvwmWindow *found;
@@ -555,6 +630,36 @@ void CMD_Current(F_CMD_ARGS)
   }
 }
 
+void CMD_PointerWindow(F_CMD_ARGS)
+{
+	char *restofline;
+	FvwmWindow *t;
+	WindowConditionMask mask;
+	char *flags;
+
+	t = get_pointer_fvwm_window();
+	if (t == NULL)
+	{
+		return;
+	}
+	flags = CreateFlagString(action, &restofline);
+	DefaultConditionMask(&mask);
+	CreateConditionMask(flags, &mask);
+	if (flags)
+	{
+		free(flags);
+	}
+	mask.my_flags.use_circulate_hit = 1;
+	mask.my_flags.use_circulate_hit_icon = 1;
+	if (MatchesConditionMask(t, &mask))
+	{
+		old_execute_function(
+			restofline, t, eventp, C_WINDOW, *Module, 0, NULL);
+	}
+
+	return;
+}
+
 void CMD_All(F_CMD_ARGS)
 {
   FvwmWindow *t, **g;
@@ -570,7 +675,6 @@ void CMD_All(F_CMD_ARGS)
     free(flags);
   mask.my_flags.use_circulate_hit = 1;
   mask.my_flags.use_circulate_hit_icon = 1;
-  mask.my_flags.use_circulate_hit_shaded = 1;
 
   num = 0;
   for (t = Scr.FvwmRoot.next; t; t = t->next)
@@ -601,14 +705,6 @@ void CMD_All(F_CMD_ARGS)
   return;
 }
 
-static void GetDirectionReference(FvwmWindow *w, rectangle *r)
-{
-  if (IS_ICONIFIED(w))
-    *r = w->icon_g;
-  else
-    *r = w->frame_g;
-}
-
 /**********************************************************************
  * Execute a function to the closest window in the given
  * direction.
@@ -677,14 +773,19 @@ void CMD_Direction(F_CMD_ARGS)
    * Otherwise we use the pointer as a starting point. */
   if (tmp_win)
   {
-    GetDirectionReference(tmp_win, &my_g);
+    get_visible_window_or_icon_geometry(tmp_win, &my_g);
     my_cx = my_g.x + my_g.width / 2;
     my_cy = my_g.y + my_g.height / 2;
   }
   else
   {
-    XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
-		  &my_g.x, &my_g.y, &JunkX, &JunkY, &JunkMask);
+    if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
+		      &my_g.x, &my_g.y, &JunkX, &JunkY, &JunkMask) == False)
+    {
+      /* pointer is on a different screen */
+      my_g.x = 0;
+      my_g.y = 0;
+    }
     my_g.width = 1;
     my_g.height = 1;
     my_cx = my_g.x;
@@ -705,7 +806,7 @@ void CMD_Direction(F_CMD_ARGS)
       continue;
 
     /* Calculate relative location of the window. */
-    GetDirectionReference(window, &his_g);
+    get_visible_window_or_icon_geometry(window, &his_g);
     his_g.x -= my_cx;
     his_g.y -= my_cy;
     his_cx = his_g.x + his_g.width / 2;
@@ -784,9 +885,6 @@ void CMD_Pick(F_CMD_ARGS)
 
   flags = CreateFlagString(action, &restofline);
   DefaultConditionMask(&mask);
-  mask.my_flags.use_circulate_hit = 1;
-  mask.my_flags.use_circulate_hit_icon = 1;
-  mask.my_flags.use_circulate_hit_shaded = 1;
   CreateConditionMask(flags, &mask);
   if (flags)
     free(flags);
@@ -795,39 +893,6 @@ void CMD_Pick(F_CMD_ARGS)
     old_execute_function(
       restofline, tmp_win, eventp, C_WINDOW, *Module, 0, NULL);
   }
-  FreeConditionMask(&mask);
-}
-
-void CMD_ThisWindow(F_CMD_ARGS)
-{
-	char *restofline;
-	char *flags;
-	WindowConditionMask mask;
-
-	if (!tmp_win)
-	{
-		return;
-	}
-
-	flags = CreateFlagString(action, &restofline);
-	DefaultConditionMask(&mask);
-	mask.my_flags.use_circulate_hit = 1;
-	mask.my_flags.use_circulate_hit_icon = 1;
-	CreateConditionMask(flags, &mask);
-	if (flags)
-	{
-		free(flags);
-	}
-
-	if (MatchesConditionMask(tmp_win, &mask) && restofline)
-	{
-		old_execute_function(
-			restofline, tmp_win, eventp, C_WINDOW, *Module, 0,
-			NULL);
-	}
-
-	FreeConditionMask(&mask);
-	return;
 }
 
 void CMD_WindowId(F_CMD_ARGS)
