@@ -44,7 +44,7 @@
 #include "borders.h"
 #include "virtual.h"
 #include "stack.h"
-#include "colormaps.h"
+#include "geometry.h"
 
 static Bool lastFocusType;
 /* Last window which Fvwm gave the focus to NOT the window that really has the
@@ -64,7 +64,9 @@ Bool do_accept_input_focus(FvwmWindow *tmp_win)
  * Sets the input focus to the indicated window.
  *
  **********************************************************************/
-static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
+static void DoSetFocus(
+  Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp,
+  Bool do_allow_force_broadcast)
 {
   extern Time lastTimestamp;
   FvwmWindow *sf;
@@ -83,7 +85,7 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
     else
     {
       /* make sure the window is not hilighted */
-      DrawDecorations(Fw, DRAW_ALL, False, False, None);
+      DrawDecorations(Fw, DRAW_ALL, False, False, None, CLEAR_ALL);
     }
     return;
   }
@@ -97,8 +99,7 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
    * Focus-by-mouse type focus events */
   /* Watch out: Fw may not be on the windowlist and the windowlist may be
    * empty */
-  if (Fw && Fw != get_focus_window() && Fw != &Scr.FvwmRoot &&
-      !IS_SCHEDULED_FOR_DESTROY(Fw))
+  if (Fw && Fw != get_focus_window() && Fw != &Scr.FvwmRoot)
   {
     if (FocusByMouse) /* pluck window from list and deposit at top */
     {
@@ -160,12 +161,10 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
     if (IS_ICONIFIED(Fw))
     {
       rectangle r;
+      Bool rc;
 
-      r.x = Fw->icon_g.x;
-      r.y = Fw->icon_g.y;
-      r.width = Fw->icon_g.width;
-      r.height = Fw->icon_p_height + Fw->icon_g.height;
-      if (!IsRectangleOnThisPage(&r, Fw->Desk))
+      rc = get_visible_icon_geometry(Fw, &r);
+      if (!rc || !IsRectangleOnThisPage(&r, Fw->Desk))
       {
         Fw = NULL;
         w = Scr.NoFocusWin;
@@ -201,9 +200,9 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
   {
     Bool is_window_selected = False;
 
-    if (Fw->icon_w)
+    if (Fw->icon_title_w)
     {
-      w = Fw->icon_w;
+      w = Fw->icon_title_w;
       is_window_selected = True;
     }
     if ((!is_window_selected || WAS_ICON_HINT_PROVIDED(Fw)) &&
@@ -217,7 +216,10 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
   {
     FOCUS_SET(w);
     set_focus_window(Fw);
-    SET_FOCUS_CHANGE_BROADCAST_PENDING(Fw, 1);
+    if (do_allow_force_broadcast)
+    {
+      SET_FOCUS_CHANGE_BROADCAST_PENDING(Fw, 1);
+    }
     Scr.UnknownWinFocused = None;
   }
   else if (!Fw || !(Fw->wmhints) || !(Fw->wmhints->flags & InputHint) ||
@@ -228,7 +230,10 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
     set_focus_window(Fw);
     if (Fw)
     {
-      SET_FOCUS_CHANGE_BROADCAST_PENDING(Fw, 1);
+      if (do_allow_force_broadcast)
+      {
+	SET_FOCUS_CHANGE_BROADCAST_PENDING(Fw, 1);
+      }
     }
     Scr.UnknownWinFocused = None;
   }
@@ -248,7 +253,8 @@ static void DoSetFocus(Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp)
 }
 
 static void MoveFocus(
-  Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp, Bool do_force)
+  Window w, FvwmWindow *Fw, Bool FocusByMouse, Bool NoWarp, Bool do_force,
+  Bool do_allow_force_broadcast)
 {
   FvwmWindow *ffw_old = get_focus_window();
   Bool accepts_input_focus = do_accept_input_focus(Fw);
@@ -276,7 +282,7 @@ static void MoveFocus(
     }
     return;
   }
-  DoSetFocus(w, Fw, FocusByMouse, NoWarp);
+  DoSetFocus(w, Fw, FocusByMouse, NoWarp, do_allow_force_broadcast);
   if (get_focus_window() != ffw_old)
   {
     if (accepts_input_focus)
@@ -300,24 +306,26 @@ static void MoveFocus(
   }
 }
 
-void SetFocusWindow(FvwmWindow *Fw, Bool FocusByMouse)
+void SetFocusWindow(
+  FvwmWindow *Fw, Bool FocusByMouse, Bool do_allow_force_broadcast)
 {
-  MoveFocus(Fw->w, Fw, FocusByMouse, False, False);
+  MoveFocus(Fw->w, Fw, FocusByMouse, False, False, do_allow_force_broadcast);
 }
 
 void ReturnFocusWindow(FvwmWindow *Fw, Bool FocusByMouse)
 {
-  MoveFocus(Fw->w, Fw, FocusByMouse, True, False);
+  MoveFocus(Fw->w, Fw, FocusByMouse, True, False, True);
 }
 
-void DeleteFocus(Bool FocusByMouse)
+void DeleteFocus(Bool FocusByMouse, Bool do_allow_force_broadcast)
 {
-  MoveFocus(Scr.NoFocusWin, NULL, FocusByMouse, False, False);
+  MoveFocus(
+    Scr.NoFocusWin, NULL, FocusByMouse, False, False, do_allow_force_broadcast);
 }
 
 void ForceDeleteFocus(Bool FocusByMouse)
 {
-  MoveFocus(Scr.NoFocusWin, NULL, FocusByMouse, False, True);
+  MoveFocus(Scr.NoFocusWin, NULL, FocusByMouse, False, True, True);
 }
 
 /* When a window is unmapped (or destroyed) this function takes care of
@@ -349,9 +357,9 @@ void restore_focus_after_unmap(
     {
       for (t = tmp_win->next; t != NULL; t = t->next)
       {
-	if (t->Desk == tmp_win->Desk && !DO_SKIP_CIRCULATE(t) &&
-	    !(IS_ICONIFIED(t) && (
-		      DO_SKIP_ICON_CIRCULATE(t) || IS_ICON_SUPPRESSED(t))) &&
+	if (t->Desk == tmp_win->Desk &&
+	    !DO_SKIP_CIRCULATE(t) &&
+	    !(DO_SKIP_ICON_CIRCULATE(t) && IS_ICONIFIED(t)) &&
 	    (!do_skip_marked_transients || !IS_IN_TRANSIENT_SUBTREE(t)))
 	{
 	  /* If it is on a different desk we have to look for another window */
@@ -364,19 +372,17 @@ void restore_focus_after_unmap(
 	set_focus_to->Desk == tmp_win->Desk)
     {
       /* Don't transfer focus to windows on other desks */
-      SetFocusWindow(set_focus_to, 1);
+      SetFocusWindow(set_focus_to, True, True);
     }
     if (tmp_win == get_focus_window())
     {
-      DeleteFocus(1);
+      DeleteFocus(True, True);
     }
   }
   if (tmp_win == Scr.pushed_window)
     Scr.pushed_window = NULL;
   if (tmp_win == colormap_win)
-  {
-    InstallWindowColormaps(set_focus_to);
-  }
+    colormap_win = NULL;
 
   return;
 }
@@ -392,7 +398,6 @@ void SetPointerEventPosition(XEvent *eventp, int x, int y)
   case MotionNotify:
     eventp->xbutton.x_root = x;
     eventp->xbutton.y_root = y;
-    eventp->xmotion.same_screen = True;
     break;
   default:
     break;
@@ -418,7 +423,7 @@ void FocusOn(FvwmWindow *t, Bool FocusByMouse, char *action)
     if (t)
     {
       /* give the window a chance to take the focus itself */
-      MoveFocus(t->w, t, FocusByMouse, 1, 0);
+      MoveFocus(t->w, t, FocusByMouse, True, False, True);
     }
     return;
   }
@@ -432,8 +437,16 @@ void FocusOn(FvwmWindow *t, Bool FocusByMouse, char *action)
 
     if (IS_ICONIFIED(t))
     {
-      cx = t->icon_xl_loc + t->icon_g.width/2;
-      cy = t->icon_g.y + t->icon_p_height + ICON_HEIGHT(t) / 2;
+      rectangle g;
+      Bool rc;
+
+      rc = get_visible_icon_title_geometry(t, &g);
+      if (rc == False)
+      {
+	      get_visible_icon_picture_geometry(t, &g);
+      }
+      cx = g.x + g.width / 2;
+      cy = g.y + g.height / 2;
     }
     else
     {
@@ -460,7 +473,7 @@ void FocusOn(FvwmWindow *t, Bool FocusByMouse, char *action)
   UngrabEm(GRAB_NORMAL);
   if (t->Desk == Scr.CurrentDesk)
   {
-    MoveFocus(t->w, t, FocusByMouse, do_not_warp, 0);
+    MoveFocus(t->w, t, FocusByMouse, do_not_warp, False, True);
   }
 
   return;
@@ -478,7 +491,7 @@ static void warp_to_fvwm_window(
   int cx,cy;
   int x,y;
 
-  if(t == (FvwmWindow *)0 || (IS_ICONIFIED(t) && t->icon_w == None))
+  if(t == (FvwmWindow *)0 || (IS_ICONIFIED(t) && t->icon_title_w == None))
     return;
 
   if(t->Desk != Scr.CurrentDesk)
@@ -488,8 +501,16 @@ static void warp_to_fvwm_window(
 
   if(IS_ICONIFIED(t))
   {
-    cx = t->icon_xl_loc + t->icon_g.width/2;
-    cy = t->icon_g.y + t->icon_p_height + ICON_HEIGHT(t) / 2;
+    rectangle g;
+    Bool rc;
+
+    rc = get_visible_icon_title_geometry(t, &g);
+    if (rc == False)
+    {
+      get_visible_icon_picture_geometry(t, &g);
+    }
+    cx = g.x + g.width / 2;
+    cy = g.y + g.height / 2;
   }
   else
   {
@@ -504,8 +525,16 @@ static void warp_to_fvwm_window(
 
   if(IS_ICONIFIED(t))
   {
-    x = t->icon_xl_loc + t->icon_g.width / 2;
-    y = t->icon_g.y + t->icon_p_height + ICON_HEIGHT(t) / 2;
+    rectangle g;
+    Bool rc;
+
+    rc = get_visible_icon_title_geometry(t, &g);
+    if (rc == False)
+    {
+      get_visible_icon_picture_geometry(t, &g);
+    }
+    x = g.x + g.width / 2;
+    y = g.y + g.height / 2;
   }
   else
   {
@@ -594,17 +623,14 @@ void CMD_WarpToWindow(F_CMD_ARGS)
 	return;
       }
       if (val1_unit != Scr.MyDisplayWidth)
-	x = val1;
+	x = wx + val1;
       else
-	x = (ww - 1) * val1 / 100;
+	x = wx + (ww - 1) * val1 / 100;
+
       if (val2_unit != Scr.MyDisplayHeight)
-	y = val2;
+	y = wy + val2;
       else
-	y = (wh - 1) * val2 / 100;
-      if (x < 0)
-	x += ww;
-      if (y < 0)
-	y += wh;
+	y = wy + (wh - 1) * val2 / 100;
     }
     XWarpPointer(dpy, None, w, 0, 0, 0, 0, x, y);
   }
@@ -622,35 +648,25 @@ void focus_grab_buttons(FvwmWindow *tmp_win, Bool is_focused)
   Bool do_grab_window = False;
   unsigned char grab_buttons = Scr.buttons2grab;
 
-  if (!tmp_win || IS_SCHEDULED_FOR_DESTROY(tmp_win))
+  if (!tmp_win)
   {
-    /* It's pointless to grab buttons on dying windows */
     return;
   }
   accepts_input_focus = do_accept_input_focus(tmp_win);
-  if (HAS_SLOPPY_FOCUS(tmp_win) || HAS_MOUSE_FOCUS(tmp_win) ||
-       HAS_NEVER_FOCUS(tmp_win))
+  if ((HAS_SLOPPY_FOCUS(tmp_win) || HAS_MOUSE_FOCUS(tmp_win) ||
+       HAS_NEVER_FOCUS(tmp_win)) &&
+      DO_RAISE_MOUSE_FOCUS_CLICK(tmp_win) &&
+      (!is_focused || !is_on_top_of_layer(tmp_win)))
   {
-    if (DO_RAISE_MOUSE_FOCUS_CLICK(tmp_win) &&
-        (!is_focused || !is_on_top_of_layer(tmp_win)))
-    {
-      grab_buttons = ((1 << NUMBER_OF_MOUSE_BUTTONS) - 1);
-      do_grab_window = True;
-    }
+    grab_buttons = (char)(DEFAULT_ALL_BUTTONS_MOTION_MASK / Button1Mask);
+    do_grab_window = True;
   }
-  else if (HAS_CLICK_FOCUS(tmp_win))
+  else if (HAS_CLICK_FOCUS(tmp_win) &&
+	   (!is_focused || !is_on_top_of_layer(tmp_win)) &&
+	   (!DO_NOT_RAISE_CLICK_FOCUS_CLICK(tmp_win) || accepts_input_focus))
   {
-    if (is_focused && !is_on_top_of_layer(tmp_win) &&
-        DO_NOT_RAISE_CLICK_FOCUS_CLICK(tmp_win) && accepts_input_focus)
-    {
-      /* don't grab */
-    }
-    else if ((!is_focused || !is_on_top_of_layer(tmp_win)) &&
-             (!DO_NOT_RAISE_CLICK_FOCUS_CLICK(tmp_win) || accepts_input_focus))
-    {
-      grab_buttons = ((1 << NUMBER_OF_MOUSE_BUTTONS) - 1);
-      do_grab_window = True;
-    }
+    grab_buttons = (char)(DEFAULT_ALL_BUTTONS_MOTION_MASK / Button1Mask);
+    do_grab_window = True;
   }
 
 #if 0
@@ -705,22 +721,21 @@ void focus_grab_buttons(FvwmWindow *tmp_win, Bool is_focused)
 	    if (do_grab)
 	    {
 	      XGrabButton(
-		dpy, i + 1, mods, tmp_win->Parent, True, ButtonPressMask,
-		GrabModeSync, GrabModeAsync, None, None);
+		      dpy, i + 1, mods, tmp_win->Parent, True, ButtonPressMask,
+		      GrabModeSync, GrabModeAsync, None, None);
               /*  Set each FvwmWindow flag accordingly, as we grab or ungrab. */
               tmp_win->grabbed_buttons |= (1<<i);
 	    }
 	    else
 	    {
 	      XUngrabButton(dpy, (i+1), mods, tmp_win->Parent);
-              tmp_win->grabbed_buttons &= ~(1<<i);
+              tmp_win->grabbed_buttons &= !(1<<i);
 	    }
 	  }
 	} /* for */
       }
     } /* for */
     MyXUngrabServer (dpy);
-
   }
 
   return;
