@@ -46,10 +46,12 @@
 #include "module_interface.h"
 #include "misc.h"
 #include "screen.h"
+#include "geometry.h"
 #include "repeat.h"
 #include "read.h"
 #include "virtual.h"
 #include "libs/Colorset.h"
+#include "ewmh.h"
 
 extern XEvent Event;
 extern FvwmWindow *Tmp_win;
@@ -90,11 +92,9 @@ void CMD_Title(F_CMD_ARGS)
 static const func_type func_config[] =
 {
   CMD_ENTRY("+", CMD_Plus, F_ADDMENU2, 0),
-#ifdef MULTISTYLE
   CMD_ENTRY("addbuttonstyle", CMD_AddButtonStyle, F_ADD_BUTTON_STYLE,
 	    FUNC_DECOR),
   CMD_ENTRY("addtitlestyle", CMD_AddTitleStyle, F_ADD_TITLE_STYLE, FUNC_DECOR),
-#endif /* MULTISTYLE */
 #ifdef USEDECOR
   CMD_ENTRY("addtodecor", CMD_AddToDecor, F_ADD_DECOR, 0),
 #endif /* USEDECOR */
@@ -103,6 +103,7 @@ static const func_type func_config[] =
   CMD_ENTRY("all", CMD_All, F_ALL, 0),
   CMD_ENTRY("animatedmove", CMD_AnimatedMove, F_ANIMATED_MOVE,
 	    FUNC_NEEDS_WINDOW),
+  CMD_ENTRY("any", CMD_Any, F_ANY, 0),
   CMD_ENTRY("beep", CMD_Beep, F_BEEP, 0),
   CMD_ENTRY("borderstyle", CMD_BorderStyle, F_BORDERSTYLE, FUNC_DECOR),
   CMD_ENTRY("bugopts", CMD_BugOpts, F_BUG_OPTS, 0),
@@ -129,6 +130,7 @@ static const func_type func_config[] =
   CMD_ENTRY("defaultlayers", CMD_DefaultLayers, F_DFLT_LAYERS, 0),
   CMD_ENTRY("delete", CMD_Delete, F_DELETE, FUNC_NEEDS_WINDOW),
   CMD_ENTRY("desk", CMD_Desk, F_GOTO_DESK, 0),
+  CMD_ENTRY("desktopname", CMD_DesktopName, F_DESKTOP_NAME, 0),
   CMD_ENTRY("desktopsize", CMD_DesktopSize, F_SETDESK, 0),
   CMD_ENTRY("destroy", CMD_Destroy, F_DESTROY, FUNC_NEEDS_WINDOW),
 #ifdef USEDECOR
@@ -146,6 +148,9 @@ static const func_type func_config[] =
   CMD_ENTRY("edgethickness", CMD_EdgeThickness, F_NOP, 0),
   CMD_ENTRY("emulate", CMD_Emulate, F_EMULATE, 0),
   CMD_ENTRY("escapefunc", CMD_EscapeFunc, F_ESCAPE_FUNC, 0),
+  CMD_ENTRY("ewmhbasestrut", CMD_EwmhBaseStrut, F_EWMH_BASE_STRUT, 0),
+  CMD_ENTRY("ewmhnumberofdesktops", CMD_EwmhNumberOfDesktops,
+    F_EWMH_NUMBER_OF_DESKTOPS, 0),
   CMD_ENTRY("exec", CMD_Exec, F_EXEC, 0),
   CMD_ENTRY("execuseshell", CMD_ExecUseShell, F_EXEC_SETUP, 0),
   CMD_ENTRY("fakeclick", CMD_FakeClick, F_FAKE_CLICK, 0),
@@ -195,6 +200,7 @@ static const func_type func_config[] =
   CMD_ENTRY("pixmappath", CMD_PixmapPath, F_PIXMAP_PATH, 0),
   CMD_ENTRY("placeagain", CMD_PlaceAgain, F_PLACEAGAIN, FUNC_NEEDS_WINDOW),
   CMD_ENTRY("pointerkey", CMD_PointerKey, F_POINTERKEY, 0),
+  CMD_ENTRY("pointerwindow", CMD_PointerWindow, F_POINTERWINDOW, 0),
   CMD_ENTRY("popup", CMD_Popup, F_POPUP, 0),
   CMD_ENTRY("prev", CMD_Prev, F_PREV, 0),
   CMD_ENTRY("quit", CMD_Quit, F_QUIT, 0),
@@ -209,7 +215,11 @@ static const func_type func_config[] =
   CMD_ENTRY("refreshwindow", CMD_RefreshWindow, F_REFRESH, FUNC_NEEDS_WINDOW),
   CMD_ENTRY(PRE_REPEAT, CMD_Repeat, F_REPEAT, FUNC_DONT_REPEAT),
   CMD_ENTRY("resize", CMD_Resize, F_RESIZE, FUNC_NEEDS_WINDOW),
+  CMD_ENTRY("resizemaximize", CMD_ResizeMaximize, F_RESIZE_MAXIMIZE,
+	    FUNC_NEEDS_WINDOW),
   CMD_ENTRY("resizemove", CMD_ResizeMove, F_RESIZEMOVE, FUNC_NEEDS_WINDOW),
+  CMD_ENTRY("resizemovemaximize", CMD_ResizeMoveMaximize,
+	    F_RESIZEMOVE_MAXIMIZE, FUNC_NEEDS_WINDOW),
   CMD_ENTRY("restart", CMD_Restart, F_RESTART, 0),
   CMD_ENTRY("savequitsession", CMD_SaveQuitSession, F_SAVE_QUIT_SESSION, 0),
   CMD_ENTRY("savesession", CMD_SaveSession, F_SAVE_SESSION, 0),
@@ -235,7 +245,6 @@ static const func_type func_config[] =
   CMD_ENTRY("strokefunc", CMD_StrokeFunc, F_STROKE_FUNC, 0),
 #endif /* HAVE_STROKE */
   CMD_ENTRY("style", CMD_Style, F_STYLE, 0),
-  CMD_ENTRY("thiswindow", CMD_ThisWindow, F_THISWINDOW, 0),
   CMD_ENTRY("title", CMD_Title, F_TITLE, 0),
   CMD_ENTRY("titlestyle", CMD_TitleStyle, F_TITLESTYLE, FUNC_DECOR),
   CMD_ENTRY("unsetenv", CMD_UnsetEnv, F_SETENV, 0),
@@ -312,34 +321,41 @@ static char *function_vars[] =
   "w.y",
   "w.width",
   "w.height",
+  "cw.x",
+  "cw.y",
+  "cw.width",
+  "cw.height",
+  "it.x",
+  "it.y",
+  "it.width",
+  "it.height",
+  "ip.x",
+  "ip.y",
+  "ip.width",
+  "ip.height",
+  "i.x",
+  "i.y",
+  "i.width",
+  "i.height",
   "screen",
-  "desk.n",
-  "w.id",
-  "w.name",
-  "w.iconname",
-  "w.class",
-  "w.resource",
-  "version.num",
-  "version.info",
-  "version.line",
-  "desk.pagesx",
-  "desk.pagesy",
+  "desk.name",
   NULL
 };
 
-static signed int expand_extended_var(
+static int expand_extended_var(
   char *var_name, char *output, FvwmWindow *tmp_win)
 {
+  char *s;
   char *rest;
   char dummy[64];
   char *target = (output) ? output : dummy;
   int cs = -1;
   int n;
   int i;
+  int l;
   Pixel pixel = 0;
   int val = -12345678;
   Bool is_numeric = False;
-  const char *string = NULL;
 
   /* allow partial matches for *.cs variables */
   switch ((i = GetTokenIndex(var_name, function_vars, -1, &rest)))
@@ -350,14 +366,14 @@ static signed int expand_extended_var(
   case 3:
     if (!isdigit(*rest) || (*rest == '0' && *(rest + 1) != 0))
       /* not a non-negative integer without leading zeros */
-      return -1;
+      return 0;
     if (sscanf(rest, "%d%n", &cs, &n) < 1)
-      return -1;
+      return 0;
     if (*(rest + n) != 0)
       /* trailing characters */
-      return -1;
+      return 0;
     if (cs < 0)
-      return -1;
+      return 0;
     AllocColorset(cs);
     switch (i)
     {
@@ -379,6 +395,35 @@ static signed int expand_extended_var(
       break;
     }
     return pixel_to_color_string(dpy, Pcmap, pixel, target, False);
+    break;
+  case 33:
+    if (sscanf(rest, "%d%n", &cs, &n) < 1)
+      return 0;
+    if (*(rest + n) != 0)
+      /* trailing characters */
+      return 0;
+    s = GetDesktopName(cs);
+    if (s == NULL)
+    {
+      s = (char *)safemalloc(23*sizeof(char));
+      sprintf(s,"Desk %i", cs);
+      l = strlen(s);
+      if (output)
+      {
+	strcpy(output,s);
+      }
+      free(s);
+    }
+    else
+    {
+      l = strlen(s);
+      if (output)
+      {
+	strcpy(output,s);
+      }
+    }
+    return l;
+    break;
   default:
     break;
   }
@@ -426,124 +471,218 @@ static signed int expand_extended_var(
     is_numeric = True;
     val = (int)(Scr.Vy / Scr.MyDisplayHeight);
     break;
-
   case 12:
   case 13:
   case 14:
   case 15:
-    if (!tmp_win || IS_ICONIFIED(tmp_win))
-      return -1;
+    if (!tmp_win || IS_ICONIFIED(tmp_win) || IS_EWMH_DESKTOP(tmp_win->w))
+      return 0;
     else
     {
+      rectangle g;
+
       is_numeric = True;
+      get_unshaded_geometry(tmp_win, &g);
       switch (i)
       {
       case 12:
 	/* w.x */
-	val = tmp_win->frame_g.x;
+	val = g.x;
 	break;
       case 13:
 	/* w.y */
-	val = tmp_win->frame_g.y;
+	val = g.y;
 	break;
       case 14:
 	/* w.width */
-	val = tmp_win->frame_g.width;
+	val = g.width;
 	break;
       case 15:
 	/* w.height */
-	val = tmp_win->frame_g.height;
+	val = g.height;
 	break;
       default:
-	return -1;
+	return 0;
       }
     }
     break;
   case 16:
+  case 17:
+  case 18:
+  case 19:
+    if (!tmp_win || IS_ICONIFIED(tmp_win) || IS_EWMH_DESKTOP(tmp_win->w))
+      return 0;
+    else
+    {
+      rectangle g;
+
+      is_numeric = True;
+      get_client_geometry(tmp_win, &g);
+      switch (i)
+      {
+      case 16:
+	/* cw.x */
+	val = g.x;
+	break;
+      case 17:
+	/* cw.y */
+	val = g.y;
+	break;
+      case 18:
+	/* cw.width */
+	val = g.width;
+	break;
+      case 19:
+	/* cw.height */
+	val = g.height;
+	break;
+      default:
+	return 0;
+      }
+    }
+    break;
+  case 20:
+  case 21:
+  case 22:
+  case 23:
+    if (!tmp_win || IS_EWMH_DESKTOP(tmp_win->w))
+      return 0;
+    else
+    {
+      rectangle g;
+
+      if (get_visible_icon_title_geometry(tmp_win, &g) == False)
+      {
+	return 0;
+      }
+      is_numeric = True;
+      switch (i)
+      {
+      case 20:
+	/* it.x */
+	val = g.x;
+	break;
+      case 21:
+	/* it.y */
+	val = g.y;
+	break;
+      case 22:
+	/* it.width */
+	val = g.width;
+	break;
+      case 23:
+	/* it.height */
+	val = g.height;
+	break;
+      default:
+	return 0;
+      }
+    }
+    break;
+  case 24:
+  case 25:
+  case 26:
+  case 27:
+    if (!tmp_win || IS_EWMH_DESKTOP(tmp_win->w))
+      return 0;
+    else
+    {
+      rectangle g;
+
+      if (get_visible_icon_picture_geometry(tmp_win, &g) == False)
+      {
+	return 0;
+      }
+      is_numeric = True;
+      switch (i)
+      {
+      case 24:
+	/* ip.x */
+	val = g.x;
+	break;
+      case 25:
+	/* ip.y */
+	val = g.y;
+	break;
+      case 26:
+	/* ip.width */
+	val = g.width;
+	break;
+      case 27:
+	/* ip.height */
+	val = g.height;
+	break;
+      default:
+	return 0;
+      }
+    }
+    break;
+  case 28:
+  case 29:
+  case 30:
+  case 31:
+    if (!tmp_win || IS_EWMH_DESKTOP(tmp_win->w))
+      return 0;
+    else
+    {
+      rectangle g;
+
+      if (get_visible_icon_geometry(tmp_win, &g) == False)
+      {
+	return 0;
+      }
+      is_numeric = True;
+      switch (i)
+      {
+      case 28:
+	/* i.x */
+	val = g.x;
+	break;
+      case 29:
+	/* i.y */
+	val = g.y;
+	break;
+      case 30:
+	/* i.width */
+	val = g.width;
+	break;
+      case 31:
+	/* i.height */
+	val = g.height;
+	break;
+      default:
+	return 0;
+      }
+    }
+    break;
+  case 32:
     /* screen */
     is_numeric = True;
     val = Scr.screen;
     break;
-  case 17:
-    /* desk.n */
-    is_numeric = True;
-    val = Scr.CurrentDesk;
-    break;
-  case 18:
-    /* w.id */
-    if (tmp_win)
-    {
-      sprintf(target, "0x%x", (unsigned int)tmp_win->w);
-      string = target;
-    }
-    break;
-  case 19:
-    /* w.name */
-    if (tmp_win)
-    {
-      string = tmp_win->name;
-    }
-    break;
-  case 20:
-    /* w.iconname */
-    if (tmp_win)
-    {
-      string = tmp_win->icon_name;
-    }
-    break;
-  case 21:
-    /* w.class */
-    if (tmp_win)
-    {
-      string = tmp_win->class.res_class;
-    }
-    break;
-  case 22:
-    /* w.resource */
-    if (tmp_win)
-    {
-      string = tmp_win->class.res_name;
-    }
-    break;
-  case 23:
-    /* version.num */
-    string = VERSION;
-    break;
-  case 24:
-    /* version.info */
-    string = "";
-    break;
-  case 25:
-    /* version.line */
-    string = Fvwm_VersionInfo;
-    break;
-  case 26:
-    /* desk.pagesx */
-    is_numeric = True;
-    val = (int)(Scr.VxMax / Scr.MyDisplayWidth) + 1;
-    break;
-  case 27:
-    /* desk.pagesy */
-    is_numeric = True;
-    val = (int)(Scr.VyMax / Scr.MyDisplayHeight) + 1;
-    break;
   default:
     /* unknown variable - try to find it in the environment */
-    string = getenv(var_name);
+    s = getenv(var_name);
+    if (s)
+    {
+      if (output)
+      {
+	strcpy(output, s);
+      }
+      return strlen(s);
+    }
+    else
+    {
+      return 0;
+    }
   }
   if (is_numeric)
   {
     sprintf(target, "%d", val);
     return strlen(target);
   }
-  else
-  {
-    if (output && string)
-    {
-      strcpy(output, string);
-    }
-    return string ? strlen(string) : -1;
-  }
+
+  return 0;
 }
 
 static char *expand(
@@ -590,7 +729,7 @@ static char *expand(
 	  if (!addto)
 	  {
 	    xlen = expand_extended_var(var, NULL, tmp_win);
-	    if (xlen >= 0)
+	    if (xlen > 0)
 	      l2 += xlen - (k + 2);
 	  }
 	  i += k + 2;
@@ -629,23 +768,31 @@ static char *expand(
 	i++;
 	break;
       case 'c':
-	if (tmp_win && tmp_win->class.res_class &&
-	    tmp_win->class.res_class[0])
-	{
-	  string = tmp_win->class.res_class;
-	}
-	break;
       case 'r':
-	if (tmp_win && tmp_win->class.res_name &&
-	    tmp_win->class.res_name[0])
-	{
-	  string = tmp_win->class.res_name;
-	}
-	break;
       case 'n':
-	if (tmp_win && tmp_win->name && tmp_win->name[0])
+	if (tmp_win && !IS_EWMH_DESKTOP(tmp_win->w))
 	{
-	  string = tmp_win->name;
+	  switch(input[i+1])
+	  {
+	  case 'c':
+	    if (tmp_win->class.res_class && tmp_win->class.res_class[0])
+	    {
+	      string = tmp_win->class.res_class;
+	    }
+	    break;
+	  case 'r':
+	    if (tmp_win->class.res_name && tmp_win->class.res_name[0])
+	    {
+	      string = tmp_win->class.res_name;
+	    }
+	    break;
+	  case 'n':
+	    if (tmp_win->name && tmp_win->name[0])
+	    {
+	      string = tmp_win->name;
+	    }
+	    break;
+	  }
 	}
 	break;
       case 'v':
@@ -698,7 +845,7 @@ static char *expand(
 	  k = strlen(var);
 	  xlen = expand_extended_var(var, &out[j], tmp_win);
 	  input[m] = ']';
-	  if (xlen >= 0)
+	  if (xlen > 0)
 	  {
 	    j += xlen;
 	    i += k + 2;
@@ -758,7 +905,7 @@ static char *expand(
 	is_string = True;
 	break;
       case 'w':
-	if(tmp_win)
+	if(tmp_win && !IS_EWMH_DESKTOP(tmp_win->w))
 	  sprintf(&out[j],"0x%x",(unsigned int)tmp_win->w);
 	else
 	  sprintf(&out[j],"$w");
@@ -782,25 +929,31 @@ static char *expand(
 	break;
 
       case 'c':
-	if (tmp_win && tmp_win->class.res_class &&
-	    tmp_win->class.res_class[0])
-	{
-	  string = tmp_win->class.res_class;
-	}
-	is_string = True;
-	break;
       case 'r':
-	if (tmp_win && tmp_win->class.res_name &&
-	    tmp_win->class.res_name[0])
-	{
-	  string = tmp_win->class.res_name;
-	}
-	is_string = True;
-	break;
       case 'n':
-	if (tmp_win && tmp_win->name && tmp_win->name[0])
+	if (tmp_win && !IS_EWMH_DESKTOP(tmp_win->w))
 	{
-	  string = tmp_win->name;
+	  switch(input[i+1])
+	  {
+	  case 'c':
+	    if (tmp_win->class.res_class && tmp_win->class.res_class[0])
+	    {
+	      string = tmp_win->class.res_class;
+	    }
+	    break;
+	  case 'r':
+	    if (tmp_win->class.res_name && tmp_win->class.res_name[0])
+	    {
+	      string = tmp_win->class.res_name;
+	    }
+	    break;
+	  case 'n':
+	    if (tmp_win->name && tmp_win->name[0])
+	    {
+	      string = tmp_win->name;
+	    }
+	    break;
+	  }
 	}
 	is_string = True;
 	break;
@@ -856,8 +1009,7 @@ static char *expand(
  *
  ****************************************************************************/
 static cfunc_action_type CheckActionType(
-  int x, int y, XEvent *d, Bool may_time_out, Bool is_button_pressed,
-  int *ret_button)
+  int x, int y, XEvent *d, Bool may_time_out, Bool is_button_pressed)
 {
   int xcurrent,ycurrent,total = 0;
   Time t0;
@@ -898,13 +1050,8 @@ static cfunc_action_type CheckActionType(
       switch (d->xany.type)
       {
       case ButtonRelease:
-	*ret_button = d->xbutton.button;
 	return CF_CLICK;
       case MotionNotify:
-	if (d->xmotion.same_screen == False)
-	{
-	  break;
-	}
 	if ((d->xmotion.state & DEFAULT_ALL_BUTTONS_MASK) ||
             !is_button_pressed)
 	{
@@ -917,7 +1064,6 @@ static cfunc_action_type CheckActionType(
 	}
 	break;
       case ButtonPress:
-	*ret_button = d->xbutton.button;
 	if (may_time_out)
 	  is_button_pressed = True;
 	break;
@@ -999,7 +1145,7 @@ void execute_function(exec_func_args_type *efa)
       arguments[j] = NULL;
   }
 
-  if (efa->tmp_win == NULL)
+  if (efa->tmp_win == NULL || IS_EWMH_DESKTOP(efa->tmp_win->w))
   {
     if (efa->flags.is_window_unmanaged)
       w = efa->win;
@@ -1239,7 +1385,8 @@ int DeferExecution(
 
   original_w = *w;
 
-  if((*context != C_ROOT)&&(*context != C_NO_CONTEXT)&&(*tmp_win != NULL))
+  if((*context != C_ROOT)&&(*context != C_NO_CONTEXT)&&(*tmp_win != NULL)
+     && *context != C_EWMH_DESKTOP)
   {
     if((FinishEvent == ButtonPress)||((FinishEvent == ButtonRelease) &&
                                       (eventp->type != ButtonPress)))
@@ -1318,7 +1465,7 @@ int DeferExecution(
     *w = eventp->xbutton.subwindow;
     eventp->xany.window = *w;
   }
-  if (*w == Scr.Root)
+  if (*w == Scr.Root || IS_EWMH_DESKTOP(*w))
   {
     *context = C_ROOT;
     XBell(dpy, 0);
@@ -1344,7 +1491,8 @@ int DeferExecution(
   /* this ugly mess attempts to ensure that the release and press
    * are in the same window. */
   if(*w != original_w && original_w != Scr.Root &&
-     original_w != None && original_w != Scr.NoFocusWin)
+     original_w != None && original_w != Scr.NoFocusWin &&
+     !IS_EWMH_DESKTOP(original_w))
   {
     if (*w != (*tmp_win)->frame || original_w != (*tmp_win)->w)
     {
@@ -1353,6 +1501,14 @@ int DeferExecution(
       return TRUE;
     }
   }
+
+  if (IS_EWMH_DESKTOP((*tmp_win)->w))
+  {
+    *context = C_ROOT;
+    XBell(dpy, 0);
+    return TRUE;
+  }
+
   *context = GetContext(*tmp_win,eventp,&dummy);
 
   return FALSE;
@@ -1607,7 +1763,6 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
   XEvent d, *ev;
   FvwmFunction *func;
   static unsigned int depth = 0;
-  int button;
 
   /* FindFunction expects a token, not just a quoted string */
   func_name = PeekToken(action, &taction);
@@ -1744,43 +1899,37 @@ static void execute_complex_function(F_CMD_ARGS, Bool *desperate)
   case ButtonRelease:
     x = eventp->xbutton.x_root;
     y = eventp->xbutton.y_root;
-    button = eventp->xbutton.button;
     /* Take the click which started this fuction off the
      * Event queue.  -DDN- Dan D Niles dniles@iname.com */
     XCheckMaskEvent(dpy, ButtonPressMask, &d);
     break;
   default:
-    XQueryPointer( dpy, Scr.Root, &JunkRoot, &JunkChild,
-		   &x,&y,&JunkX, &JunkY, &JunkMask);
-    button = 0;
+    if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
+		      &x, &y, &JunkX, &JunkY, &JunkMask) == False)
+    {
+      /* pointer is on a different screen */
+      x = 0;
+      y = 0;
+    }
     break;
   }
 
   /* Wait and see if we have a click, or a move */
   /* wait forever, see if the user releases the button */
-  type = CheckActionType(x, y, &d, HaveHold, True, &button);
+  type = CheckActionType(x, y, &d, HaveHold, True);
   if (type == CF_CLICK)
   {
     ev = &d;
     /* If it was a click, wait to see if its a double click */
     if(HaveDoubleClick)
     {
-      int button2;
-
-      type = CheckActionType(x, y, &d, True, False, &button2);
+      type = CheckActionType(x, y, &d, True, False);
       switch (type)
       {
+      case CF_CLICK:
       case CF_HOLD:
       case CF_MOTION:
-      case CF_CLICK:
-	if (button == button2)
-	{
-	  type = CF_DOUBLE_CLICK;
-	}
-	else
-	{
-	  type = CF_CLICK;
-	}
+	type = CF_DOUBLE_CLICK;
 	ev = &d;
 	break;
       case CF_TIMEOUT:
