@@ -91,6 +91,7 @@ enum
   F_ADDMENU,
   F_ADDMENU2,
   F_ALL,
+  F_ANY,
   F_BEEP,
   F_BUG_OPTS,
   F_BUSY_CURSOR,
@@ -106,6 +107,7 @@ enum
   F_COPY_MENU_STYLE,
   F_CURRENT,
   F_CURSOR_STYLE,
+  F_DESKTOP_NAME,
   F_DESTROY_FUNCTION,
   F_DESTROY_MENU,
   F_DESTROY_MENUSTYLE,
@@ -120,6 +122,8 @@ enum
   F_EDGE_SCROLL,
   F_EMULATE,
   F_ESCAPE_FUNC,
+  F_EWMH_BASE_STRUT,
+  F_EWMH_NUMBER_OF_DESKTOPS,
   F_EXEC,
   F_EXEC_SETUP,
   F_FAKE_CLICK,
@@ -150,6 +154,7 @@ enum
   F_PICK,
   F_PIXMAP_PATH,
   F_POINTERKEY,
+  F_POINTERWINDOW,
   F_POPUP,
   F_PREV,
   F_QUIT,
@@ -178,7 +183,6 @@ enum
   STROKE_ARG(F_STROKE)
   STROKE_ARG(F_STROKE_FUNC)
   F_STYLE,
-  F_THISWINDOW,
   F_TITLE,
   F_TITLESTYLE,
   F_TOGGLE_PAGE,
@@ -218,7 +222,9 @@ enum
   F_RAISE,
   F_RAISELOWER,
   F_RESIZE,
+  F_RESIZE_MAXIMIZE,
   F_RESIZEMOVE,
+  F_RESIZEMOVE_MAXIMIZE,
   F_SEND_STRING,
   F_STICK,
   F_UPDATE_DECOR,
