@@ -55,6 +55,7 @@
 #include "libs/Colorset.h"
 #include "icccm2.h"
 #include "gnome.h"
+#include "ewmh.h"
 #include "icons.h"
 #include "add_window.h"
 #include "fvwmsignal.h"
@@ -75,7 +76,7 @@
 /* need to get prototype for XrmUniqueQuark for XUniqueContext call */
 #include <X11/Xresource.h>
 
-#if defined(I18N_MB) || defined(COMPOUND_TEXT)
+#ifdef I18N_MB
 #include <X11/Xlocale.h>
 #endif
 
@@ -202,25 +203,9 @@ int main(int argc, char **argv)
 
   setVersionInfo();
 
-#if defined(I18N_MB) || defined(COMPOUND_TEXT)
-  {
-    char *s;
-
-    if ((s = setlocale(LC_CTYPE, "")) == NULL)
-    {
-      fvwm_msg(ERR, "main","Can't set locale. Check your $LC_CTYPE or $LANG.\n");
-    }
-    else if (!XSupportsLocale())
-    {
-      fvwm_msg(WARN, "main", "X does not support current locale: %s\n", s);
-    }
 #ifdef I18N_MB
-    else if (XSetLocaleModifiers("") == NULL)
-    {
-      fvwm_msg(WARN, "main", "Cannot set locale modifiers\n");
-    }
-#endif
-  }
+  if (setlocale(LC_CTYPE, "") == NULL)
+    fvwm_msg(ERR, "main", "Can't set locale. Check your $LC_CTYPE or $LANG.\n");
 #endif
 
   /* Put the default module directory into the environment so it can be used
@@ -727,6 +712,7 @@ int main(int argc, char **argv)
 
   SessionInit();
 
+  EWMH_Init();
   GNOME_Init();
 
   DBUG("main","Entering HandleEvents loop...");
@@ -785,20 +771,17 @@ void StartupStuff(void)
 
   fFvwmInStartup = False;
 
+  /* Make sure the geometry window uses the current font */
+  resize_geometry_window();
+
   /* Make sure we have the correct click time now. */
   if (Scr.ClickTime < 0)
     Scr.ClickTime = -Scr.ClickTime;
 
-# if 0
-  /* It is safe to ungrab here: if not, and one of the init functions
-   * does not finish, we've got a complete freeze! */
-  /* DV (15-Jul-2004): No, it is not safe to ungrab.  If another application
-   * grabs the pointer before execute_function gets it, the start functions
-   * are not executed.  And the pointer is grabbed during function execution
-   * anyway, so releasing it here buys us nothing. */
+  /* It is safe to Ungrab here: if not and one of the init functions not
+   * finish we've got a complete freeze ! */
   UngrabEm(GRAB_STARTUP);
   XUngrabPointer(dpy, CurrentTime);
-#endif
 
   /* migo (04-Sep-1999): execute StartFunction */
   if (FindFunction(startFuncName)) {
@@ -813,9 +796,6 @@ void StartupStuff(void)
     old_execute_function(action, NULL, &Event, C_ROOT, 1, 0, NULL);
     free(action);
   }
-  /* see comment above */
-  UngrabEm(GRAB_STARTUP);
-  XUngrabPointer(dpy, CurrentTime);
 
   /*
      This should be done after the initialization is finished, since
@@ -907,6 +887,9 @@ Atom _XA_WM_WINDOW_ROLE;
 Atom _XA_WM_CLIENT_LEADER;
 Atom _XA_SM_CLIENT_ID;
 
+Atom _XA_XROOTPMAP_ID;
+Atom _XA_XSETROOT_ID;
+
 static void InternUsefulAtoms (void)
 {
   /*
@@ -939,6 +922,9 @@ static void InternUsefulAtoms (void)
   _XA_WM_WINDOW_ROLE=XInternAtom(dpy, "WM_WINDOW_ROLE",False);
   _XA_WM_CLIENT_LEADER=XInternAtom(dpy, "WM_CLIENT_LEADER",False);
   _XA_SM_CLIENT_ID=XInternAtom(dpy, "SM_CLIENT_ID",False);
+
+  _XA_XROOTPMAP_ID=XInternAtom(dpy, "_XROOTPMAP_ID",False);
+  _XA_XSETROOT_ID=XInternAtom(dpy, "_XSETROOT_ID",False);
   return;
 }
 
@@ -1466,8 +1452,7 @@ static void InitVariables(void)
    * compilations */
   Scr.CurrentDesk = 0;
 
-  Scr.EdgeScrollX = DEFAULT_EDGE_SCROLL * Scr.MyDisplayWidth / 100;
-  Scr.EdgeScrollY = DEFAULT_EDGE_SCROLL * Scr.MyDisplayHeight / 100;
+  Scr.EdgeScrollX = Scr.EdgeScrollY = DEFAULT_EDGE_SCROLL;
   Scr.ScrollResistance = DEFAULT_SCROLL_RESISTANCE;
   Scr.MoveResistance = DEFAULT_MOVE_RESISTANCE;
   Scr.XiMoveResistance = DEFAULT_XIMOVE_RESISTANCE;
@@ -1487,6 +1472,21 @@ static void InitVariables(void)
   Scr.randomx = Scr.randomy = 0;
   Scr.buttons2grab = 0;
 
+  /* initialisation of the head of the desktops info */
+  Scr.Desktops = (DesktopsInfo *)safemalloc(sizeof(DesktopsInfo));
+  Scr.Desktops->name = NULL;
+  Scr.Desktops->desk = 0; /* not desk 0 */
+  Scr.Desktops->ewmh_dyn_working_area.x = Scr.Desktops->ewmh_working_area.x = 0;
+  Scr.Desktops->ewmh_dyn_working_area.y = Scr.Desktops->ewmh_working_area.y = 0;
+  Scr.Desktops->ewmh_dyn_working_area.width =
+    Scr.Desktops->ewmh_working_area.width = Scr.MyDisplayWidth;
+  Scr.Desktops->ewmh_dyn_working_area.height =
+    Scr.Desktops->ewmh_working_area.height = Scr.MyDisplayHeight;
+  Scr.Desktops->next = NULL;
+
+  /* ewmh desktop */
+  Scr.EwmhDesktop = NULL;
+
   InitFvwmDecor(&Scr.DefaultDecor);
 #ifdef USEDECOR
   Scr.DefaultDecor.tag = "Default";
@@ -1515,15 +1515,13 @@ static void InitVariables(void)
   Scr.bo.DisableConfigureNotify = 0;
   Scr.bo.InstallRootCmap = 0;
   Scr.bo.FlickeringQtDialogsWorkaround = 1;
+  Scr.bo.EWMHIconicStateWorkaround = 0;
 
   Scr.gs.EmulateMWM = DEFAULT_EMULATE_MWM;
   Scr.gs.EmulateWIN = DEFAULT_EMULATE_WIN;
   Scr.gs.use_active_down_buttons = DEFAULT_USE_ACTIVE_DOWN_BUTTONS;
   Scr.gs.use_inactive_buttons = DEFAULT_USE_INACTIVE_BUTTONS;
   /* Not the right place for this, should only be called once somewhere .. */
-  Scr.flags.is_pointer_on_this_screen = !!XQueryPointer(
-	  dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX, &JunkY, &JunkX, &JunkY,
-	  &JunkMask);
 
   return;
 }
@@ -1577,6 +1575,7 @@ void Done(int restart, char *command)
     Reborder();
   }
 
+  EWMH_ExitStuff();
   if (restart)
   {
     Bool doPreserveState = True;
