@@ -165,6 +165,14 @@ typedef struct
   Pixel shadow;
 } color_quad;
 
+typedef struct
+{
+  int left;
+  int right;
+  int top;
+  int bottom;
+} ewmh_strut;
+
 #ifdef USEDECOR
 struct FvwmDecor;		/* definition in screen.h */
 #endif
@@ -187,6 +195,7 @@ typedef struct
     unsigned do_iconify_window_groups : 1;
     unsigned do_ignore_gnome_hints : 1;
     unsigned do_ignore_icon_boxes : 1;
+    unsigned do_ignore_mouse_focus_click_motion : 1;
     unsigned do_ignore_restack : 1;
     unsigned do_use_window_group_hint : 1;
     unsigned do_lower_transient : 1;
@@ -220,10 +229,22 @@ typedef struct
 #define NO_ICON_OVERRIDE        2
 #define ICON_OVERRIDE_MASK      3
     unsigned is_fixed : 1;
+    unsigned is_fixed_ppos : 1;
     unsigned is_icon_sticky : 1;
     unsigned is_icon_suppressed : 1;
     unsigned is_lenient : 1;
+    unsigned is_size_fixed : 1;
+    unsigned is_psize_fixed : 1;
     unsigned use_icon_position_hint : 1;
+    unsigned use_indexed_window_name : 1;
+    unsigned use_indexed_icon_name : 1;
+    unsigned do_ewmh_mini_icon_override : 1;
+    unsigned do_ewmh_donate_icon : 1;
+    unsigned do_ewmh_donate_mini_icon : 1;
+    unsigned do_ewmh_use_stacking_hints : 1;
+    unsigned do_ewmh_ignore_strut_hints : 1;
+    unsigned do_ewmh_ignore_state_hints : 1;
+    unsigned ewmh_maximize_mode : 2; /* see ewmh.h */
   } s;
 } common_flags_type;
 
@@ -292,6 +313,30 @@ typedef struct
 #define ICON_HINT_MULTIPLE 2
   unsigned was_icon_hint_provided : 2;
   unsigned was_icon_name_provided : 1;
+  unsigned has_ewmh_wm_name : 1;
+  unsigned has_ewmh_wm_icon_name : 1;
+#define EWMH_NO_ICON     0 /* the application does not provide an ewmh icon */
+#define EWMH_TRUE_ICON   1 /* the application does provide an ewmh icon */
+#define EWMH_FVWM_ICON   2 /* the ewmh icon has been set by fvwm */
+  unsigned has_ewmh_wm_icon_hint : 2;
+  unsigned has_ewmh_mini_icon : 1; /* says if the app have an ewmh icon of
+				    * acceptable size for a mini icon in its
+				    * list of icons */
+  unsigned use_ewmh_icon : 1; /* the ewmh icon is used as icon pixmap */
+  unsigned is_ewmh_modal : 1;
+#define EWMH_STATE_UNDEFINED_HINT 0
+#define EWMH_STATE_NO_HINT        1
+#define EWMH_STATE_HAS_HINT       2
+  unsigned has_ewmh_init_fullscreen_state : 2;
+  unsigned has_ewmh_init_hidden_state : 2;
+  unsigned has_ewmh_init_maxhoriz_state : 2;
+  unsigned has_ewmh_init_maxvert_state : 2;
+  unsigned has_ewmh_init_modal_state : 2;
+  unsigned has_ewmh_init_shaded_state : 2;
+  unsigned has_ewmh_init_skip_pager_state : 2;
+  unsigned has_ewmh_init_skip_taskbar_state : 2;
+  unsigned has_ewmh_init_sticky_state : 2;
+  unsigned has_ewmh_init_wm_desktop : 2;
 } window_flags;
 
 /* Window mask for Circulate and Direction functions */
@@ -303,8 +348,13 @@ typedef struct WindowConditionMask
     unsigned needs_current_desk : 1;
     unsigned needs_current_page : 1;
     unsigned needs_current_global_page : 1;
+#define NEEDS_ANY        0
+#define NEEDS_TRUE       1
+#define NEEDS_FALSE      2
+    unsigned needs_focus : 2;
     unsigned needs_name : 1;
     unsigned needs_not_name : 1;
+    unsigned needs_pointer : 2;
     unsigned use_circulate_hit : 1;
     unsigned use_circulate_hit_icon : 1;
     unsigned use_circulate_hit_shaded : 1;
@@ -346,6 +396,7 @@ typedef struct
 #define PLACE_MINOVERLAP        0x7
 #define PLACE_MASK              0x7
   unsigned placement_mode : 3;
+  unsigned ewmh_placement_mode : 2; /* see ewmh.h */
   unsigned do_save_under : 1;
   unsigned do_start_lowered : 1;
   unsigned has_border_width : 1;
@@ -392,6 +443,8 @@ typedef struct
   unsigned manual_placement_honors_starts_on_page : 1;
   unsigned capture_honors_starts_on_page : 1;
   unsigned recapture_honors_starts_on_page : 1;
+  unsigned has_placement_penalty : 1;
+  unsigned has_placement_percentage_penalty : 1;
 } style_flags;
 
 /* only style.c and add_window.c are allowed to access this struct!! */
@@ -431,6 +484,9 @@ typedef struct window_style
   int max_window_height;
   int shade_anim_steps;
   icon_boxes *icon_boxes;
+  float norm_placement_penalty;
+  float placement_penalty[6];
+  int placement_percentage_penalty[4];
   style_flags flags;
   style_flags flag_mask;
   style_flags change_mask;
@@ -448,6 +504,10 @@ typedef struct FvwmWindow
   char **name_list;           /* window name list */
   char **icon_name_list;      /* icon name list */
 #endif
+  char *visible_name;
+  char *visible_icon_name;
+  int name_count;
+  int icon_name_count;
   struct FvwmWindow *next;    /* next fvwm window */
   struct FvwmWindow *prev;    /* prev fvwm window */
   struct FvwmWindow *stack_next; /* next (lower) fvwm window in stacking
@@ -474,8 +534,8 @@ typedef struct FvwmWindow
 #ifdef USEDECOR
   struct FvwmDecor *decor;
 #endif
-  Window icon_w;              /* the icon window */
-  Window icon_pixmap_w;       /* the icon window */
+  Window icon_title_w;        /* icon title window */
+  Window icon_pixmap_w;       /* icon picture window */
   int wShaped;               /* is this a shaped window */
   Pixmap title_background_pixmap;
   Pixmap button_background_pixmap[NUMBER_OF_BUTTONS];
@@ -486,14 +546,15 @@ typedef struct FvwmWindow
 
   rectangle title_g;
   short title_top_height;     /* title height/0 for top/bottom titles */
-  rectangle icon_g;           /* geometry of the icon window */
-  short icon_border_width;
-  int icon_xl_loc;            /* icon label window x coordinate */
-  int icon_t_width;           /* width of the icon title window */
-  int icon_p_width;           /* width of the icon pixmap window */
-  int icon_p_height;          /* height of the icon pixmap window */
-  Pixmap iconPixmap;          /* pixmap for the icon */
+  struct
+  {
+     rectangle picture_w_g;   /* geometry of the icon picture window */
+     rectangle title_w_g;     /* geometry of the icon title window */
+     int title_text_width;    /* width of the text in the icon title */
+  } icon_g;
+
   int iconDepth;              /* Drawable depth for the icon */
+  Pixmap iconPixmap;          /* pixmap for the icon */
   Pixmap icon_maskPixmap;     /* pixmap for the icon mask */
   FvwmFont title_font;
   FvwmFont icon_font;
@@ -546,14 +607,32 @@ typedef struct FvwmWindow
 #define FM_GLOBALLY_ACTIVE 3
   unsigned char focus_model;
 
+  float placement_penalty[6];
+  int placement_percentage_penalty[4];
+
+#define EWMH_WINDOW_TYPE_NONE_ID      0
+#define EWMH_WINDOW_TYPE_DESKTOP_ID   1
+#define EWMH_WINDOW_TYPE_DIALOG_ID    2
+#define EWMH_WINDOW_TYPE_DOCK_ID      3
+#define EWMH_WINDOW_TYPE_MENU_ID      4
+#define EWMH_WINDOW_TYPE_NORMAL_ID    5
+#define EWMH_WINDOW_TYPE_TOOLBAR_ID   6
+
+  int ewmh_window_type;
+  rectangle ewmh_icon_geometry; /* icon geometry */
+  ewmh_strut strut;             /* for computing the working area */
+  ewmh_strut dyn_strut;         /* for the dynamic working area */
+  int ewmh_icon_height;         /* memories for the icons we set on the */
+  int ewmh_icon_width;          /* _NET_WM_ICON */
+  int ewmh_mini_icon_height;
+  int ewmh_mini_icon_width;
+
+  int ewmh_hint_layer;        /* memory for the initial _NET_WM_STATE */
+  unsigned long ewmh_hint_desktop;    /* memory for the initial _NET_WM_STATE */
+
   unsigned char initial_backing_store;
 
-  /* multi purpose scratch structure */
-  struct
-  {
-    void *p;
-    int i;
-  } scratch;
+  void *pscratch;             /* multi purpose scratch pointer */
 } FvwmWindow;
 
 /* include this down here because FvwmWindows must be defined when including
