@@ -26,16 +26,17 @@
 #include "externs.h"
 #include "cursor.h"
 #include "functions.h"
-#include "bindings.h"
 #include "misc.h"
 #include "screen.h"
 #include "gnome.h"
+#include "move_resize.h"
 #include "stack.h"
 #include "update.h"
 #include "style.h"
 #include "virtual.h"
 #include "window_flags.h"
 #include "borders.h"
+#include "decorations.h"
 
 
 /*********************************************************/
@@ -469,7 +470,7 @@ GNOME_SetHints(FvwmWindow *fwin)
     val |= WIN_STATE_STICKY;
   if (IS_SHADED(fwin))
     val |= WIN_STATE_SHADED;
-  if (IS_FIXED(fwin))
+  if (!is_function_allowed(F_MOVE, NULL, fwin, True, False))
     val |= WIN_STATE_FIXED_POSITION;
 
   XChangeProperty(dpy, fwin->w, atom_set, XA_CARDINAL, 32,
@@ -487,7 +488,7 @@ GNOME_GetStyle (FvwmWindow *fwin, window_style *style)
   unsigned char *retval;
   int size;
 
-  if (DO_IGNORE_GNOME_HINTS(fwin))
+  if (SDO_IGNORE_GNOME_HINTS(style->flags))
     return;
   /* Desktop */
   atom_get = XInternAtom(dpy, XA_WIN_WORKSPACE, False);
@@ -536,7 +537,7 @@ GNOME_GetStyle (FvwmWindow *fwin, window_style *style)
       */
     }
 
-    if (*(int*)retval & WIN_STATE_FIXED_POSITION)
+    if ((*(int*)retval & WIN_STATE_FIXED_POSITION))
     {
       SFSET_IS_FIXED(*style, 1);
       SMSET_IS_FIXED(*style, 1);
@@ -710,42 +711,31 @@ GNOME_SetDeskNames(void)
 void
 GNOME_SetClientList(void)
 {
-	Atom atom_set;
-	Window *wl=NULL;
-	int displayed=0;
-	int i = 0;
-	FvwmWindow *t;
+  Atom atom_set;
+  Window wl[256];
+  int i = 0;
+  FvwmWindow *t;
 
-	/* Find out how many window pointers we need to store, allocate
-	 * the space and go through the list again to actually store them.
-	 */
-	for (t = Scr.FvwmRoot.next; t; t = t->next)
-	{
-		if (!DO_SKIP_WINDOW_LIST(t))
-		{
-			displayed++;
-		}
-	}
-	if(displayed)
-	{
-		wl=(Window*)safemalloc(sizeof(Window*)*displayed);
-		for (t = Scr.FvwmRoot.next; t; t = t->next)
-		{
-			if (!DO_SKIP_WINDOW_LIST(t))
-			{
-				wl[i++] = t->w;
-			}
-		}
-	}
-	/* Update X */
-	atom_set = XInternAtom(dpy, XA_WIN_CLIENT_LIST, False);
-	XChangeProperty(
-		dpy, Scr.Root, atom_set, XA_CARDINAL, 32,
-		PropModeReplace, (unsigned char *)wl, i);
-	if(wl)
-		free(wl);
+  atom_set = XInternAtom(dpy, XA_WIN_CLIENT_LIST, False);
 
-	return;
+  for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+  {
+    if (!DO_SKIP_WINDOW_LIST(t))
+    {
+      wl[i++] = t->w;
+    }
+  }
+
+  if (i > 0)
+  {
+    XChangeProperty(dpy, Scr.Root, atom_set, XA_CARDINAL, 32,
+		    PropModeReplace, (unsigned char *)wl, i);
+  }
+  else
+  {
+    XChangeProperty(dpy, Scr.Root, atom_set, XA_CARDINAL, 32,
+		    PropModeReplace, (unsigned char *)NULL, 0);
+  }
 }
 
 void
@@ -876,9 +866,8 @@ CMD_GnomeShowDesks(F_CMD_ARGS)
 int
 GNOME_ProcessClientMessage(FvwmWindow *fwin, XEvent *ev)
 {
+  int x, y;
   Atom a;
-  long p0 = 0;
-  long p1 = 0;
 
   if (fwin != NULL  && DO_IGNORE_GNOME_HINTS(fwin))
     return 0;
@@ -886,40 +875,24 @@ GNOME_ProcessClientMessage(FvwmWindow *fwin, XEvent *ev)
   if (ev->xclient.message_type == a)
   {
     /* convert to integer grid */
-    p0 = ev->xclient.data.l[0] * Scr.MyDisplayWidth;
-    p1 = ev->xclient.data.l[1] * Scr.MyDisplayHeight;
-    if (p0 < 0 || p0 > 0x7fffffff || p1 < 0 || p1 > 0x7fffffff)
-    {
-      return 0;
-    }
-    MoveViewport(p0, p1, 1);
+    x = ev->xclient.data.l[0] * Scr.MyDisplayWidth;
+    y = ev->xclient.data.l[1] * Scr.MyDisplayHeight;
 
+    MoveViewport(x, y, 1);
     return 1;
   }
 
   a = XInternAtom(dpy, XA_WIN_WORKSPACE, False);
   if (ev->xclient.message_type == a)
   {
-    p0 = ev->xclient.data.l[0];
-    if (p0 < 0 || p0 > 0x7fffffff)
-    {
-      return 0;
-    }
-    goto_desk(p0);
-
+    goto_desk(ev->xclient.data.l[0]);
     return 1;
   }
 
   a = XInternAtom(dpy, XA_WIN_LAYER, False);
   if (ev->xclient.message_type == a && fwin)
   {
-    p0 = ev->xclient.data.l[0];
-    if (p0 < 0 || p0 > 0x7fffffff)
-    {
-      return 0;
-    }
-    new_layer(fwin, p0);
-
+    new_layer(fwin, ev->xclient.data.l[0]);
     return 1;
   }
 
@@ -1000,7 +973,7 @@ void GNOME_HandlePropRequest(unsigned int propm,
     {
       SET_STICKY(fwin, 0);
     }
-    DrawDecorations(fwin, DRAW_TITLE, (Scr.Hilite==fwin), True, None);
+    DrawDecorations(fwin, DRAW_TITLE, (Scr.Hilite==fwin), True, None, CLEAR_ALL);
   }
 /*-----------------------------------------------------------------------
     WindowShade
