@@ -94,6 +94,7 @@ typedef struct MenuFeel
   } flags;
   int PopdownDelay10ms;
   int PopupOffsetPercent;
+  int ActiveAreaPercent;
   int PopupOffsetAdd;
   int PopupDelay10ms;
   int DoubleClickTime;
@@ -305,6 +306,10 @@ typedef struct MenuStyle
 #define MST_POPUP_OFFSET_PERCENT(m)   ((m)->s->ms->feel.PopupOffsetPercent)
 #define ST_POPUP_OFFSET_ADD(s)        ((s)->feel.PopupOffsetAdd)
 #define MST_POPUP_OFFSET_ADD(m)       ((m)->s->ms->feel.PopupOffsetAdd)
+#define ST_ACTIVE_AREA_PERCENT(s)  \
+        ((s)->feel.ActiveAreaPercent)
+#define MST_ACTIVE_AREA_PERCENT(m) \
+       ((m)->s->ms->feel.ActiveAreaPercent)
 #define ST_POPDOWN_DELAY(s)           ((s)->feel.PopdownDelay10ms)
 #define MST_POPDOWN_DELAY(m)          ((m)->s->ms->feel.PopdownDelay10ms)
 #define ST_POPUP_DELAY(s)             ((s)->feel.PopupDelay10ms)
@@ -692,9 +697,6 @@ extern MenuInfo Menus;
 
 
 MenuRoot *FollowMenuContinuations(MenuRoot *mr,MenuRoot **pmrPrior);
-void AnimatedMoveOfWindow(Window w,int startX,int startY,int endX, int endY,
-			  Bool fWarpPointerToo, int cusDelay,
-			  float *ppctMovement );
 MenuRoot *NewMenuRoot(char *name);
 void AddToMenu(MenuRoot *, char *, char *, Bool, Bool);
 void do_menu(MenuParameters *pmp, MenuReturn *pret);
@@ -707,5 +709,6 @@ void change_mr_menu_style(MenuRoot *mr, char *stylename);
 void UpdateAllMenuStyles(void);
 void UpdateMenuColorset(int cset);
 void SetMenuCursor(Cursor cursor);
+void ParentalMenuRePaint(void);
 
 #endif /* _MENUS_ */
