@@ -40,6 +40,7 @@
 #include "libs/fvwmlib.h"
 #include "libs/Colorset.h"
 #include "libs/FScreen.h"
+#include "libs/Module.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -50,6 +51,7 @@
 #include "module_interface.h"
 #include "read.h"
 #include "events.h"
+#include "geometry.h"
 #include "fvwmsignal.h"
 
 /*
@@ -62,15 +64,21 @@
 int npipes;
 int *readPipes;
 int *writePipes;
-int *pipeOn;
+static int *pipeOn;
 char **pipeName;
 #ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
 char **pipeAlias;  /* as given in: Module FvwmPager MyAlias */
 #endif
 
-unsigned long *PipeMask;
-unsigned long *SyncMask;
-unsigned long *NoGrabMask;
+typedef struct
+{
+	unsigned long m1;
+	unsigned long m2;
+} msg_masks_type;
+
+static msg_masks_type *PipeMask;
+static msg_masks_type *SyncMask;
+static msg_masks_type *NoGrabMask;
 struct queue_buff_struct **pipeQueue;
 
 extern fd_set init_fdset;
@@ -79,6 +87,53 @@ static void DeleteMessageQueueBuff(int module);
 static void AddToMessageQueue(int module, unsigned long *ptr, int size, int done);
 static void AddToCommandQueue(Window w, int module, char * command);
 
+/*
+ * Returns zero if the msg is not selected by the mask. Takes care of normal
+ * and extended messages.
+ */
+static int is_message_in_mask(msg_masks_type *mask, unsigned long msg)
+{
+	unsigned long rc;
+
+	if (msg & M_EXTENDED_MSG)
+	{
+		rc = (mask->m2 & msg);
+	}
+	else
+	{
+		rc = (mask->m1 & msg);
+	}
+
+	return rc;
+}
+
+/*
+ * Sets the mask to the specific value.  If M_EXTENDED_MSG is set in mask, the
+ * function operates only on the extended messages, otherwise it operates only
+ * on normal messages.
+ */
+static void set_message_mask(msg_masks_type *mask, unsigned long msg)
+{
+	if (msg & M_EXTENDED_MSG)
+	{
+		mask->m2 = (msg & ~M_EXTENDED_MSG);
+	}
+	else
+	{
+		mask->m1 = msg;
+	}
+
+	return;
+}
+
+/*
+ * Returns non zero if one of the specified messages is selected for the module
+ */
+int is_message_selected(int module, unsigned long msg_mask)
+{
+	return is_message_in_mask(&PipeMask[module], msg_mask);
+}
+
 void initModules(void)
 {
   int i;
@@ -88,9 +143,9 @@ void initModules(void)
   writePipes = (int *)safemalloc(sizeof(int)*npipes);
   readPipes = (int *)safemalloc(sizeof(int)*npipes);
   pipeOn = (int *)safemalloc(sizeof(int)*npipes);
-  PipeMask = (unsigned long *)safemalloc(sizeof(unsigned long)*npipes);
-  SyncMask = (unsigned long *)safemalloc(sizeof(unsigned long)*npipes);
-  NoGrabMask = (unsigned long *)safemalloc(sizeof(unsigned long)*npipes);
+  PipeMask = (msg_masks_type *)safemalloc(sizeof(msg_masks_type)*npipes);
+  SyncMask = (msg_masks_type *)safemalloc(sizeof(msg_masks_type)*npipes);
+  NoGrabMask = (msg_masks_type *)safemalloc(sizeof(msg_masks_type)*npipes);
   pipeName = (char **)safemalloc(sizeof(char *)*npipes);
 #ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
   pipeAlias= (char **)safemalloc(sizeof(char *)*npipes);
@@ -98,20 +153,23 @@ void initModules(void)
   pipeQueue=(struct queue_buff_struct **)
     safemalloc(sizeof(struct queue_buff_struct *)*npipes);
 
-  for(i=0;i<npipes;i++)
-    {
-      writePipes[i]= -1;
-      readPipes[i]= -1;
-      pipeOn[i] = -1;
-      PipeMask[i] = MAX_MASK;
-      SyncMask[i] = 0;
-      NoGrabMask[i] = 0;
-      pipeQueue[i] = (struct queue_buff_struct *)NULL;
-      pipeName[i] = NULL;
+  for (i=0; i < npipes; i++)
+  {
+    writePipes[i]= -1;
+    readPipes[i]= -1;
+    pipeOn[i] = -1;
+    PipeMask[i].m1 = DEFAULT_MASK;
+    PipeMask[i].m2 = DEFAULT_XMASK;
+    SyncMask[i].m1 = 0;
+    SyncMask[i].m2 = 0;
+    NoGrabMask[i].m1 = 0;
+    NoGrabMask[i].m2 = 0;
+    pipeQueue[i] = (struct queue_buff_struct *)NULL;
+    pipeName[i] = NULL;
 #ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
-      pipeAlias[i] = NULL;
+    pipeAlias[i] = NULL;
 #endif
-    }
+  }
   DBUG("initModules", "Zeroing init module array\n");
   FD_ZERO(&init_fdset);
 }
@@ -296,9 +354,12 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
     writePipes[i] = fvwm_to_app[1];
     readPipes[i] = app_to_fvwm[0];
     pipeOn[i] = -1;
-    PipeMask[i] = MAX_MASK;
-    SyncMask[i] = 0;
-    NoGrabMask[i] = 0;
+    PipeMask[i].m1 = DEFAULT_MASK;
+    PipeMask[i].m2 = DEFAULT_XMASK;
+    SyncMask[i].m1 = 0;
+    SyncMask[i].m2 = 0;
+    NoGrabMask[i].m1 = 0;
+    NoGrabMask[i].m2 = 0;
     free(arg1);
     pipeQueue[i] = NULL;
     if (DoingCommandLine) {
@@ -895,7 +956,7 @@ static void BroadcastNewPacket(unsigned long event_type,
 
 
 /* this is broken, the flags may not fit in a word */
-#define CONFIGARGS(_t) 24,\
+#define CONFIGARGS(_t) 27,\
             (_t)->w,\
             (_t)->frame,\
             (unsigned long)(_t),\
@@ -915,14 +976,17 @@ static void BroadcastNewPacket(unsigned long event_type,
             (_t)->hints.min_height,\
             (_t)->hints.max_width,\
             (_t)->hints.max_height,\
-            (_t)->icon_w,\
+            (_t)->icon_title_w,\
             (_t)->icon_pixmap_w,\
             (_t)->hints.win_gravity,\
             (_t)->colors.fore,\
-            (_t)->colors.back
+            (_t)->colors.back,\
+            (_t)->ewmh_hint_layer,\
+            (_t)->ewmh_hint_desktop,\
+            (_t)->ewmh_window_type
 
 #ifndef DISABLE_MBC
-#define OLDCONFIGARGS(_t) 24,\
+#define OLDCONFIGARGS(_t) 27,\
             (_t)->w,\
             (_t)->frame,\
             (unsigned long)(_t),\
@@ -942,11 +1006,14 @@ static void BroadcastNewPacket(unsigned long event_type,
             (_t)->hints.min_height,\
             (_t)->hints.max_width,\
             (_t)->hints.max_height,\
-            (_t)->icon_w,\
+            (_t)->icon_title_w,\
             (_t)->icon_pixmap_w,\
             (_t)->hints.win_gravity,\
             (_t)->colors.fore,\
-            (_t)->colors.back
+            (_t)->colors.back,\
+            (_t)->ewmh_hint_layer,\
+            (_t)->ewmh_hint_desktop,\
+            (_t)->ewmh_window_type
 
 #define SETOLDFLAGS \
 { int i = 1; \
@@ -996,7 +1063,7 @@ static void BroadcastNewPacket(unsigned long event_type,
         as a dummy to preserve alignment of the other fields in the
         old packet: we should drop this before the next release.
 *****************************************************************/
-#define CONFIGARGSNEW(_t) 25,\
+#define CONFIGARGSNEW(_t) 28,\
 	    (unsigned long)(sizeof(unsigned long)),\
             &(*(_t))->w,\
 	    (unsigned long)(sizeof(unsigned long)),\
@@ -1036,7 +1103,7 @@ static void BroadcastNewPacket(unsigned long event_type,
 	    (unsigned long)(0),\
             &(*(_t))->hints.max_height,\
 	    (unsigned long)(0),\
-            &(*(_t))->icon_w,\
+            &(*(_t))->icon_title_w,\
 	    (unsigned long)(sizeof(unsigned long)),\
             &(*(_t))->icon_pixmap_w,\
 	    (unsigned long)(0),\
@@ -1045,6 +1112,12 @@ static void BroadcastNewPacket(unsigned long event_type,
             &(*(_t))->colors.fore,\
 	    (unsigned long)(sizeof(unsigned long)),\
             &(*(_t))->colors.back,\
+            (unsigned long)(0),\
+            &(*(_t))->ewmh_hint_layer,\
+	    (unsigned long)(sizeof(unsigned long)),\
+            &(*(_t))->ewmh_hint_desktop,\
+	    (unsigned long)(0),\
+            &(*(_t))->ewmh_window_type,\
 	    (unsigned long)(sizeof((*(_t))->flags)),\
             &(*(_t))->flags
 
@@ -1091,7 +1164,7 @@ const FvwmWindow **t1 = &t;
 
 static unsigned long *
 make_named_packet(int *len, unsigned long event_type, const char *name,
-                  int num, ...)
+		  int num, ...)
 {
   unsigned long *body;
   va_list ap;
@@ -1101,8 +1174,8 @@ make_named_packet(int *len, unsigned long event_type, const char *name,
   *len =
     FvwmPacketHeaderSize + num + (strlen(name) / sizeof(unsigned long)) + 1;
   /* truncate long packets */
-  if (*len > FvwmPacketMaxSize)
-    *len = FvwmPacketMaxSize;
+  if (*len > FvwmPacketMaxSize_byte)
+    *len = FvwmPacketMaxSize_byte;
 
   body = (unsigned long *)safemalloc(*len * sizeof(unsigned long));
   body[*len-1] = 0; /* Zero out end of memory to avoid uninit memory access. */
@@ -1111,8 +1184,7 @@ make_named_packet(int *len, unsigned long event_type, const char *name,
   make_vpacket(body, event_type, num, ap);
   va_end(ap);
 
-  strncpy((char *)&body[FvwmPacketHeaderSize+num], name,
-	  (*len - FvwmPacketHeaderSize - num)*sizeof(unsigned long) - 1);
+  strcpy((char *)&body[FvwmPacketHeaderSize+num], name);
   body[2] = *len;
 
 #if 0
@@ -1159,6 +1231,23 @@ BroadcastName(unsigned long event_type,
   free(body);
 }
 
+void
+BroadcastWindowIconNames(FvwmWindow *t, Bool window, Bool icon)
+{
+  if (window)
+  {
+    BroadcastName(M_WINDOW_NAME,t->w,t->frame,(unsigned long)t,t->name);
+    BroadcastName(M_VISIBLE_NAME,t->w,t->frame,(unsigned long)t,
+		  t->visible_name);
+  }
+  if (icon)
+  {
+    BroadcastName(M_ICON_NAME,t->w,t->frame,(unsigned long)t,t->icon_name);
+    BroadcastName(MX_VISIBLE_ICON_NAME,t->w,t->frame,
+		  (unsigned long)t,t->visible_icon_name);
+  }
+}
+
 #ifdef MINI_ICONS
 static void
 SendMiniIcon(int module, unsigned long event_type,
@@ -1245,6 +1334,20 @@ void broadcast_xinerama_state(void)
 }
 
 
+/**********************************************************************
+ * Broadcasts the ignored modifiers to all modules as M_CONFIG_INFO.
+ **********************************************************************/
+void broadcast_ignore_modifiers(void)
+{
+  char msg[32];
+
+  sprintf(msg, "IgnoreModifiers %d", GetUnusedModifiers());
+  BroadcastConfigInfoString(msg);
+
+  return;
+}
+
+
 /*
 ** send an arbitrary string to all instances of a module
 */
@@ -1259,9 +1362,10 @@ void CMD_SendToModule(F_CMD_ARGS)
   /* UngrabEm(); */
   if (!action)
     return;
-  str = GetNextToken(action, &module);
+  GetNextToken(action,&module);
   if (!module)
     return;
+  str = safestrdup(action + strlen(module) + 1);
 
   if (tmp_win) {
     /* Modules may need to know which window this applies to */
@@ -1287,6 +1391,7 @@ void CMD_SendToModule(F_CMD_ARGS)
   }
 
   free(module);
+  free(str);
 }
 
 
@@ -1298,12 +1403,17 @@ extern char *ModuleUnlock;		/* defined in libs/Module.c */
 void PositiveWrite(int module, unsigned long *ptr, int size)
 {
   extern int moduleTimeout;
+  msg_masks_type mask;
 
   if(ptr == NULL)
   {
    return;
   }
-  if((pipeOn[module]<0)||(!((PipeMask[module]) & ptr[1])))
+  if (pipeOn[module] < 0)
+  {
+    return;
+  }
+  if (!is_message_in_mask(&PipeMask[module], ptr[1]))
   {
     return;
   }
@@ -1312,7 +1422,9 @@ void PositiveWrite(int module, unsigned long *ptr, int size)
   /* would be better to send RecaptureStart and RecaptureEnd messages. */
   /* If module is lock on send for iconify message and it's an
    * iconify event and server grabbed, then return */
-  if ((NoGrabMask[module] & SyncMask[module] & ptr[1]) && (myxgrabcount != 0))
+  mask.m1 = (NoGrabMask[module].m1 & SyncMask[module].m1);
+  mask.m2 = (NoGrabMask[module].m2 & SyncMask[module].m2);
+  if (is_message_in_mask(&mask, ptr[1]) && myxgrabcount != 0)
   {
     return;
   }
@@ -1327,7 +1439,7 @@ void PositiveWrite(int module, unsigned long *ptr, int size)
   /* migo (19-Aug-2000): removed !myxgrabcount to sync M_DESTROY_WINDOW */
 /*if ((SyncMask[module] & ptr[1]) && !myxgrabcount) */
 
-  if ((SyncMask[module] & ptr[1]))
+  if (is_message_in_mask(&SyncMask[module], ptr[1]))
   {
     Window targetWindow;
     fd_set readSet;
@@ -1365,32 +1477,18 @@ void PositiveWrite(int module, unsigned long *ptr, int size)
       if (rc <= 0 || read(channel, &targetWindow, sizeof(targetWindow))
            != sizeof(targetWindow))
       {
-        char *name;
-
-        if (pipeName[module] != NULL)
-        {
-#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
-          if (pipeAlias[module] != NULL)
-          {
-            name = CatString3(pipeName[module], " ", pipeAlias[module]);
-          }
-          else
-#endif
-          {
-            name = pipeName[module];
-          }
-        }
-        else
-        {
-          name = "(null)";
-        }
-
         /* Doh! Something has gone wrong - get rid of the offender!! */
         fvwm_msg(ERR, "PositiveWrite",
                  "Failed to read descriptor from '%s':\n"
                  "- data available=%c\n"
                  "- terminate signal=%c\n",
-                 name,
+#ifndef WITHOUT_KILLMODULE_ALIAS_SUPPORT
+                 pipeAlias[module]
+                   ? CatString3(pipeName[module], " ", pipeAlias[module])
+                   : pipeName[module],
+#else
+                 pipeName[module],
+#endif
                  (FD_ISSET(channel, &readSet) ? 'Y' : 'N'),
                  isTerminated ? 'Y' : 'N');
         KillModule(module);
@@ -1614,7 +1712,10 @@ void CMD_Send_WindowList(F_CMD_ARGS)
 		   (unsigned long)t,t->name);
 	  SendName(*Module,M_ICON_NAME,t->w,t->frame,
 		   (unsigned long)t,t->icon_name);
-
+	  SendName(*Module,M_VISIBLE_NAME,t->w,t->frame,
+		   (unsigned long)t,t->visible_name);
+	  SendName(*Module,MX_VISIBLE_ICON_NAME,t->w,t->frame,
+		   (unsigned long)t,t->visible_icon_name);
           if (t->icon_bitmap_file != NULL
               && t->icon_bitmap_file != Scr.DefaultIcon)
             SendName(*Module,M_ICON_FILE,t->w,t->frame,
@@ -1626,10 +1727,17 @@ void CMD_Send_WindowList(F_CMD_ARGS)
 		   (unsigned long)t,t->class.res_name);
 
 	  if((IS_ICONIFIED(t))&&(!IS_ICON_UNMAPPED(t)))
-	    SendPacket(*Module, M_ICONIFY, 7, t->w, t->frame,
-		       (unsigned long)t,
-		       t->icon_g.x, t->icon_g.y,
-		       t->icon_g.width, t->icon_g.height+t->icon_p_height);
+	  {
+	    rectangle r;
+	    Bool rc;
+
+	    rc = get_visible_icon_geometry(t, &r);
+	    if (rc == True)
+	    {
+	      SendPacket(*Module, M_ICONIFY, 7, t->w, t->frame,
+			 (unsigned long)t, r.x, r.y, r.width, r.height);
+	    }
+	  }
 
 	  if((IS_ICONIFIED(t))&&(IS_ICON_UNMAPPED(t)))
 	    SendPacket(*Module, M_ICONIFY, 7, t->w, t->frame,
@@ -1671,26 +1779,30 @@ void CMD_Send_WindowList(F_CMD_ARGS)
 
 void CMD_set_mask(F_CMD_ARGS)
 {
-  int val = 0;
+  unsigned long val;
 
-  GetIntegerArguments(action, NULL, &val, 1);
-  PipeMask[*Module] = (unsigned long)val;
+  /*GetIntegerArguments(action, NULL, &val, 1);*/
+  if (!action || sscanf(action,"%lu",&val) != 1)
+    val = 0;
+  set_message_mask(&PipeMask[*Module], (unsigned long)val);
 }
 
 void CMD_set_sync_mask(F_CMD_ARGS)
 {
-  int val = 0;
+  unsigned long val;
 
-  GetIntegerArguments(action, NULL, &val, 1);
-  SyncMask[*Module] = (unsigned long)val;
+  if (!action || sscanf(action,"%lu",&val) != 1)
+    val = 0;
+  set_message_mask(&SyncMask[*Module], (unsigned long)val);
 }
 
 void CMD_set_nograb_mask(F_CMD_ARGS)
 {
-  int val = 0;
+  unsigned long val;
 
-  GetIntegerArguments(action, NULL, &val, 1);
-  NoGrabMask[*Module] = (unsigned long)val;
+  if (!action || sscanf(action,"%lu",&val) != 1)
+    val = 0;
+  set_message_mask(&NoGrabMask[*Module], (unsigned long)val);
 }
 
 /*
@@ -1715,3 +1827,4 @@ char *skipModuleAliasToken(const char *string)
   }
   return NULL;
 }
+
