@@ -48,10 +48,12 @@
 #include "borders.h"
 #include "geometry.h"
 #include "gnome.h"
+#include "ewmh.h"
 #include "colormaps.h"
 #include "virtual.h"
 #include "decorations.h"
 #include "events.h"
+#include "menus.h"
 #include <X11/keysym.h>
 
 /* ----- move globals ----- */
@@ -159,8 +161,12 @@ static int GetOnePositionArgument(
     int x = 0;
     int y = 0;
 
-    XQueryPointer(
-      dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX, &JunkY, &x, &y, &JunkMask);
+    if (XQueryPointer(
+	  dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX, &JunkY, &x, &y,
+	  &JunkMask) == False)
+    {
+      /* pointer is on a different screen - that's okay here */
+    }
     *pFinalX += (is_x) ? x : y;
   }
 
@@ -197,7 +203,6 @@ static int GetMoveArguments(
   if (s1 && fPointer && StrEquals(s1, "pointer"))
   {
       *fPointer = True;
-      *paction = action;
       free(s1);
       return 0;
   }
@@ -581,8 +586,7 @@ static void DisplaySize(
 #endif
 }
 
-
-void CMD_ResizeMove(F_CMD_ARGS)
+static Bool resize_move_window(F_CMD_ARGS)
 {
   int FinalX = 0;
   int FinalY = 0;
@@ -592,20 +596,16 @@ void CMD_ResizeMove(F_CMD_ARGS)
   int x,y;
   Bool fWarp = False;
   Bool fPointer = False;
-  Bool has_focus;
   int dx;
   int dy;
 
-  if (DeferExecution(eventp,&w,&tmp_win,&context, CRS_RESIZE, ButtonPress))
-    return;
-  if (tmp_win == NULL)
-    return;
-  if (IS_FIXED(tmp_win))
-    return;
-  if (check_if_function_allowed(F_RESIZE,tmp_win,True,NULL) == 0)
+  if (!is_function_allowed(F_MOVE, NULL, tmp_win, True, False))
   {
-    XBell(dpy, 0);
-    return;
+    return False;
+  }
+  if (!is_function_allowed(F_RESIZE, NULL, tmp_win, True, True))
+  {
+    return False;
   }
 
   /* gotta have a window */
@@ -614,7 +614,7 @@ void CMD_ResizeMove(F_CMD_ARGS)
 		    (unsigned int *)&FinalH, &JunkBW, &JunkDepth))
   {
     XBell(dpy, 0);
-    return;
+    return False;
   }
 
   FinalX = x;
@@ -627,7 +627,9 @@ void CMD_ResizeMove(F_CMD_ARGS)
     tmp_win->boundary_width, tmp_win->title_g.height,
     &FinalX, &FinalY, &FinalW, &FinalH, &fWarp, &fPointer);
   if (n < 4)
-    return;
+  {
+    return False;
+  }
 
   if (IS_MAXIMIZED(tmp_win))
   {
@@ -635,7 +637,7 @@ void CMD_ResizeMove(F_CMD_ARGS)
      * depressed. */
     SET_MAXIMIZED(tmp_win, 0);
     DrawDecorations(
-      tmp_win, DRAW_BUTTONS, (tmp_win == Scr.Hilite), True, None);
+      tmp_win, DRAW_BUTTONS, (tmp_win == Scr.Hilite), True, None, CLEAR_ALL);
   }
   dx = FinalX - tmp_win->frame_g.x;
   dy = FinalY - tmp_win->frame_g.y;
@@ -662,13 +664,27 @@ void CMD_ResizeMove(F_CMD_ARGS)
     tmp_win->normal_g.x += dx;
     tmp_win->normal_g.y += dy;
   }
-  has_focus = (tmp_win == get_focus_window())? True : False;
-  DrawDecorations(tmp_win, DRAW_ALL, has_focus, True, None);
+  DrawDecorations(tmp_win, DRAW_ALL, True, True, None, CLEAR_ALL);
   update_absolute_geometry(tmp_win);
   maximize_adjust_offset(tmp_win);
   XSync(dpy, 0);
   GNOME_SetWinArea(tmp_win);
 
+  return True;
+}
+
+void CMD_ResizeMove(F_CMD_ARGS)
+{
+  if (DeferExecution(eventp,&w,&tmp_win,&context, CRS_RESIZE, ButtonPress))
+  {
+    return;
+  }
+  if (tmp_win == NULL || IS_ICONIFIED(tmp_win))
+  {
+    return;
+  }
+  resize_move_window(F_PASS_ARGS);
+
   return;
 }
 
@@ -696,8 +712,8 @@ static void InteractiveMove(
   if (do_start_at_pointer)
   {
     if (XQueryPointer(
-      dpy, Scr.Root, &JunkRoot, &JunkChild, &DragX, &DragY, &JunkX, &JunkY,
-      &JunkMask) == False)
+	  dpy, Scr.Root, &JunkRoot, &JunkChild, &DragX, &DragY, &JunkX, &JunkY,
+	  &JunkMask) == False)
     {
       /* pointer is on a different screen */
       DragX = 0;
@@ -790,14 +806,14 @@ static void InteractiveMove(
  * entry */
 static void AnimatedMoveAnyWindow(
   FvwmWindow *tmp_win, Window w, int startX, int startY, int endX, int endY,
-  Bool fWarpPointerToo, int cmsDelay, float *ppctMovement)
+  Bool fWarpPointerToo, int cmsDelay, float *ppctMovement, Bool ParentalMenu)
 {
   int pointerX, pointerY;
   int currentX, currentY;
   int lastX, lastY;
   int deltaX, deltaY;
 
-  if (tmp_win && IS_FIXED(tmp_win))
+  if (!is_function_allowed(F_MOVE, NULL, tmp_win, True, False))
     return;
 
   /* set our defaults */
@@ -839,23 +855,24 @@ static void AnimatedMoveAnyWindow(
       /* don't waste time in the same spot */
       continue;
     XMoveWindow(dpy,w,currentX,currentY);
+    if (ParentalMenu)
+    {
+      ParentalMenuRePaint();
+    }
     if (fWarpPointerToo == True)
     {
-      if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
-			&JunkX,&JunkY,&pointerX,&pointerY,&JunkMask) == False)
+      if (XQueryPointer(
+	    dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX, &JunkY, &pointerX,
+	    &pointerY,&JunkMask) == False)
       {
-	/* pointer is on a different screen */
-	pointerX = currentX;
-	pointerY = currentY;
+	/* pointer is on a different screen - that's okay here */
       }
-      else
-      {
-	pointerX += currentX - lastX;
-	pointerY += currentY - lastY;
-      }
-      XWarpPointer(dpy,None,Scr.Root,0,0,0,0, pointerX,pointerY);
+      pointerX += currentX - lastX;
+      pointerY += currentY - lastY;
+      XWarpPointer(dpy,None,Scr.Root,0,0,0,0,
+		   pointerX,pointerY);
     }
-    if (tmp_win && !IS_SHADED(tmp_win) && !Scr.bo.DisableConfigureNotify)
+    if (tmp_win && !IS_SHADED(tmp_win))
     {
       /* send configure notify event for windows that care about their
        * location */
@@ -893,6 +910,10 @@ static void AnimatedMoveAnyWindow(
       StashEventTime(&Event);
       /* finish the move immediately */
       XMoveWindow(dpy,w,endX,endY);
+      if (ParentalMenu)
+      {
+	ParentalMenuRePaint();
+      }
       break;
     }
     lastX = currentX;
@@ -909,11 +930,12 @@ static void AnimatedMoveAnyWindow(
 
 /* used for moving menus, not a client window */
 void AnimatedMoveOfWindow(Window w, int startX, int startY,
-                             int endX, int endY, Bool fWarpPointerToo,
-                             int cmsDelay, float *ppctMovement)
+			  int endX, int endY, Bool fWarpPointerToo,
+			  int cmsDelay, float *ppctMovement,
+			  Bool ParentalMenu)
 {
   AnimatedMoveAnyWindow(NULL, w, startX, startY, endX, endY, fWarpPointerToo,
-                        cmsDelay, ppctMovement);
+                        cmsDelay, ppctMovement, ParentalMenu);
 }
 
 /* used for moving client windows */
@@ -923,7 +945,7 @@ void AnimatedMoveFvwmWindow(FvwmWindow *tmp_win, Window w, int startX,
 			    float *ppctMovement)
 {
   AnimatedMoveAnyWindow(tmp_win, w, startX, startY, endX, endY,
-			fWarpPointerToo, cmsDelay, ppctMovement);
+			fWarpPointerToo, cmsDelay, ppctMovement, False);
 }
 
 
@@ -951,17 +973,10 @@ static void move_window_doit(F_CMD_ARGS, Bool do_animate, int mode)
   {
     return;
   }
-
   if (tmp_win == NULL)
     return;
-
-  if (IS_FIXED(tmp_win))
+  if (!is_function_allowed(F_MOVE, NULL, tmp_win, True, False))
     return;
-  if (check_if_function_allowed(F_MOVE, tmp_win, True, NULL) == 0)
-  {
-    return;
-  }
-
 
   /* gotta have a window */
   w = tmp_win->frame;
@@ -970,11 +985,11 @@ static void move_window_doit(F_CMD_ARGS, Bool do_animate, int mode)
     if(tmp_win->icon_pixmap_w != None)
     {
       w = tmp_win->icon_pixmap_w;
-      XUnmapWindow(dpy,tmp_win->icon_w);
+      XUnmapWindow(dpy,tmp_win->icon_title_w);
     }
     else
     {
-      w = tmp_win->icon_w;
+      w = tmp_win->icon_title_w;
     }
   }
   if (!XGetGeometry(dpy, w, &JunkRoot, &x, &y, &width, &height,
@@ -1046,10 +1061,6 @@ static void move_window_doit(F_CMD_ARGS, Bool do_animate, int mode)
 
     if (n != 2 || fPointer)
       InteractiveMove(&w, tmp_win, &FinalX, &FinalY, eventp, fPointer);
-    else if (IS_ICONIFIED(tmp_win))
-    {
-      SET_ICON_MOVED(tmp_win, 1);
-    }
   }
 
   if (w == tmp_win->frame)
@@ -1081,43 +1092,50 @@ static void move_window_doit(F_CMD_ARGS, Bool do_animate, int mode)
   }
   else /* icon window */
   {
-    tmp_win->icon_g.x = FinalX;
-    tmp_win->icon_xl_loc = FinalX -
-      (tmp_win->icon_g.width - tmp_win->icon_p_width)/2;
-    tmp_win->icon_g.y = FinalY;
-    BroadcastPacket(M_ICON_LOCATION, 7,
-		    tmp_win->w, tmp_win->frame,
-		    (unsigned long)tmp_win,
-		    tmp_win->icon_g.x, tmp_win->icon_g.y,
-		    tmp_win->icon_p_width,
-		    tmp_win->icon_g.height + tmp_win->icon_p_height);
-    if (do_animate)
+    rectangle gt;
+    rectangle gp;
+    Bool has_icon_title;
+    Bool has_icon_picture;
+
+    set_icon_position(tmp_win, FinalX, FinalY);
+    broadcast_icon_geometry(tmp_win, False);
+    has_icon_title = get_visible_icon_title_geometry(tmp_win, &gt);
+    has_icon_picture = get_visible_icon_picture_geometry(tmp_win, &gp);
+    if (has_icon_picture)
     {
-      AnimatedMoveOfWindow(tmp_win->icon_w,-1,-1,tmp_win->icon_xl_loc,
-			   FinalY+tmp_win->icon_p_height, fWarp,-1,NULL);
-    }
-    else
-    {
-      XMoveWindow(dpy,tmp_win->icon_w, tmp_win->icon_xl_loc,
-		  FinalY+tmp_win->icon_p_height);
-      if (fWarp)
-	XWarpPointer(dpy, None, None, 0, 0, 0, 0, FinalX - x, FinalY - y);
-    }
-    if(tmp_win->icon_pixmap_w != None)
-    {
-      XMapWindow(dpy,tmp_win->icon_w);
       if (do_animate)
       {
-	AnimatedMoveOfWindow(tmp_win->icon_pixmap_w, -1,-1,
-			     tmp_win->icon_g.x,FinalY,fWarp,-1,NULL);
+	AnimatedMoveOfWindow(
+	  tmp_win->icon_pixmap_w, -1, -1, gp.x, gp.y, fWarp, -1, NULL, False);
       }
       else
       {
-	XMoveWindow(dpy, tmp_win->icon_pixmap_w, tmp_win->icon_g.x, FinalY);
+	XMoveWindow(dpy, tmp_win->icon_pixmap_w, gp.x, gp.y);
+	if (fWarp)
+	{
+	  XWarpPointer(dpy, None, None, 0, 0, 0, 0, FinalX - x, FinalY - y);
+	}
+      }
+      if (has_icon_title)
+      {
+	XMoveWindow(dpy, tmp_win->icon_title_w, gt.x, gt.y);
+	XMapWindow(dpy, tmp_win->icon_title_w);
+      }
+      XMapWindow(dpy, w);
+    }
+    else if (has_icon_title)
+    {
+      if (do_animate)
+      {
+	AnimatedMoveOfWindow(
+	  tmp_win->icon_title_w, -1, -1, gt.x, gt.y, fWarp, -1, NULL, False);
+      }
+      else
+      {
+	XMoveWindow(dpy, tmp_win->icon_title_w, gt.x, gt.y);
 	if (fWarp)
 	  XWarpPointer(dpy, None, None, 0, 0, 0, 0, FinalX - x, FinalY - y);
       }
-      XMapWindow(dpy,w);
     }
     XSync(dpy, 0);
   }
@@ -1153,6 +1171,8 @@ static void DoSnapAttract(
   int nyt,nxl,dist,closestLeft,closestRight,closestBottom,closestTop;
   rectangle self, other;
   FvwmWindow *tmp;
+  rectangle g;
+  Bool rc;
 
   /* resist based on window edges */
   closestTop = Scr.SnapAttraction;
@@ -1165,9 +1185,10 @@ static void DoSnapAttract(
   self.y = *py;
   self.width = Width;
   self.height = Height;
-  if (IS_ICONIFIED(tmp_win) && !HAS_NO_ICON_TITLE(tmp_win))
+  rc = get_visible_icon_title_geometry(tmp_win, &g);
+  if (rc == True)
   {
-    self.height += tmp_win->icon_g.height;
+    self.height += g.height;
   }
 
   /*
@@ -1210,32 +1231,7 @@ static void DoSnapAttract(
 	break;
       }
       /* get other window dimensions */
-      if (IS_ICONIFIED(tmp))
-      {
-	if (tmp->icon_p_height > 0)
-	{
-	  other.width = tmp->icon_p_width;
-	  other.height = tmp->icon_p_height;
-	}
-	else
-	{
-	  other.width = tmp->icon_g.width;
-	  other.height = tmp->icon_g.height;
-	}
-	other.x = tmp->icon_g.x;
-	other.y = tmp->icon_g.y;
-	if (!HAS_NO_ICON_TITLE(tmp))
-	{
-	  other.height += tmp->icon_g.height;
-	}
-      }
-      else
-      {
-	other.width = tmp->frame_g.width;
-	other.height = tmp->frame_g.height;
-	other.x = tmp->frame_g.x;
-	other.y = tmp->frame_g.y;
-      }
+      get_visible_window_or_icon_geometry(tmp, &other);
       /* prevent that window snaps off screen */
       if (other.x <= 0)
       {
@@ -1560,8 +1556,8 @@ Bool moveLoop(FvwmWindow *tmp_win, int XOffset, int YOffset, int Width,
   {
     if (tmp_win->icon_pixmap_w != None)
       move_w = tmp_win->icon_pixmap_w;
-    else if (tmp_win->icon_w != None)
-      move_w = tmp_win->icon_w;
+    else if (tmp_win->icon_title_w != None)
+      move_w = tmp_win->icon_title_w;
   }
   else
   {
@@ -1575,8 +1571,11 @@ Bool moveLoop(FvwmWindow *tmp_win, int XOffset, int YOffset, int Width,
 
   if (IS_ICONIFIED(tmp_win))
   {
-    orig_icon_x = tmp_win->icon_g.x;
-    orig_icon_y = tmp_win->icon_g.y;
+    rectangle g;
+
+    get_visible_icon_geometry(tmp_win, &g);
+    orig_icon_x = g.x;
+    orig_icon_y = g.y;
   }
 
   /* make a copy of the tmp_win structure for sending to the pager */
@@ -1584,20 +1583,16 @@ Bool moveLoop(FvwmWindow *tmp_win, int XOffset, int YOffset, int Width,
   /* prevent flicker when paging */
   SET_WINDOW_BEING_MOVED_OPAQUE(tmp_win, do_move_opaque);
 
-  if (XQueryPointer(
-	dpy, Scr.Root, &JunkRoot, &JunkChild, &xl, &yt, &JunkX, &JunkY,
-	&button_mask) == False)
+  if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild, &xl, &yt,
+		    &JunkX, &JunkY, &button_mask) == False)
   {
     /* pointer is on a different screen */
     xl = 0;
     yt = 0;
   }
-  else
-  {
-    xl += XOffset;
-    yt += YOffset;
-  }
   button_mask &= DEFAULT_ALL_BUTTONS_MASK;
+  xl += XOffset;
+  yt += YOffset;
   xl_orig = xl;
   yt_orig = yt;
 
@@ -1610,17 +1605,13 @@ Bool moveLoop(FvwmWindow *tmp_win, int XOffset, int YOffset, int Width,
 
   while (!finished && bad_window != tmp_win->w)
   {
-    int rc = 0;
-
     /* wait until there is an interesting event */
-    while (rc != -1 && (!XPending(dpy) ||
+    while (!XPending(dpy) ||
 	   !XCheckMaskEvent(dpy, ButtonPressMask | ButtonReleaseMask |
 			    KeyPressMask | PointerMotionMask |
-			    ButtonMotionMask | ExposureMask, &Event)))
+			    ButtonMotionMask | ExposureMask, &Event))
     {
-      rc = HandlePaging(
-	      dx, dy, &xl, &yt, &delta_x, &delta_y, False, False, True);
-      if (rc == 1)
+      if (HandlePaging(dx, dy, &xl,&yt, &delta_x,&delta_y, False, False, True))
       {
 	/* Fake an event to force window reposition */
 	if (delta_x)
@@ -1647,13 +1638,6 @@ Bool moveLoop(FvwmWindow *tmp_win, int XOffset, int YOffset, int Width,
 	break;
       }
     }
-    if (rc == -1)
-    {
-      /* block until an event arrives */
-      XMaskEvent(dpy, ButtonPressMask | ButtonReleaseMask |
-		 KeyPressMask | PointerMotionMask |
-		 ButtonMotionMask | ExposureMask, &Event);
-    }
     StashEventTime(&Event);
 
     /* discard any extra motion events before a logical release */
@@ -1767,8 +1751,8 @@ Bool moveLoop(FvwmWindow *tmp_win, int XOffset, int YOffset, int Width,
     case ButtonRelease:
       if(!do_move_opaque)
 	switch_move_resize_grid(False);
-      xl2 = Event.xbutton.x_root + XOffset + x_virtual_offset;
-      yt2 = Event.xbutton.y_root + YOffset + y_virtual_offset;
+      xl2 = Event.xbutton.x_root + XOffset;
+      yt2 = Event.xbutton.y_root + YOffset;
       /* ignore the position of the button release if it was on a
        * different page. */
       if (!(((xl <  0 && xl2 >= 0) || (xl >= 0 && xl2 <  0) ||
@@ -1792,10 +1776,6 @@ Bool moveLoop(FvwmWindow *tmp_win, int XOffset, int YOffset, int Width,
       break;
 
     case MotionNotify:
-      if (Event.xmotion.same_screen == False)
-      {
-	continue;
-      }
       xl = Event.xmotion.x_root;
       yt = Event.xmotion.y_root;
       if (xl > 0 && xl < Scr.MyDisplayWidth - 1)
@@ -1826,15 +1806,9 @@ Bool moveLoop(FvwmWindow *tmp_win, int XOffset, int YOffset, int Width,
 	{
 	  if (IS_ICONIFIED(tmp_win))
 	  {
-	    tmp_win->icon_g.x = xl ;
-	    tmp_win->icon_xl_loc = xl -
-	      (tmp_win->icon_g.width - tmp_win->icon_p_width)/2;
-	    tmp_win->icon_g.y = yt;
-	    if(tmp_win->icon_pixmap_w != None)
-	      XMoveWindow(dpy, tmp_win->icon_pixmap_w, tmp_win->icon_g.x,yt);
-	    else if (tmp_win->icon_w != None)
-	      XMoveWindow(dpy, tmp_win->icon_w,tmp_win->icon_xl_loc,
-			  yt+tmp_win->icon_p_height);
+	    set_icon_position(tmp_win, xl, yt);
+	    move_icon_to_position(tmp_win);
+	    broadcast_icon_geometry(tmp_win, False);
 	  }
 	  else
 	  {
@@ -1893,8 +1867,7 @@ Bool moveLoop(FvwmWindow *tmp_win, int XOffset, int YOffset, int Width,
     } /* switch */
     xl += x_virtual_offset;
     yt += y_virtual_offset;
-    if (do_move_opaque && !IS_ICONIFIED(tmp_win) && !IS_SHADED(tmp_win) &&
-	!Scr.bo.DisableConfigureNotify)
+    if (do_move_opaque && !IS_ICONIFIED(tmp_win) && !IS_SHADED(tmp_win))
     {
       /* send configure notify event for windows that care about their
        * location; don't send anything if position didn't change */
@@ -2219,10 +2192,10 @@ void CMD_XorPixmap(F_CMD_ARGS)
  * Starts a window resize operation
  *
  ****************************************************************************/
-void CMD_Resize(F_CMD_ARGS)
+static Bool resize_window(F_CMD_ARGS)
 {
   extern Window bad_window;
-  Bool finished = False, done = False, abort = False;
+  Bool finished = False, is_done = False, is_aborted = False;
   Bool do_resize_opaque;
   int x,y,delta_x,delta_y,stashed_x,stashed_y;
   Window ResizeWindow;
@@ -2246,20 +2219,19 @@ void CMD_Resize(F_CMD_ARGS)
   Bool called_from_title = False;
 
   bad_window = False;
-  if (DeferExecution(eventp,&w,&tmp_win,&context, CRS_RESIZE, ButtonPress))
-    return;
-  if (tmp_win == NULL)
-    return;
-
   ResizeWindow = tmp_win->frame;
-  XQueryPointer( dpy, ResizeWindow, &JunkRoot, &JunkChild,
-		 &JunkX, &JunkY, &px, &py, &button_mask);
+  if (XQueryPointer(
+	dpy, ResizeWindow, &JunkRoot, &JunkChild, &JunkX, &JunkY, &px, &py,
+	&button_mask) == False)
+  {
+    /* pointer is on a different screen - that's okay here */
+  }
   button_mask &= DEFAULT_ALL_BUTTONS_MASK;
 
-  if (!check_if_function_allowed(F_RESIZE, tmp_win, True, NULL))
+  if (!is_function_allowed(F_RESIZE, NULL, tmp_win, True, True))
   {
     XBell(dpy, 0);
-    return;
+    return False;
   }
 
   was_maximized = IS_MAXIMIZED(tmp_win);
@@ -2269,7 +2241,7 @@ void CMD_Resize(F_CMD_ARGS)
     /* must redraw the buttons now so that the 'maximize' button does not stay
      * depressed. */
     DrawDecorations(
-      tmp_win, DRAW_BUTTONS, (tmp_win == Scr.Hilite), True, None);
+      tmp_win, DRAW_BUTTONS, (tmp_win == Scr.Hilite), True, None, CLEAR_ALL);
   }
 
   if (IS_SHADED(tmp_win) || !IS_MAPPED(tmp_win))
@@ -2310,12 +2282,12 @@ void CMD_Resize(F_CMD_ARGS)
       ForceSetupFrame(tmp_win, tmp_win->frame_g.x, tmp_win->frame_g.y,
 		      drag->width, drag->height, False);
     }
-    DrawDecorations(tmp_win, DRAW_ALL, True, True, None);
+    DrawDecorations(tmp_win, DRAW_ALL, True, True, None, CLEAR_ALL);
     update_absolute_geometry(tmp_win);
     maximize_adjust_offset(tmp_win);
     GNOME_SetWinArea(tmp_win);
     ResizeWindow = None;
-    return;
+    return True;
   }
 
   if (Scr.bo.InstallRootCmap)
@@ -2326,7 +2298,7 @@ void CMD_Resize(F_CMD_ARGS)
   if(!GrabEm(CRS_RESIZE, GRAB_NORMAL))
   {
     XBell(dpy, 0);
-    return;
+    return False;
   }
 
   /* handle problems with edge-wrapping while resizing */
@@ -2349,7 +2321,7 @@ void CMD_Resize(F_CMD_ARGS)
     {
       MyXUngrabServer(dpy);
     }
-    return;
+    return False;
   }
   if (IS_SHADED(tmp_win))
   {
@@ -2537,8 +2509,14 @@ void CMD_Resize(F_CMD_ARGS)
    * press */
   if (eventp->type == KeyPress)
   {
-    XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
-		  &stashed_x,&stashed_y,&JunkX, &JunkY, &JunkMask);
+    if (XQueryPointer(
+	  dpy, Scr.Root, &JunkRoot, &JunkChild, &stashed_x, &stashed_y, &JunkX,
+	  &JunkY, &JunkMask) == False)
+    {
+      /* pointer is on a different screen */
+      stashed_x = 0;
+      stashed_y = 0;
+    }
     DoResize(stashed_x, stashed_y, tmp_win, drag, orig, &xmotion, &ymotion,
 	     do_resize_opaque);
   }
@@ -2548,18 +2526,13 @@ void CMD_Resize(F_CMD_ARGS)
   /* loop to resize */
   while(!finished && bad_window != tmp_win->w)
   {
-    int rc = 0;
-
     /* block until there is an interesting event */
-    while (rc != -1 &&
-	   (!XCheckMaskEvent(dpy, ButtonPressMask | ButtonReleaseMask |
-			     KeyPressMask | PointerMotionMask |
-			     ButtonMotionMask | ExposureMask, &Event)))
+    while (!XCheckMaskEvent(dpy, ButtonPressMask | ButtonReleaseMask |
+			    KeyPressMask | PointerMotionMask |
+			    ButtonMotionMask | ExposureMask, &Event))
     {
-      rc = HandlePaging(
-	      Scr.EdgeScrollX, Scr.EdgeScrollY, &x, &y, &delta_x, &delta_y,
-	      False, False, True);
-      if (rc == 1)
+      if (HandlePaging(Scr.EdgeScrollX, Scr.EdgeScrollY, &x, &y,
+		       &delta_x, &delta_y, False, False, True))
       {
 	/* Fake an event to force window reposition */
 	Event.type = MotionNotify;
@@ -2568,12 +2541,6 @@ void CMD_Resize(F_CMD_ARGS)
 	break;
       }
     }
-    if (rc == -1)
-    {
-      XMaskEvent(dpy, ButtonPressMask | ButtonReleaseMask |
-		 KeyPressMask | PointerMotionMask |
-		 ButtonMotionMask | ExposureMask, &Event);
-    }
     StashEventTime(&Event);
 
     if (Event.type == MotionNotify)
@@ -2588,7 +2555,7 @@ void CMD_Resize(F_CMD_ARGS)
       }
     }
 
-    done = False;
+    is_done = False;
     /* Handle a limited number of key press events to allow mouseless
      * operation */
     if(Event.type == KeyPress)
@@ -2596,7 +2563,7 @@ void CMD_Resize(F_CMD_ARGS)
     switch(Event.type)
     {
     case ButtonPress:
-      done = True;
+      is_done = True;
       if (Event.xbutton.button <= NUMBER_OF_MOUSE_BUTTONS &&
 	  ((Button1Mask << (Event.xbutton.button - 1)) & button_mask))
       {
@@ -2614,7 +2581,7 @@ void CMD_Resize(F_CMD_ARGS)
       /* simple code to bag out of move - CKH */
       if (XLookupKeysym(&(Event.xkey),0) == XK_Escape || fButtonAbort)
       {
-	abort = True;
+	is_aborted = True;
 	finished = True;
 	/* return pointer if aborted resize was invoked with key */
 	if (stashed_x >= 0)
@@ -2627,22 +2594,18 @@ void CMD_Resize(F_CMD_ARGS)
 	  DoResize(
 	    start_g.x, start_g.y, tmp_win, &start_g, orig, &xmotion, &ymotion,
 	    do_resize_opaque);
-	  DrawDecorations(tmp_win, DRAW_ALL, True, True, None);
+	  DrawDecorations(tmp_win, DRAW_ALL, True, True, None, CLEAR_ALL);
 	}
       }
-      done = True;
+      is_done = True;
       break;
 
     case ButtonRelease:
       finished = True;
-      done = True;
+      is_done = True;
       break;
 
     case MotionNotify:
-      if (Event.xmotion.same_screen == False)
-      {
-	continue;
-      }
       if (!fForceRedraw)
       {
 	x = Event.xmotion.x_root;
@@ -2667,13 +2630,11 @@ void CMD_Resize(F_CMD_ARGS)
 	  x, y, tmp_win, drag, orig, &xmotion, &ymotion, do_resize_opaque);
       }
       fForceRedraw = False;
-      done = True;
-      break;
-
+      is_done = True;
     default:
       break;
     }
-    if(!done)
+    if(!is_done)
     {
       if (!do_resize_opaque)
 	/* must undraw the rubber band in case the event causes some drawing */
@@ -2703,7 +2664,7 @@ void CMD_Resize(F_CMD_ARGS)
   if (!Scr.gs.do_hide_resize_window)
     XUnmapWindow(dpy, Scr.SizeWindow);
 
-  if(!abort && bad_window != tmp_win->w)
+  if(!is_aborted && bad_window != tmp_win->w)
   {
     /* size will be >= to requested */
     constrain_size(
@@ -2729,13 +2690,13 @@ void CMD_Resize(F_CMD_ARGS)
 	tmp_win, drag->x, drag->y, drag->width, drag->height, False);
     }
   }
-  if (abort && was_maximized)
+  if (is_aborted && was_maximized)
   {
     /* since we aborted the resize, the window is still maximized */
     SET_MAXIMIZED(tmp_win, 1);
     /* force redraw */
     DrawDecorations(
-      tmp_win, DRAW_BUTTONS, (tmp_win == Scr.Hilite), True, None);
+      tmp_win, DRAW_BUTTONS, (tmp_win == Scr.Hilite), True, None, CLEAR_ALL);
   }
 
   if (bad_window == tmp_win->w)
@@ -2767,10 +2728,29 @@ void CMD_Resize(F_CMD_ARGS)
   maximize_adjust_offset(tmp_win);
   GNOME_SetWinArea(tmp_win);
 
-  return;
+  if (is_aborted)
+  {
+    return False;
+  }
+
+  return True;
 }
 
+void CMD_Resize(F_CMD_ARGS)
+{
+	if (DeferExecution(
+		    eventp,&w,&tmp_win,&context, CRS_RESIZE, ButtonPress))
+	{
+		return;
+	}
+	if (tmp_win == NULL || IS_ICONIFIED(tmp_win))
+	{
+		return;
+	}
+	resize_window(F_PASS_ARGS);
 
+	return;
+}
 
 /***********************************************************************
  *
@@ -2897,29 +2877,29 @@ static int get_outline_rects(
 
   for (i = 0; i < n; i++)
   {
-    rects[i].x = x + i;
-    rects[i].y = y + i;
-    rects[i].width = width - (i << 1);
-    rects[i].height = height - (i << 1);
+    rects[i+i].x = x + i;
+    rects[i+i].y = y + i;
+    rects[i+i].width = width - (i << 1);
+    rects[i+i].height = height - (i << 1);
   }
   if (width - (n << 1) >= 5 && height - (n << 1) >= 5)
   {
     if (width - (n << 1) >= 10)
     {
       int off = (width - (n << 1)) / 3 + n;
-      rects[i].x = x + off;
-      rects[i].y = y + n;
-      rects[i].width = width - (off << 1);
-      rects[i].height = height - (n << 1);
+      rects[i+i].x = x + off;
+      rects[i+i].y = y + n;
+      rects[i+i].width = width - (off << 1);
+      rects[i+i].height = height - (n << 1);
       i++;
     }
     if (height - (n << 1) >= 10)
     {
       int off = (height - (n << 1)) / 3 + n;
-      rects[i].x = x + n;
-      rects[i].y = y + off;
-      rects[i].width = width - (n << 1);
-      rects[i].height = height - (off << 1);
+      rects[i+i].x = x + n;
+      rects[i+i].y = y + off;
+      rects[i+i].width = width - (n << 1);
+      rects[i+i].height = height - (off << 1);
       i++;
     }
   }
@@ -2964,7 +2944,7 @@ static void draw_move_resize_grid(int x, int  y, int  width, int height)
     move_resize_grid.flags.is_enabled = 0;
     nrects +=
       get_outline_rects(
-	&(rects[0]), move_resize_grid.geom.x, move_resize_grid.geom.y,
+	&(rects[1]), move_resize_grid.geom.x, move_resize_grid.geom.y,
 	move_resize_grid.geom.width, move_resize_grid.geom.height);
   }
   if (width && height)
@@ -2974,11 +2954,11 @@ static void draw_move_resize_grid(int x, int  y, int  width, int height)
     move_resize_grid.geom.y = y;
     move_resize_grid.geom.width = width;
     move_resize_grid.geom.height = height;
-    nrects += get_outline_rects(&(rects[nrects]), x, y, width, height);
+    nrects += get_outline_rects(&(rects[0]), x, y, width, height);
   }
   if (nrects > 0)
   {
-    XDrawRectangles(dpy, Scr.Root, Scr.XorGC, rects, nrects);
+    XDrawRectangles(dpy, Scr.Root, Scr.XorGC, rects, 10);
     XSync(dpy, 0);
   }
 
@@ -3058,13 +3038,15 @@ static void move_sticky_window_to_same_page(
 static void MaximizeHeight(
   FvwmWindow *win, unsigned int win_width, int win_x, unsigned int *win_height,
   int *win_y, Bool grow_up, Bool grow_down,
-  int top_border, int bottom_border)
+  int top_border, int bottom_border, Bool layer_grow)
 {
   FvwmWindow *cwin;
   int x11, x12, x21, x22;
   int y11, y12, y21, y22;
   int new_y1, new_y2;
   Bool is_sticky;
+  rectangle g;
+  Bool rc;
 
   x11 = win_x;             /* Start x */
   y11 = *win_y;            /* Start y */
@@ -3079,24 +3061,18 @@ static void MaximizeHeight(
       is_sticky = True;
     else
       is_sticky = False;
-    if (cwin == win || (cwin->Desk != win->Desk && !is_sticky))
+    if (cwin == win || (cwin->Desk != win->Desk && !is_sticky) ||
+	(layer_grow && cwin->layer <= win->layer))
       continue;
-    if (IS_ICONIFIED(cwin))
+    rc = get_visible_window_or_icon_geometry(cwin, &g);
+    if (rc == False)
     {
-      if(cwin->icon_w == None || IS_ICON_UNMAPPED(cwin))
-	continue;
-      x21 = cwin->icon_g.x;
-      y21 = cwin->icon_g.y;
-      x22 = x21 + cwin->icon_p_width;
-      y22 = y21 + cwin->icon_p_height + cwin->icon_g.height;
-    }
-    else
-    {
-      x21 = cwin->frame_g.x;
-      y21 = cwin->frame_g.y;
-      x22 = x21 + cwin->frame_g.width;
-      y22 = y21 + cwin->frame_g.height;
+      continue;
     }
+    x21 = g.x;
+    y21 = g.y;
+    x22 = y21 + g.width;
+    y22 = y21 + g.height;
     if (is_sticky)
     {
       move_sticky_window_to_same_page(
@@ -3127,13 +3103,15 @@ static void MaximizeHeight(
 static void MaximizeWidth(
   FvwmWindow *win, unsigned int *win_width, int *win_x, unsigned int win_height,
   int win_y, Bool grow_left, Bool grow_right,
-  int left_border, int right_border)
+  int left_border, int right_border, Bool layer_grow)
 {
   FvwmWindow *cwin;
   int x11, x12, x21, x22;
   int y11, y12, y21, y22;
   int new_x1, new_x2;
   Bool is_sticky;
+  rectangle g;
+  Bool rc;
 
   x11 = *win_x;            /* Start x */
   y11 = win_y;             /* Start y */
@@ -3148,24 +3126,18 @@ static void MaximizeWidth(
       is_sticky = True;
     else
       is_sticky = False;
-    if (cwin == win || (cwin->Desk != win->Desk && !is_sticky))
+    if (cwin == win || (cwin->Desk != win->Desk && !is_sticky) ||
+	(layer_grow && cwin->layer <= win->layer))
       continue;
-    if (IS_ICONIFIED(cwin))
+    rc = get_visible_window_or_icon_geometry(cwin, &g);
+    if (rc == False)
     {
-      if(cwin->icon_w == None || IS_ICON_UNMAPPED(cwin))
-	continue;
-      x21 = cwin->icon_g.x;
-      y21 = cwin->icon_g.y;
-      x22 = x21 + cwin->icon_p_width;
-      y22 = y21 + cwin->icon_p_height + cwin->icon_g.height;
-    }
-    else
-    {
-      x21 = cwin->frame_g.x;
-      y21 = cwin->frame_g.y;
-      x22 = x21 + cwin->frame_g.width;
-      y22 = y21 + cwin->frame_g.height;
+      continue;
     }
+    x21 = g.x;
+    y21 = g.y;
+    x22 = y21 + g.width;
+    y22 = y21 + g.height;
     if (is_sticky)
     {
       move_sticky_window_to_same_page(
@@ -3193,6 +3165,57 @@ static void MaximizeWidth(
   *win_x = new_x1;
 }
 
+static void unmaximize_fvwm_window(
+	FvwmWindow *tmp_win)
+{
+	SET_MAXIMIZED(tmp_win, 0);
+	get_relative_geometry(&tmp_win->frame_g, &tmp_win->normal_g);
+	if (IS_SHADED(tmp_win))
+	{
+		get_shaded_geometry(
+			tmp_win, &tmp_win->frame_g, &tmp_win->frame_g);
+	}
+	ForceSetupFrame(
+		tmp_win, tmp_win->frame_g.x, tmp_win->frame_g.y,
+		tmp_win->frame_g.width, tmp_win->frame_g.height, True);
+	DrawDecorations(tmp_win, DRAW_ALL, True, True, None, CLEAR_ALL);
+
+	return;
+}
+
+static void maximize_fvwm_window(
+	FvwmWindow *tmp_win, rectangle *geometry)
+{
+	SET_MAXIMIZED(tmp_win, 1);
+	tmp_win->max_g_defect.width = 0;
+	tmp_win->max_g_defect.height = 0;
+	constrain_size(
+		tmp_win, (unsigned int*)&(geometry->width),
+		(unsigned int *)&(geometry->height), 0, 0,
+		CS_UPDATE_MAX_DEFECT);
+	tmp_win->max_g = *geometry;
+	if (IS_SHADED(tmp_win))
+	{
+		get_shaded_geometry(tmp_win, geometry, &tmp_win->max_g);
+	}
+	SetupFrame(
+		tmp_win, geometry->x, geometry->y, geometry->width,
+		geometry->height, True);
+	DrawDecorations(
+		tmp_win, DRAW_ALL, (Scr.Hilite == tmp_win), True, None,
+		CLEAR_ALL);
+	update_absolute_geometry(tmp_win);
+	/* remember the offset between old and new position in case the
+	 * maximized  window is moved more than the screen width/height. */
+	tmp_win->max_offset.x = tmp_win->normal_g.x - tmp_win->max_g.x;
+	tmp_win->max_offset.y = tmp_win->normal_g.y - tmp_win->max_g.y;
+#if 0
+fprintf(stderr,"%d %d %d %d, max_offset.x = %d, max_offset.y = %d\n", tmp_win->max_g.x, tmp_win->max_g.y, tmp_win->max_g.width, tmp_win->max_g.height, tmp_win->max_offset.x, tmp_win->max_offset.y);
+#endif
+
+    return;
+}
+
 /***********************************************************************
  *
  *  Procedure:
@@ -3212,32 +3235,57 @@ void CMD_Maximize(F_CMD_ARGS)
   Bool grow_right = False;
   Bool do_force_maximize = False;
   Bool is_screen_given = False;
+  Bool ignore_working_area = False;
+  Bool layer_grow = False;
+  Bool global_flag_parsed = False;
   int  scr_x, scr_y, scr_w, scr_h;
+  int sx, sy, sw, sh;
   rectangle new_g;
   FvwmWindow *sf;
 
-  if (DeferExecution(eventp,&w,&tmp_win,&context, CRS_SELECT,ButtonRelease))
+  if (DeferExecution(eventp, &w, &tmp_win, &context, CRS_SELECT, ButtonRelease))
     return;
-  if (tmp_win == NULL)
+  if (tmp_win == NULL || IS_ICONIFIED(tmp_win))
     return;
 
-  if (check_if_function_allowed(F_MAXIMIZE,tmp_win,True,NULL) == 0)
+  if (!is_function_allowed(F_MAXIMIZE, NULL, tmp_win, True, False))
   {
     XBell(dpy, 0);
     return;
   }
   /* Check for "global" flag ("absolute" is for compatibility with E) */
-  token = PeekToken(action, &taction);
-  if (token)
+  while (!global_flag_parsed)
   {
-    if (StrEquals(token, "screen"))
+    token = PeekToken(action, &taction);
+    if (!token)
     {
-      int scr;
+      global_flag_parsed = True;
+    }
+    else
+    {
+      if (StrEquals(token, "screen"))
+      {
+	int scr;
 
-      is_screen_given = True;
-      token = PeekToken(taction, &action);
-      scr = FScreenGetScreenArgument(token, FSCREEN_SPEC_PRIMARY);
-      FScreenGetScrRect(NULL, scr, &scr_x, &scr_y, &scr_w, &scr_h);
+	is_screen_given = True;
+	token = PeekToken(taction, &action);
+	scr = FScreenGetScreenArgument(token, FSCREEN_SPEC_PRIMARY);
+	FScreenGetScrRect(NULL, scr, &scr_x, &scr_y, &scr_w, &scr_h);
+      }
+      else if (StrEquals(token, "ewmhiwa"))
+      {
+	ignore_working_area = True;
+	action = taction;
+      }
+      else if (StrEquals(token, "layer"))
+      {
+	layer_grow = True;
+	action = taction;
+      }
+      else
+      {
+	global_flag_parsed = True;
+      }
     }
   }
   toggle = ParseToggleArgument(action, &action, -1, 0);
@@ -3285,13 +3333,22 @@ void CMD_Maximize(F_CMD_ARGS)
     FScreenGetScrRect(&fscr, FSCREEN_XYPOS, &scr_x, &scr_y, &scr_w, &scr_h);
   }
 
+  sx = scr_x;
+  sy = scr_y;
+  sw = scr_w;
+  sh = scr_h;
+  if (!ignore_working_area)
+  {
+    EWMH_GetWorkAreaIntersection(
+      tmp_win, &sx, &sy, &sw, &sh, EWMH_MAXIMIZE_MODE(tmp_win));
+  }
 #if 0
   fprintf(stderr, "%s: page=(%d,%d), scr=(%d,%d, %dx%d)\n", __FUNCTION__,
           page_x, page_y, scr_x, scr_y, scr_w, scr_h);
 #endif
 
   /* parse first parameter */
-  val1_unit = scr_w;
+  val1_unit = sw;
   token = PeekToken(action, &taction);
   if (token && StrEquals(token, "grow"))
   {
@@ -3317,24 +3374,24 @@ void CMD_Maximize(F_CMD_ARGS)
     if (GetOnePercentArgument(token, &val1, &val1_unit) == 0)
     {
       val1 = 100;
-      val1_unit = scr_w;
+      val1_unit = sw;
     }
     else if (val1 < 0)
     {
       /* handle negative offsets */
-      if (val1_unit == scr_w)
+      if (val1_unit == sw)
       {
 	val1 = 100 + val1;
       }
       else
       {
-	val1 = scr_w + val1;
+	val1 = sw + val1;
       }
     }
   }
 
   /* parse second parameter */
-  val2_unit = scr_h;
+  val2_unit = sh;
   token = PeekToken(taction, NULL);
   if (token && StrEquals(token, "grow"))
   {
@@ -3360,32 +3417,41 @@ void CMD_Maximize(F_CMD_ARGS)
     if (GetOnePercentArgument(token, &val2, &val2_unit) == 0)
     {
       val2 = 100;
-      val2_unit = scr_h;
+      val2_unit = sh;
     }
     else if (val2 < 0)
     {
       /* handle negative offsets */
-      if (val2_unit == scr_h)
+      if (val2_unit == sh)
       {
 	val2 = 100 + val2;
       }
       else
       {
-	val2 = scr_h + val2;
+	val2 = sh + val2;
       }
     }
   }
 
+  if (!grow_left && !grow_right)
+  {
+    scr_x = sx;
+    scr_w = sw;
+  }
+  if (!grow_up && !grow_down)
+  {
+    scr_y = sy;
+    scr_h = sh;
+  }
+
+#if 0
+  fprintf(stderr, "%s: page=(%d,%d), scr=(%d,%d, %dx%d)\n", __FUNCTION__,
+          page_x, page_y, scr_x, scr_y, scr_w, scr_h);
+#endif
+
   if (IS_MAXIMIZED(tmp_win) && !do_force_maximize)
   {
-    SET_MAXIMIZED(tmp_win, 0);
-    get_relative_geometry(&tmp_win->frame_g, &tmp_win->normal_g);
-    if (IS_SHADED(tmp_win))
-      get_shaded_geometry(tmp_win, &tmp_win->frame_g, &tmp_win->frame_g);
-    ForceSetupFrame(
-      tmp_win, tmp_win->frame_g.x, tmp_win->frame_g.y, tmp_win->frame_g.width,
-      tmp_win->frame_g.height, True);
-    DrawDecorations(tmp_win, DRAW_ALL, True, True, None);
+    unmaximize_fvwm_window(tmp_win);
   }
   else /* maximize */
   {
@@ -3395,7 +3461,7 @@ void CMD_Maximize(F_CMD_ARGS)
       MaximizeHeight(
 	tmp_win, new_g.width, new_g.x, (unsigned int *)&new_g.height,
 	&new_g.y, grow_up, grow_down,
-	page_y + scr_y, page_y + scr_y + scr_h);
+	page_y + scr_y, page_y + scr_y + scr_h, layer_grow);
     }
     else if(val2 > 0)
     {
@@ -3407,7 +3473,7 @@ void CMD_Maximize(F_CMD_ARGS)
       MaximizeWidth(
 	tmp_win, (unsigned int *)&new_g.width, &new_g.x, new_g.height,
 	new_g.y, grow_left, grow_right,
-	page_x + scr_x, page_x + scr_x + scr_w);
+	page_x + scr_x, page_x + scr_x + scr_w, layer_grow);
     }
     else if(val1 >0)
     {
@@ -3422,33 +3488,90 @@ void CMD_Maximize(F_CMD_ARGS)
       new_g.width = scr_w;
     }
     /* now maximize it */
-    SET_MAXIMIZED(tmp_win, 1);
-    tmp_win->max_g_defect.width = 0;
-    tmp_win->max_g_defect.height = 0;
-    constrain_size(tmp_win, (unsigned int *)&new_g.width,
-		   (unsigned int *)&new_g.height, 0, 0, CS_UPDATE_MAX_DEFECT);
-    tmp_win->max_g = new_g;
-    if (IS_SHADED(tmp_win))
-      get_shaded_geometry(tmp_win, &new_g, &tmp_win->max_g);
-    SetupFrame(
-      tmp_win, new_g.x, new_g.y, new_g.width, new_g.height, True);
-    DrawDecorations(tmp_win, DRAW_ALL, (Scr.Hilite == tmp_win), True, None);
-    update_absolute_geometry(tmp_win);
-    /* remember the offset between old and new position in case the maximized
-     * window is moved more than the screen width/height. */
-    tmp_win->max_offset.x = tmp_win->normal_g.x - tmp_win->max_g.x;
-    tmp_win->max_offset.y = tmp_win->normal_g.y - tmp_win->max_g.y;
-#if 0
-fprintf(stderr,"%d %d %d %d, max_offset.x = %d, max_offset.y = %d\n", tmp_win->max_g.x, tmp_win->max_g.y, tmp_win->max_g.width, tmp_win->max_g.height, tmp_win->max_offset.x, tmp_win->max_offset.y);
-#endif
+    maximize_fvwm_window(tmp_win, &new_g);
   }
   if ((sf = get_focus_window()))
   {
     focus_grab_buttons(sf, True);
   }
+  EWMH_SetWMState(tmp_win, False);
   GNOME_SetWinArea(tmp_win);
 }
 
+/****************************************************************************
+ *
+ * Same as CMD_Resize and CMD_ResizeMove, but the window ends up maximized
+ * without touching the normal geometry.
+ *
+ ****************************************************************************/
+void CMD_ResizeMaximize(F_CMD_ARGS)
+{
+	rectangle normal_g;
+	rectangle max_g;
+	Bool was_resized;
+
+	if (DeferExecution(eventp, &w, &tmp_win, &context, CRS_RESIZE,
+			   ButtonPress))
+	{
+		return;
+	}
+	if (tmp_win == NULL || IS_ICONIFIED(tmp_win))
+	{
+		return;
+	}
+	/* keep a copy of the old geometry */
+	normal_g = tmp_win->normal_g;
+	/* resize the window normally */
+	was_resized = resize_window(F_PASS_ARGS);
+	if (was_resized == True)
+	{
+		/* set the new geometry as the maximized geometry and restore
+		 * the old normal geometry */
+		max_g = tmp_win->normal_g;
+                max_g.x -= Scr.Vx;
+                max_g.y -= Scr.Vy;
+		tmp_win->normal_g = normal_g;
+		/* and mark it as maximized */
+		maximize_fvwm_window(tmp_win, &max_g);
+	}
+
+	return;
+}
+
+void CMD_ResizeMoveMaximize(F_CMD_ARGS)
+{
+	rectangle normal_g;
+	rectangle max_g;
+	Bool was_resized;
+
+	if (DeferExecution(
+		    eventp, &w, &tmp_win, &context, CRS_RESIZE, ButtonPress))
+	{
+		return;
+	}
+	if (tmp_win == NULL || IS_ICONIFIED(tmp_win))
+	{
+		return;
+	}
+	/* keep a copy of the old geometry */
+	normal_g = tmp_win->normal_g;
+	/* resize the window normally */
+	was_resized = resize_move_window(F_PASS_ARGS);
+	if (was_resized == True)
+	{
+		/* set the new geometry as the maximized geometry and restore
+		 * the old normal geometry */
+		max_g = tmp_win->normal_g;
+                max_g.x -= Scr.Vx;
+                max_g.y -= Scr.Vy;
+		tmp_win->normal_g = normal_g;
+		/* and mark it as maximized */
+		maximize_fvwm_window(tmp_win, &max_g);
+	}
+
+	return;
+}
+
 /* ----------------------------- stick code -------------------------------- */
 
 void handle_stick(F_CMD_ARGS, int toggle)
@@ -3482,11 +3605,14 @@ void handle_stick(F_CMD_ARGS, int toggle)
   }
   BroadcastConfig(M_CONFIGURE_WINDOW,tmp_win);
   DrawDecorations(
-    tmp_win, DRAW_TITLE | DRAW_BUTTONS, (Scr.Hilite==tmp_win), True, None);
+    tmp_win, DRAW_TITLE | DRAW_BUTTONS, (Scr.Hilite==tmp_win),
+    True, None, CLEAR_ALL);
   if ((sf = get_focus_window()))
   {
     focus_grab_buttons(sf, True);
   }
+  EWMH_SetWMState(tmp_win, False);
+  EWMH_SetWMDesktop(tmp_win);
   GNOME_SetHints(tmp_win);
 }
 
