@@ -46,6 +46,7 @@
 #include "virtual.h"
 #include "stack.h"
 #include "add_window.h"
+#include "ewmh.h"
 
 #ifndef MIN
 #define MIN(A,B) ((A)<(B)? (A):(B))
@@ -60,10 +61,10 @@ static int get_next_x(
 static int get_next_y(
   FvwmWindow *t, rectangle *screen_g, int y, int pdeltay, int use_percent);
 static float test_fit(
-  FvwmWindow *t, rectangle *screen_g,
+  FvwmWindow *t, style_flags *sflags, rectangle *screen_g,
   int x11, int y11, float aoimin, int pdeltax, int pdeltay, int use_percent);
 static void CleverPlacement(
-  FvwmWindow *t, rectangle *screen_g,
+  FvwmWindow *t, style_flags *sflags, rectangle *screen_g,
   int *x, int *y, int pdeltax, int pdeltay, int use_percent);
 
 static int SmartPlacement(
@@ -86,7 +87,8 @@ static int SmartPlacement(
   FvwmWindow *test_window;
   int stickyx;
   int stickyy;
-  Bool do_test;
+  rectangle g;
+  Bool rc;
 
   test_x = PageLeft;
   test_y = PageTop;
@@ -102,8 +104,9 @@ static int SmartPlacement(
       test_window = Scr.FvwmRoot.next;
       for ( ; test_window != NULL && loc_ok; test_window = test_window->next)
       {
-	if (t == test_window)
+	if (t == test_window || IS_EWMH_DESKTOP(test_window->w))
 	  continue;
+
         /*  RBW - account for sticky windows...  */
         if (test_window->Desk == t->Desk || IS_STICKY(test_window))
         {
@@ -117,27 +120,14 @@ static int SmartPlacement(
 	    stickyx = 0;
 	    stickyy = 0;
 	  }
-	  do_test = False;
-          if(PLACEMENT_AVOID_ICON != 0 && IS_ICONIFIED(test_window) &&
-	     !IS_ICON_UNMAPPED(test_window) && test_window->icon_w)
-          {
-            tw=test_window->icon_p_width;
-            th=test_window->icon_p_height+
-              test_window->icon_g.height;
-            tx = test_window->icon_g.x - stickyx;
-            ty = test_window->icon_g.y - stickyy;
-	    do_test = True;
-          }
-          else if(!IS_ICONIFIED(test_window))
-          {
-            tw = test_window->frame_g.width;
-            th = test_window->frame_g.height;
-            tx = test_window->frame_g.x - stickyx;
-            ty = test_window->frame_g.y - stickyy;
-	    do_test = True;
-          }
-	  if (do_test)
+	  rc = get_visible_window_or_icon_geometry(test_window, &g);
+	  if (rc == True &&
+	      (PLACEMENT_AVOID_ICON == 0 || !IS_ICONIFIED(test_window)))
 	  {
+	    tx = g.x - stickyx;
+	    ty = g.y - stickyy;
+	    tw = g.width;
+	    th = g.height;
             if (tx < test_x + width  && test_x < tx + tw &&
 		ty < test_y + height && test_y < ty + th)
             {
@@ -171,7 +161,7 @@ static int SmartPlacement(
  * interference, fine.  Otherwise, it places it so that the area of of
  * interference between the new window and the other windows is minimized */
 static void CleverPlacement(
-  FvwmWindow *t, rectangle *screen_g,
+  FvwmWindow *t, style_flags *sflags, rectangle *screen_g,
   int *x, int *y, int pdeltax, int pdeltay, int use_percent)
 {
   int test_x;
@@ -187,7 +177,7 @@ static void CleverPlacement(
   test_x = PageLeft;
   test_y = PageTop;
   aoi = test_fit(
-    t, screen_g, test_x, test_y, -1, pdeltax, pdeltay, use_percent);
+    t, sflags, screen_g, test_x, test_y, -1, pdeltax, pdeltay, use_percent);
   aoimin = aoi;
   xbest = test_x;
   ybest = test_y;
@@ -207,10 +197,10 @@ static void CleverPlacement(
       test_y = get_next_y(t, screen_g, test_y, pdeltay, use_percent);
     }
     aoi = test_fit(
-      t, screen_g, test_x, test_y, aoimin, pdeltax, pdeltay, use_percent);
-    /* I've added +0.00001 because whith my machine the < test fail with
+      t, sflags, screen_g, test_x,test_y, aoimin, pdeltax, pdeltay, use_percent);
+    /* I've added +0.0001 because whith my machine the < test fail with
      * certain *equal* float numbers! */
-    if (aoi >= 0 && aoi + 0.00001 < aoimin)
+    if (aoi >= 0 && aoi + 0.0001 < aoimin)
     {
       xbest = test_x;
       ybest = test_y;
@@ -223,6 +213,7 @@ static void CleverPlacement(
   return;
 }
 
+#define GET_NEXT_STEP 5
 static int get_next_x(
   FvwmWindow *t, rectangle *screen_g, int x, int y, int pdeltax, int pdeltay,
   int use_percent)
@@ -232,19 +223,37 @@ static int get_next_x(
   int xtest;
   int PageLeft = screen_g->x - pdeltax;
   int PageRight = PageLeft + screen_g->width;
-  int minx_inc = screen_g->width / 20;
   int stickyx;
   int stickyy;
+  int start,i;
+  int win_left;
+  rectangle g;
+  Bool rc;
+
+  if (use_percent)
+    start = 0;
+  else
+    start = GET_NEXT_STEP;
 
   /* Test window at far right of screen */
   xnew = PageRight;
   xtest = PageRight - t->frame_g.width;
+  if (xtest > x)
+    xnew = MIN(xnew, xtest);
+  /* test the borders of the working area */
+  xtest = PageLeft + Scr.Desktops->ewmh_working_area.x;
+  if (xtest > x)
+    xnew = MIN(xnew, xtest);
+  xtest = PageLeft +
+    (Scr.Desktops->ewmh_working_area.x + Scr.Desktops->ewmh_working_area.width)
+    - t->frame_g.width;
   if (xtest > x)
     xnew = MIN(xnew, xtest);
   /* Test the values of the right edges of every window */
   for (testw = Scr.FvwmRoot.next ; testw != NULL ; testw = testw->next)
   {
-    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw)))
+    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw)) ||
+	IS_EWMH_DESKTOP(testw->w))
       continue;
 
     if (IS_STICKY(testw))
@@ -260,34 +269,50 @@ static int get_next_x(
 
     if (IS_ICONIFIED(testw))
     {
-      if (y < testw->icon_p_height + testw->icon_g.height + testw->icon_g.y
-	  - stickyy &&
-	  testw->icon_g.y - stickyy < t->frame_g.height + y)
+      rc = get_visible_icon_geometry(testw, &g);
+      if (rc == True &&
+	  y < g.y + g.height - stickyy && g.y - stickyy < t->frame_g.height + y)
       {
-        xtest = PageLeft + testw->icon_p_width + testw->icon_g.x - stickyx;
-        if (xtest > x)
-          xnew = MIN(xnew, xtest);
-        xtest = PageLeft + testw->icon_g.x - stickyx - t->frame_g.width;
-        if (xtest > x)
-          xnew = MIN(xnew, xtest);
+	win_left = PageLeft + g.x - stickyx - t->frame_g.width;
+	for (i = start; i <= GET_NEXT_STEP; i++)
+	{
+	  xtest = win_left + g.width * (GET_NEXT_STEP - i) / GET_NEXT_STEP;
+	  if (xtest > x)
+	    xnew = MIN(xnew, xtest);
+	}
+	win_left = PageLeft + g.x - stickyx;
+	for(i=start; i <= GET_NEXT_STEP; i++)
+	{
+	  xtest = (win_left) + g.width * i / GET_NEXT_STEP;
+	  if (xtest > x)
+	    xnew = MIN(xnew, xtest);
+	}
       }
     }
     else if (y < testw->frame_g.height + testw->frame_g.y - stickyy &&
 	     testw->frame_g.y - stickyy < t->frame_g.height + y)
     {
-      xtest = PageLeft + testw->frame_g.width + testw->frame_g.x - stickyx;
-      if (xtest > x)
-        xnew = MIN(xnew, xtest);
-      xtest = PageLeft + testw->frame_g.x - stickyx - t->frame_g.width;
-      if (xtest > x)
-        xnew = MIN(xnew, xtest);
+      win_left = PageLeft + testw->frame_g.x - stickyx - t->frame_g.width;
+      for(i=start; i <= GET_NEXT_STEP; i++)
+      {
+	xtest = (win_left) + (testw->frame_g.width) *
+	  (GET_NEXT_STEP - i)/GET_NEXT_STEP;
+	if (xtest > x)
+	  xnew = MIN(xnew, xtest);
+      }
+      win_left = PageLeft + testw->frame_g.x - stickyx;
+      for(i=start; i <= GET_NEXT_STEP; i++)
+      {
+	xtest = (win_left) + (testw->frame_g.width) * i/GET_NEXT_STEP;
+	if (xtest > x)
+	  xnew = MIN(xnew, xtest);
+      }
     }
   }
-  if (use_percent)
-    xnew = MIN(xnew, x + minx_inc);
 
   return xnew;
 }
+
 static int get_next_y(
   FvwmWindow *t, rectangle *screen_g, int y, int pdeltay, int use_percent)
 {
@@ -295,18 +320,36 @@ static int get_next_y(
   int ynew;
   int ytest;
   int PageBottom = screen_g->y + screen_g->height - pdeltay;
-  int miny_inc = (screen_g->height / 20);
   int stickyy;
+  int win_top;
+  int start;
+  int i;
+  rectangle g;
+
+  if (use_percent)
+    start = 0;
+  else
+    start = GET_NEXT_STEP;
 
   /* Test window at far bottom of screen */
   ynew = PageBottom;
   ytest = PageBottom - t->frame_g.height;
+  if (ytest > y)
+    ynew = MIN(ynew, ytest);
+  /* test the borders of the working area */
+  ytest = screen_g->y + Scr.Desktops->ewmh_working_area.y - pdeltay;
+  if (ytest > y)
+    ynew = MIN(ynew, ytest);
+  ytest = screen_g->y +
+    (Scr.Desktops->ewmh_working_area.y + Scr.Desktops->ewmh_working_area.height)
+    - t->frame_g.height;
   if (ytest > y)
     ynew = MIN(ynew, ytest);
   /* Test the values of the bottom edge of every window */
   for (testw = Scr.FvwmRoot.next ; testw != NULL ; testw = testw->next)
   {
-    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw)))
+    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw))
+	|| IS_EWMH_DESKTOP(testw->w))
       continue;
 
     if (IS_STICKY(testw))
@@ -320,32 +363,47 @@ static int get_next_y(
 
     if (IS_ICONIFIED(testw))
     {
-      ytest =
-	testw->icon_p_height + testw->icon_g.height + testw->icon_g.y - stickyy;
-      if (ytest > y)
-        ynew = MIN(ynew, ytest);
-      ytest = testw->icon_g.y - stickyy - t->frame_g.height;
-      if (ytest > y)
-        ynew = MIN(ynew, ytest);
+      get_visible_icon_geometry(testw, &g);
+      win_top = g.y - stickyy;
+      for(i=start; i <= GET_NEXT_STEP; i++)
+      {
+	ytest = win_top + g.height * i / GET_NEXT_STEP;
+	if (ytest > y)
+	  ynew = MIN(ynew, ytest);
+      }
+      win_top = g.y - stickyy - t->frame_g.height;
+      for(i=start; i <= GET_NEXT_STEP; i++)
+      {
+	ytest = win_top + g.height * (GET_NEXT_STEP - i) / GET_NEXT_STEP;
+	if (ytest > y)
+	  ynew = MIN(ynew, ytest);
+      }
     }
     else
     {
-      ytest = testw->frame_g.height + testw->frame_g.y - stickyy;
-      if (ytest > y)
-        ynew = MIN(ynew, ytest);
-      ytest = testw->frame_g.y - stickyy - t->frame_g.height;
-      if (ytest > y)
-        ynew = MIN(ynew, ytest);
+      win_top = testw->frame_g.y - stickyy;;
+      for(i=start; i <= GET_NEXT_STEP; i++)
+      {
+	ytest = (win_top) + (testw->frame_g.height) * i/GET_NEXT_STEP;
+	if (ytest > y)
+	  ynew = MIN(ynew, ytest);
+      }
+      win_top = testw->frame_g.y - stickyy - t->frame_g.height;
+      for(i=start; i <= GET_NEXT_STEP; i++)
+      {
+	ytest = win_top + (testw->frame_g.height) *
+	  (GET_NEXT_STEP - i)/GET_NEXT_STEP;
+	if (ytest > y)
+	  ynew = MIN(ynew, ytest);
+      }
     }
   }
-  if (use_percent)
-    ynew = MIN(ynew,y+miny_inc);
 
   return ynew;
 }
 
 static float test_fit(
-  FvwmWindow *t, rectangle *screen_g,
+  FvwmWindow *t, style_flags *sflags, rectangle *screen_g,
   int x11, int y11, float aoimin, int pdeltax, int pdeltay, int use_percent)
 {
   FvwmWindow *testw;
@@ -363,11 +421,13 @@ static float test_fit(
   /* area of interference */
   float aoi = 0;
   float anew;
-  float cover_factor = 1;
+  float cover_factor = 0;
   float avoidance_factor;
   int PageRight  = screen_g->x + screen_g->width - pdeltax;
   int PageBottom = screen_g->y + screen_g->height - pdeltay;
   int stickyx, stickyy;
+  rectangle g;
+  Bool rc;
 
   x12 = x11 + t->frame_g.width;
   y12 = y11 + t->frame_g.height;
@@ -378,7 +438,8 @@ static float test_fit(
     return -2;
   for (testw = Scr.FvwmRoot.next ; testw != NULL ; testw = testw->next)
   {
-    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw)))
+    if (testw == t || (testw->Desk != t->Desk && !IS_STICKY(testw))
+	|| IS_EWMH_DESKTOP(testw->w))
        continue;
 
     if (IS_STICKY(testw))
@@ -392,22 +453,11 @@ static float test_fit(
       stickyy = 0;
     }
 
-    if (IS_ICONIFIED(testw))
-    {
-       if (testw->icon_w == None || IS_ICON_UNMAPPED(testw))
-	  continue;
-       x21 = testw->icon_g.x - stickyx;
-       y21 = testw->icon_g.y - stickyy;
-       x22 = x21 + testw->icon_p_width;
-       y22 = y21 + testw->icon_p_height + testw->icon_g.height;
-    }
-    else
-    {
-       x21 = testw->frame_g.x - stickyx;
-       y21 = testw->frame_g.y - stickyy;
-       x22 = x21 + testw->frame_g.width;
-       y22 = y21 + testw->frame_g.height;
-    }
+    rc = get_visible_window_or_icon_geometry(testw, &g);
+    x21 = g.x - stickyx;
+    y21 = g.y - stickyy;
+    x22 = x21 + g.width;
+    y22 = y21 + g.height;
     if (x11 < x22 && x12 > x21 &&
 	y11 < y22 && y12 > y21)
     {
@@ -418,44 +468,59 @@ static float test_fit(
       yb = MIN(y12, y22);
       anew = (xr - xl) * (yb - yt);
       if (IS_ICONIFIED(testw))
-        avoidance_factor = PLACEMENT_AVOID_ICON;
+        avoidance_factor = ICON_PLACEMENT_PENALTY(testw);
       else if(compare_window_layers(testw, t) > 0)
-        avoidance_factor = PLACEMENT_AVOID_ONTOP;
+        avoidance_factor = ONTOP_PLACEMENT_PENALTY(testw);
       else if(compare_window_layers(testw, t) < 0)
-        avoidance_factor = PLACEMENT_AVOID_BELOW;
+        avoidance_factor = BELOW_PLACEMENT_PENALTY(testw);
       else if(IS_STICKY(testw))
-        avoidance_factor = PLACEMENT_AVOID_STICKY;
+        avoidance_factor = STICKY_PLACEMENT_PENALTY(testw);
       else
-        avoidance_factor = 1.0;
+        avoidance_factor = NORMAL_PLACEMENT_PENALTY(testw);
 
       if (use_percent)
       {
-	/* normalisation */
+	cover_factor = 0;
 	if ((x22 - x21) * (y22 - y21) != 0 && (x12 - x11) * (y12 - y11) != 0)
 	{
 	  anew = 100 * MAX(anew / ((x22 - x21) * (y22 - y21)),
-			   anew / ((x12 - x11) * (y12 - y11)));
+			      anew / ((x12 - x11) * (y12 - y11)));
 	  if (anew >= 99)
-	    cover_factor = PLACEMENT_AVOID_COVER_99;
-	  else if (anew >= 95)
-	    cover_factor = PLACEMENT_AVOID_COVER_95;
-	  else if (anew >= 85)
-	    cover_factor = PLACEMENT_AVOID_COVER_85;
-	  else if (anew >= 75)
-	    cover_factor = PLACEMENT_AVOID_COVER_75;
+	    cover_factor = PERCENTAGE_99_PENALTY(testw);
+	  else if (anew > 94)
+	    cover_factor = PERCENTAGE_95_PENALTY(testw);
+	  else if (anew > 84)
+	    cover_factor = PERCENTAGE_85_PENALTY(testw);
+	  else if (anew > 74)
+	    cover_factor = PERCENTAGE_75_PENALTY(testw);
 	}
-	if (avoidance_factor>1)
-	  avoidance_factor =
-	    avoidance_factor + (cover_factor > 1) ? cover_factor : 0;
-	else
-	  avoidance_factor = cover_factor;
+	avoidance_factor += (avoidance_factor >= 1)? cover_factor : 0;
       }
+
+      if (SEWMH_PLACEMENT_MODE(sflags) == EWMH_USE_DYNAMIC_WORKING_AREA &&
+	  !DO_EWMH_IGNORE_STRUT_HINTS(testw) &&
+	  (testw->dyn_strut.left > 0 || testw->dyn_strut.right > 0 ||
+	   testw->dyn_strut.top > 0 || testw->dyn_strut.bottom > 0))
+      {
+	/* if we intersect a window which reserves space */
+	avoidance_factor += (avoidance_factor >= 1)?
+	  EWMH_STRUT_PLACEMENT_PENALTY(t) : 0;
+      }
+
       anew *= avoidance_factor;
       aoi += anew;
       if (aoi > aoimin && aoimin != -1)
+      {
         return aoi;
+      }
     }
   }
+  /* now handle the working area */
+  if (SEWMH_PLACEMENT_MODE(sflags) == EWMH_USE_WORKING_AREA)
+  {
+    aoi += EWMH_STRUT_PLACEMENT_PENALTY(t) *
+      EWMH_GetStrutIntersection(x11, y11, x12, y12, use_percent);
+  }
   return aoi;
 }
 
@@ -582,6 +647,15 @@ Bool PlaceWindow(
       NULL, FSCREEN_CURRENT,
       &screen_g.x, &screen_g.y, &screen_g.width, &screen_g.height);
   }
+
+  if (SPLACEMENT_MODE(sflags) != PLACE_MINOVERLAPPERCENT &&
+      SPLACEMENT_MODE(sflags) != PLACE_MINOVERLAP)
+  {
+    EWMH_GetWorkAreaIntersection(tmp_win,
+      &screen_g.x, &screen_g.y, &screen_g.width, &screen_g.height,
+      SEWMH_PLACEMENT_MODE(sflags));
+  }
+
   PageLeft   = screen_g.x - pdeltax;
   PageTop    = screen_g.y - pdeltay;
   PageRight  = PageLeft + screen_g.width;
@@ -628,8 +702,11 @@ Bool PlaceWindow(
       /* Try to find the parent's desktop */
       for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
       {
-	if(t->w == tmp_win->transientfor)
+	if (t->w == tmp_win->transientfor)
+        {
 	  tmp_win->Desk = t->Desk;
+          break;
+        }
       }
     }
 
@@ -818,7 +895,10 @@ Bool PlaceWindow(
 	DragWidth = tmp_win->frame_g.width;
 	DragHeight = tmp_win->frame_g.height;
 
-	XMapRaised(dpy, Scr.SizeWindow);
+	if (Scr.SizeWindow != None)
+	{
+	  XMapRaised(dpy, Scr.SizeWindow);
+	}
 	FScreenGetScrRect(NULL, FSCREEN_GLOBAL, &mx, &my, NULL, NULL);
 	if (moveLoop(tmp_win, mx, my, DragWidth, DragHeight,
 		     &xl, &yt, False))
@@ -831,7 +911,10 @@ Bool PlaceWindow(
 	  /* ok */
 	  rc = False;
 	}
-	XUnmapWindow(dpy, Scr.SizeWindow);
+	if (Scr.SizeWindow != None)
+	{
+	  XUnmapWindow(dpy, Scr.SizeWindow);
+	}
 	MyXUngrabKeyboard(dpy);
 	MyXUngrabServer(dpy);
 	UngrabEm(GRAB_NORMAL);
@@ -852,7 +935,7 @@ Bool PlaceWindow(
       tmp_win->attr.x = xl;
       break;
     case PLACE_MINOVERLAPPERCENT:
-      CleverPlacement(tmp_win, &screen_g, &xl, &yt, pdeltax, pdeltay, 1);
+      CleverPlacement(tmp_win, sflags, &screen_g, &xl, &yt, pdeltax, pdeltay, 1);
       flags.is_smartly_placed = True;
       break;
     case PLACE_TILECASCADE:
@@ -898,19 +981,9 @@ Bool PlaceWindow(
 	  2*tmp_win->boundary_width;
 	Scr.randomy = 0;
       }
-
-      /* the left and top sides are more important in huge windows */
-      if (tmp_win->attr.x < PageLeft)
-      {
-	tmp_win->attr.x = PageLeft;
-      }
-      if (tmp_win->attr.y < PageTop)
-      {
-	tmp_win->attr.y = PageTop;
-      }
       break;
     case PLACE_MINOVERLAP:
-      CleverPlacement(tmp_win, &screen_g, &xl, &yt, pdeltax, pdeltay, 0);
+      CleverPlacement(tmp_win, sflags, &screen_g, &xl, &yt, pdeltax, pdeltay, 0);
       flags.is_smartly_placed = True;
       break;
     default:
@@ -960,25 +1033,18 @@ Bool PlaceWindow(
     if ( ( DO_NOT_SHOW_ON_MAP(tmp_win) && flags.do_honor_starts_on_page )  &&
 
 	 ( (!(IS_TRANSIENT(tmp_win)) ||
-	    SUSE_START_ON_PAGE_FOR_TRANSIENT(sflags))
+	    SUSE_START_ON_PAGE_FOR_TRANSIENT(sflags)) &&
 
-#if 0
-	   /* dv 08-Jul-2003:  Do not use this.  Instead, force the
-	    * window on the requested page even if the application
-	    * requested a different position. */
-	   && ((SUSE_NO_PPOSITION(sflags)) ||
-	       !(tmp_win->hints.flags & PPosition))
+	   ((SUSE_NO_PPOSITION(sflags)) ||
+	    !(tmp_win->hints.flags & PPosition)) &&
 
-	   /* dv 08-Jul-2003:  This condition is always true because we
-	    * already checked for flags.do_honor_starts_on_page above. */
            /*  RBW - allow StartsOnPage to go through, even if iconic.  */
-           && ( ((!((tmp_win->wmhints)&&
-		    (tmp_win->wmhints->flags & StateHint)&&
-		    (tmp_win->wmhints->initial_state == IconicState)))
-		 || (flags.do_honor_starts_on_page)) )
+           ( ((!((tmp_win->wmhints)&&
+		 (tmp_win->wmhints->flags & StateHint)&&
+		 (tmp_win->wmhints->initial_state == IconicState)))
+              || (flags.do_honor_starts_on_page)) )
 
-#endif
-		 ) )
+	   ) )
     {
       /*
        * We're placing a SkipMapping window - either capturing one that's
