@@ -54,6 +54,13 @@
 #define COLORMAP_FOLLOWS_MOUSE 1 /* default */
 #define COLORMAP_FOLLOWS_FOCUS 2
 
+#ifdef FANCY_TITLEBARS
+#define TITLE_PADDING 5
+enum tb_pixmap_enum {TBP_MAIN, TBP_LEFT_MAIN, TBP_RIGHT_MAIN, TBP_UNDER_TEXT,
+                     TBP_LEFT_OF_TEXT, TBP_RIGHT_OF_TEXT, TBP_LEFT_END,
+                     TBP_RIGHT_END, TBP_BUTTONS, TBP_LEFT_BUTTONS,
+                     TBP_RIGHT_BUTTONS, NUM_TB_PIXMAPS};
+#endif
 
 typedef struct
 {
@@ -64,14 +71,17 @@ typedef struct
 typedef enum
 {
     /* button types */
-    DefaultVectorButton         ,
-    VectorButton                ,
-    SimpleButton                ,
-    GradientButton              ,
-    PixmapButton                ,
-    TiledPixmapButton           ,
+    DefaultVectorButton       ,
+    VectorButton              ,
+    SimpleButton              ,
+    GradientButton            ,
+    PixmapButton              ,
+    TiledPixmapButton         ,
+#ifdef FANCY_TITLEBARS
+    MultiPixmap               ,
+#endif
 #ifdef MINI_ICONS
-    MiniIconButton              ,
+    MiniIconButton            ,
 #endif
     SolidButton
 } DecorFaceType;
@@ -88,7 +98,11 @@ typedef enum
 
 typedef struct
 {
+#ifdef FANCY_TITLEBARS
+  unsigned face_type : 4;
+#else
   unsigned face_type : 3; /* was DecorFaceType : 3 */
+#endif
   struct
   {
     unsigned h_justification : 2; /* was JustificationType : 2 */
@@ -123,6 +137,10 @@ typedef struct DecorFace
   struct
   {
     Picture *p;
+#ifdef FANCY_TITLEBARS
+    Picture **multi_pixmaps;
+    short multi_stretch_flags;
+#endif
     Pixel back;
     struct
     {
@@ -136,12 +154,11 @@ typedef struct DecorFace
       int *x;
       int *y;
       unsigned long line_style;
+      unsigned long use_fgbg;
     } vector;
   } u;
 
-#ifdef MULTISTYLE
   struct DecorFace *next;
-#endif
 
   struct
   {
@@ -151,13 +168,14 @@ typedef struct DecorFace
 
 enum ButtonState
 {
-  ActiveUp,
-  ActiveDown,
-  Inactive,
-  ToggledActiveUp,
-  ToggledActiveDown,
-  ToggledInactive,
-  MaxButtonState
+  BS_All = -1,
+  BS_ActiveUp,
+  BS_ActiveDown,
+  BS_Inactive,
+  BS_ToggledActiveUp,
+  BS_ToggledActiveDown,
+  BS_ToggledInactive,
+  BS_MaxButtonState
 };
 
 typedef enum
@@ -181,17 +199,23 @@ typedef enum
 typedef struct
 {
   unsigned just : 2; /* was JustificationType : 2 */
+  int layer;
   struct
   {
     unsigned has_changed : 1;
     mwm_flags mwm_decor_flags : 9;
+    /* Support {ButtonStyle <number> - Layer 4} construction, so button
+     * can be rendered 'pressed in' when the window is assigned to a
+     * particular layer. */
+    unsigned has_layer : 1;
   } flags;
-  DecorFace state[MaxButtonState];
+  DecorFace state[BS_MaxButtonState];
 } TitleButton;
 
 #define TB_FLAGS(tb)              ((tb).flags)
 #define TB_STATE(tb)              ((tb).state)
 #define TB_JUSTIFICATION(tb)      ((tb).just)
+#define TB_LAYER(tb)              ((tb).layer)
 #define TB_MWM_DECOR_FLAGS(tb)    ((tb).flags.mwm_decor_flags)
 #define TB_HAS_CHANGED(tb)     \
   (!!((tb).flags.has_changed))
@@ -229,6 +253,28 @@ typedef struct FvwmDecor
   } flags;
 } FvwmDecor;
 
+typedef struct DesktopsInfo
+{
+  int desk;
+  char *name;
+
+  struct
+  {
+    int x;
+    int y;
+    int width;
+    int height;
+  } ewmh_working_area;
+  struct
+  {
+    int x;
+    int y;
+    int width;
+    int height;
+  } ewmh_dyn_working_area;
+
+  struct DesktopsInfo *next;
+} DesktopsInfo;
 
 typedef struct ScreenInfo
 {
@@ -285,6 +331,8 @@ typedef struct ScreenInfo
   GC XorGC;			/* GC to draw lines for move and resize */
   GC ScratchGC1;
   GC ScratchGC2;
+  GC ScratchGC3;
+  GC ScratchGC4;
   GC TitleGC;
   int SizeStringWidth;	        /* minimum width of size window */
 
@@ -297,16 +345,10 @@ typedef struct ScreenInfo
   FvwmWindow *Hilite;		/* the fvwm window that is highlighted
 				 * except for networking delays, this is the
 				 * window which REALLY has the focus */
-  FvwmWindow *Focus;            /* Last window which Fvwm gave the focus to
-                                 * NOT the window that really has the focus */
   Window UnknownWinFocused;      /* None, if the focus is nowhere or on an fvwm
 				 * managed window. Set to id of otherwindow
 				 * with focus otherwise */
   FvwmWindow *Ungrabbed;
-  FvwmWindow *PreviousFocus;    /* Window which had focus before fvwm stole it
-				 * to do moves/menus/etc. */
-  FvwmWindow *LastScreenFocus;  /* Window which had focus before the pointer
-				 * moved to a different screen. */
   int EdgeScrollX;              /* #pixels to scroll on screen edge */
   int EdgeScrollY;              /* #pixels to scroll on screen edge */
   unsigned char buttons2grab;   /* buttons to grab in click to focus mode */
@@ -321,6 +363,7 @@ typedef struct ScreenInfo
   int ClickTime;               /*Max button-click delay for Function built-in*/
   int ScrollResistance;        /* resistance to scrolling in desktop */
   int MoveResistance;          /* res to moving windows over viewport edge */
+  int XiMoveResistance;        /* the same for edges of xinerama screens */
   int MoveThreshold;           /* number of pixels of mouse motion to decide
 				* it's a move operation */
   int SnapAttraction;          /* attractiveness of window edges */
@@ -353,6 +396,8 @@ typedef struct ScreenInfo
     unsigned DisableConfigureNotify : 1;
     unsigned InstallRootCmap : 1;
     unsigned RaiseOverUnmanaged : 1;
+    unsigned FlickeringQtDialogsWorkaround : 1;
+    unsigned EWMHIconicStateWorkaround : 1;
   } bo; /* bug workaround control options */
   struct
   {
@@ -372,6 +417,7 @@ typedef struct ScreenInfo
     unsigned has_default_color_changed : 1;
     unsigned has_mouse_binding_changed : 1;
     unsigned has_nr_buttons_changed : 1;
+    unsigned has_xinerama_state_changed : 1;
     unsigned is_executing_complex_function : 1;
     unsigned is_map_desk_in_progress : 1;
     unsigned is_pointer_on_this_screen : 1;
@@ -384,6 +430,10 @@ typedef struct ScreenInfo
     unsigned edge_wrap_y : 1;
   } flags;
 
+  DesktopsInfo *Desktops; /* info for some desktops; the first entries should be
+			  * generic info correct for any desktop not in the
+			  * list */
+  FvwmWindow *EwmhDesktop;   /* the window of desktop type if any */
   struct
   {
     last_added_item_type type;
@@ -391,6 +441,9 @@ typedef struct ScreenInfo
   } last_added_item;
 } ScreenInfo;
 
+/* A macro to to simplify he "ewmh desktop code" */
+#define IS_EWMH_DESKTOP(win) (Scr.EwmhDesktop && win == Scr.EwmhDesktop->w)
+
 /*
    Macro which gets specific decor or default decor.
    This saves an indirection in case you don't want
