@@ -24,11 +24,8 @@
 #include "config.h"
 
 #include <stdio.h>
-#ifdef HAVE_GETPWUID
 #include <pwd.h>
-#endif
 #include <signal.h>
-#include <fcntl.h>
 
 #include "libs/fvwmlib.h"
 #include "fvwm.h"
@@ -70,6 +67,7 @@ typedef struct _match
   int                 layer;
   int                 used;
   int                 gravity;
+  unsigned long       ewmh_hint_desktop;
   window_flags        flags;
 }
 Match;
@@ -150,30 +148,13 @@ static void setRealStateFilename(char *filename)
 static char *getUniqueStateFilename(void)
 {
   const char *path = getenv("SM_SAVE_DIR");
-  char *filename;
-  int fd;
+  struct passwd *pwd;
 
-  if ( ! path)
-    path = getenv ("HOME");
-
-#ifdef HAVE_GETPWUID
   if (!path) {
-    struct passwd *pwd = getpwuid(getuid());
-    if (pwd)
-      path = pwd->pw_dir;
+    pwd = getpwuid(getuid());
+    path = pwd->pw_dir;
   }
-#endif
-  if ( ! path)
-    return NULL;
-
-  filename = tempnam(path, ".fs-");
-  if ((fd = open (filename, O_WRONLY | O_EXCL | O_CREAT, 0600)) == -1) {
-    free (filename);
-    filename = NULL;
-  } else {
-    close (fd);
-  }
-  return filename;
+  return tempnam(path, ".fs-");
 }
 
 #else
@@ -333,8 +314,7 @@ char *get_version_string()
   /* migo (14-Mar-2001): it is better to manually update a version string
    * in the stable branch, othervise saving sessions becomes useless */
   /*return CatString3(VERSION, ", ",__DATE__);*/
-
-  return "2.4-4";
+  return "2.5-4";
 }
 
 /*
@@ -412,7 +392,9 @@ SaveWindowStates(FILE *f)
   int wm_command_count;
   FvwmWindow *ewin;
   rectangle save_g;
+  rectangle ig;
   int i;
+  int layer;
 
   for (ewin = get_next_window_in_stack_ring(&Scr.FvwmRoot);
        ewin != &Scr.FvwmRoot;
@@ -477,17 +459,24 @@ SaveWindowStates(FILE *f)
       save_g.x -= Scr.Vx;
       save_g.y -= Scr.Vy;
     }
+    get_visible_icon_geometry(ewin, &ig);
     fprintf(
       f, "  [GEOMETRY] %i %i %i %i %i %i %i %i %i %i %i %i %i %i %i\n",
       save_g.x, save_g.y, save_g.width, save_g.height,
       ewin->max_g.x, ewin->max_g.y, ewin->max_g.width, ewin->max_g.height,
       ewin->max_g_defect.width, ewin->max_g_defect.height,
-      ewin->icon_g.x + ((!is_icon_sticky) ? Scr.Vx : 0),
-      ewin->icon_g.y + ((!is_icon_sticky) ? Scr.Vy : 0),
+      ig.x + ((!is_icon_sticky) ? Scr.Vx : 0),
+      ig.y + ((!is_icon_sticky) ? Scr.Vy : 0),
       ewin->hints.win_gravity,
       ewin->max_offset.x, ewin->max_offset.y);
     fprintf(f, "  [DESK] %i\n", ewin->Desk);
-    fprintf(f, "  [LAYER] %i\n", get_layer(ewin));
+    /* set the layer to the default layer if the layer has been set by
+     * an ewmh hint */
+    layer = get_layer(ewin);
+    if (layer == ewin->ewmh_hint_layer && layer > 0)
+      layer = Scr.DefaultLayer;
+    fprintf(f, "  [LAYER] %i\n", layer);
+    fprintf(f, "  [EWMH_DESKTOP] %lu\n", ewin->ewmh_hint_desktop);
     fprintf(f, "  [FLAGS] ");
     for (i = 0; i < sizeof(window_flags); i++)
       fprintf(f, "%02x ", (int)(((unsigned char *)&(ewin->flags))[i]));
@@ -508,10 +497,8 @@ LoadWindowStates(char *filename)
 {
   FILE *f;
   char s[4096], s1[4096];
-  char *s2;
   int i, pos, pos1;
   unsigned long w;
-  int n;
 
   if (!VerifyVersionInfo(filename))
     return;
@@ -533,8 +520,7 @@ LoadWindowStates(char *filename)
 
   while (fgets(s, sizeof(s), f))
   {
-    n = 0;
-    sscanf(s, "%4000s%n", s1, &n);
+    sscanf(s, "%4000s", s1);
 #ifdef SESSION
     /* migo: temporarily */
     if (!strcmp(s1, "[REAL_STATE_FILENAME]"))
@@ -605,6 +591,11 @@ LoadWindowStates(char *filename)
       sscanf(s, "%*s %i",
 	     &(matches[num_match - 1].layer));
     }
+    else if (!strcmp(s1, "[EWMH_DESKTOP]"))
+    {
+      sscanf(s, "%*s %lu",
+	     &(matches[num_match - 1].ewmh_hint_desktop));
+    }
     else if (!strcmp(s1, "[FLAGS]"))
     {
       char *ts = s;
@@ -625,52 +616,27 @@ LoadWindowStates(char *filename)
     }
     else if (!strcmp(s1, "[CLIENT_ID]"))
     {
-      s2 = s + n;
-      if (*s2 != 0)
-      {
-	s2++;
-      }
-      sscanf(s2, "%[^\n]", s1);
+      sscanf(s, "%*s %[^\n]", s1);
       matches[num_match - 1].client_id = duplicate(s1);
     }
     else if (!strcmp(s1, "[WINDOW_ROLE]"))
     {
-      s2 = s + n;
-      if (*s2 != 0)
-      {
-	s2++;
-      }
-      sscanf(s2, "%[^\n]", s1);
+      sscanf(s, "%*s %[^\n]", s1);
       matches[num_match - 1].window_role = duplicate(s1);
     }
     else if (!strcmp(s1, "[RES_NAME]"))
     {
-      s2 = s + n;
-      if (*s2 != 0)
-      {
-	s2++;
-      }
-      sscanf(s2, "%[^\n]", s1);
+      sscanf(s, "%*s %[^\n]", s1);
       matches[num_match - 1].res_name = duplicate(s1);
     }
     else if (!strcmp(s1, "[RES_CLASS]"))
     {
-      s2 = s + n;
-      if (*s2 != 0)
-      {
-	s2++;
-      }
-      sscanf(s2, "%[^\n]", s1);
+      sscanf(s, "%*s %[^\n]", s1);
       matches[num_match - 1].res_class = duplicate(s1);
     }
     else if (!strcmp(s1, "[WM_NAME]"))
     {
-      s2 = s + n;
-      if (*s2 != 0)
-      {
-	s2++;
-      }
-      sscanf(s2, "%[^\n]", s1);
+      sscanf(s, "%*s %[^\n]", s1);
       matches[num_match - 1].wm_name = duplicate(s1);
     }
     else if (!strcmp(s1, "[WM_COMMAND]"))
@@ -821,11 +787,9 @@ MatchWinToSM(FvwmWindow *ewin, int *do_shade, int *do_max)
 	SET_FOCUS_MODE(ewin, GET_FOCUS_MODE(&(matches[i])));
 	if (matches[i].wm_name)
 	{
-	  if (ewin->name)
-	  {
-	    free(ewin->name);
-	  }
+	  free_window_names(ewin, True, False);
 	  ewin->name = matches[i].wm_name;
+	  setup_visible_name(ewin, False);
 	}
       }
       SET_PLACED_WB3(ewin,IS_PLACED_WB3(&(matches[i])));
@@ -867,18 +831,32 @@ MatchWinToSM(FvwmWindow *ewin, int *do_shade, int *do_max)
       SET_STICKY(ewin, IS_STICKY(&(matches[i])));
       ewin->Desk = (IS_STICKY(ewin)) ? Scr.CurrentDesk : matches[i].desktop;
       set_layer(ewin, matches[i].layer);
-
       /* this is not enough to fight fvwms attempts to
 	 put icons on the current page */
-      ewin->icon_g.x = matches[i].icon_x;
-      ewin->icon_g.y = matches[i].icon_y;
+      set_icon_position(ewin, matches[i].icon_x, matches[i].icon_y);
       if (!IS_STICKY(&(matches[i])) &&
 	  !(IS_ICONIFIED(&(matches[i])) && IS_ICON_STICKY(&(matches[i]))))
       {
-	ewin->icon_g.x -= Scr.Vx;
-	ewin->icon_g.y -= Scr.Vy;
+	set_icon_position(
+	  ewin, matches[i].icon_x - Scr.Vx, matches[i].icon_y - Scr.Vy);
       }
-
+      /* Note: the Modal, skip pager, skip taskbar and "stacking order" state
+       * are not restored here: there are restored in EWMH_ExitStuff */
+      ewin->ewmh_hint_desktop = matches[i].ewmh_hint_desktop;
+      SET_HAS_EWMH_INIT_WM_DESKTOP(
+	  ewin, HAS_EWMH_INIT_WM_DESKTOP(&(matches[i])));
+      SET_HAS_EWMH_INIT_FULLSCREEN_STATE(
+	  ewin, HAS_EWMH_INIT_FULLSCREEN_STATE(&(matches[i])));
+      SET_HAS_EWMH_INIT_HIDDEN_STATE(
+	  ewin, HAS_EWMH_INIT_HIDDEN_STATE(&(matches[i])));
+      SET_HAS_EWMH_INIT_MAXHORIZ_STATE(
+	  ewin, HAS_EWMH_INIT_MAXHORIZ_STATE(&(matches[i])));
+      SET_HAS_EWMH_INIT_MAXVERT_STATE(
+	  ewin, HAS_EWMH_INIT_MAXVERT_STATE(&(matches[i])));
+      SET_HAS_EWMH_INIT_SHADED_STATE(
+	  ewin, HAS_EWMH_INIT_SHADED_STATE(&(matches[i])));
+      SET_HAS_EWMH_INIT_STICKY_STATE(
+	  ewin, HAS_EWMH_INIT_STICKY_STATE(&(matches[i])));
       return True;
     }
   }
