@@ -35,21 +35,12 @@
 #include "events.h"
 #include "borders.h"
 #include "virtual.h"
-#include "icons.h"
+#include "geometry.h"
 #include "gnome.h"
+#include "ewmh.h"
 
 /* ----------------------------- stack ring code --------------------------- */
 
-/* If more than this many transients are in a single branch of a transient
- * tree, they will end up in more or less random stacking order. */
-#define MAX_TRANSIENTS_IN_BRANCH 200000
-/* Same for total levels of transients. */
-#define MAX_TRANSIENT_LEVELS     10000
-/* This number must fit in a signed int! */
-#define LOWER_PENALTY (MAX_TRANSIENTS_IN_BRANCH * MAX_TRANSIENT_LEVELS)
-
-static void __RaiseOrLowerWindow(
-  FvwmWindow *t, Bool do_lower, Bool allow_recursion, Bool is_new_window);
 static void RaiseOrLowerWindow(
   FvwmWindow *t, Bool do_lower, Bool allow_recursion, Bool is_new_window);
 #if 0
@@ -57,8 +48,6 @@ static void ResyncFvwmStackRing(void);
 #endif
 static void ResyncXStackingOrder(void);
 static void BroadcastRestack(FvwmWindow *s1, FvwmWindow *s2);
-static int collect_transients_recursive(
-  FvwmWindow *t, FvwmWindow *list_head, int layer, Bool do_lower);
 
 #define DEBUG_STACK_RING 1
 #ifdef DEBUG_STACK_RING
@@ -341,7 +330,17 @@ static void raise_over_unmanaged(FvwmWindow *t)
 	 t2 = t2->stack_next)
     {
       count++;
-      count += get_visible_icon_window_count(t2);
+      if (IS_ICONIFIED(t2) && ! IS_ICON_SUPPRESSED(t2))
+      {
+	if (t2->icon_title_w != None)
+	{
+	  count++;
+        }
+	if (t2->icon_pixmap_w != None)
+	{
+	  count++;
+        }
+      }
       if (t2 == t)
       {
 	break;
@@ -357,9 +356,9 @@ static void raise_over_unmanaged(FvwmWindow *t)
 	wins[i++] = t2->frame;
 	if (IS_ICONIFIED(t2) && ! IS_ICON_SUPPRESSED(t2))
 	{
-	  if (t2->icon_w != None)
+	  if (t2->icon_title_w != None)
 	  {
-	    wins[i++] = t2->icon_w;
+	    wins[i++] = t2->icon_title_w;
           }
 	  if (t2->icon_pixmap_w != None)
 	  {
@@ -389,8 +388,10 @@ static void raise_over_unmanaged(FvwmWindow *t)
 
 
 static Bool must_move_transients(
-  FvwmWindow *t, Bool do_lower)
+  FvwmWindow *t, Bool do_lower, Bool *found_transient)
 {
+  *found_transient = False;
+
   if (IS_ICONIFIED(t))
   {
     return False;
@@ -402,14 +403,18 @@ static Bool must_move_transients(
     Bool scanning_above_window = True;
     FvwmWindow *q;
 
-    for (q = Scr.FvwmRoot.stack_next;
-	 q != &Scr.FvwmRoot && t->layer <= q->layer; q = q->stack_next)
+    for (q = Scr.FvwmRoot.stack_next; q != &Scr.FvwmRoot; q = q->stack_next)
     {
       if (t->layer < q->layer)
       {
 	/* We're not interested in higher layers. */
 	continue;
       }
+      if (t->layer > q->layer)
+      {
+	/* We are at the end of this layer. Stop scanning windows. */
+	break;
+      }
       else if (t == q)
       {
 	/* We found our window. All further transients are below it. */
@@ -417,12 +422,22 @@ static Bool must_move_transients(
       }
       else if (IS_TRANSIENT(q) && (q->transientfor == t->w))
       {
-	return True;
+	/* found a transient */
+	*found_transient = True;
+
+        /* transients are not allowed below main in the same layer */
+        if ( !scanning_above_window )
+        {
+          return True;
+        }
       }
-      else if (scanning_above_window && !do_lower)
+      else if ((scanning_above_window && !do_lower) ||
+	       (*found_transient && do_lower))
       {
 	/* raise: The window is not raised, so itself and all transients will
 	 * be raised. */
+	/* lower: There is a transient above some other window, so we have to
+	 * lower. */
 	return True;
       }
     }
@@ -430,53 +445,34 @@ static Bool must_move_transients(
   return False;
 }
 
-static Window __get_stacking_sibling(FvwmWindow *fw, Bool do_stack_below)
-{
-  Window w;
-
-  /* default to frame window */
-  w = fw->frame;
-  if (IS_ICONIFIED(fw) && do_stack_below == True)
-  {
-    /* override with icon windows when stacking below */
-    if (fw->icon_pixmap_w != None)
-    {
-      w = fw->icon_pixmap_w;
-    }
-    else if (fw->icon_w != None)
-    {
-      w = fw->icon_w;
-    }
-  }
-
-  return w;
-}
-
 static void restack_windows(
-  FvwmWindow *r, FvwmWindow *s, int count, Bool do_broadcast_all, Bool do_lower)
+  FvwmWindow *r, FvwmWindow *s, int count, Bool do_broadcast_all)
 {
   FvwmWindow *t;
   unsigned int flags;
   int i;
   XWindowChanges changes;
   Window *wins;
-  int do_stack_above;
-  int is_reversed;
 
   if (count <= 0)
   {
-    for (count = 0, t = r->stack_next; t != s; t = t->stack_next)
+    FvwmWindow *prev;
+
+    for (t = r, prev = NULL; prev != s; prev = t, t = t->stack_next)
     {
       count++;
-      count += get_visible_icon_window_count(t);
+      if (IS_ICONIFIED(t) && !IS_ICON_SUPPRESSED(t))
+      {
+	count += 2;
+      }
     }
   }
   /* restack the windows between r and s */
-  wins = (Window*) safemalloc ((count + 3) * sizeof (Window));
+  wins = (Window*) safemalloc (count * sizeof (Window));
   i = 0;
   for (t = r->stack_next; t != s; t = t->stack_next)
   {
-    if (i > count)
+    if (i >= count)
     {
       fvwm_msg (ERR, "restack_windows", "more transients than expected");
       break;
@@ -484,40 +480,30 @@ static void restack_windows(
     wins[i++] = t->frame;
     if (IS_ICONIFIED(t) && !IS_ICON_SUPPRESSED(t))
     {
-      if(t->icon_w != None)
-	wins[i++] = t->icon_w;
+      if(t->icon_title_w != None)
+	wins[i++] = t->icon_title_w;
       if(t->icon_pixmap_w != None)
 	wins[i++] = t->icon_pixmap_w;
     }
   }
 
-  changes.sibling = __get_stacking_sibling(r, True);
-  if (changes.sibling == None)
+  changes.sibling = s->frame;
+  if (changes.sibling != None)
   {
-    changes.sibling = __get_stacking_sibling(s, False);
-    is_reversed = 1;
+    changes.stack_mode = Above;
+    flags = CWSibling|CWStackMode;
   }
   else
   {
-    is_reversed = 0;
-  }
-  if (changes.sibling == None)
-  {
-    do_stack_above = !do_lower;
+    changes.stack_mode = Below;
     flags = CWStackMode;
   }
-  else
-  {
-    do_stack_above = 0;
-    flags = CWStackMode | CWSibling;
-  }
-  changes.stack_mode = (do_stack_above ^ is_reversed) ? Above : Below;
   XConfigureWindow (dpy, r->stack_next->frame, flags, &changes);
-  if (count > 1)
-  {
-    XRestackWindows (dpy, wins, count);
-  }
+  XRestackWindows (dpy, wins, count);
   free(wins);
+
+  EWMH_SetClientListStacking();
+
   if (do_broadcast_all)
   {
     /* send out M_RESTACK for all windows, to make sure we don't forget
@@ -533,145 +519,22 @@ static void restack_windows(
   return;
 }
 
-static void __sort_transient_ring(FvwmWindow *ring)
-{
-  FvwmWindow *s;
-  FvwmWindow *t;
-  FvwmWindow *u;
-  FvwmWindow *prev;
-
-  if (ring->stack_next->stack_next == ring)
-  {
-    /* only one or zero windows */
-    return;
-  }
-  /* Implementation note:  this sorting algorithm is about the most inefficient
-   * possible.  It just swaps the position of two adjacent windows in the ring
-   * if they are in the wrong order.  Since transient windows are rare, this
-   * should not cause any notable performance hit.  Because it is important that
-   * the order of windows with the same key is not changed, we can not just use
-   * qsort() here. */
-  for (t = ring->stack_next, prev = ring; t->stack_next != ring;
-       prev = t->stack_prev)
-  {
-    s = t->stack_next;
-    if (t->scratch.i < s->scratch.i)
-    {
-      /* swap windows */
-      u = s->stack_next;
-      s->stack_next = t;
-      t->stack_next = u;
-      u = t->stack_prev;
-      t->stack_prev = s;
-      s->stack_prev = u;
-      s->stack_prev->stack_next = s;
-      t->stack_next->stack_prev = t;
-      if (prev != ring)
-      {
-	/* move further up the ring? */
-	t = prev;
-      }
-      else
-      {
-	/* hit start of ring */
-      }
-    }
-    else
-    {
-      /* correct order, advance one window */
-      t = t->stack_next;
-    }
-  }
-
-  return;
-}
-
-static Bool __restack_window(
-  FvwmWindow *t, Bool do_lower, Bool do_restack_transients, Bool is_new_window)
-{
-  FvwmWindow *s, *r, tmp_r;
-  int count;
-  int test_layer;
-
-  /* detach t, so it doesn't make trouble in the loops */
-  remove_window_from_stack_ring(t);
-
-  count = 0;
-  if (do_restack_transients)
-  {
-    /* collect the transients in a temp list */
-    tmp_r.stack_prev = &tmp_r;
-    tmp_r.stack_next = &tmp_r;
-    count = collect_transients_recursive(t, &tmp_r, t->layer, do_lower);
-    if (count == 0)
-    {
-      do_restack_transients = False;
-    }
-  }
-  count += 1 + get_visible_icon_window_count(t);
-
-  test_layer = t->layer;
-  if (do_lower)
-  {
-    test_layer--;
-  }
-  /* now find the place to reinsert t and friends */
-  for (s = Scr.FvwmRoot.stack_next; s != &Scr.FvwmRoot; s = s->stack_next)
-  {
-    if (test_layer >= s->layer)
-    {
-      break;
-    }
-  }
-  r = s->stack_prev;
-
-  if (do_restack_transients)
-  {
-    /* re-sort the transient windows according to their scratch.i register */
-    __sort_transient_ring(&tmp_r);
-    /* insert all transients between r and s. */
-    add_windowlist_to_stack_ring_after(&tmp_r, r);
-  }
-
-  /*
-  ** Re-insert t - below transients
-  */
-  add_window_to_stack_ring_after(t, s->stack_prev);
-
-  if (is_new_window && IS_TRANSIENT(t) && DO_STACK_TRANSIENT_PARENT(t) &&
-      !IS_ICONIFIED(t))
-  {
-    /* now that the new transient is properly positioned in the stack ring,
-     * raise/lower it again so that its parent is raised/lowered too */
-    RaiseOrLowerWindow(t, do_lower, True, False);
-    /* make sure the stacking order is correct - may be the sledge-hammer
-     * method, but the recursion ist too hard to understand. */
-    ResyncXStackingOrder();
-    return True;
-  }
-  else
-  {
-    /* restack the windows between r and s */
-    restack_windows(r, s, count, do_restack_transients, do_lower);
-  }
-
-  return False;
-}
-
-/* Important note:  __RaiseOrLowerWindow must *only* be called by
- * RaiseOrLowerWindow()! */
-static void __RaiseOrLowerWindow(
+static void RaiseOrLowerWindow(
   FvwmWindow *t, Bool do_lower, Bool allow_recursion, Bool is_new_window)
 {
-  FvwmWindow *t2;
+  FvwmWindow *s, *r, *t2, *next, tmp_r;
+  int count;
   Bool do_move_transients;
+  Bool found_transient;
+  Bool no_movement;
+  int test_layer;
 
   /* Do not raise this window after command execution (see HandleButtonPress()).
    */
   SET_SCHEDULED_FOR_RAISE(t, 0);
 
   /* New windows are simply raised/lowered without touching the transientfor
-   * at first.  Then, further down in the code, __RaiseOrLowerWindow() is called
+   * at first.  Then, further down in the code, RaiseOrLowerWindow() is called
    * again to raise/lower the transientfor if necessary.  We can not do the
    * recursion stuff for new windows because the must_move_transients() call
    * needs a properly ordered stack ring - but the new window is still at the
@@ -696,31 +559,14 @@ static void __RaiseOrLowerWindow(
       {
         if (t2->w == t->transientfor)
         {
-	  if (t2 == t)
+	  if (IS_ICONIFIED(t2))
 	  {
 	    break;
 	  }
-	  if (IS_ICONIFIED(t2) || t->layer != t2->layer)
-	  {
-	    break;
-	  }
-	  if (do_lower && (!IS_TRANSIENT(t2) || !DO_STACK_TRANSIENT_PARENT(t2)))
-	  {
-	    /* hit the highest level transient; lower this subtree below all
-	     * other subtrees of the same window */
-	    t->scratch.i = -LOWER_PENALTY;
-	  }
-	  else
-	  {
-	    /* Add a bonus to the stack ring position for this branch of the
-	     * transient tree over all other branches. */
-	    t->scratch.i = MAX_TRANSIENTS_IN_BRANCH;
-	  }
-	  __RaiseOrLowerWindow(t2, do_lower, True, False);
           if ((!do_lower && DO_RAISE_TRANSIENT(t2)) ||
               (do_lower && DO_LOWER_TRANSIENT(t2))  )
           {
-	    /* moving the parent moves our window already */
+	    RaiseOrLowerWindow(t2,do_lower,False,False);
 	    return;
           }
         }
@@ -730,16 +576,109 @@ static void __RaiseOrLowerWindow(
 
   if (is_new_window)
   {
+    no_movement = False;
     do_move_transients = False;
   }
   else
   {
-    do_move_transients = must_move_transients(t, do_lower);
+    do_move_transients = must_move_transients(t, do_lower, &found_transient);
+
+    /*
+     * This part implements (Dont)FlipTransient style.
+     *
+     * must_move_transients() believes that main should always be below
+     * its transients. Therefore if it finds a transient but does not wish
+     * move it, this means main and its superior transients are already
+     * in the correct position at the top or bottom of the layer - depending
+     * on do_lower.
+     */
+    no_movement = found_transient && !do_move_transients;
   }
-  if (__restack_window(
-	t, do_lower, do_move_transients, is_new_window) == True)
+
+  if (!no_movement)
   {
-    return;
+    /* detach t, so it doesn't make trouble in the loops */
+    remove_window_from_stack_ring(t);
+
+    count = 1;
+    if (IS_ICONIFIED(t) && !IS_ICON_SUPPRESSED(t))
+    {
+      count += 2;
+    }
+
+    if (do_move_transients)
+    {
+      /* collect the transients in a temp list */
+      tmp_r.stack_prev = &tmp_r;
+      tmp_r.stack_next = &tmp_r;
+      for (t2 = Scr.FvwmRoot.stack_next; t2 != &Scr.FvwmRoot; t2 = next)
+      {
+	next = t2->stack_next;
+	if ((IS_TRANSIENT(t2)) && (t2->transientfor == t->w) &&
+	    (t2->layer == t->layer) && !IS_ICONIFIED(t2))
+	{
+	  /* t2 is a transient to lower */
+	  count++;
+	  if (IS_ICONIFIED(t2) && !IS_ICON_SUPPRESSED(t2))
+	  {
+	    count += 2;
+	  }
+
+	  /* unplug it */
+	  remove_window_from_stack_ring(t2);
+	  /* put it above tmp_r */
+	  add_window_to_stack_ring_after(t2, tmp_r.stack_prev);
+	}
+      }
+      if (tmp_r.stack_next == &tmp_r)
+      {
+	do_move_transients = False;
+      }
+    }
+
+    test_layer = t->layer;
+    if (do_lower)
+    {
+      test_layer--;
+    }
+    /* now find the place to reinsert t and friends */
+    for (s = Scr.FvwmRoot.stack_next; s != &Scr.FvwmRoot; s = s->stack_next)
+    {
+      if (test_layer >= s->layer)
+      {
+	break;
+      }
+    }
+    r = s->stack_prev;
+
+    if (do_move_transients && tmp_r.stack_next != &tmp_r)
+    {
+      /* insert all transients between r and s. */
+      add_windowlist_to_stack_ring_after(&tmp_r, r);
+    }
+
+    /*
+    ** Re-insert t - below transients
+    */
+    add_window_to_stack_ring_after(t, s->stack_prev);
+
+    if (is_new_window && IS_TRANSIENT(t) && DO_STACK_TRANSIENT_PARENT(t) &&
+	!IS_ICONIFIED(t))
+    {
+      /* now that the new transient is properly positioned in the stack ring,
+       * raise/lower it again so that its parent is raised/lowered too */
+      RaiseOrLowerWindow(t, do_lower, True, False);
+      /* make sure the stacking order is correct - may be the sledge-hammer
+       * method, but the recursion ist too hard to understand. */
+      ResyncXStackingOrder();
+      return;
+    }
+    else
+    {
+      /* restack the windows between r and s */
+      restack_windows(
+	r, s, count, (do_move_transients && tmp_r.stack_next != &tmp_r));
+    }
   }
 
   if (!do_lower)
@@ -753,6 +692,7 @@ static void __RaiseOrLowerWindow(
       raise_over_unmanaged(t);
     }
 
+
     /*
      * The following is a hack to raise X windows over native windows
      * which is needed for some (all ?) X servers running under Windows
@@ -784,31 +724,18 @@ static void __RaiseOrLowerWindow(
         XRaiseWindow (dpy, t2->frame);
       }
     }
+
+
     /*  This needs to be done after all the raise hacks.  */
     raisePanFrames();
-    /* If the window has been raised, make sure the decorations are updated
-     * immediately in case we are in a complex function (e.g. raise, unshade).
-     */
+  }
+  /* If the window has been raised, make sure the decorations are updated
+   * immediately in case we are in a complex function (e.g. raise, unshade). */
+  if (!do_lower)
+  {
     XSync(dpy, 0);
     handle_all_expose();
   }
-
-  return;
-}
-
-static void RaiseOrLowerWindow(
-  FvwmWindow *t, Bool do_lower, Bool allow_recursion, Bool is_new_window)
-{
-  FvwmWindow *fw;
-
-  /* clean the auxiliary registers used in stacking transients */
-  for (fw = Scr.FvwmRoot.next; fw != NULL; fw = fw->next)
-  {
-    fw->scratch.i = 0;
-  }
-  __RaiseOrLowerWindow(t, do_lower, allow_recursion, is_new_window);
-
-  return;
 }
 
 /*
@@ -849,47 +776,31 @@ intersect (int x0, int y0, int w0, int h0,
 }
 
 static Bool
-overlap_box (FvwmWindow *r, int x, int y, int w, int h)
+overlap(FvwmWindow *r, FvwmWindow *s)
 {
-  if (IS_ICONIFIED(r))
-  {
-    return ((r->icon_pixmap_w) &&
-             intersect (x, y, w, h, r->icon_g.x, r->icon_g.y,
-			r->icon_p_width, r->icon_p_height)) ||
-           ((r->icon_w) &&
-             intersect (x, y, w, h, r->icon_xl_loc,
-			r->icon_g.y + r->icon_p_height,
-			r->icon_g.width, r->icon_g.height));
-  }
-  else
-  {
-    return intersect (x, y, w, h, r->frame_g.x, r->frame_g.y,
-                           r->frame_g.width, r->frame_g.height);
-  }
-}
+	rectangle g1;
+	rectangle g2;
+	Bool rc;
 
-static Bool
-overlap (FvwmWindow *r, FvwmWindow *s)
-{
-  if (r->Desk != s->Desk)
-  {
-    return 0;
-  }
+	if (r->Desk != s->Desk)
+	{
+		return False;
+	}
+	rc = get_visible_window_or_icon_geometry(r, &g1);
+	if (rc == False)
+	{
+		return False;
+	}
+	rc = get_visible_window_or_icon_geometry(s, &g2);
+	if (rc == False)
+	{
+		return False;
+	}
+	rc = intersect(
+		g1.x, g1.y, g1.width, g1.height,
+		g2.x, g2.y, g2.width, g2.height);
 
-  if (IS_ICONIFIED(r))
-  {
-    return ((r->icon_pixmap_w) &&
-             overlap_box (s, r->icon_g.x, r->icon_g.y,
-                             r->icon_p_width, r->icon_p_height)) ||
-           ((r->icon_w) &&
-             overlap_box (s, r->icon_xl_loc, r->icon_g.y + r->icon_p_height,
-                             r->icon_g.width, r->icon_g.height));
-  }
-  else
-  {
-    return overlap_box (s, r->frame_g.x, r->frame_g.y,
-                           r->frame_g.width, r->frame_g.height);
-  }
+	return rc;
 }
 
 /* return true if stacking order changed */
@@ -982,7 +893,7 @@ static void ResyncFvwmStackRing (void)
     {
       if (IS_ICONIFIED(t1) && !IS_ICON_SUPPRESSED(t1))
       {
-	if (t1->icon_w == children[i] || t1->icon_pixmap_w == children[i])
+	if (t1->icon_title_w == children[i] || t1->icon_pixmap_w == children[i])
 	{
 	  break;
 	}
@@ -1041,8 +952,8 @@ static void ResyncXStackingOrder(void)
       wins[i++] = t->frame;
       if (IS_ICONIFIED(t) && !IS_ICON_SUPPRESSED(t))
       {
-	if (t->icon_w != None)
-	  wins[i++] = t->icon_w;
+	if (t->icon_title_w != None)
+	  wins[i++] = t->icon_title_w;
 	if (t->icon_pixmap_w != None)
 	  wins[i++] = t->icon_pixmap_w;
       }
@@ -1059,12 +970,9 @@ static void ResyncXStackingOrder(void)
 /* send RESTACK packets for all windows between s1 and s2 */
 static void BroadcastRestack(FvwmWindow *s1, FvwmWindow *s2)
 {
-  FvwmWindow *fw;
-  int num;
-  int i;
-  int n;
+  FvwmWindow *t, *t2;
+  int num, i;
   unsigned long *body, *bp, length;
-  unsigned long max_wins_per_packet;
   extern Time lastTimestamp;
 
   if (s2 == &Scr.FvwmRoot)
@@ -1075,51 +983,46 @@ static void BroadcastRestack(FvwmWindow *s1, FvwmWindow *s2)
   }
   if (s1 == &Scr.FvwmRoot)
   {
-    s1 = s1->stack_next;
-    if (s1 == &Scr.FvwmRoot)
+    t = s1->stack_next;
+    if (t == &Scr.FvwmRoot)
       return;
-    /* s1 has been moved to the top of stack */
-    BroadcastPacket (M_RAISE_WINDOW, 3, s1->w, s1->frame, (unsigned long)s1);
-    if (s1->stack_next == s2)
+    /* t has been moved to the top of stack */
+    BroadcastPacket (M_RAISE_WINDOW, 3, t->w, t->frame, (unsigned long)t);
+    if (t->stack_next == s2)
     {
       /* avoid sending empty RESTACK packet */
       return;
     }
   }
+  else
+  {
+    t = s1;
+  }
   if (s1 == s2)
   {
     /* A useful M_RESTACK packet must contain at least two windows. */
     return;
   }
-  for (fw = s1, num = 1; fw != s2 && fw != &Scr.FvwmRoot;
-       fw = fw->stack_next, num++)
+  for (t2 = t, num = 1 ; t2 != s2 && t != &Scr.FvwmRoot;
+       t2 = t2->stack_next, num++)
+    ;
+  length = FvwmPacketHeaderSize + 3*num;
+  body = (unsigned long *) safemalloc (length*sizeof(unsigned long));
+
+  bp = body;
+  *(bp++) = START_FLAG;
+  *(bp++) = M_RESTACK;
+  *(bp++) = length;
+  *(bp++) = lastTimestamp;
+  for (t2 = t; num != 0; num--, t2 = t2->stack_next)
   {
-	  /* nothing */
-  }
-  max_wins_per_packet = (FvwmPacketMaxSize - FvwmPacketHeaderSize) / 3;
-  /* split packet if it is too long */
-  for ( ; num > 1; s1 = fw, num -= n)
-  {
-    n = min(num, max_wins_per_packet) - 1;
-    length = FvwmPacketHeaderSize + 3 * (n + 1);
-    body = (unsigned long *)safemalloc(length * sizeof(unsigned long));
-    bp = body;
-    *(bp++) = START_FLAG;
-    *(bp++) = M_RESTACK;
-    *(bp++) = length;
-    *(bp++) = lastTimestamp;
-    for (fw = s1, i = 0; i <= n; i++, fw = fw->stack_next)
-    {
-      *(bp++) = fw->w;
-      *(bp++) = fw->frame;
-      *(bp++) = (unsigned long)fw;
-    }
-    for (i = 0; i < npipes; i++)
-    {
-      PositiveWrite(i, body, length*sizeof(unsigned long));
-    }
-    free(body);
+    *(bp++) = t2->w;
+    *(bp++) = t2->frame;
+    *(bp++) = (unsigned long)t2;
   }
+  for (i = 0; i < npipes; i++)
+    PositiveWrite(i, body, length*sizeof(unsigned long));
+  free(body);
 #ifdef DEBUG_STACK_RING
   verify_stack_ring_consistency();
 #endif
@@ -1211,11 +1114,11 @@ void mark_transient_subtree(
     SET_IN_TRANSIENT_SUBTREE(s, 0);
     if (IS_TRANSIENT(s) && (layer < 0 || layer == s->layer))
     {
-      s->scratch.p = get_transientfor_fvwmwindow(s);
+      s->pscratch = get_transientfor_fvwmwindow(s);
     }
     else
     {
-      s->scratch.p = NULL;
+      s->pscratch = NULL;
     }
   }
   /* now loop over the windows and mark the ones we need to move */
@@ -1234,7 +1137,7 @@ void mark_transient_subtree(
 
       if (IS_IN_TRANSIENT_SUBTREE(s))
 	continue;
-      if (use_window_group_hint && DO_ICONIFY_WINDOW_GROUPS(s) && s->wmhints &&
+      if (DO_ICONIFY_WINDOW_GROUPS(s) && s->wmhints &&
 	  (s->wmhints->flags & WindowGroupHint) &&
 	  (s->wmhints->window_group != None) &&
 	  (s->wmhints->window_group != s->w) &&
@@ -1244,9 +1147,9 @@ void mark_transient_subtree(
       }
       if (!IS_TRANSIENT(s) && !use_group_hint)
 	continue;
-      if (do_ignore_icons && IS_ICONIFIED(s))
+      r = (FvwmWindow *)s->pscratch;
+      if (do_ignore_icons && r && IS_ICONIFIED(r))
 	continue;
-      r = (FvwmWindow *)s->scratch.p;
       if (IS_TRANSIENT(s))
       {
 	if (r && IS_IN_TRANSIENT_SUBTREE(r) &&
@@ -1256,8 +1159,6 @@ void mark_transient_subtree(
 	{
 	  /* have to move this one too */
 	  SET_IN_TRANSIENT_SUBTREE(s, 1);
-	  /* used for stacking transients */
-	  s->scratch.i += r->scratch.i + 1;
 	  /* need another scan through the list */
 	  is_finished = False;
 	  continue;
@@ -1306,7 +1207,8 @@ static int collect_transients_recursive(
       remove_window_from_stack_ring(s);
       add_window_to_stack_ring_after(s, list_head->stack_prev);
       count++;
-      count += get_visible_icon_window_count(t);
+      if (IS_ICONIFIED(t) && !IS_ICON_SUPPRESSED(t))
+	count += 2;
     }
     s = tmp;
   }
@@ -1322,12 +1224,7 @@ void new_layer(FvwmWindow *tmp_win, int layer)
   FvwmWindow list_head;
   int add_after_layer;
   int count;
-  Bool do_lower;
 
-  if (layer < 0)
-  {
-    layer = 0;
-  }
   tmp_win = get_transientfor_top_fvwmwindow(tmp_win);
   if (layer == tmp_win->layer)
     return;
@@ -1346,13 +1243,11 @@ void new_layer(FvwmWindow *tmp_win, int layer)
   {
     /* lower below the windows in the new (lower) layer */
     add_after_layer = layer;
-    do_lower = True;
   }
   else
   {
     /* raise above the windows in the new (higher) layer */
     add_after_layer = layer + 1;
-    do_lower = False;
   }
   /* find the place to insert the windows */
   for (target = Scr.FvwmRoot.stack_next; target != &Scr.FvwmRoot;
@@ -1371,11 +1266,15 @@ void new_layer(FvwmWindow *tmp_win, int layer)
        prev = s, s = s->stack_next)
   {
     s->layer = layer;
+    /* redraw title and buttons to update layer buttons */
+    DrawDecorations(
+      s, DRAW_TITLE | DRAW_BUTTONS, (Scr.Hilite == tmp_win), True, None,
+      CLEAR_NONE);
     GNOME_SetLayer(tmp_win);
+    EWMH_SetWMState(tmp_win, False);
   }
   /* move the windows without modifying their stacking order */
-  restack_windows(
-    list_head.stack_next->stack_prev, target, count, (count > 1), do_lower);
+  restack_windows(list_head.stack_next->stack_prev, target, count, (count > 1));
 
   return;
 }
@@ -1401,15 +1300,6 @@ static Bool is_on_top_of_layer_ignore_rom(FvwmWindow *fw)
   FvwmWindow *t;
   Bool ontop = True;
 
-  if (IS_SCHEDULED_FOR_DESTROY(fw))
-  {
-    /* stack ring members are no longer valid */
-    return False;
-  }
-  if (DO_RAISE_TRANSIENT(fw))
-  {
-    mark_transient_subtree(fw, fw->layer, MARK_RAISE, True, False);
-  }
   for (t = fw->stack_prev; t != &Scr.FvwmRoot; t = t->stack_prev)
   {
     if (t->layer > fw->layer)
@@ -1425,7 +1315,8 @@ static Bool is_on_top_of_layer_ignore_rom(FvwmWindow *fw)
      * windows, but on top of its layer, it would be considered on top. */
     if (Scr.bo.RaiseOverUnmanaged || overlap(fw, t))
     {
-      if (!DO_RAISE_TRANSIENT(fw) || (!IS_IN_TRANSIENT_SUBTREE(t) && t != fw))
+      if (!IS_TRANSIENT(t) || t->transientfor != fw->w ||
+	  !DO_RAISE_TRANSIENT(fw))
       {
 	ontop = False;
 	break;
