@@ -53,6 +53,7 @@
 #include "libs/Colorset.h"
 #include "borders.h"
 #include "gnome.h"
+#include "ewmh.h"
 #include "icons.h"
 #include "focus.h"
 #include "geometry.h"
@@ -60,6 +61,7 @@
 #include "add_window.h"
 #include "module_interface.h"
 #include "focus.h"
+#include "stack.h"
 
 static void init_style(
   FvwmWindow *old_t, FvwmWindow *t, window_style *pstyle, short *pbuttons)
@@ -77,6 +79,15 @@ static void apply_window_updates(
   short buttons;
   Bool is_style_initialised = False;
 
+  if (flags->do_update_gnome_styles)
+  {
+    if (!SDO_IGNORE_GNOME_HINTS(pstyle->flags))
+    {
+      GNOME_GetStyle(t, pstyle);
+    }
+  }
+
+  t->title_top_height = (HAS_BOTTOM_TITLE(t)) ? 0 : t->title_g.height;
   if (flags->do_update_window_grabs)
   {
     focus_grab_buttons(t, False);
@@ -114,9 +125,38 @@ static void apply_window_updates(
 #ifdef MINI_ICONS
   if (flags->do_update_mini_icon)
   {
-    change_mini_icon(t, pstyle);
+    if (!HAS_EWMH_MINI_ICON(t) || DO_EWMH_MINI_ICON_OVERRIDE(t))
+    {
+      change_mini_icon(t, pstyle);
+    }
+    else
+    {
+      if (EWMH_SetIconFromWMIcon(t, NULL, 0, True))
+	SET_HAS_EWMH_MINI_ICON(t, True);
+      else
+      {
+	/* "should" not happen */
+	SET_HAS_EWMH_MINI_ICON(t, False);
+	change_mini_icon(t, pstyle);
+      }
+    }
   }
 #endif
+  if (flags->do_update_visible_window_name)
+  {
+    setup_visible_name(t, False);
+    BroadcastName(M_VISIBLE_NAME,t->w,t->frame,
+		(unsigned long)t,t->visible_name);
+    EWMH_SetVisibleName(t, False);
+  }
+
+  if (flags->do_update_visible_icon_name)
+  {
+    setup_visible_name(t, True);
+    BroadcastName(MX_VISIBLE_ICON_NAME,t->w,t->frame,
+		  (unsigned long)t,t->visible_icon_name);
+    EWMH_SetVisibleName(t, True);
+  }
 
   if (flags->do_update_window_font || flags->do_update_window_font_height)
   {
@@ -186,15 +226,20 @@ static void apply_window_updates(
     rectangle old_g;
 
     setup_frame_size_limits(t, pstyle);
-    gravity_constrain_size(t->hints.win_gravity, t, &(t->frame_g));
     old_g = t->frame_g;
     t->frame_g = t->normal_g;
+    gravity_constrain_size(t->hints.win_gravity, t, &(t->frame_g), 0);
+    t->normal_g = t->frame_g;
     if (IS_MAXIMIZED(t))
     {
       t->frame_g = t->max_g;
-      gravity_constrain_size(t->hints.win_gravity, t, &(t->max_g));
+      gravity_constrain_size(
+	t->hints.win_gravity, t, &(t->frame_g), CS_UPDATE_MAX_DEFECT);
+      t->max_g = t->frame_g;
     }
     t->frame_g = old_g;
+    gravity_constrain_size(t->hints.win_gravity, t, &(t->frame_g), 0);
+
     flags->do_setup_frame = True;
     flags->do_redraw_decoration = True;
   }
@@ -202,15 +247,13 @@ static void apply_window_updates(
   {
     if (IS_SHADED(t))
     {
-      rectangle *new_g;
-
-      new_g = (IS_MAXIMIZED(t)) ? &t->max_g : &t->normal_g;
-      get_relative_geometry(&t->frame_g, new_g);
+      get_unshaded_geometry(t, &t->frame_g);
       get_shaded_geometry(t, &t->frame_g, &t->frame_g);
     }
     ForceSetupFrame(t, t->frame_g.x, t->frame_g.y, t->frame_g.width,
 		    t->frame_g.height, True);
     GNOME_SetWinArea(t);
+    EWMH_SetFrameStrut(t);
   }
   if (flags->do_update_window_color)
   {
@@ -226,15 +269,7 @@ static void apply_window_updates(
   }
   if (flags->do_redraw_decoration)
   {
-    FvwmWindow *u = Scr.Hilite;
-
-    if (IS_SHADED(t))
-      XRaiseWindow(dpy, t->decor_w);
-    /* domivogt (6-Jun-2000): Don't check if the window is visible here.  If we
-     * do, some updates are not applied and when the window becomes visible
-     * again, the X Server may not redraw the window. */
-    DrawDecorations(t, DRAW_ALL, (Scr.Hilite == t), 2, None);
-    Scr.Hilite = u;
+    RedrawDecorations(t);
   }
   if (flags->do_update_icon_font)
   {
@@ -253,18 +288,25 @@ static void apply_window_updates(
   if (flags->do_update_icon)
   {
     change_icon(t, pstyle);
-    if (IS_ICONIFIED(t))
-    {
-      SET_ICONIFIED(t, 0);
-      Iconify(t, 0, 0);
-    }
+    flags->do_update_icon_placement = True;
     flags->do_update_icon_title = False;
+    flags->do_update_ewmh_icon = True;
   }
   if (flags->do_update_icon_title)
   {
     RedoIconName(t);
     if (IS_ICONIFIED(t))
+    {
       DrawIconWindow(t);
+    }
+  }
+  if (flags->do_update_icon_placement)
+  {
+    if (IS_ICONIFIED(t))
+    {
+      SET_ICONIFIED(t, 0);
+      Iconify(t, 0, 0);
+    }
   }
   if (flags->do_setup_focus_policy)
   {
@@ -274,10 +316,78 @@ static void apply_window_updates(
   {
     setup_frame_attributes(t, pstyle);
   }
-  if (flags->do_update_modules_flags) {
+  if (flags->do_update_ewmh_state_hints)
+  {
+     EWMH_SetWMState(t, False);
+  }
+  if (flags->do_update_modules_flags)
+  {
     BroadcastConfig(M_CONFIGURE_WINDOW,t);
   }
+  if (flags->do_update_ewmh_mini_icon || flags->do_update_ewmh_icon)
+  {
+    EWMH_DoUpdateWmIcon(
+      t, flags->do_update_ewmh_mini_icon, flags->do_update_ewmh_icon);
+  }
+  if (flags->do_update_placement_penalty)
+  {
+    setup_placement_penalty(t, pstyle);
+  }
+  if (flags->do_update_working_area)
+  {
+    EWMH_UpdateWorkArea();
+  }
+  if (flags->do_update_ewmh_stacking_hints)
+  {
+    if (DO_EWMH_USE_STACKING_HINTS(t))
+    {
+      if (t->ewmh_hint_layer > 0 && t->layer != t->ewmh_hint_layer)
+	new_layer(t, t->ewmh_hint_layer);
+    }
+    else
+    {
+      if (t->ewmh_hint_layer > 0)
+	new_layer(t, Scr.DefaultLayer);
+    }
+  }
+  if (flags->do_update_ewmh_allowed_actions)
+  {
+    EWMH_SetAllowedActions(t);
+  }
   t->shade_anim_steps = pstyle->shade_anim_steps;
+
+  return;
+}
+
+/* similar, but takes only care of destroying windows that have to go away. */
+void destroy_scheduled_windows(void)
+{
+  FvwmWindow *t;
+  FvwmWindow *next;
+  Bool do_need_ungrab = False;
+
+  if (Scr.flags.is_executing_complex_function != 0 ||
+      Scr.flags.is_window_scheduled_for_destroy == 0)
+    return;
+  /* Grab the server during the style update! */
+  if (GrabEm(CRS_WAIT, GRAB_BUSY))
+    do_need_ungrab = True;
+  MyXGrabServer(dpy);
+  Scr.flags.is_window_scheduled_for_destroy = 0;
+  /* need to destroy one or more windows before looking at the window list */
+  for (t = Scr.FvwmRoot.next, next = NULL; t != NULL; t = next)
+  {
+    next = t->next;
+    if (IS_SCHEDULED_FOR_DESTROY(t))
+    {
+      SET_SCHEDULED_FOR_DESTROY(t, 0);
+      destroy_window(t);
+    }
+  }
+  MyXUngrabServer(dpy);
+  if (do_need_ungrab)
+    UngrabEm(GRAB_BUSY);
+
   return;
 }
 
@@ -292,7 +402,8 @@ void apply_decor_change(FvwmWindow *tmp_win)
   memset(&flags, 0, sizeof(flags));
   flags.do_redecorate = True;
   flags.do_update_window_font_height = True;
-  apply_window_updates(tmp_win, &flags, &style, Scr.Focus);
+  apply_window_updates(tmp_win, &flags, &style, get_focus_window());
+  Scr.flags.do_need_window_update = 1;
 }
 
 /* Check and apply new style to each window if the style has changed. */
@@ -304,17 +415,15 @@ void flush_window_updates(void)
   Bool do_need_ungrab = False;
   update_win flags;
 
-  memset(&flags, 0, sizeof(update_win));
   /* Grab the server during the style update! */
   if (GrabEm(CRS_WAIT, GRAB_BUSY))
     do_need_ungrab = True;
   MyXGrabServer(dpy);
-  XSync(dpy,0);
 
   /* This is necessary in case the focus policy changes. With ClickToFocus some
    * buttons have to be grabbed/ungrabbed. */
-  focus_fw = Scr.Focus;
-  DeleteFocus(1);
+  focus_fw = get_focus_window();
+  DeleteFocus(True, False);
 
   /* Apply the new default font and colours first */
   if (Scr.flags.has_default_color_changed || Scr.flags.has_default_font_changed)
@@ -323,7 +432,13 @@ void flush_window_updates(void)
   /* update styles for all windows */
   for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
   {
+    memset(&flags, 0, sizeof(update_win));
     check_window_style_change(t, &flags, &style);
+    if (Scr.flags.has_xinerama_state_changed)
+    {
+      flags.do_update_icon_boxes = True;
+      flags.do_update_icon_placement = True;
+    }
     if (Scr.flags.has_nr_buttons_changed)
       flags.do_redecorate = True;
     /* TODO: this is not optimised for minimal redrawing yet*/
@@ -356,14 +471,16 @@ void flush_window_updates(void)
    * window is now NeverFocus */
   if (focus_fw)
   {
-    SetFocusWindow(focus_fw, 1);
+    SetFocusWindow(focus_fw, True, False);
     if (Scr.flags.has_mouse_binding_changed)
     {
       focus_grab_buttons(focus_fw, True);
     }
   }
   else
-    DeleteFocus(1);
+  {
+    DeleteFocus(True, True);
+  }
 
   /* finally clean up the change flags */
   reset_style_changes();
@@ -373,16 +490,16 @@ void flush_window_updates(void)
   Scr.flags.has_default_color_changed = 0;
   Scr.flags.has_mouse_binding_changed = 0;
   Scr.flags.has_nr_buttons_changed = 0;
+  Scr.flags.has_xinerama_state_changed = 0;
 
   MyXUngrabServer(dpy);
   if (do_need_ungrab)
     UngrabEm(GRAB_BUSY);
-  XSync(dpy, 0);
 
   return;
 }
 
-void update_styles_func(F_CMD_ARGS)
+void CMD_UpdateStyles(F_CMD_ARGS)
 {
   if (Scr.flags.do_need_window_update)
   {
