@@ -32,8 +32,10 @@
 #include "module_interface.h"
 #include "focus.h"
 #include "gnome.h"
+#include "ewmh.h"
 #include "move_resize.h"
 #include "borders.h"
+#include "geometry.h"
 #include "icons.h"
 #include "stack.h"
 
@@ -84,7 +86,7 @@ static unsigned int prev_desk_and_page_page_y = 0;
  * read (or if action is empty).
  *
  **************************************************************************/
-static int GetDeskNumber(char *action, int current_desk)
+static int GetDeskNumber(char *action)
 {
   int n;
   int m;
@@ -99,11 +101,11 @@ static int GetDeskNumber(char *action, int current_desk)
 
   n = GetIntegerArguments(action, NULL, &(val[0]), 4);
   if (n <= 0)
-    return current_desk;
+    return Scr.CurrentDesk;
   if (n == 1)
-    return current_desk + val[0];
+    return Scr.CurrentDesk + val[0];
 
-  desk = current_desk;
+  desk = Scr.CurrentDesk;
   m = 0;
 
   if (val[0] == 0)
@@ -189,8 +191,8 @@ static void unmap_window(FvwmWindow *t)
   {
     if (t->icon_pixmap_w != None)
       XUnmapWindow(dpy,t->icon_pixmap_w);
-    if (t->icon_w != None)
-      XUnmapWindow(dpy,t->icon_w);
+    if (t->icon_title_w != None)
+      XUnmapWindow(dpy,t->icon_title_w);
   }
   else
   {
@@ -199,7 +201,8 @@ static void unmap_window(FvwmWindow *t)
     /* this is required by the ICCCM2 */
     XUnmapWindow(dpy, t->w);
 #endif
-    SetMapStateProp(t, IconicState);
+    if (!Scr.bo.EWMHIconicStateWorkaround)
+      SetMapStateProp(t, IconicState);
   }
   if (ret)
   {
@@ -239,8 +242,8 @@ static void map_window(FvwmWindow *t)
   {
     if(t->icon_pixmap_w != None)
       XMapWindow(dpy,t->icon_pixmap_w);
-    if(t->icon_w != None)
-      XMapWindow(dpy,t->icon_w);
+    if(t->icon_title_w != None)
+      XMapWindow(dpy,t->icon_title_w);
   }
   else if(IS_MAPPED(t))
   {
@@ -251,7 +254,8 @@ static void map_window(FvwmWindow *t)
     /* this is required by the ICCCM2 */
     XMapWindow(dpy, t->w);
 #endif
-    SetMapStateProp(t, NormalState);
+    if (!Scr.bo.EWMHIconicStateWorkaround)
+      SetMapStateProp(t, NormalState);
   }
   if (ret)
   {
@@ -442,16 +446,10 @@ void CMD_DesktopSize(F_CMD_ARGS)
  *
  * Check to see if the pointer is on the edge of the screen, and scroll/page
  * if needed
- *
- * Returns
- *  0: no paging
- *  1: paging occured
- * -1: no need to call the function again before a new event arrives
  ***************************************************************************/
-int HandlePaging(
-	int HorWarpSize, int VertWarpSize, int *xl, int *yt,
-	int *delta_x, int *delta_y, Bool Grab, Bool fLoop,
-	Bool do_continue_previous)
+Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
+		  int *delta_x, int *delta_y, Bool Grab, Bool fLoop,
+		  Bool do_continue_previous)
 {
   static unsigned int add_time = 0;
   int x,y;
@@ -465,38 +463,30 @@ int HandlePaging(
 
   *delta_x = 0;
   *delta_y = 0;
+
+  /* Only continue to count the time, but do not start counting from
+   * scratch. */
   if (!is_timestamp_valid && do_continue_previous)
   {
-    /* don't call me again until something has happened */
-    return -1;
-  }
-  if (!is_timestamp_valid && Event.type == MotionNotify)
-  {
-    x = Event.xmotion.x_root;
-    y = Event.xmotion.y_root;
-    /* need to move the viewport */
-    if ((Scr.VxMax == 0 ||
-	 (x >= edge_thickness &&
-	  x < Scr.MyDisplayWidth  - edge_thickness)) &&
-	(Scr.VyMax == 0 ||
-	 (y >= edge_thickness &&
-	  y < Scr.MyDisplayHeight - edge_thickness)))
-    {
-      return -1;
-    }
+    /* Free some CPU */
+    usleep(10000);
+    return False;
   }
+
   if (Scr.ScrollResistance >= 10000 || (HorWarpSize == 0 && VertWarpSize==0))
   {
     is_timestamp_valid = False;
     add_time = 0;
-    return 0;
+    return False;
   }
+
   if (Scr.VxMax == 0 && Scr.VyMax == 0)
   {
     is_timestamp_valid = False;
     add_time = 0;
-    return 0;
+    return False;
   }
+
   if (!is_timestamp_valid)
   {
     is_timestamp_valid = True;
@@ -527,14 +517,14 @@ int HandlePaging(
       StashEventTime(&Event);
       is_timestamp_valid = False;
       add_time = 0;
-      return 0;
+      return False;
     }
     else if (XCheckMaskEvent(dpy, ButtonPressMask | ButtonReleaseMask, &e))
     {
       XPutBackEvent(dpy, &e);
       is_timestamp_valid = False;
       add_time = 0;
-      return 0;
+      return False;
     }
     /* get pointer location */
     GetLocationFromEventOrQuery(dpy, Scr.Root, &Event, &x, &y);
@@ -549,7 +539,7 @@ int HandlePaging(
       discard_window_events(Scr.PanFrameBottom.win, LeaveWindowMask);
       discard_window_events(Scr.PanFrameLeft.win, LeaveWindowMask);
       discard_window_events(Scr.PanFrameRight.win, LeaveWindowMask);
-      return 0;
+      return False;
     }
     if (!fLoop && is_last_position_valid &&
 	(x - last_x > MAX_PAGING_MOVE_DISTANCE ||
@@ -565,7 +555,7 @@ int HandlePaging(
       add_time = 0;
       last_x = x;
       last_y = y;
-      return 0;
+      return False;
     }
     last_x = x;
     last_y = y;
@@ -577,7 +567,7 @@ int HandlePaging(
 
   if (lastTimestamp - my_timestamp + add_time < Scr.ScrollResistance)
   {
-    return 0;
+    return False;
   }
 
   /* Move the viewport */
@@ -672,7 +662,7 @@ int HandlePaging(
   add_time = 0;
   if (*delta_x == 0 && *delta_y == 0)
   {
-    return 0;
+    return False;
   }
 
   if(Grab)
@@ -681,12 +671,18 @@ int HandlePaging(
   switch_move_resize_grid(False);
   XWarpPointer(dpy,None,Scr.Root,0,0,0,0,*xl,*yt);
   MoveViewport(Scr.Vx + *delta_x,Scr.Vy + *delta_y,False);
-  XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
-		xl, yt, &JunkX, &JunkY, &JunkMask);
+  if (XQueryPointer(
+	dpy, Scr.Root, &JunkRoot, &JunkChild, xl, yt, &JunkX, &JunkY,
+	&JunkMask) == False)
+  {
+    /* pointer is on a different screen */
+    *xl = 0;
+    *yt = 0;
+  }
   if(Grab)
     MyXUngrabServer(dpy);
 
-  return 1;
+  return True;
 }
 
 
@@ -979,11 +975,11 @@ void MoveViewport(int newx, int newy, Bool grab)
   }
   Scr.Vx = newx;
   Scr.Vy = newy;
+  BroadcastPacket(
+    M_NEW_PAGE, 5, Scr.Vx, Scr.Vy, Scr.CurrentDesk, Scr.VxMax, Scr.VyMax);
 
   if (deltax || deltay)
   {
-     BroadcastPacket(
-       M_NEW_PAGE, 5, Scr.Vx, Scr.Vy, Scr.CurrentDesk, Scr.VxMax, Scr.VyMax);
 
     /*
      * RBW - 11/13/1998	 - new:	 chase the chain bidirectionally, all at once!
@@ -1028,24 +1024,9 @@ void MoveViewport(int newx, int newy, Bool grab)
 	{
 	  if (IS_ICONIFIED(t))
 	  {
-	    t->icon_g.x += deltax;
-	    t->icon_xl_loc += deltax;
-	    t->icon_g.y += deltay;
-	    if(t->icon_pixmap_w != None)
-	      XMoveWindow(dpy,t->icon_pixmap_w,t->icon_g.x,
-			  t->icon_g.y);
-	    if(t->icon_w != None)
-	      XMoveWindow(dpy,t->icon_w,t->icon_g.x,
-			  t->icon_g.y+t->icon_p_height);
-	    if(!(IS_ICON_UNMAPPED(t)))
-	    {
-	      BroadcastPacket(M_ICON_LOCATION, 7,
-			      t->w, t->frame,
-			      (unsigned long)t,
-			      t->icon_g.x, t->icon_g.y,
-			      t->icon_p_width,
-			      t->icon_g.height+t->icon_p_height);
-	    }
+	    modify_icon_position(t, deltax, deltay);
+	    move_icon_to_position(t);
+	    broadcast_icon_geometry(t, False);
 	  }
 	  SetupFrame(t, t->frame_g.x+ deltax, t->frame_g.y + deltay,
 		     t->frame_g.width, t->frame_g.height, False);
@@ -1076,30 +1057,13 @@ void MoveViewport(int newx, int newy, Bool grab)
 	{
 	  if (IS_ICONIFIED(t1))
 	  {
-	    t1->icon_g.x += deltax;
-	    t1->icon_xl_loc += deltax;
-	    t1->icon_g.y += deltay;
-	    if(t1->icon_pixmap_w != None)
-	      XMoveWindow(dpy,t1->icon_pixmap_w,
-			  t1->icon_g.x,
-			  t1->icon_g.y);
-	    if(t1->icon_w != None)
-	      XMoveWindow(dpy,t1->icon_w,t1->icon_g.x,
-			  t1->icon_g.y+t1->icon_p_height);
-	    if(!IS_ICON_UNMAPPED(t1))
-	    {
-	      BroadcastPacket(M_ICON_LOCATION, 7,
-			      t1->w, t1->frame,
-			      (unsigned long)t1,
-			      t1->icon_g.x, t1->icon_g.y,
-			      t1->icon_p_width,
-			      t1->icon_g.height +
-			      t1->icon_p_height);
-	    }
+	    modify_icon_position(t1, deltax, deltay);
+	    move_icon_to_position(t1);
+	    broadcast_icon_geometry(t1, False);
 	  }
-	  SetupFrame(t1, t1->frame_g.x+ deltax,
-		     t1->frame_g.y + deltay, t1->frame_g.width,
-		     t1->frame_g.height, False);
+	  SetupFrame(
+	    t1, t1->frame_g.x + deltax, t1->frame_g.y + deltay,
+	    t1->frame_g.width, t1->frame_g.height, False);
 	}
       }
       /*  Bump to next win...	 */
@@ -1167,15 +1131,13 @@ static void UnmapDesk(int desk, Bool grab)
 	{
 	  t->flags.is_focused_on_other_desk = 1;
 	  t->FocusDesk = desk;
-	  DeleteFocus(1);
+	  DeleteFocus(True, True);
 	}
 	else
 	{
 	  t->flags.is_focused_on_other_desk = 0;
 	}
 	unmap_window(t);
-	SET_FULLY_VISIBLE(t, 0);
-	SET_PARTIALLY_VISIBLE(t, 0);
       }
     }
     else
@@ -1269,7 +1231,7 @@ static void MapDesk(int desk, Bool grab)
     }
     else if (FocusWin && !HAS_NEVER_FOCUS(FocusWin))
     {
-      DeleteFocus(1);
+      DeleteFocus(True, True);
     }
   }
   Scr.flags.is_map_desk_in_progress = 0;
@@ -1285,7 +1247,7 @@ static void MapDesk(int desk, Bool grab)
  *************************************************************************/
 void CMD_GotoDesk(F_CMD_ARGS)
 {
-  goto_desk(GetDeskNumber(action, Scr.CurrentDesk));
+  goto_desk(GetDeskNumber(action));
 }
 
 void CMD_Desk(F_CMD_ARGS)
@@ -1318,6 +1280,7 @@ void goto_desk(int desk)
      * This should be fixed in the pager, but right now the pager doesn't
      * maintain the stacking order. */
     BroadcastRestackAllWindows();
+    EWMH_SetCurrentDesktop();
     GNOME_SetCurrentDesk();
     GNOME_SetDeskCount();
   }
@@ -1347,12 +1310,7 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
     val[1] = prev_desk_and_page_page_x;
     val[2] = prev_desk_and_page_page_y;
   }
-  else if (GetIntegerArguments(action, NULL, val, 3) == 3)
-  {
-    val[1] *= Scr.MyDisplayWidth;
-    val[2] *= Scr.MyDisplayHeight;
-  }
-  else
+  else if (GetIntegerArguments(action, NULL, val, 3) != 3)
   {
     return;
   }
@@ -1364,7 +1322,7 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
   }
   prev_desk_and_page_page_x = Scr.Vx;
   prev_desk_and_page_page_y = Scr.Vy;
-  MoveViewport(val[1], val[2], True);
+  MoveViewport(val[1] * Scr.MyDisplayWidth, val[2] * Scr.MyDisplayHeight, True);
   if (is_new_desk)
   {
     prev_desk = Scr.CurrentDesk;
@@ -1387,6 +1345,7 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
     BroadcastPacket(M_NEW_DESK, 1, Scr.CurrentDesk);
   }
 
+  EWMH_SetCurrentDesktop();
   GNOME_SetCurrentDesk();
   GNOME_SetDeskCount();
 
@@ -1431,11 +1390,9 @@ void do_move_window_to_desk(FvwmWindow *tmp_win, int desk)
       tmp_win->Desk = desk;
       if (tmp_win == get_focus_window())
       {
-	DeleteFocus(0);
+	DeleteFocus(False, True);
       }
       unmap_window(tmp_win);
-      SET_FULLY_VISIBLE(tmp_win, 0);
-      SET_PARTIALLY_VISIBLE(tmp_win, 0);
     }
     else if(desk == Scr.CurrentDesk)
     {
@@ -1451,6 +1408,7 @@ void do_move_window_to_desk(FvwmWindow *tmp_win, int desk)
     }
     BroadcastConfig(M_CONFIGURE_WINDOW,tmp_win);
   }
+  EWMH_SetWMDesktop(tmp_win);
   GNOME_SetDeskCount();
   GNOME_SetDesk(tmp_win);
   GNOME_SetWinArea(tmp_win);
@@ -1463,7 +1421,7 @@ void CMD_MoveToDesk(F_CMD_ARGS)
 
   if (DeferExecution(eventp,&w,&tmp_win,&context,CRS_SELECT,ButtonRelease))
     return;
-  desk = GetDeskNumber(action, tmp_win->Desk);
+  desk = GetDeskNumber(action);
   if (desk == tmp_win->Desk)
     return;
   do_move_window_to_desk(tmp_win, desk);
@@ -1603,3 +1561,106 @@ Bool get_page_arguments(char *action, int *page_x, int *page_y)
 
   return True;
 }
+
+/**************************************************************************
+ *
+ * Defines the name of a desktop
+ *
+ *************************************************************************/
+void CMD_DesktopName(F_CMD_ARGS)
+{
+  int desk;
+  DesktopsInfo *t, *d, *new, **prev;
+
+  if (GetIntegerArguments(action, &action, &desk, 1) != 1)
+  {
+    fvwm_msg(ERR,"CMD_DesktopName",
+	     "First argument to DesktopName must be an integer: %s", action);
+    return;
+  }
+
+  d = Scr.Desktops->next;
+  while(d != NULL && d->desk != desk)
+  {
+    d = d->next;
+  }
+
+  if (d != NULL)
+  {
+    if (d->name != NULL)
+    {
+      free(d->name);
+      d->name = NULL;
+    }
+    if (action != NULL && *action && *action != '\n')
+      CopyString(&d->name, action);
+  }
+  else
+  {
+    /* new deskops entries: add it in order */
+    d = Scr.Desktops->next;
+    t = Scr.Desktops;
+    prev = &(Scr.Desktops->next);
+    while (d != NULL && d->desk < desk)
+    {
+      t = t->next;
+      prev = &(d->next);
+      d = d->next;
+    }
+    if (d == NULL)
+    {
+      /* add it at the end */
+      *prev = (DesktopsInfo *)safemalloc(sizeof(DesktopsInfo));
+      memset(*prev, 0, sizeof(DesktopsInfo));
+      (*prev)->desk = desk;
+      if (action != NULL && *action && *action != '\n')
+	CopyString(&((*prev)->name), action);
+    }
+    else
+    {
+      /* instert it */
+      new = (DesktopsInfo *)safemalloc(sizeof(DesktopsInfo));
+      memset(new, 0, sizeof(DesktopsInfo));
+      new->desk = desk;
+      if (action != NULL && *action && *action != '\n')
+	CopyString(&(new->name), action);
+      t->next = new;
+      new->next = d;
+     }
+    /* should check/set the working areas */
+  }
+
+  if (!fFvwmInStartup)
+  {
+    char *msg;
+    /* should send the info to the FvwmPager and set the EWMH desktop names */
+    if (action != NULL && *action && *action != '\n')
+    {
+      msg = (char *)safemalloc(strlen(action) + 44);
+      sprintf(msg, "DesktopName %d %s", desk, action);
+    }
+    else
+    {
+      msg = (char *)safemalloc(84);
+      sprintf(msg, "DesktopName %d Desk %d", desk, desk);
+    }
+    BroadcastConfigInfoString(msg);
+    free(msg);
+    EWMH_SetDesktopNames();
+  }
+}
+
+char *GetDesktopName(int desk)
+{
+  DesktopsInfo *d;
+
+  d = Scr.Desktops->next;
+  while(d != NULL && d->desk != desk)
+  {
+    d = d->next;
+  }
+  if (d == NULL)
+    return NULL;
+  else
+    return d->name;
+}
