@@ -16,7 +16,7 @@
 #ifndef _VIRTUAL_
 #define _VIRTUAL_
 
-int HandlePaging(int, int, int *, int *, int *, int *, Bool, Bool, Bool);
+Bool HandlePaging(int, int, int *, int *, int *, int *, Bool, Bool, Bool);
 void checkPanFrames(void);
 void raisePanFrames(void);
 void initPanFrames(void);
@@ -24,5 +24,6 @@ void MoveViewport(int newx, int newy,Bool);
 void goto_desk(int desk);
 void do_move_window_to_desk(FvwmWindow *tmp_win, int desk);
 Bool get_page_arguments(char *action, int *page_x, int *page_y);
+char *GetDesktopName(int desk);
 
 #endif /* _VIRTUAL_ */
