@@ -40,6 +40,12 @@
           (fw)->flags.common.s.do_not_raise_click_focus_click = !!(x)
 #define SETM_DO_NOT_RAISE_CLICK_FOCUS_CLICK(fw,x) \
           (fw)->flag_mask.common.s.do_not_raise_click_focus_click = !!(x)
+#define DO_IGNORE_MOUSE_FOCUS_CLICK_MOTION(fw)  \
+          ((fw)->flags.common.s.do_ignore_mouse_focus_click_motion)
+#define SET_DO_IGNORE_MOUSE_FOCUS_CLICK_MOTION(fw,x) \
+          (fw)->flags.common.s.do_ignore_mouse_focus_click_motion = !!(x)
+#define SETM_DO_IGNORE_MOUSE_FOCUS_CLICK_MOTION(fw,x) \
+          (fw)->flag_mask.common.s.do_ignore_mouse_focus_click_motion = !!(x)
 #define DO_RAISE_MOUSE_FOCUS_CLICK(fw)  \
           ((fw)->flags.common.s.do_raise_mouse_focus_click)
 #define SET_DO_RAISE_MOUSE_FOCUS_CLICK(fw,x) \
@@ -113,6 +119,60 @@
           (fw)->flags.common.s.use_icon_position_hint = !!(x)
 #define SETM_USE_ICON_POSITION_HINT(fw,x) \
           (fw)->flag_mask.common.s.use_icon_position_hint = !!(x)
+#define USE_INDEXED_WINDOW_NAME(fw) \
+          ((fw)->flags.common.s.use_indexed_window_name)
+#define SET_USE_INDEXED_WINDOW_NAME(fw,x) \
+          (fw)->flags.common.s.use_indexed_window_name = !!(x)
+#define SETM_USE_INDEXED_WINDOW_NAME(fw,x) \
+          (fw)->flag_mask.common.s.use_indexed_window_name = !!(x)
+#define USE_INDEXED_ICON_NAME(fw) \
+          ((fw)->flags.common.s.use_indexed_icon_name)
+#define SET_USE_INDEXED_ICON_NAME(fw,x) \
+          (fw)->flags.common.s.use_indexed_icon_name = !!(x)
+#define SETM_USE_INDEXED_ICON_NAME(fw,x) \
+          (fw)->flag_mask.common.s.use_indexed_icon_name = !!(x)
+#define DO_EWMH_MINI_ICON_OVERRIDE(fw) \
+          ((fw)->flags.common.s.do_ewmh_mini_icon_override)
+#define SET_DO_EWMH_MINI_ICON_OVERRIDE(fw,x) \
+          (fw)->flags.common.s.do_ewmh_mini_icon_override = !!(x)
+#define SETM_DO_EWMH_MINI_ICON_OVERRIDE(fw,x) \
+          (fw)->flag_mask.common.s.do_ewmh_mini_icon_override = !!(x)
+#define DO_EWMH_DONATE_ICON(fw) \
+          ((fw)->flags.common.s.do_ewmh_donate_icon)
+#define SET_DO_EWMH_DONATE_ICON(fw,x) \
+          (fw)->flags.common.s.do_ewmh_donate_icon = !!(x)
+#define SETM_DO_EWMH_DONATE_ICON(fw,x) \
+          (fw)->flag_mask.common.s.do_ewmh_donate_icon = !!(x)
+#define DO_EWMH_DONATE_MINI_ICON(fw) \
+          ((fw)->flags.common.s.do_ewmh_donate_mini_icon)
+#define SET_DO_EWMH_DONATE_MINI_ICON(fw,x) \
+          (fw)->flags.common.s.do_ewmh_donate_mini_icon = !!(x)
+#define SETM_DO_EWMH_DONATE_MINI_ICON(fw,x) \
+          (fw)->flag_mask.common.s.do_ewmh_donate_mini_icon = !!(x)
+#define DO_EWMH_USE_STACKING_HINTS(fw) \
+          ((fw)->flags.common.s.do_ewmh_use_stacking_hints)
+#define SET_DO_EWMH_USE_STACKING_HINTS(fw,x) \
+          (fw)->flags.common.s.do_ewmh_use_stacking_hints = !!(x)
+#define SETM_DO_EWMH_USE_STACKING_HINTS(fw,x) \
+          (fw)->flag_mask.common.s.do_ewmh_use_stacking_hints = !!(x)
+#define DO_EWMH_IGNORE_STRUT_HINTS(fw) \
+          ((fw)->flags.common.s.do_ewmh_ignore_strut_hints)
+#define SET_DO_EWMH_IGNORE_STRUT_HINTS(fw,x) \
+          (fw)->flags.common.s.do_ewmh_ignore_strut_hints = !!(x)
+#define SETM_DO_EWMH_IGNORE_STRUT_HINTS(fw,x) \
+          (fw)->flag_mask.common.s.do_ewmh_ignore_strut_hints = !!(x)
+#define DO_EWMH_IGNORE_STATE_HINTS(fw) \
+          ((fw)->flags.common.s.do_ewmh_ignore_state_hints)
+#define SET_DO_EWMH_IGNORE_STATE_HINTS(fw,x) \
+          (fw)->flags.common.s.do_ewmh_ignore_state_hints = !!(x)
+#define SETM_DO_EWMH_IGNORE_STATE_HINTS(fw,x) \
+          (fw)->flag_mask.common.s.do_ewmh_ignore_state_hints = !!(x)
+#define EWMH_MAXIMIZE_MODE(fw) \
+          ((fw)->flags.common.s.ewmh_maximize_mode)
+#define SET_EWMH_MAXIMIZE_MODE(fw,x) \
+          (fw)->flags.common.s.ewmh_maximize_mode = (x)
+#define SETM_EWMH_MAXIMIZE_MODE(fw,x) \
+          (fw)->flag_mask.common.s.ewmh_maximize_mode = (x)
 #define IS_ICON_SUPPRESSED(fw) \
           ((fw)->flags.common.s.is_icon_suppressed)
 #define SET_ICON_SUPPRESSED(fw,x)  \
@@ -173,6 +233,17 @@
 #define IS_FIXED(fw)           ((fw)->flags.common.s.is_fixed)
 #define SET_FIXED(fw,x)        (fw)->flags.common.s.is_fixed = !!(x)
 #define SETM_FIXED(fw,x)       (fw)->flag_mask.common.s.is_fixed = !!(x)
+#define IS_FIXED_PPOS(fw)      ((fw)->flags.common.s.is_fixed_ppos)
+#define SET_FIXED_PPOS(fw,x)   (fw)->flags.common.s.is_fixed_ppos = !!(x)
+#define SETM_FIXED_PPOS(fw,x)  (fw)->flag_mask.common.s.is_fixed_ppos = !!(x)
+#define IS_SIZE_FIXED(fw)      ((fw)->flags.common.s.is_size_fixed)
+#define SET_SIZE_FIXED(fw,x)   (fw)->flags.common.s.is_size_fixed = !!(x)
+#define SETM_SIZE_FIXED(fw,x)  (fw)->flag_mask.common.s.is_size_fixed = !!(x)
+#define IS_PSIZE_FIXED(fw) ((fw)->flags.common.s.is_psize_fixed)
+#define SET_PSIZE_FIXED(fw,x) \
+                               (fw)->flags.common.s.is_psize_fixed = !!(x)
+#define SETM_PSIZE_FIXED(fw,x) \
+                            (fw)->flag_mask.common.s.is_psize_fixed = !!(x)
 #define HAS_DEPRESSABLE_BORDER(fw) ((fw)->flags.common.s.has_depressable_border)
 
 
@@ -356,6 +427,86 @@
           (fw)->flags.was_icon_name_provided = (x)
 #define SETM_WAS_ICON_NAME_PROVIDED(fw,x) \
           (fw)->flag_mask.was_icon_name_provided = (x)
-
-
+#define HAS_EWMH_WM_NAME(fw)        ((fw)->flags.has_ewmh_wm_name)
+#define SET_HAS_EWMH_WM_NAME(fw,x)  (fw)->flags.has_ewmh_wm_name = !!(x)
+#define SETM_HAS_EWMH_WM_NAME(fw,x) (fw)->flag_mask.has_ewmh_wm_name = !!(x)
+#define HAS_EWMH_WM_ICON_NAME(fw) \
+          ((fw)->flags.has_ewmh_wm_icon_name)
+#define SET_HAS_EWMH_WM_ICON_NAME(fw,x) \
+          (fw)->flags.has_ewmh_wm_icon_name = !!(x)
+#define SETM_HAS_EWMH_WM_ICON_NAME(fw,x) \
+          (fw)->flag_mask.has_ewmh_wm_icon_name = !!(x)
+#define HAS_EWMH_WM_ICON_HINT(fw) ((fw)->flags.has_ewmh_wm_icon_hint)
+#define SET_HAS_EWMH_WM_ICON_HINT(fw,x)   (fw)->flags.has_ewmh_wm_icon_hint = (x)
+#define SETM_HAS_EWMH_WM_ICON_HINT(fw,x)  (fw)->flag_mask.has_ewmh_wm_icon_hint = (x)
+#define USE_EWMH_ICON(fw) ((fw)->flags.use_ewmh_icon)
+#define SET_USE_EWMH_ICON(fw,x) (fw)->flags.use_ewmh_icon = !!(x)
+#define SETM_USE_EWMH_ICON(fw,x) (fw)->flag_mask.use_ewmh_icon = !!(x)
+#define HAS_EWMH_MINI_ICON(fw) ((fw)->flags.has_ewmh_mini_icon)
+#define SET_HAS_EWMH_MINI_ICON(fw,x) (fw)->flags.has_ewmh_mini_icon = !!(x)
+#define SETM_HAS_EWMH_MINI_ICON(fw,x) (fw)->flag_mask.has_ewmh_mini_icon = !!(x)
+#define IS_EWMH_MODAL(fw)     ((fw)->flags.is_ewmh_modal)
+#define SET_EWMH_MODAL(fw,x)  (fw)->flags.is_ewmh_modal = !!(x)
+#define SETM_EWMH_MODAL(fw,x) \
+          (fw)->flag_mask.is_ewmh_modal = !!(x)
+#define SET_HAS_EWMH_INIT_FULLSCREEN_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_fullscreen_state = (x)
+#define SETM_HAS_EWMH_INIT_FULLSCREEN_STATE(fw,x) \
+          (fw)->flags_mask.has_ewmh_init_fullscreen_state = (x)
+#define HAS_EWMH_INIT_FULLSCREEN_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_fullscreen_state)
+#define SET_HAS_EWMH_INIT_HIDDEN_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_hidden_state = (x)
+#define SETM_HAS_EWMH_INIT_HIDDEN_STATE(fw,x) \
+          (fw)->flags_mask.has_ewmh_init_hidden_state = (x)
+#define HAS_EWMH_INIT_HIDDEN_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_hidden_state)
+#define SET_HAS_EWMH_INIT_MAXHORIZ_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_maxhoriz_state = (x)
+#define SETM_HAS_EWMH_INIT_MAXHORIZ_STATE(fw,x) \
+          (fw)->flags_mask.has_ewmh_init_maxhoriz_state = (x)
+#define HAS_EWMH_INIT_MAXHORIZ_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_maxhoriz_state)
+#define SET_HAS_EWMH_INIT_MAXVERT_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_maxvert_state = (x)
+#define SETM_HAS_EWMH_INIT_MAXVERT_STATE(fw,x) \
+          (fw)->flags_mask.has_ewmh_init_maxvert_state = (x)
+#define HAS_EWMH_INIT_MAXVERT_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_fullscreen_state)
+#define SET_HAS_EWMH_INIT_MODAL_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_modal_state = (x)
+#define SETM_HAS_EWMH_INIT_MODAL_STATE(fw,x) \
+          (fw)->flags_mask.has_ewmh_init_modal_state = (x)
+#define HAS_EWMH_INIT_MODAL_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_modal_state)
+#define SET_HAS_EWMH_INIT_SHADED_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_shaded_state = (x)
+#define SETM_HAS_EWMH_INIT_SHADED_STATE(fw,x) \
+          (fw)->flags_mask.has_ewmh_init_shaded_state = (x)
+#define HAS_EWMH_INIT_SHADED_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_shaded_state)
+#define SET_HAS_EWMH_INIT_SKIP_PAGER_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_skip_pager_state = (x)
+#define SETM_HAS_EWMH_INIT_SKIP_PAGER_STATE(fw,x) \
+          (fw)->flags_mask.has_ewmh_init_skip_pager_state = (x)
+#define HAS_EWMH_INIT_SKIP_PAGER_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_skip_pager_state)
+#define SET_HAS_EWMH_INIT_SKIP_TASKBAR_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_skip_taskbar_state = (x)
+#define SETM_HAS_EWMH_INIT_SKIP_TASKBAR_STATE(fw,x) \
+          (fw)->flags_mask.has_ewmh_init_skip_taskbar_state = (x)
+#define HAS_EWMH_INIT_SKIP_TASKBAR_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_skip_taskbar_state)
+#define SET_HAS_EWMH_INIT_STICKY_STATE(fw,x) \
+          (fw)->flags.has_ewmh_init_sticky_state = (x)
+#define SETM_HAS_EWMH_INIT_STICKY_STATE(fw,x) \
+          (fw)->flags_mask.has_ewmh_init_sticky_state = (x)
+#define HAS_EWMH_INIT_STICKY_STATE(fw) \
+          ((fw)->flags.has_ewmh_init_sticky_state)
+#define SET_HAS_EWMH_INIT_WM_DESKTOP(fw,x) \
+          (fw)->flags.has_ewmh_init_wm_desktop = (x)
+#define SETM_HAS_EWMH_INIT_WM_DESKTOP(fw,x) \
+          (fw)->flags_mask.has_ewmh_init_wm_desktop = (x)
+#define HAS_EWMH_INIT_WM_DESKTOP(fw) \
+          ((fw)->flags.has_ewmh_init_wm_desktop)
 #endif /* _WINDOW_FLAGS_ */
