@@ -39,6 +39,7 @@ static Bool is_grabbing_everything = False;
 static struct charstring win_contexts[]=
 {
   {'w',C_WINDOW},
+  {'d',C_EWMH_DESKTOP},
   {'t',C_TITLE},
   {'i',C_ICON},
   {'r',C_ROOT},
@@ -75,22 +76,6 @@ static struct charstring key_modifiers[]=
   {0,0}
 };
 
-/* Table to translate a modifier map index to a modifier that we define that
- * generates that index.  This mapping can be chosen by each client, but the
- * settings below try to emulate the usual terminal behaviour. */
-static unsigned int modifier_mapindex_to_mask[8] =
-{
-	ShiftMask,
-	Mod3Mask, /* Alt Gr */
-	Mod3Mask | ShiftMask,
-	/* Just guessing below here - LockMask is not used anywhere*/
-	ControlMask,
-	Mod1Mask, /* Alt/Meta on XFree86 */
-	Mod2Mask, /* Num lock on XFree86 */
-	Mod4Mask,
-	Mod5Mask, /* Scroll lock on XFree86 */
-};
-
 static int key_min;
 static int key_max;
 
@@ -278,7 +263,7 @@ Bool MatchBinding(Display *dpy, Binding *b,
   static int kc_maxlen = 0;
   static KeySym last_keysym = NoSymbol;
 
-  modifier &= (used_modifiers & ALL_MODIFIERS);
+  modifier &= used_modifiers;
   if (((b->Modifier & used_modifiers) != modifier &&
        b->Modifier != AnyModifier) ||
       (b->Context & Context) == 0 ||
@@ -365,7 +350,6 @@ int AddBinding(
   int max;
   int maxmods;
   int m;
-  int mask;
   int count = 0;
   KeySym tkeysym;
   Binding *temp;
@@ -422,19 +406,20 @@ int AddBinding(
 	    check_bound_mask = 1;
 	  }
 	  break;
-	default:
-	  /* key generates the key sym with undefined modifiers depressed -
-	   * let's make an educated guess at what modifiers the user expected
-	   * based on the XFree86 default configuration. */
-	  mask = modifier_mapindex_to_mask[m - 1];
-	  if (modifiers != AnyModifier && !(modifiers & mask) != mask)
+	case 2:
+	  /* key generates the key sym with caps-lock depressed */
+	  if (modifiers != AnyModifier && !(modifiers & LockMask))
 	  {
-	    add_modifiers = mask;
-	    bind_mask = (1 << m);
+	    add_modifiers = LockMask;
+	    bind_mask = 4;
 	    /* but don't bind it again if already bound without modifiers */
 	    check_bound_mask = 1;
 	  }
 	  break;
+	default:
+	  /* key generates the key sym with unknown modifiers depressed -
+	  * can't map that to specific modifiers - trat as no modifiers */
+	  break;
 	}
 	if ((bind_mask & bound_mask) || (check_bound_mask & bound_mask))
 	{
@@ -566,7 +551,7 @@ void *CheckBinding(Binding *blist,
   Binding *b;
   unsigned int used_modifiers = ~dead_modifiers;
 
-  modifier &= (used_modifiers & ALL_MODIFIERS);
+  modifier &= used_modifiers;
 
   for (b = blist; b != NULL; b = b->NextBinding)
   {
@@ -595,7 +580,6 @@ Bool MatchBindingExactly(
   int button, KeyCode keycode, unsigned int modifier, int Context,
   BindingType type)
 {
-  modifier &= ALL_MODIFIERS;
   if (b->type == type)
   {
     if (
@@ -642,7 +626,6 @@ void GrabWindowKey(Display *dpy, Window w, Binding *binding,
 {
   /* remove unnecessary bits from dead_modifiers */
   dead_modifiers &= ~(binding->Modifier & dead_modifiers);
-  dead_modifiers &= ALL_MODIFIERS;
 
   if((binding->Context & contexts) && (binding->type == KEY_BINDING))
   {
@@ -701,7 +684,6 @@ void GrabWindowButton(Display *dpy, Window w, Binding *binding,
     return;
 
   dead_modifiers &= ~(binding->Modifier & dead_modifiers); /* dje */
-  dead_modifiers &= ALL_MODIFIERS;
 
   if ((binding->Context & contexts) &&
       ((binding->type == MOUSE_BINDING)
