@@ -184,6 +184,7 @@ Picture *LoadPicture(Display *dpy, Window Root, char *path, int color_limit)
 #endif
 
   p = (Picture*)safemalloc(sizeof(Picture));
+  memset(p, 0, sizeof(Picture));
   p->count = 1;
   p->name = path;
   p->next = NULL;
@@ -233,12 +234,12 @@ Picture *LoadPicture(Display *dpy, Window Root, char *path, int color_limit)
   /* If no XPM support, or XPM loading failed, try bitmap */
   if(XReadBitmapFile(dpy,Root,path,&p->width,&p->height,&p->picture,&l,&l)
      == BitmapSuccess)
-    {
-      p->depth = 0;
-      p->mask = None;
-      p->nalloc_pixels = 0;
-      return p;
-    }
+  {
+    p->depth = 0;
+    p->mask = None;
+    p->nalloc_pixels = 0;
+    return p;
+  }
 
   free(p);
   return NULL;
@@ -341,6 +342,56 @@ void DestroyPicture(Display *dpy, Picture *p)
   free(p);
 }
 
+Picture *LoadPictureFromPixmap(Display *dpy, Window Root, char *name,
+			       Pixmap pixmap, Pixmap mask, int width, int height)
+{
+  Picture *q;
+
+  q = (Picture*)safemalloc(sizeof(Picture));
+  memset(q, 0, sizeof(Picture));
+  q->count = 1;
+  q->name = name;
+  q->next = NULL;
+  q->stamp = pixmap;
+  q->picture = pixmap;
+  q->mask = mask;
+  q->width = width;
+  q->height = height;
+  q->depth = Pdepth;
+  q->alloc_pixels = 0;
+  q->nalloc_pixels = 0;
+
+  return q;
+}
+
+Picture *CachePictureFromPixmap(Display *dpy, Window Root,char *name,
+				Pixmap pixmap, Pixmap mask,
+				int width, int height)
+{
+  Picture *p=PictureList;
+  
+  /* See if the picture is already cached */
+  for(;p!=NULL;p=p->next)
+  {
+#if 0
+    /* at th present time no good way to cache a pixmap */
+    if (!strcmp(p->name,name)) {
+      p->count++;
+      return p;
+    }
+#endif
+  }
+
+  /* Not previously cached, have to load. Put it first in list */
+  p = LoadPictureFromPixmap(dpy, Root, name, pixmap, mask, width, height);
+  if(p)
+  {
+    p->next=PictureList;
+    PictureList=p;
+  }
+
+  return p;
+}
 
 #ifdef XPM
 /* This structure is used to quickly access the RGB values of the colors */
@@ -575,7 +626,6 @@ Pixel GetColor(char *name)
   int cs;
   char *rest;
   XColor color;
-  color.pixel = 0;
 
   switch ((i = GetTokenIndex(name, colorset_names, -1, &rest)))
   {
@@ -604,14 +654,25 @@ Pixel GetColor(char *name)
     switch (i)
     {
     case 0:
-      return Colorset[cs].fg;
+      color.pixel = Colorset[cs].fg;
+      break;
     case 1:
-      return Colorset[cs].bg;
+      color.pixel = Colorset[cs].bg;
+      break;
     case 2:
-      return Colorset[cs].hilite;
+      color.pixel = Colorset[cs].hilite;
+      break;
     case 3:
-      return Colorset[cs].shadow;
+      color.pixel = Colorset[cs].shadow;
+      break;
     }
+    if (!XAllocColor(Pdpy, Pcmap, &color))
+    {
+      fprintf(stderr, "Cannot allocate color %d from colorset %d\n", i, cs);
+      return 0;
+    }
+    return color.pixel;
+
   default:
     break;
   }
