@@ -96,32 +96,6 @@ void CopyString(char **dest, const char *source)
   (*dest)[len]=0;
 }
 
-
-void CopyStringWithQuotes(char **dest, const char *src)
-{
-	while (src && src[0] == ' ')
-	{
-		src++;
-	}
-	if (src && src[0] == '"')
-	{
-		int len;
-
-		src++;
-		CopyString(dest, src);
-		len = strlen(*dest);
-		if (len > 0 && (*dest)[len - 1] == '"')
-		{
-			(*dest)[len - 1] = '\0';
-		}
-	}
-	else
-	{
-		CopyString(dest, src);
-	}
-}
-
-
 /****************************************************************************
  *
  * Copies a string into a new, malloc'ed string
