@@ -122,8 +122,13 @@ void fvwmlib_keyboard_shortcuts(
     int x_def_new = 0;
     int y_def_new = 0;
 
-    XQueryPointer(dpy, RootWindow(dpy, screen), &JunkRoot, &Event->xany.window,
-		  &x_root, &y_root, &x, &y, &JunkMask);
+    if (XQueryPointer(
+	  dpy, RootWindow(dpy, screen), &JunkRoot, &Event->xany.window,
+	  &x_root, &y_root, &x, &y, &JunkMask) == False)
+    {
+      /* pointer is on a different screen - do nothing */
+      return;
+    }
     if (x + x_move < 0)
     {
       x_def_new = x + x_move;
