@@ -50,6 +50,7 @@
  (Button3MotionMask * (NUMBER_OF_MOUSE_BUTTONS >= 3)) | \
  (Button4MotionMask * (NUMBER_OF_MOUSE_BUTTONS >= 4)) | \
  (Button5MotionMask * (NUMBER_OF_MOUSE_BUTTONS >= 5)))
+#define DEFAULT_MODS_UNUSED                LockMask
 
 
 /***************************************************************
@@ -72,7 +73,7 @@
 #define MAX_MENU_BORDER_WIDTH		  50 /* pixels */
 #define MAX_MENU_ITEM_RELIEF_THICKNESS	  50 /* pixels */
 #define PARENT_MENU_FORCE_VISIBLE_WIDTH	  20 /* pixels */
-#define MENU_POPUP_NOW_RATIO		0.75 /* % of item width */
+#define DEFAULT_MENU_POPUP_NOW_RATIO	  75 /* % of item width */
 /*   default item formats for left and right submenus. */
 #define DEFAULT_MENU_ITEM_FORMAT	  "%s%.1|%.5i%.5l%.5l%.5r%.5i%2.3>%1|"
 #define DEFAULT_LEFT_MENU_ITEM_FORMAT	  "%.1|%3.2<%5i%5l%5l%5r%5i%1|%s"
@@ -156,7 +157,7 @@
 #define WINDOW_TITLE_TEXT_OFFSET     10 /* pixels */
 
 /*** window placement (MinOverlap(Percent)Placement) ***/
-
+/** Now these values are configurable by using styles **/
 /* The following factors represent the amount of area that these types of
  * windows are counted as.  For example, by default the area of ONTOP windows
  * is counted 5 times as much as normal windows.  So CleverPlacement will
@@ -172,13 +173,36 @@
 #define PLACEMENT_AVOID_STICKY		   1.0
 #define PLACEMENT_AVOID_ONTOP		   5
 #define PLACEMENT_AVOID_ICON              10
-/* used in MinOverlapPercentPlacement to forbid complete covering (99%, 95%
+#define PLACEMENT_AVOID_EWMH_STRUT        50
+/* used in MinOverlap*Placement to forbid complete covering (99%, 95%
    85% and 75%) of windows */
 #define PLACEMENT_AVOID_COVER_99          12
 #define PLACEMENT_AVOID_COVER_95           6
 #define PLACEMENT_AVOID_COVER_85           4
 #define PLACEMENT_AVOID_COVER_75           1
 
+
+/*** icon layout ***/
+/* width of the relief around the icon and icon title */
+#define ICON_RELIEF_WIDTH                  2 /* pixels */
+/* height of stick lines */
+#define ICON_TITLE_STICK_HEIGHT            WINDOW_TITLE_STICK_HEIGHT
+/* vertical distance between stick lines */
+#define ICON_TITLE_STICK_VERT_DIST         WINDOW_TITLE_STICK_VERT_DIST
+/* vertical offset for icon title */
+#define ICON_TITLE_VERT_TEXT_OFFSET       -3 /* pixels */
+/* minimum width of stick lines */
+#define ICON_TITLE_STICK_MIN_WIDTH         3 /* pixels */
+/* number of blank pixels before and after a collapsed title */
+#define ICON_TITLE_TEXT_GAP_COLLAPSED      1 /* pixels */
+/* number of blank pixels before and after an expanded title */
+#define ICON_TITLE_TEXT_GAP_EXPANDED       4 /* pixels */
+/* extra blank pixels if the icon is sticky */
+#define ICON_TITLE_TO_STICK_EXTRA_GAP      1 /* pixels */
+/* minimum distance of icons in icon box */
+#define MIN_ICON_BOX_DIST                  3 /* pixels */
+
+
 /*** general keyboard shortcuts used in move, resize, ... ***/
 #define DEFAULT_KDB_SHORTCUT_MOVE_DISTANCE 5 /* pixels */
 #define KDB_SHORTCUT_MOVE_DISTANCE_SMALL   1 /* pixels */
@@ -244,6 +268,9 @@
 /* Replace with -1 to switch off "primary screen" concept by default */
 #define DEFAULT_PRIMARY_SCREEN             0
 
+/*** string sent to modules when fvwm2 detect that the root background change ***/
+#define ROOT_BG_CHANGE_STRING       "RootBgChange"
+
 /*** Very long window names (600000 characters or more) seem to hang the X
  *** server. ***/
 #define MAX_WINDOW_NAME_LEN		 200 /* characters */
