@@ -122,6 +122,7 @@ static void GetPanelGeometry(
 void swallow(unsigned long*);
 void AddButtonAction(button_info*,int,char*);
 char *GetButtonAction(button_info*,int);
+static void update_root_transparency(XEvent *Event);
 static void change_colorset(int colorset);
 
 void DebugEvents(XEvent*);
@@ -357,10 +358,6 @@ static void DeadPipeCleanup(void)
 	  XMoveWindow(Dpy,swin,b->x,b->y);
 	  XResizeWindow(Dpy,swin,b->w,b->h);
 	  XSetWindowBorderWidth(Dpy,swin,b->bw);
-	  if ((b->flags & b_Panel))
-	  {
-	    XMapWindow(Dpy, swin);
-	  }
 	}
       }
 #ifdef DEBUG_HANGON
@@ -645,7 +642,6 @@ void SetTransparentBackground(button_info *ub,int w,int h)
       Dpy, MyWindow, FShapeBounding, 0, 0, pmap_mask, FShapeSet);
     XFreePixmap(Dpy, pmap_mask);
     XDestroyRegion(shape_r);
-    XFlush(Dpy);
   }
 
   return;
@@ -840,7 +836,7 @@ int main(int argc, char **argv)
   /* parse the geometry string */
   if (geom_option_argc != 0)
   {
-    parse_window_geometry(argv[geom_option_argc], 0);
+    parse_window_geometry(argv[geom_option_argc]);
     has_button_geometry = 0;
   }
 
@@ -970,6 +966,8 @@ void Loop(void)
   KeySym keysym;
   char buffer[10],*tmp,*act;
   int i,i2,button;
+  int x;
+  int y;
   button_info *ub,*b;
 #ifndef OLD_EXPOSE
   int ex=10000,ey=10000,ex2=0,ey2=0;
@@ -1051,7 +1049,7 @@ void Loop(void)
       {
 	unsigned int depth,tw,th,border_width;
 	Window root;
-
+ 
 	if (!XGetGeometry(Dpy, MyWindow, &root, &x, &y, &tw, &th, &border_width,
 			  &depth))
 	{
@@ -1064,16 +1062,25 @@ void Loop(void)
 	  SetButtonSize(UberButton,Width,Height);
 	  button=-1;
 	  ub=UberButton;
-	  for (i = 0; i < nColorsets; i++)
-	    change_colorset(i);
-	  while(NextButton(&ub,&b,&button,0))
-	    MakeButton(b);
 	  if (FShapesSupported && UberButton->c->flags & b_TransBack)
 	  {
 	    SetTransparentBackground(UberButton, Width, Height);
 	  }
+	  for (i = 0; i < nColorsets; i++)
+	    change_colorset(i);
+	  while(NextButton(&ub,&b,&button,0))
+	    MakeButton(b);
+	  XClearWindow(Dpy, MyWindow);
 	  RedrawWindow(NULL);
 	}
+	if (Event.xconfigure.window == MyWindow &&
+	    Event.xconfigure.send_event && (UberButton->c->flags & b_Colorset) &&
+	      Colorset[UberButton->c->colorset].pixmap == ParentRelative)
+	{
+	  UberButton->x = Event.xconfigure.x;
+	  UberButton->y = Event.xconfigure.y;
+	  update_root_transparency(&Event);
+	} /* moved && ParentRelative */
       }
       break;
 
@@ -1108,8 +1115,21 @@ void Loop(void)
 	{
 	  break;
 	}
+        if (Event.xbutton.window == MyWindow)
+        {
+          x = Event.xbutton.x;
+          y = Event.xbutton.y;
+        }
+        else
+        {
+          Window dummy;
+
+          XTranslateCoordinates(
+            Dpy, Event.xbutton.window, MyWindow, Event.xbutton.x,
+            Event.xbutton.y, &x, &y, &dummy);
+        }
 	CurrentButton = b =
-	  select_button(UberButton,Event.xbutton.x,Event.xbutton.y);
+	  select_button(UberButton, x, y);
 	is_pointer_in_current_button = True;
 
 	act = NULL;
@@ -1150,7 +1170,20 @@ void Loop(void)
 	  CurrentButton = NULL;
 	  break;
 	}
-	b = select_button(UberButton,Event.xbutton.x,Event.xbutton.y);
+        if (Event.xbutton.window == MyWindow)
+        {
+          x = Event.xbutton.x;
+          y = Event.xbutton.y;
+        }
+        else
+        {
+          Window dummy;
+
+          XTranslateCoordinates(
+            Dpy, Event.xbutton.window, MyWindow, Event.xbutton.x,
+            Event.xbutton.y, &x, &y, &dummy);
+        }
+	b = select_button(UberButton, x, y);
 	act = GetButtonAction(b,Event.xbutton.button);
 	if (b && !act && (b->flags & b_Panel))
 	{
@@ -2292,8 +2325,6 @@ void SpawnSome(void)
   static char first=1;
   button_info *b,*ub=UberButton;
   int button=-1;
-  char *p;
-
   if(!first)
     return;
   first=0;
@@ -2306,12 +2337,7 @@ void SpawnSome(void)
 	fprintf(stderr,"%s: Button 0x%06x did not find a \"%s\" window, %s",
 		MyName,(ushort)b,b->hangon,"spawning own\n");
 #endif
-	p = expand_action(b->spawn, NULL);
-	if (p)
-	{
-	  MySendText(fd, p, 0);
-	  free(p);
-	}
+	SendText(fd,b->spawn,0);
       }
 }
 
@@ -2385,6 +2411,54 @@ static void recursive_change_colorset(container_info *c, int colorset)
   return;
 }
 
+static void update_root_transparency(XEvent *Event)
+{
+  button_info *ub,*b;
+  int button,bx,by,bpx,bpy,bf;
+
+  if (!(UberButton->c->flags & b_Colorset) ||
+      Colorset[UberButton->c->colorset].pixmap != ParentRelative)
+  {
+    return;
+  }
+
+  XClearWindow(Dpy, MyWindow);
+  RedrawWindow(NULL);
+  /* Handle the swallowed window */
+  ub = UberButton;
+  button=-1;
+  while(NextButton(&ub,&b,&button,0))
+  {
+    if (buttonSwallowCount(b) == 3 && (b->flags & b_Swallow) &&
+	((b->flags & b_Colorset) || (b->flags & b_ColorsetParent)) &&
+	Colorset[buttonColorset(b)].pixmap == ParentRelative) 
+    {
+      /* should handle 2 cases: may cause problems with complex
+       * functions, but this may be considered as a feature if
+       * we document the rules used */
+      if (b->spawn != NULL &&
+	  (strncasecmp(b->spawn,"module",6) == 0 ||
+	   strncasecmp(b->spawn,"fvwm",4) == 0))
+      {
+	/* it is a module (or a complex fct) */
+	if (Event != NULL)
+	{
+	  buttonInfo(b,&bx,&by,&bpx,&bpy,&bf);
+	  Event->xconfigure.x = UberButton->x + bx;
+	  Event->xconfigure.y = UberButton->y + by;
+	  Event->xconfigure.window = SwallowedWindow(b);  
+	  XSendEvent(Dpy, SwallowedWindow(b), False, NoEventMask, Event);
+	}
+      }
+      else
+      {
+	/* it is an application (or a complex fct) */
+	change_swallowed_window_colorset(b, True);
+      }
+    }
+  }
+}
+
 static void change_colorset(int colorset)
 {
   if (UberButton->c->flags & b_Colorset && colorset == UberButton->c->colorset)
@@ -2415,6 +2489,10 @@ static void handle_config_info_packet(unsigned long *body)
   {
     FScreenConfigureModule(tline);
   }
+  else if (StrEquals(token, ROOT_BG_CHANGE_STRING))
+  {
+    update_root_transparency(NULL);
+  }
 
   return;
 }
@@ -2766,6 +2844,7 @@ void swallow(unsigned long *body)
   char *temp;
   button_info *ub=UberButton,*b;
   int button=-1;
+  int i;
   unsigned int d;
   Window p;
 
@@ -2835,6 +2914,21 @@ void swallow(unsigned long *body)
 	  return;
 	}
 	/*error checking*/
+        for (i = 0; !(b->flags & b_ActionIgnoresClientWindow) &&
+		     i <= NUMBER_OF_MOUSE_BUTTONS; i++)
+        {
+          if (b->action != NULL && b->action[i] != NULL)
+          {
+            XGrabButton(
+              Dpy, i, 0, b->IconWin, False,
+              ButtonPressMask | ButtonReleaseMask, GrabModeAsync,
+              GrabModeAsync, None, None);
+            if (i == 0)
+            {
+              break;
+            }
+          }
+        }
 	XReparentWindow(Dpy,swin,MyWindow,-32768,-32768);
 	XSync(Dpy, 0);
 	MyXUngrabServer(Dpy);
