@@ -780,7 +780,7 @@ static void ParseButton(button_info **uberb,char *s)
       "padding",
       "swallow",
       "panel",
-      "action",
+      "actionignoresclientwindow",
       "container",
       "end",
       "nosize",
@@ -789,6 +789,7 @@ static void ParseButton(button_info **uberb,char *s)
       "right",
       "center",
       "colorset",
+      "action",
       NULL
     };
     s = trimleft(s);
@@ -796,18 +797,11 @@ static void ParseButton(button_info **uberb,char *s)
     {
       Bool is_swallow = False;
 
-      if (*s == ',')
-      {
-	      s++;
-	      s = trimleft(s);
-	      continue;
-      }
       if((*s>='0' && *s<='9') || *s=='+' || *s=='-')
       {
 	char *geom;
 	int x,y,flags;
 	unsigned int w,h;
-
 	geom=seekright(&s);
 	if (geom)
 	{
@@ -835,6 +829,8 @@ static void ParseButton(button_info **uberb,char *s)
 	s = trimleft(s);
 	continue;
       }
+      if(*s==',' && s++)
+	s = trimleft(s);
       switch(GetTokenIndex(s,opts,-1,&s))
       {
       case 0: /* Back */
@@ -1058,52 +1054,8 @@ static void ParseButton(button_info **uberb,char *s)
 	}
 	break;
 
-	/* --------------------------- action ------------------------ */
-
-      case 9: /* Action */
-	s = trimleft(s);
-	i=0;
-	if(*s=='(')
-	{
-	  s++;
-	  if(strncasecmp(s,"mouse",5)!=0)
-	  {
-	    fprintf(stderr,"%s: Couldn't parse action\n",MyName);
-	  }
-	  s+=5;
-	  i=strtol(s,&t,10);
-	  s=t;
-	  while(*s && *s!=')')
-	    s++;
-	  if(*s==')')
-            s++;
-	}
-        {
-          char *r;
-          char *u = s;
-
-          s = GetQuotedString(s, &t, ",)", NULL, "(", ")");
-          r = s;
-          if (t && r > u + 1)
-          {
-            /* remove unquoted trailing spaces */
-            r -= 2;
-            while (r >= u && isspace(*r))
-              r--;
-            r++;
-            if (isspace(*r))
-            {
-              t[strlen(t) - (s - r - 1)] = 0;
-            }
-          }
-        }
-	if(t)
-	{
-	  AddButtonAction(b,i,t);
-	  free(t);
-	}
-	else
-	  fprintf(stderr,"%s: Missing action argument\n",MyName);
+      case 9: /* ActionIgnoresClientWindow */
+	b->flags |= b_ActionIgnoresClientWindow;
 	break;
 
 	/* -------------------------- container ---------------------- */
@@ -1176,6 +1128,54 @@ static void ParseButton(button_info **uberb,char *s)
 	}
 	break;
 
+	/* --------------------------- action ------------------------ */
+
+      case 18: /* Action */
+	s = trimleft(s);
+	i=0;
+	if(*s=='(')
+	{
+	  s++;
+	  if(strncasecmp(s,"mouse",5)!=0)
+	  {
+	    fprintf(stderr,"%s: Couldn't parse action\n",MyName);
+	  }
+	  s+=5;
+	  i=strtol(s,&t,10);
+	  s=t;
+	  while(*s && *s!=')')
+	    s++;
+	  if(*s==')')
+            s++;
+	}
+        {
+          char *r;
+          char *u = s;
+
+          s = GetQuotedString(s, &t, ",)", NULL, "(", ")");
+          r = s;
+          if (t && r > u + 1)
+          {
+            /* remove unquoted trailing spaces */
+            r -= 2;
+            while (r >= u && isspace(*r))
+              r--;
+            r++;
+            if (isspace(*r))
+            {
+              t[strlen(t) - (s - r - 1)] = 0;
+            }
+          }
+        }
+	if(t)
+	{
+	  AddButtonAction(b,i,t);
+	  free(t);
+	}
+	else
+	  fprintf(stderr,"%s: Missing action argument\n",MyName);
+	break;
+
       default:
 	t=seekright(&s);
 	fprintf(stderr,"%s: Illegal button option \"%s\"\n",MyName,
@@ -1308,7 +1308,7 @@ static void ParseConfigLine(button_info **ubb,char *s)
     i=sscanf(s,"%63s",geom);
     if(i==1)
     {
-      parse_window_geometry(geom, 0);
+      parse_window_geometry(geom);
     }
     break;
   }
@@ -1319,14 +1319,39 @@ static void ParseConfigLine(button_info **ubb,char *s)
     i=sscanf(s,"%63s",geom);
     if(i==1)
     {
-      parse_window_geometry(geom, 1);
+      int flags;
+      int g_x;
+      int g_y;
+      unsigned int width;
+      unsigned int height;
+
+      flags = XParseGeometry(geom,&g_x,&g_y,&width,&height);
+      if (!flags)
+	break;
+      UberButton->w = 0;
+      UberButton->h = 0;
+      UberButton->x = 0;
+      UberButton->y = 0;
+      if (flags&WidthValue)
+	button_width = width;
+      if (flags&HeightValue)
+	button_height = height;
+      if (flags&XValue)
+	UberButton->x = g_x;
+      if (flags&YValue)
+	UberButton->y = g_y;
+      if (flags&XNegative)
+	UberButton->w = 1;
+      if (flags&YNegative)
+	UberButton->h = 1;
+      has_button_geometry = 1;
     }
     break;
   }
   case 2:/* Font */
     if (ub->c->font_string)
       free(ub->c->font_string);
-    CopyStringWithQuotes(&ub->c->font_string, s);
+    CopyString(&ub->c->font_string, s);
     break;
   case 3:/* Padding */
     i=sscanf(s,"%d %d",&j,&k);
@@ -1607,7 +1632,7 @@ char *expand_action(char *in_action, button_info *b)
   return action;
 }
 
-void parse_window_geometry(char *geom, int is_button_geometry)
+void parse_window_geometry(char *geom)
 {
   int flags;
   int g_x;
@@ -1618,20 +1643,12 @@ void parse_window_geometry(char *geom, int is_button_geometry)
   flags = FScreenParseGeometry(geom,&g_x,&g_y,&width,&height);
   UberButton->w = 0;
   UberButton->h = 0;
-  if (is_button_geometry)
-  {
-    if (flags&WidthValue)
-      button_width = width;
-    if (flags&HeightValue)
-      button_height = height;
-  }
-  else
-  {
-    if (flags&WidthValue)
-      w = width;
-    if (flags&HeightValue)
-      h = height;
-  }
+  UberButton->x = 0;
+  UberButton->y = 0;
+  if (flags&WidthValue)
+    w = width;
+  if (flags&HeightValue)
+    h = height;
   if (flags&XValue)
     UberButton->x = g_x;
   if (flags&YValue)
@@ -1640,7 +1657,7 @@ void parse_window_geometry(char *geom, int is_button_geometry)
     UberButton->w = 1;
   if (flags&YNegative)
     UberButton->h = 1;
-  has_button_geometry = is_button_geometry;
+  has_button_geometry = 0;
 
   return;
 }
