@@ -179,6 +179,9 @@ int main(int argc, char **argv)
 
   fd_width = GetFdWidth();
 
+  /* select all extended messages */
+  SetMessageMask(fd, M_EXTENDED_MSG | MAX_XMSG_MASK);
+
   /* Create a list of all windows */
   /* Request a list of all windows,
    * wait for ConfigureWindow packets */
@@ -262,10 +265,18 @@ void process_message(unsigned long type, const unsigned long *body)
       fprintf(output,"window name\n");
       list_window_name(body);
       break;
+    case M_VISIBLE_NAME:
+      fprintf(output,"visible window name\n");
+      list_window_name(body);
+      break;
     case M_ICON_NAME:
       fprintf(output,"icon name\n");
       list_window_name(body);
       break;
+    case MX_VISIBLE_ICON_NAME:
+      fprintf(output,"visible icon name\n");
+      list_window_name(body);
+      break;
     case M_RES_CLASS:
       fprintf(output,"window class\n");
       list_window_name(body);
@@ -281,6 +292,10 @@ void process_message(unsigned long type, const unsigned long *body)
       fprintf(output,"icon location\n");
       list_icon(body);
       break;
+    case M_ICON_FILE:
+      fprintf(output,"icon file\n");
+      list_window_name(body);
+      break;
     case M_MAP:
       fprintf(output,"map\n");
       list_winid(body);
