@@ -275,6 +275,10 @@ void veryLongLoop()
 	case Expose:
 	  drawDragWellButton(&dragBut); /*draws the drag icon*/
 	  break;
+	case ConfigureNotify:
+	  if (xg.colorset >= 0 && Colorset[xg.colorset].pixmap == ParentRelative)
+	    XClearArea(xg.dpy, xg.win, 0,0,0,0, True);
+	  break;
 	}
     } else if (eventType==FOUND_FVWM_MESSAGE) { /*we have a fvwm2 message */
       /* two cases, one where item is a file, other where we have a subdir*/
@@ -610,6 +614,13 @@ void parseOptions(void)
     } else if (StrEquals(resource, "Geometry")) {
       int g_x,g_y,flags;
       unsigned width,height;
+      xg.w = 48;
+      xg.h = 48;
+      xg.x = 0;
+      xg.y = 0;
+      xg.xneg = 0;
+      xg.yneg = 0;
+      xg.usposition = 0;
       flags = FScreenParseGeometry(arg1,&g_x,&g_y,&width,&height);
       if (flags & WidthValue) {
 	xg.w = width;
@@ -634,6 +645,10 @@ void parseOptions(void)
     } else if (StrEquals(resource, "DragWellGeometry")) {
       int g_x,g_y,flags;
       unsigned width,height;
+      xg.dbw = 30;
+      xg.dbh = 30;
+      xg.dbx = 9;
+      xg.dby = 9;
       flags = FScreenParseGeometry(arg1,&g_x,&g_y,&width,&height);
       if (flags & WidthValue) {
 	xg.dbw = width;
