@@ -87,8 +87,8 @@ int Channel[2];
    . copied into items during parse
    . displayed in the customization dialog
    */
-static int colorset = -1;
-static int itemcolorset = 0;
+int colorset = -1;
+int itemcolorset = 0;
 
 /* prototypes */
 static void AssignDrawTable(Item *);
@@ -433,6 +433,10 @@ static void ct_Geometry(char *cp)
   int flags;
   unsigned int dummy;
 
+  CF.gx = 0;
+  CF.gy = 0;
+  CF.xneg = 0;
+  CF.yneg = 0;
   while (isspace(*cp))
     cp++;
   flags = FScreenParseGeometry(cp, &x, &y, &dummy, &dummy);
@@ -505,14 +509,14 @@ static void ct_Font(char *cp)
 {
   if (font_names[f_text])
     free(font_names[f_text]);
-  CopyStringWithQuotes(&font_names[f_text], cp);
+  font_names[f_text] = safestrdup(cp);
   myfprintf((stderr, "Font: %s\n", font_names[f_text]));
 }
 static void ct_ButtonFont(char *cp)
 {
   if (font_names[f_button])
     free(font_names[f_button]);
-  CopyStringWithQuotes(&font_names[f_button], cp);
+  font_names[f_button] = safestrdup(cp);
   myfprintf((stderr, "ButtonFont: %s\n", font_names[f_button]));
 }
 static void ct_ButtonInPointer(char *cp)
@@ -539,7 +543,7 @@ static void ct_InputFont(char *cp)
 {
   if (font_names[f_input])
     free(font_names[f_input]);
-  CopyStringWithQuotes(&font_names[f_input], cp);
+  font_names[f_input] = safestrdup(cp);
   myfprintf((stderr, "InputFont: %s\n", font_names[f_input]));
 }
 static void ct_InputPointer(char *cp)
@@ -929,8 +933,7 @@ static void PutDataInForm(char *cp)
         strcpy(item->input.init_value,cp); /* new initial value in field */
 	if (item->input.value)
 	  free(item->input.value);
-        item->input.buf = var_len+1;
-        item->input.value = safemalloc(item->input.buf);
+        item->input.value = safemalloc(var_len+1);
         strcpy(item->input.value,cp);     /* new value in field */
         /* New value, but don't change length */
         free(var_name);                 /* goto's have their uses */
@@ -1568,6 +1571,28 @@ void RedrawItem (Item *item, int click)
   XFlush(dpy);
 }
 
+/* update transparency if backgroude colorset is transparent */
+
+void UpdateRootTransapency(void)
+{
+  Item *item;
+
+  if (colorset > -1 && Colorset[colorset].pixmap == ParentRelative)
+  {
+    /* window has moved redraw the background if it is transparent */
+    XClearArea(dpy, CF.frame, 0,0,0,0, True);
+    if (itemcolorset > -1 &&
+	Colorset[itemcolorset].pixmap == ParentRelative)
+    {
+      for (item = root_item_ptr; item != 0; item = item->header.next)
+      {
+	if (item->header.win != None)
+	  XClearArea(dpy, item->header.win, 0,0,0,0, True);
+      }
+    }
+  }
+}
+
 /* execute a command */
 void DoCommand (Item *cmd)
 {
@@ -1946,6 +1971,11 @@ static void ParseActiveMessage(char *buf)
     FScreenConfigureModule(buf + sizeof(XINERAMA_CONFIG_STRING)-1);
     return;
   }
+  if (strncasecmp(buf, ROOT_BG_CHANGE_STRING, sizeof(ROOT_BG_CHANGE_STRING)) == 0)
+  {
+    UpdateRootTransapency();
+    return;
+  }
   if (strncasecmp(buf, MyName, MyNameLen) != 0) {/* If its not for me */
     return;
   } /* Now I know its for me. */
