@@ -41,6 +41,7 @@
 
 #include "config.h"
 #include "libs/fvwmlib.h"
+#include "libs/Colorset.h"
 
 #include <stdio.h>
 #include <ctype.h>
@@ -79,7 +80,16 @@ void ReadXServer ()
     if (event.xany.window == CF.frame) {
       switch (event.type) {
       case ConfigureNotify:             /* has window be reconfigured */
-        ResizeFrame();                  /* adjust yourself... */
+	if (colorset > -1 && Colorset[colorset].pixmap == ParentRelative && 
+	    event.xconfigure.send_event)
+	{
+	  UpdateRootTransapency();
+	}
+	else
+	{
+	  /* adjust yourself... */
+	  ResizeFrame();
+	}
         break;
 #if 0
       case SelectionClear:
