@@ -125,9 +125,10 @@ int UWidth, UHeight;
 		       LeaveWindowMask | PointerMotionMask)
 
 unsigned long m_mask = M_CONFIGURE_WINDOW|M_ADD_WINDOW|M_DESTROY_WINDOW|
-		 M_END_WINDOWLIST| M_ICONIFY|M_DEICONIFY|M_ICON_NAME|
-		 M_RES_NAME|M_RES_CLASS|M_WINDOW_NAME|M_ICON_FILE|
-		 M_DEFAULTICON|M_CONFIG_INFO|M_END_CONFIG_INFO;
+  M_END_WINDOWLIST| M_ICONIFY|M_DEICONIFY|
+  M_RES_NAME|M_RES_CLASS|M_VISIBLE_NAME|M_ICON_FILE|
+  M_DEFAULTICON|M_CONFIG_INFO|M_END_CONFIG_INFO;
+unsigned long mx_mask = MX_VISIBLE_ICON_NAME;
 
 struct icon_info *Hilite;
 int main_width, main_height;
@@ -145,7 +146,7 @@ int interval = 8;
 int motion = NONE;
 int primary = LEFT, secondary = BOTTOM;
 int xneg = 0, yneg = 0;
-int ClickTime = 150;
+int ClickTime = DEFAULT_CLICKTIME;
 unsigned int bar_width = 9;
 int redraw_flag = 3;
 int icon_relief = 4;
@@ -181,6 +182,7 @@ int sortby = UNSORT;
 char* AnimCommand = NULL;
 
 int save_color_limit = 0;                   /* color limit from config */
+static Bool have_double_click = False;
 static Bool is_dead_pipe = False;
 
 static RETSIGTYPE TerminateHandler(int);
@@ -298,11 +300,10 @@ int main(int argc, char **argv)
 
   XSetErrorHandler(myErrorHandler);
 
-  /* SetMessageMask(fd, m_mask); */
-
   ParseOptions();
-
-  SetMessageMask(fd, m_mask); /* it may have changed */
+  /* m_mask/mx_mask may have changed in the ParseOptions call*/
+  SetMessageMask(fd, m_mask);
+  SetMessageMask(fd, mx_mask);
 
   if ((local_flags & SETWMICONSIZE) && (size = XAllocIconSize()) != NULL){
     size->max_width  = size->min_width  = max_icon_width + icon_relief;
@@ -373,6 +374,13 @@ void Loop(void)
 	      break;
 
 	    case ConfigureNotify:
+	      if (Event.xconfigure.window == icon_win &&
+		  (colorset < 0 ||
+		   Colorset[colorset].pixmap != ParentRelative))
+	      {
+		/* needs this event only for transparency */
+		break;
+	      }
 	      if (!XGetGeometry(dpy,main_win,&root,&x,&y,
 				(unsigned int *)&tw,(unsigned int *)&th,
 				(unsigned int *)&border_width,
@@ -422,6 +430,14 @@ void Loop(void)
 		  RedrawWindow();
 		}
 	      }
+	      else if (ready &&
+		       (Event.xconfigure.send_event ||
+			Event.xconfigure.window == icon_win) && colorset >= 0 &&
+		       Colorset[colorset].pixmap == ParentRelative)
+	      {
+		/* moved */
+		XClearArea(dpy, icon_win, 0,0,0,0, True);
+	      }
 	      break;
 	    case KeyPress:
 	      ExecuteKey(Event);
@@ -1168,6 +1184,8 @@ void CreateWindow(void)
 				 icon_win_width,
 				 icon_win_height,
 				 0,fore_pix,back_pix);
+  /* needed for transparency */
+  XSelectInput(dpy,icon_win,StructureNotifyMask);
 
   gcm = GCForeground|GCBackground;
   gcv.foreground = hilite_pix;
@@ -1195,6 +1213,9 @@ void CreateWindow(void)
     SetWindowBackground(dpy, icon_win, icon_win_width, icon_win_height,
                         &Colorset[(colorset)], Pdepth, NormalGC,
 			True);
+    SetWindowBackground(dpy, holder_win, mysizehints.width, mysizehints.height,
+                        &Colorset[(colorset)], Pdepth, NormalGC,
+			True);
     SetWindowBackground(dpy, main_win, mysizehints.width, mysizehints.height,
                         &Colorset[(colorset)], Pdepth, NormalGC,
 			True);
@@ -1299,7 +1320,8 @@ static void change_colorset(int color)
     SetWindowBackground(
       dpy, icon_win, icon_win_width, icon_win_height, &Colorset[(colorset)],
       Pdepth, NormalGC, True);
-    XSetWindowBackground(dpy, holder_win, back_pix);
+    SetWindowBackground(dpy, holder_win, main_width - h_margin,
+      main_height - v_margin, &Colorset[(colorset)], Pdepth, NormalGC, True);
     XSetWindowBackground(dpy, h_scroll_bar, back_pix);
     XSetWindowBackground(dpy, v_scroll_bar, back_pix);
     XSetWindowBackground(dpy, l_button, back_pix);
@@ -1651,6 +1673,12 @@ void ParseOptions(void)
     {
       if (strncasecmp(tline,CatString3("*", MyName, "Geometry"),Clength+9)==0)
       {
+	num_columns = 6;
+	num_rows = 1;
+	geom_x = -10000;
+	geom_y = -10000;
+	xneg = 0;
+	yneg = 0;
 	tmp = &tline[Clength+9];
 	while(((isspace((unsigned char)*tmp))&&(*tmp != '\n'))&&(*tmp != 0))
 	  tmp++;
@@ -1715,7 +1743,7 @@ void ParseOptions(void)
       }
       else if (strncasecmp(tline,CatString3("*",MyName,"Font"),Clength+5)==0)
       {
-	CopyStringWithQuotes(&font_string,&tline[Clength+5]);
+	CopyString(&font_string,&tline[Clength+5]);
       }
       else if (strncasecmp(
 		 tline,CatString3("*",MyName,"IconFore"),Clength+9)==0)
@@ -2001,7 +2029,10 @@ void parsemouse(char *tline)
   if (strncasecmp(start, "Click", 5) == 0)
     f->type = CLICK;
   else if (strncasecmp(start, "DoubleClick", 11) == 0)
+  {
     f->type = DOUBLE_CLICK;
+    have_double_click = True;
+  }
 
   /* actions */
   tline = end;
@@ -2311,7 +2342,7 @@ void process_message(unsigned long type, unsigned long *body)
   case M_RES_CLASS:
     UpdateItem(type, body[0], (char *)&body[3]);
     break;
-  case M_WINDOW_NAME:
+  case M_VISIBLE_NAME:
     tmp = UpdateItem(type, body[0], (char *)&body[3]);
     if (!ready || tmp == NULL)
       break;
@@ -2338,7 +2369,7 @@ void process_message(unsigned long type, unsigned long *body)
     }
     }
     break;
-  case M_ICON_NAME:
+  case MX_VISIBLE_ICON_NAME:
     tmp = UpdateItem(type, body[0], (char *)&body[3]);
     if (!ready || tmp == NULL)
     break;
@@ -2629,7 +2660,7 @@ struct icon_info *UpdateItem(unsigned long type, unsigned long id, char *item)
       strcpy(str, item);
 
       switch (type){
-      case M_ICON_NAME:
+      case MX_VISIBLE_ICON_NAME:
 	if (tmp->name)
 	  free(tmp->name);
 	tmp->name = str;
@@ -2640,7 +2671,7 @@ struct icon_info *UpdateItem(unsigned long type, unsigned long id, char *item)
 	tmp->icon_file = str;
 	tmp->extra_flags &= ~DEFAULTICON;
 	return tmp;
-      case M_WINDOW_NAME:
+      case M_VISIBLE_NAME:
 	if (tmp->window_name)
 	  free(tmp->window_name);
 	tmp->window_name = str;
@@ -2877,35 +2908,73 @@ void freeitem(struct icon_info *item, int d)
 
 
 /************************************************************************
-  IsClick
-  	Based on functions.c from Fvwm:
-	Copyright 1988, Evans and Sutherland Computer Corporation,
-       	Copyright 1989, Massachusetts Institute of Technology,
-       	Copyright 1993, Robert Nation.
+ * CheckActionType
+ *   Based on functions.c from Fvwm:
  ***********************************************************************/
-Bool IsClick(int x,int y,unsigned EndMask, XEvent *d)
+static int CheckActionType(
+  int x, int y, XEvent *d, Bool may_time_out, Bool is_button_pressed)
 {
   int xcurrent,ycurrent,total = 0;
+  int dist;
+  Bool do_sleep = False;
 
   xcurrent = x;
   ycurrent = y;
-  while((total < ClickTime)&&
-        (x - xcurrent < 5)&&(x - xcurrent > -5)&&
-        (y - ycurrent < 5)&&(y - ycurrent > -5))
+  dist = DEFAULT_MOVE_THRESHOLD;
+
+  while (total < ClickTime || !may_time_out)
+  {
+    if (!(x - xcurrent <= dist && xcurrent - x <= dist &&
+	  y - ycurrent <= dist && ycurrent - y <= dist))
     {
-      usleep(10000);
-      total+=10;
-      if(XCheckMaskEvent (dpy,EndMask, d))
-	return True;
-      if(XCheckMaskEvent (dpy,ButtonMotionMask|PointerMotionMask, d))
-        {
-          xcurrent = d->xmotion.x_root;
-          ycurrent = d->xmotion.y_root;
-        }
+      return TIMEOUT;
     }
-  return False;
+
+    if (do_sleep)
+    {
+      usleep(20000);
+    }
+    else
+    {
+      usleep(1);
+      do_sleep = 1;
+    }
+    total += 20;
+    if (XCheckMaskEvent(dpy, ButtonReleaseMask|ButtonMotionMask|
+			PointerMotionMask|ButtonPressMask|ExposureMask, d))
+    {
+      do_sleep = 0;
+      switch (d->xany.type)
+      {
+      case ButtonRelease:
+	return CLICK;
+      case MotionNotify:
+	if ((d->xmotion.state & DEFAULT_ALL_BUTTONS_MASK) ||
+            !is_button_pressed)
+	{
+	  xcurrent = d->xmotion.x_root;
+	  ycurrent = d->xmotion.y_root;
+	}
+        else
+	{
+	  return CLICK;
+	}
+	break;
+      case ButtonPress:
+	if (may_time_out)
+	  is_button_pressed = True;
+	break;
+      default:
+	/* can't happen */
+	break;
+      }
+    }
+  }
+
+  return TIMEOUT;
 }
 
+
 /************************************************************************
  * ExecuteAction
 * 	Based on part of ComplexFunction() of functions.c from fvwm:
@@ -2920,26 +2989,31 @@ void ExecuteAction(int x, int y, struct icon_info *item)
   XEvent d;
   struct mousefunc *tmp;
 
-   /* Wait and see if we have a click, or a move */
-  /* wait 100 msec, see if the used releases the button */
-  if(IsClick(x,y,ButtonReleaseMask,&d))
+  /* Wait and see if we have a click, or a move */
+  /* wait forever, see if the user releases the button */
+  type = CheckActionType(x, y, &d, False, True);
+  if (type == CLICK)
+  {
+    ev = &d;
+    /* If it was a click, wait to see if its a double click */
+    if (have_double_click)
     {
-      type = CLICK;
-      ev = &d;
-    }
-
-  /* If it was a click, wait to see if its a double click */
-  if((type == CLICK) && (IsClick(x,y,ButtonPressMask, &d)))
-    {
-      type = ONE_AND_A_HALF_CLICKS;
-      ev = &d;
-    }
-  if((type == ONE_AND_A_HALF_CLICKS) &&
-     (IsClick(x,y,ButtonReleaseMask, &d)))
-    {
-      type = DOUBLE_CLICK;
-      ev = &d;
+      type = CheckActionType(x, y, &d, True, False);
+      switch (type)
+      {
+      case CLICK:
+	type = DOUBLE_CLICK;
+	ev = &d;
+	break;
+      case TIMEOUT:
+	type = CLICK;
+	break;
+      default:
+	/* can't happen */
+	break;
+      }
     }
+  }
   tmp = MouseActions;
 
   while (tmp != NULL){
