@@ -28,6 +28,7 @@
 
 
 const char *MyName;
+int mods_unused = DEFAULT_MODS_UNUSED;
 
 static RETSIGTYPE TerminateHandler(int);
 
@@ -281,13 +282,15 @@ main(int argc, char **argv)
 
   SetMessageMask(Fvwm_fd, M_CONFIGURE_WINDOW | M_RES_CLASS | M_RES_NAME |
                  M_ADD_WINDOW | M_DESTROY_WINDOW | M_ICON_NAME |
-                 M_DEICONIFY | M_ICONIFY | M_ICON_LOCATION | M_END_WINDOWLIST |
+                 M_DEICONIFY | M_ICONIFY | M_END_WINDOWLIST |
                  M_NEW_DESK | M_NEW_PAGE | M_FOCUS_CHANGE | M_WINDOW_NAME |
-                 M_CONFIG_INFO |
+                 M_CONFIG_INFO | M_VISIBLE_NAME |
 #ifdef MINI_ICONS
 		 M_MINI_ICON |
 #endif
 		 M_STRING);
+  /* extended messages */
+  SetMessageMask(Fvwm_fd, MX_VISIBLE_ICON_NAME);
 
   SendInfo(Fvwm_fd, "Send_WindowList", 0);
 
