@@ -208,7 +208,6 @@ typedef struct win_data {
 
   Ulong desknum;
   long x, y, width, height;
-  rectangle icon_g;
   Ulong app_id;
 /*  Ulong fvwm_flags; */
   window_flags flags;
@@ -224,6 +223,8 @@ typedef struct win_data {
   char *classname;
   char *titlename;
   char *iconname;
+  char *visible_name;
+  char *visible_icon_name;
 } WinData;
 
 typedef struct button {
@@ -374,6 +375,7 @@ extern Display *theDisplay;
 extern char *Module;
 extern int ModuleLen;
 extern ContextDefaults contextDefaults[];
+extern int mods_unused;
 
 extern void ReadFvwmPipe(void);
 extern void *Malloc (size_t size);
