@@ -24,7 +24,7 @@
 #include <libs/Module.h>
 
 static char const rcsid[] =
-  "$Id: fvwm.c,v 1.34.2.3 2003/03/16 11:24:39 domivogt Exp $";
+  "$Id: fvwm.c,v 1.38 2002/01/19 21:33:15 domivogt Exp $";
 
 static WinData *fvwm_focus_win = NULL;
 
@@ -149,21 +149,9 @@ int win_in_viewport (WinData *win)
   WinManager *manager = win->manager;
   int flag = 0;
   int reverse = 0;
-  rectangle g;
 
   assert (manager);
 
-  if (IS_ICONIFIED(win) && !IS_ICON_SUPPRESSED(win))
-  {
-    g = win->icon_g;
-  }
-  else
-  {
-    g.x = win->x;
-    g.y = win->y;
-    g.width = win->width;
-    g.height = win->height;
-  }
   switch (manager->res) {
   case SHOW_GLOBAL:
     flag = 1;
@@ -186,7 +174,7 @@ int win_in_viewport (WinData *win)
     } else if (win->desknum == globals.desknum) {
       /* win and screen intersect if they are not disjoint in x and y */
       flag = RECTANGLES_INTERSECT(
-	g.x, g.y, g.width, g.height,
+	win->x, win->y, win->width, win->height,
 	manager->managed_g.x, manager->managed_g.y,
 	manager->managed_g.width, manager->managed_g.height);
     }
@@ -199,7 +187,7 @@ int win_in_viewport (WinData *win)
     if (win->desknum == globals.desknum) {
       /* win and screen intersect if they are not disjoint in x and y */
       flag = RECTANGLES_INTERSECT(
-	g.x, g.y, g.width, g.height,
+	win->x, win->y, win->width, win->height,
 	manager->managed_g.x, manager->managed_g.y,
 	manager->managed_g.width, manager->managed_g.height);
     }
@@ -251,6 +239,22 @@ static void handle_config_info (unsigned long *body)
   {
     FScreenConfigureModule(tline);
   }
+  else if (StrEquals(token, "IgnoreModifiers"))
+  {
+    sscanf(tline, "%d", &mods_unused);
+  }
+  else if (StrEquals(token, ROOT_BG_CHANGE_STRING))
+  {
+    int j;
+    WinManager *man;
+
+    for (j = 0; j < globals.num_managers; j++)
+    {
+      man = &globals.managers[j];
+      if (man->pixmap[DEFAULT] == ParentRelative)
+	XClearArea(theDisplay, man->theWindow, 0, 0, 0, 0, True);
+    }
+  }
 }
 
 static void configure_window (FvwmPacketBody *body)
@@ -349,6 +353,29 @@ static void icon_name (FvwmPacketBody *body)
 
   copy_string (&win->iconname, (char *)name);
   ConsoleDebug (FVWM, "new icon name: %s\n", win->iconname);
+
+  ConsoleDebug (FVWM, "Exiting icon_name\n");
+}
+
+static void visible_icon_name (FvwmPacketBody *body)
+{
+  WinData *win;
+  Ulong app_id;
+  Uchar *name = body->name_data.name.name;
+
+  ConsoleDebug (FVWM, "In visible_icon_name\n");
+
+  app_id = body->name_data.app_id;
+
+  win = id_to_win (app_id);
+
+  if (win->visible_icon_name && !strcmp (win->visible_icon_name, (char *)name)) {
+    ConsoleDebug (FVWM, "No icon change: %s %s\n", win->iconname, name);
+    return;
+  }
+
+  copy_string (&win->visible_icon_name, (char *)name);
+  ConsoleDebug (FVWM, "new visible icon name: %s\n", win->visible_icon_name);
   if (change_windows_manager (win) == 0 && win->button &&
       (win->manager->format_depend & ICON_NAME)) {
     if (win->manager->sort) {
@@ -379,25 +406,61 @@ static void window_name (FvwmPacketBody *body)
   }
 
   copy_string (&win->titlename, (char *)name);
+
+  ConsoleDebug (FVWM, "Exiting window_name\n");
+}
+
+static void visible_name (FvwmPacketBody *body)
+{
+  WinData *win;
+  Ulong app_id;
+  Uchar *name = body->name_data.name.name;
+
+  ConsoleDebug (FVWM, "In visible_name\n");
+
+  app_id = body->name_data.app_id;
+
+  win = id_to_win (app_id);
+
+  /* This is necessary because bash seems to update the window title on
+     every keystroke regardless of whether anything changes */
+  if (win->visible_name && !strcmp (win->visible_name, (char *)name)) {
+    ConsoleDebug (FVWM, "No visible name change: %s %s\n",
+		  win->visible_name, name);
+    return;
+  }
+
+  copy_string (&win->visible_name, (char *)name);
   if (change_windows_manager (win) == 0 && win->button &&
       (win->manager->format_depend & TITLE_NAME)) {
     if (win->manager->sort) {
       resort_windows_button (win);
     }
   }
-  ConsoleDebug (FVWM, "Exiting window_name\n");
+  ConsoleDebug (FVWM, "Exiting visible_name\n");
 }
 
+
 static void new_window (FvwmPacketBody *body)
 {
   WinData *win;
 
-  win = id_to_win (body->add_config_data.w);
+  win = new_windata();
   memcpy(&(win->flags), &(body->add_config_data.flags), sizeof(win->flags));
-  set_win_configuration (win, body);
-  got_configure (win->manager);
-  check_win_complete (win);
-  check_in_window (win);
+  if (!(IS_TRANSIENT(win)))
+  {
+    win->app_id = body->add_config_data.w;
+    win->app_id_set = 1;
+    set_win_configuration (win, body);
+
+    insert_win_hashtab (win);
+    check_win_complete (win);
+    check_in_window (win);
+  }
+  else
+  {
+    Free(win);
+  }
 }
 
 static void destroy_window (FvwmPacketBody *body)
@@ -440,32 +503,16 @@ static void mini_icon (FvwmPacketBody *body)
 }
 #endif
 
-static void icon_location(FvwmPacketBody *body)
+static void iconify (FvwmPacketBody *body, int dir)
 {
   Ulong app_id = body->minimal_data.app_id;
   WinData *win;
 
-  win = id_to_win(app_id);
-  if (win == NULL)
-  {
-    return;
-  }
-  win->icon_g.x = body->icon_data.xpos;
-  win->icon_g.y = body->icon_data.ypos;
-  win->icon_g.width = body->icon_data.icon_width;
-  win->icon_g.height = body->icon_data.icon_height;
-  check_in_window (win);
-}
+  win = id_to_win (app_id);
 
-static void iconify(FvwmPacketBody *body, int dir)
-{
-  Ulong app_id = body->minimal_data.app_id;
-  WinData *win;
+  set_win_iconified (win, dir);
 
-  win = id_to_win(app_id);
-  set_win_iconified(win, dir);
-  icon_location(body);
-  check_win_complete(win);
+  check_win_complete (win);
   check_in_window (win);
 }
 
@@ -554,11 +601,21 @@ static void ProcessMessage (Ulong type, FvwmPacketBody *body)
     window_name (body);
     break;
 
+  case M_VISIBLE_NAME:
+    ConsoleDebug (FVWM, "DEBUG::M_VISIBLE_NAME\n");
+    visible_name (body);
+    break;
+
   case M_ICON_NAME:
     ConsoleDebug (FVWM, "DEBUG::M_ICON_NAME\n");
     icon_name (body);
     break;
 
+  case MX_VISIBLE_ICON_NAME:
+    ConsoleDebug (FVWM, "DEBUG::M_VISIBLE_ICON_NAME\n");
+    visible_icon_name (body);
+    break;
+
   case M_DEICONIFY:
     ConsoleDebug (FVWM, "DEBUG::M_DEICONIFY\n");
     iconify (body, 0);
@@ -571,10 +628,6 @@ static void ProcessMessage (Ulong type, FvwmPacketBody *body)
     SendUnlockNotification(Fvwm_fd);
     break;
 
-  case M_ICON_LOCATION:
-    icon_location(body);
-    break;
-
   case M_END_WINDOWLIST:
     ConsoleDebug (FVWM, "DEBUG::M_END_WINDOWLIST\n");
     ConsoleDebug (FVWM,
