@@ -25,7 +25,7 @@
 #include <libs/Module.h>
 
 static char const rcsid[] =
-  "$Id: readconfig.c,v 1.38.2.3 2003/01/24 13:58:15 domivogt Exp $";
+  "$Id: readconfig.c,v 1.39 2002/01/07 12:30:23 domivogt Exp $";
 
 /************************************************************************
  *
@@ -726,7 +726,8 @@ Binding *ParseMouseEntry (char *tline)
 
   find_context(modifiers,&mods,key_modifiers,tline);
   if((mods & AnyModifier)&&(mods&(~AnyModifier))) {
-    ConsoleMessage ("Binding specified AnyModifier and other modifers too. Excess modifiers will be ignored.");
+    ConsoleMessage ("Binding specified AnyModifier and other modifers too. "
+                    "Excess modifiers will be ignored.");
   }
 
   new = (Binding *)safemalloc(sizeof(Binding));
@@ -897,8 +898,9 @@ static int GetConfigLineWrapper (int *fd, char **tline)
 
   GetConfigLine (fd, tline);
   if (*tline) {
-    if (strncasecmp(*tline, "Colorset", 8) == 0) {
-         LoadColorset(&(*tline)[8]);
+    if (strncasecmp(*tline, "Colorset", 8) == 0)
+    {
+      LoadColorset(&(*tline)[8]);
     }
     else if (strncasecmp(*tline, XINERAMA_CONFIG_STRING,
 			 sizeof(XINERAMA_CONFIG_STRING) - 1) == 0)
@@ -906,6 +908,10 @@ static int GetConfigLineWrapper (int *fd, char **tline)
       FScreenConfigureModule(
 	(*tline) + sizeof(XINERAMA_CONFIG_STRING) - 1);
     }
+    else if (strncasecmp(*tline, "IgnoreModifiers", 15) == 0)
+    {
+      sscanf((*tline) + 16, "%d", &mods_unused);
+    }
     temp = strchr (*tline, '\n');
     if (temp) {
       *temp = '\0';
@@ -928,8 +934,7 @@ static char *read_next_cmd (ReadOption flag)
 {
   static ReadOption status;
   static char *buffer;
-  static char *retstring, *cur_pos;
-  char *end_pos;
+  static char *retstring, displaced, *cur_pos;
 
   retstring = NULL;
   if (flag != READ_LINE && !(flag & status))
@@ -943,13 +948,13 @@ static char *read_next_cmd (ReadOption flag)
       if (!strncasecmp (Module, cur_pos, ModuleLen)) {
         retstring = cur_pos;
         cur_pos += ModuleLen;
-
-        if (*cur_pos == '*')
-          cur_pos++;
-        skip_space(&cur_pos);
-
-        if (isalnum(*cur_pos))
+        displaced = *cur_pos;
+        if (displaced == '*')
           status = READ_OPTION;
+        else if (displaced == '\0')
+          status = READ_LINE;
+        else if (iswhite (displaced))
+          status = READ_ARG;
         else
           status = READ_LINE;
         break;
@@ -958,43 +963,42 @@ static char *read_next_cmd (ReadOption flag)
     break;
 
   case READ_OPTION:
-    retstring = cur_pos;
+    *cur_pos = displaced;
+    retstring = ++cur_pos;
     while (*cur_pos != '*' && !iswhite (*cur_pos))
       cur_pos++;
-    end_pos = cur_pos;
-
-    if (*cur_pos == '*')
-      cur_pos++;
-    skip_space(&cur_pos);
-
-    if (!*cur_pos)
-      status = READ_LINE;
-    else if (isdigit(*retstring) || !strncmp(retstring, "transient", 9))
+    displaced = *cur_pos;
+    *cur_pos = '\0';
+    if (displaced == '*')
       status = READ_OPTION;
-    else if (*cur_pos)
-      status = READ_ARG;
-
-    *end_pos = '\0';
-    break;
-
-  case READ_ARG:
-    retstring = cur_pos;
-    while (!iswhite (*cur_pos))
-      cur_pos++;
-    end_pos = cur_pos;
-
-    skip_space(&cur_pos);
-
-    if (*cur_pos)
+    else if (displaced == '\0')
+      status = READ_LINE;
+    else if (iswhite (displaced))
       status = READ_ARG;
     else
       status = READ_LINE;
+    break;
 
-    *end_pos = '\0';
+  case READ_ARG:
+    *cur_pos = displaced;
+    skip_space (&cur_pos);
+    retstring = cur_pos;
+    while (!iswhite (*cur_pos))
+      cur_pos++;
+    displaced = *cur_pos;
+    *cur_pos = '\0';
+    if (displaced == '\0')
+      status = READ_LINE;
+    else if (iswhite (displaced))
+      status = READ_ARG;
+    else
+      status = READ_LINE;
     break;
 
   case READ_REST_OF_LINE:
     status = READ_LINE;
+    *cur_pos = displaced;
+    skip_space (&cur_pos);
     retstring = cur_pos;
     break;
   }
@@ -1395,11 +1399,11 @@ void read_in_resources (char *file)
 	ConsoleDebug (CONFIG, "Setting followfocus to: %d\n", i);
 	SET_MANAGER (manager, followFocus, i);
       }
-      else if (!strcasecmp (option1, "showonlyicons")) {
+      else if (!strcasecmp (option1, "showonlyiconic")) {
        p = read_next_cmd (READ_ARG);
        if (!p) {
          ConsoleMessage ("Bad line: %s\n", current_line);
-         ConsoleMessage ("Need argument to showonlyicons\n");
+         ConsoleMessage ("Need argument to showonlyiconic\n");
          continue;
        }
        if (!strcasecmp (p, "true")) {
@@ -1413,23 +1417,20 @@ void read_in_resources (char *file)
          ConsoleMessage ("What is this: %s?\n", p);
          continue;
        }
-       ConsoleDebug (CONFIG, "Setting showonlyicons to: %d\n", i);
+       ConsoleDebug (CONFIG, "Setting showonlyiconic to: %d\n", i);
        SET_MANAGER (manager, showonlyiconic, i);
       }
       else if (!strcasecmp (option1, "font")) {
-	char *f;
 	p = read_next_cmd (READ_REST_OF_LINE);
 	trim(p);
 	if (!p) {
 	  ConsoleMessage ("Bad line: %s\n", current_line);
 	  continue;
 	}
-	CopyStringWithQuotes(&f, p);
-	ConsoleDebug (CONFIG, "font: %s\n", f);
+	ConsoleDebug (CONFIG, "font: %s\n", p);
 
 	SET_MANAGER (manager, fontname,
-		     copy_string (&globals.managers[id].fontname, f));
-	free(f);
+		     copy_string (&globals.managers[id].fontname, p));
       }
       else if (!strcasecmp (option1, "foreground")) {
 	p = read_next_cmd (READ_ARG);
