@@ -23,7 +23,7 @@
 #include "libs/FShape.h"
 
 static char const rcsid[] =
-  "$Id: x.c,v 1.54 2001/09/23 07:03:58 olicha Exp $";
+  "$Id: x.c,v 1.57 2002/01/08 14:21:59 olicha Exp $";
 
 #define GRAB_EVENTS (ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|EnterWindowMask|LeaveWindowMask)
 
@@ -119,7 +119,7 @@ static void handle_buttonevent (XEvent *theEvent, WinManager *man)
 	if(((MouseEntry->Button_Key == theEvent->xbutton.button)||
 	    (MouseEntry->Button_Key == 0))&&
 	   ((MouseEntry->Modifier == AnyModifier)||
-	    (MouseEntry->Modifier == (modifier& (~LockMask)))))
+	    (MouseEntry->Modifier == (modifier& (~mods_unused)))))
 	{
 	  Function *ftype = (Function *)(MouseEntry->Action2);
 	  ConsoleDebug (X11, "\tgot a mouse binding\n");
@@ -207,6 +207,11 @@ void xevent_loop (void)
     }
 
     man = find_windows_manager (theEvent.xany.window);
+    if (!man && theEvent.type == ConfigureNotify)
+    {
+      /* swallowed icon man */
+      man = find_windows_manager (theEvent.xconfigure.window);
+    }
     if (!man) {
       ConsoleDebug (X11, "Event doesn't belong to a manager\n");
       continue;
@@ -232,7 +237,7 @@ void xevent_loop (void)
       for (key = man->bindings[KEYPRESS]; key != NULL; key = key->NextBinding)
       {
 	if ((key->Button_Key == theEvent.xkey.keycode) &&
-	    ((key->Modifier == (modifier&(~LockMask)))||
+	    ((key->Modifier == (modifier&(~mods_unused)))||
 	     (key->Modifier == AnyModifier)))
 	{
 	  Function *ftype = (Function *)(key->Action2);
@@ -309,7 +314,7 @@ void xevent_loop (void)
 	  moved = True;
       }
       theEvent = saveEvent;
-
+	      
       if (man->geometry.dir & GROW_FIXED)
       {
 	man->geometry.rows =
@@ -350,6 +355,8 @@ void xevent_loop (void)
       /* must refresh transparent windows when moved */
       if (moved && man->pixmap[DEFAULT] == ParentRelative)
       {
+	man->geometry.x = theEvent.xconfigure.x;
+	man->geometry.y = theEvent.xconfigure.y;
         XClearArea(theDisplay, man->theWindow, 0, 0, 0, 0, True);
       }
 
@@ -699,8 +706,14 @@ void X_init_manager (int man_id)
     unsigned int ujunk;
     fscreen_scr_arg fscr;
 
-    XQueryPointer(theDisplay, theRoot, &dummyroot, &dummychild,
-		  &man->geometry.x, &man->geometry.y, &junk, &junk, &ujunk);
+    if (XQueryPointer(
+	  theDisplay, theRoot, &dummyroot, &dummychild, &man->geometry.x,
+	  &man->geometry.y, &junk, &junk, &ujunk) == False)
+    {
+      /* pointer is on a different screen - that's okay here */
+      man->geometry.y = 0;
+      man->geometry.x = 0;
+    }
     fscr.xypos.x = man->geometry.x;
     fscr.xypos.y = man->geometry.y;
     FScreenGetScrRect(
