@@ -24,7 +24,7 @@
 #include "xmanager.h"
 
 static char const rcsid[] =
-  "$Id: xmanager.c,v 1.50.2.3 2002/10/09 23:07:04 domivogt Exp $";
+  "$Id: xmanager.c,v 1.53 2002/01/04 21:46:09 olicha Exp $";
 
 extern char *MyName;
 
@@ -342,7 +342,7 @@ static char *make_display_string (WinData *win, char *format, int len)
 {
 #define MAX_DISPLAY_SIZE 1024
 #define COPY(field)                                       \
-  temp_p = win->field;                                    \
+  temp_p = win->##field;                                  \
   if (temp_p)                                             \
     while (*temp_p && out_p - buf < len - 1) \
       *out_p++ = *temp_p++;                               \
@@ -361,11 +361,11 @@ static char *make_display_string (WinData *win, char *format, int len)
     if (*in_p == '%') {
       switch (*(++in_p)) {
       case 'i':
-	COPY (iconname);
+	COPY (visible_icon_name);
 	break;
 
       case 't':
-	COPY (titlename);
+	COPY (visible_name);
 	break;
 
       case 'r':
@@ -723,8 +723,8 @@ void set_win_displaystring (WinData *win)
   int maxlen;
 
   if (!man || ((man->format_depend & CLASS_NAME) && !win->classname)
-      || ((man->format_depend & ICON_NAME) && !win->iconname)
-      || ((man->format_depend & TITLE_NAME) && !win->titlename)
+      || ((man->format_depend & ICON_NAME) && !win->visible_icon_name)
+      || ((man->format_depend & TITLE_NAME) && !win->visible_name)
       || ((man->format_depend & RESOURCE_NAME) && !win->resname)) {
     return;
   }
@@ -1207,8 +1207,7 @@ void check_in_window (WinData *win)
   int is_state_selected;
 
   if (win->manager && win->complete) {
-    is_state_selected =
-      ((!win->manager->showonlyiconic || win->iconified) && !IS_TRANSIENT(win));
+    is_state_selected = (!win->manager->showonlyiconic || win->iconified);
     in_viewport = win_in_viewport (win);
     if (win->manager->usewinlist && DO_SKIP_WINDOW_LIST(win))
       in_viewport = 0;
@@ -1513,13 +1512,12 @@ void draw_manager (WinManager *man)
     update_geometry = 1;
     force_draw = 1;
   }
-  if (force_draw || man->buttons.num_windows != 0)
-    clear_empty_region (man);
 
   if (redraw_all || (man->dirty_flags & MAPPING_CHANGED)) {
     force_draw = 1;
     ConsoleDebug (X11, "manager %s: mapping changed\n", man->titlename);
   }
+
   if (FShapesSupported && man->shaped)
   {
     if (redraw_all || (man->dirty_flags & SHAPE_CHANGED)) {
@@ -1533,6 +1531,7 @@ void draw_manager (WinManager *man)
       update_geometry = 1;
     }
   }
+
   if (redraw_all || (man->dirty_flags & GEOMETRY_CHANGED)) {
     ConsoleDebug (X11, "\tredrawing all buttons\n");
     update_geometry = 1;
@@ -1543,6 +1542,15 @@ void draw_manager (WinManager *man)
       set_button_geometry (man, man->buttons.buttons[i]);
   }
 
+  if (force_draw || update_geometry)
+  {
+    clear_empty_region (man);
+    /* maybe not usefull but safe */
+    if (man->colorsets[DEFAULT] >= 0 &&
+	 Colorset[man->colorsets[DEFAULT]].pixmap == ParentRelative)
+      force_draw = True;
+  }
+
   man->dirty_flags = 0;
   man->buttons.dirty_flags = 0;
   man->buttons.drawn_num_buttons = man->buttons.num_buttons;
