@@ -65,6 +65,7 @@ int fd[2];
 PagerStringList *FindDeskStrings(int desk);
 PagerStringList *NewPagerStringItem(PagerStringList *last, int desk);
 extern XFontStruct *windowFont;
+extern Pixmap default_pixmap;
 
 /*************************************************************************
  *
@@ -98,6 +99,8 @@ unsigned int WindowBorderWidth = DEFAULT_PAGER_WINDOW_BORDER_WIDTH;
 unsigned int MinSize = 2 * DEFAULT_PAGER_WINDOW_BORDER_WIDTH + 1;
 Bool WindowBorders3d = False;
 
+Bool UseSkipList = False;
+
 Picture *PixmapBack = NULL;
 
 char *ImagePath = NULL;
@@ -149,7 +152,7 @@ Bool is_transient = False;
 Bool do_ignore_next_button_release = False;
 Bool error_occured = False;
 
-
+static void SetDeskLabel(int desk, const char *label);
 static RETSIGTYPE TerminateHandler(int);
 void ExitPager(void);
 
@@ -324,6 +327,7 @@ int main(int argc, char **argv)
 #endif
 
   SetMessageMask(fd,
+		 M_VISIBLE_NAME |
                  M_ADD_WINDOW|
                  M_CONFIGURE_WINDOW|
                  M_DESTROY_WINDOW|
@@ -337,25 +341,31 @@ int main(int argc, char **argv)
 		 M_DEICONIFY|
 		 M_RES_NAME|
 		 M_RES_CLASS|
-		 M_WINDOW_NAME|
-		 M_ICON_NAME|
 		 M_CONFIG_INFO|
 		 M_END_CONFIG_INFO|
 		 M_MINI_ICON|
 		 M_END_WINDOWLIST|
 		 M_RESTACK);
+  SetMessageMask(fd,
+		 MX_VISIBLE_ICON_NAME);
 #ifdef DEBUG
   fprintf(stderr,"[main]: calling ParseOptions\n");
 #endif
   ParseOptions();
   if (is_transient)
+  {
+    if (XQueryPointer(
+	  dpy, Scr.Root, &JunkRoot, &JunkChild, &window_x, &window_y, &JunkX,
+	  &JunkY, &JunkMask) == False)
     {
-      XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
-		    &window_x, &window_y, &JunkX, &JunkY, &JunkMask);
-      usposition = 1;
-      xneg = 0;
-      yneg = 0;
+      /* pointer is on a different screen */
+      window_x = 0;
+      window_y = 0;
     }
+    usposition = 1;
+    xneg = 0;
+    yneg = 0;
+  }
 #ifdef DEBUG
   fprintf(stderr,
 	  "[main]: back from calling ParseOptions, calling init pager\n");
@@ -538,10 +548,10 @@ void process_message( FvwmPacket* packet )
       break;
     case M_RES_CLASS:
     case M_RES_NAME:
-    case M_WINDOW_NAME:
+    case M_VISIBLE_NAME:
       list_window_name(body,type);
       break;
-    case M_ICON_NAME:
+    case MX_VISIBLE_ICON_NAME:
       list_icon_name(body);
       break;
     case M_MINI_ICON:
@@ -621,13 +631,6 @@ void list_add(unsigned long *body)
   (*prev)->border_width = cfgpacket->border_width;
   (*prev)->icon_w = cfgpacket->icon_w;
   (*prev)->icon_pixmap_w = cfgpacket->icon_pixmap_w;
-  if (IS_ICONIFIED(*prev))
-  {
-    (*prev)->icon_x = 0;
-    (*prev)->icon_y = 0;
-    (*prev)->icon_width = 0;
-    (*prev)->icon_height = 0;
-  }
 
   if (win_pix_set)
   {
@@ -636,8 +639,8 @@ void list_add(unsigned long *body)
   }
   else
   {
-    (*prev)->text = cfgpacket->TextPixel;
-    (*prev)->back = cfgpacket->BackPixel;
+    (*prev)->text = body[22];
+    (*prev)->back = body[23];
   }
   AddNewWindow(*prev);
 }
@@ -685,19 +688,19 @@ void list_configure(unsigned long *body)
   }
   else
   {
-    t->text = cfgpacket->TextPixel;
-    t->back = cfgpacket->BackPixel;
+    t->text = body[22];
+    t->back = body[23];
   }
-  if(IS_ICONIFIED(t))
+  if (IS_ICONIFIED(t))
   {
     t->x = t->icon_x;
     t->y = t->icon_y;
     t->width = t->icon_width;
     t->height = t->icon_height;
-    if(IS_ICON_SUPPRESSED(t) || t->width == 0 || t->height == 0)
+    if(IS_ICON_SUPPRESSED(t))
     {
-      t->x = -32768;
-      t->y = -32768;
+      t->x = -10000;
+      t->y = -10000;
     }
   }
   else
@@ -707,12 +710,14 @@ void list_configure(unsigned long *body)
     t->width = t->frame_width;
     t->height = t->frame_height;
   }
-  if(t->desk != cfgpacket->desk)
+  if (t->desk != cfgpacket->desk)
   {
     ChangeDeskForWindow(t, cfgpacket->desk);
   }
   else
+  {
     MoveResizePagerView(t, False);
+  }
 }
 
 /***********************************************************************
@@ -767,24 +772,15 @@ void list_focus(unsigned long *body)
   PagerWindow *t;
   Window target_w;
   extern Pixel focus_pix, focus_fore_pix;
-  Bool do_force_update = False;
-
   target_w = body[0];
+
   if (win_hi_pix_set)
   {
-    if (focus_pix != win_hi_back_pix || focus_fore_pix != win_hi_fore_pix)
-    {
-      do_force_update = True;
-    }
     focus_pix = win_hi_back_pix;
     focus_fore_pix = win_hi_fore_pix;
   }
   else
   {
-    if (focus_pix != body[4] || focus_fore_pix != body[3])
-    {
-      do_force_update = True;
-    }
     focus_pix = body[4];
     focus_fore_pix = body[3];
   }
@@ -793,12 +789,12 @@ void list_focus(unsigned long *body)
   {
     t = t->next;
   }
-  if (t != FocusWin || do_force_update)
+  if(t != FocusWin)
   {
-    if (FocusWin != NULL)
+    if(FocusWin != NULL)
       Hilight(FocusWin, False);
     FocusWin = t;
-    if (FocusWin != NULL)
+    if(FocusWin != NULL)
       Hilight(FocusWin, True);
   }
 }
@@ -969,25 +965,6 @@ void list_new_desk(unsigned long *body)
     XClearWindow(dpy, Desks[0].w);
     XClearWindow(dpy, Desks[0].title_w);
   } /* if (fAlwaysCurrentDesk && oldDesk != Scr.CurrentDesk) */
-  else if (!fAlwaysCurrentDesk)
-  {
-    int i;
-    char *name;
-    char line[100];
-
-    i = Scr.CurrentDesk - desk1;
-    if (i >= 0 && i < ndesks && Desks[i].label != NULL)
-    {
-      name = Desks[i].label;
-    }
-    else
-    {
-      sprintf(line, "Desk %d", Scr.CurrentDesk);
-      name = &(line[0]);
-    }
-      XStoreName(dpy, Scr.Pager_w, name);
-      XSetIconName(dpy, Scr.Pager_w, name);
-  }
 
   MovePage(True);
   DrawGrid(oldDesk - desk1,1);
@@ -1066,41 +1043,42 @@ void list_iconify(unsigned long *body)
 
   target_w = body[0];
   t = Start;
-  while (t != NULL && t->w != target_w)
-  {
-    t = t->next;
-  }
-  if (t != NULL)
-  {
-    t->t = (char *)body[2];
-    t->frame = body[1];
-    t->icon_x = body[3];
-    t->icon_y = body[4];
-    t->icon_width = body[5];
-    t->icon_height = body[6];
-    SET_ICONIFIED(t, True);
-    if (IS_ICON_SUPPRESSED(t) || t->icon_width == 0 ||
-	t->icon_height == 0)
+  while((t!= NULL)&&(t->w != target_w))
     {
-      t->x = -32768;
-      t->y = -32768;
+      t = t->next;
     }
-    else
+  if(t== NULL)
     {
-      t->x = t->icon_x;
-      t->y = t->icon_y;
+      return;
     }
-    t->width = t->icon_width;
-    t->height = t->icon_height;
-    /* if iconifying main pager window turn balloons on or off */
-    if ( t->w == Scr.Pager_w )
+  else
     {
-      ShowBalloons = ShowIconBalloons;
-    }
-    MoveResizePagerView(t, True);
-  }
+      t->t = (char *)body[2];
+      t->frame = body[1];
+      t->icon_x = body[3];
+      t->icon_y = body[4];
+      t->icon_width = body[5];
+      t->icon_height = body[6];
+      SET_ICONIFIED(t, True);
+      if(IS_ICON_SUPPRESSED(t))
+      {
+	t->x = -32768;
+	t->y = -32768;
+      }
+      else
+      {
+	t->x = t->icon_x;
+	t->y = t->icon_y;
+      }
+      t->width = t->icon_width;
+      t->height = t->icon_height;
 
-  return;
+      /* if iconifying main pager window turn balloons on or off */
+      if ( t->w == Scr.Pager_w )
+	ShowBalloons = ShowIconBalloons;
+
+      MoveResizePagerView(t, True);
+    }
 }
 
 
@@ -1176,7 +1154,7 @@ void list_window_name(unsigned long *body,unsigned long type)
           free(t->res_name);
         CopyString(&t->res_name,(char *)(&body[3]));
         break;
-      case M_WINDOW_NAME:
+      case M_VISIBLE_NAME:
         if(t->window_name != NULL)
           free(t->window_name);
         CopyString(&t->window_name,(char *)(&body[3]));
@@ -1371,6 +1349,33 @@ void list_config_info(unsigned long *body)
   {
     FScreenConfigureModule(tline);
   }
+  else if (StrEquals(token, ROOT_BG_CHANGE_STRING))
+  {
+    if (default_pixmap == ParentRelative)
+      ReConfigure();
+  }
+  else if (StrEquals(token, "DesktopName"))
+  {
+    int val;
+    if (GetIntegerArguments(tline, &tline, &val, 1) > 0)
+    {
+      SetDeskLabel(val, (const char *)tline);
+    }
+    else
+    {
+      return;
+    }
+    if (fAlwaysCurrentDesk)
+    {
+      if (Scr.CurrentDesk == val)
+	val = 0;
+    }
+    else if ((val >= desk1) && (val <=desk2))
+    {
+      val = val - desk1;
+    }
+    DrawGrid(val, True);
+  }
 }
 
 
@@ -1491,6 +1496,42 @@ static void ParseColorset(char *arg1, char *arg2, void *offset_deskinfo,
   return;
 }
 
+static void SetDeskLabel(int desk, const char *label)
+{
+  PagerStringList *item;
+
+  if (fAlwaysCurrentDesk)
+  {
+    item = FindDeskStrings(desk);
+    if (item->next != NULL)
+    {
+      /* replace label */
+      if (item->next->label != NULL)
+      {
+	free(item->next->label);
+	item->next->label = NULL;
+      }
+      CopyString(&(item->next->label), label);
+    }
+    else
+    {
+      /* new Dcolor and desktop */
+      item = NewPagerStringItem(item, desk);
+      CopyString(&(item->label), label);
+    }
+    if (desk == Scr.CurrentDesk)
+    {
+      free(Desks[0].label);
+      CopyString(&Desks[0].label, label);
+    }
+  }
+  else if((desk >= desk1)&&(desk <=desk2))
+  {
+    free(Desks[desk - desk1].label);
+    CopyString(&Desks[desk - desk1].label, label);
+  }
+}
+
 /*****************************************************************************
  *
  * This routine is responsible for reading and parsing the config file
@@ -1577,6 +1618,15 @@ void ParseOptions(void)
       }
       continue;
     }
+    else if (StrEquals(token, "DesktopName"))
+    {
+      int val;
+      if (GetIntegerArguments(next, &next, &val, 1) > 0)
+      {
+	SetDeskLabel(val, (const char *)next);
+      }
+      continue;
+    }
 
     tline2 = GetModuleResource(tline, &resource, MyName);
     if (!resource)
@@ -1693,38 +1743,7 @@ void ParseOptions(void)
 	desk = desk1;
 	sscanf(arg1,"%d",&desk);
       }
-      if (fAlwaysCurrentDesk)
-      {
-	PagerStringList *item;
-
-	item = FindDeskStrings(desk);
-	if (item->next != NULL)
-	{
-	  /* replace label */
-	  if (item->next->label != NULL)
-	  {
-	    free(item->next->label);
-	    item->next->label = NULL;
-	  }
-	  CopyString(&(item->next->label), arg2);
-	}
-	else
-	{
-	  /* new Dcolor and desktop */
-	  item = NewPagerStringItem(item, desk);
-	  CopyString(&(item->label), arg2);
-	}
-	if (desk == Scr.CurrentDesk)
-	{
-	  free(Desks[0].label);
-	  CopyString(&Desks[0].label, arg2);
-	}
-      }
-      else if((desk >= desk1)&&(desk <=desk2))
-      {
-	free(Desks[desk - desk1].label);
-	CopyString(&Desks[desk - desk1].label, arg2);
-      }
+      SetDeskLabel(desk, (const char *)arg2);
     }
     else if (StrEquals(resource, "Font"))
     {
@@ -2042,6 +2061,10 @@ void ParseOptions(void)
 	free(WindowLabelFormat);
       CopyString(&WindowLabelFormat,arg1);
     }
+    else if (StrEquals(resource,"UseSkipList"))
+    {
+      UseSkipList = True;
+    }
     else if (StrEquals(resource, "MoveThreshold"))
     {
       int val;
@@ -2115,7 +2138,7 @@ void ParseOptions(void)
     {
       if (BalloonFont)
 	free(BalloonFont);
-      CopyStringWithQuotes(&BalloonFont, next);
+      CopyString(&BalloonFont, next);
     }
 
     else if (StrEquals(resource, "BalloonBorderColor"))
