@@ -75,6 +75,10 @@ typedef struct pager_window
   Pixel text;
   Pixel back;
   window_flags flags;
+  struct
+  {
+    unsigned is_mapped : 1;
+  } myflags;
   Window icon_w;
   Window icon_pixmap_w;
   char *icon_name;
@@ -87,8 +91,6 @@ typedef struct pager_window
   int pager_view_y;
   int pager_view_width;
   int pager_view_height;
-  int icon_view_x;
-  int icon_view_y;
   int icon_view_width;
   int icon_view_height;
 
