@@ -10,12 +10,12 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 /***********************************************************************
@@ -69,6 +69,7 @@ extern Window BalloonView;
 extern unsigned int WindowBorderWidth;
 extern unsigned int MinSize;
 extern Bool WindowBorders3d;
+extern Bool UseSkipList;
 extern Picture *PixmapBack;
 extern Picture *HilightPixmap;
 extern int HilightDesks;
@@ -124,12 +125,12 @@ static char l_g_bits[] = {0x08, 0x02};
 static char s_g_bits[] = {0x01, 0x02, 0x04, 0x08};
 
 
-Window icon_win;	       /* icon window */
+Window icon_win;               /* icon window */
 
 static char *GetBalloonLabel(const PagerWindow *pw,const char *fmt);
 extern void ExitPager(void);
 
-static Pixmap default_pixmap = None;
+Pixmap default_pixmap = None;
 
 /* discard certain events on a window */
 static void discard_events(long event_type, Window w, XEvent *last_ev)
@@ -156,10 +157,10 @@ static void discard_events(long event_type, Window w, XEvent *last_ev)
  *	given the real window size.
  *	You can always tell bad code by the size of the comments.
  *
- *	dv: Some people say that good code does not nee lengthy comments. :-)
+ *      dv: Some people say that good code does not nee lengthy comments. :-)
  ***********************************************************************/
 static void CalcGeom(PagerWindow *t, int win_w, int win_h,
-		     int *x_ret, int *y_ret, int *w_ret, int *h_ret)
+                     int *x_ret, int *y_ret, int *w_ret, int *h_ret)
 {
   int virt, virt2, edge, edge2, size, page, over;
 
@@ -202,14 +203,14 @@ static void CalcGeom(PagerWindow *t, int win_w, int win_h,
     if (page == ((virt - t->width + 1) / Scr.MyDisplayWidth)) {
       /* calculate how far the mini-window right edge overlaps the page line */
       /* beware that the "over" is actually one greater than on screen, but
-	 this discrepancy is catered for in the next two lines */
+         this discrepancy is catered for in the next two lines */
       over = edge + size - ((page + 1) * win_w * Scr.MyDisplayWidth) /
 	Scr.VWidth;
 
       /* if the mini-window right edge is beyond the mini-window pager grid */
       if (over > 0) {
-	/* move it left by the amount of pager grid overlap (!== the growth) */
-	edge -= over;
+        /* move it left by the amount of pager grid overlap (!== the growth) */
+        edge -= over;
       }
     }
   }
@@ -233,7 +234,7 @@ static void CalcGeom(PagerWindow *t, int win_w, int win_h,
       over = edge + size - ((page + 1) * win_h * Scr.MyDisplayHeight) /
 	Scr.VHeight;
       if (over > 0)
-	edge -= over;
+        edge -= over;
     }
   }
   *y_ret = edge;
@@ -244,7 +245,7 @@ static void CalcGeom(PagerWindow *t, int win_w, int win_h,
  *
  *  Procedure:
  *	Initialize_viz_pager - creates a temp window of the correct visual
- *	so that pixmaps may be created for use with the main window
+ *      so that pixmaps may be created for use with the main window
  ***********************************************************************/
 void initialize_viz_pager(void)
 {
@@ -272,64 +273,6 @@ void initialize_viz_pager(void)
     default_pixmap = ParentRelative;
 }
 
-void draw_desk_background(int i, int page_w, int page_h)
-{
-	if (Desks[i].highcolorset > -1)
-	{
-		XSetForeground(
-			dpy, Desks[i].HiliteGC,
-			Colorset[Desks[i].highcolorset].bg);
-		XSetForeground(
-			dpy, Desks[i].rvGC, Colorset[Desks[i].highcolorset].fg);
-		if (HilightDesks && Scr.CurrentDesk - desk1 == i)
-		{
-			SetWindowBackground(
-				dpy, Desks[i].CPagerWin, page_w, page_h,
-				&Colorset[Desks[i].highcolorset], Pdepth,
-				Scr.NormalGC, True);
-		}
-	}
-	if (Desks[i].colorset > -1)
-	{
-		XSetWindowBorder(
-			dpy, Desks[i].title_w, Colorset[Desks[i].colorset].fg);
-		XSetWindowBorder(
-			dpy, Desks[i].w, Colorset[Desks[i].colorset].fg);
-		XSetForeground(
-			dpy, Desks[i].NormalGC,Colorset[Desks[i].colorset].fg);
-		XSetForeground(
-			dpy, Desks[i].DashedGC,Colorset[Desks[i].colorset].fg);
-		if (uselabel)
-		{
-			SetWindowBackground(
-				dpy, Desks[i].title_w, desk_w, desk_h + label_h,
-				&Colorset[Desks[i].colorset], Pdepth,
-				Scr.NormalGC, True);
-		}
-		if (label_h != 0 && uselabel && !LabelsBelow)
-		{
-			SetWindowBackgroundWithOffset(
-				dpy, Desks[i].w, 0, -label_h, desk_w,
-				desk_h + label_h, &Colorset[Desks[i].colorset],
-				Pdepth, Scr.NormalGC, True);
-		}
-		else
-		{
-			SetWindowBackground(
-				dpy, Desks[i].w, desk_w, desk_h + label_h,
-				&Colorset[Desks[i].colorset], Pdepth,
-				Scr.NormalGC, True);
-		}
-	}
-	XClearArea(dpy,Desks[i].w, 0, 0, 0, 0,True);
-	if (uselabel)
-	{
-		XClearArea(dpy,Desks[i].title_w, 0, 0, 0, 0,True);
-	}
-
-	return;
-}
-
 /***********************************************************************
  *
  *  Procedure:
@@ -343,13 +286,13 @@ char *pager_name = "Fvwm Pager";
 XSizeHints sizehints =
 {
   (PMinSize | PResizeInc | PBaseSize | PWinGravity),
-  0, 0, 100, 100,			/* x, y, width and height */
-  1, 1,					/* Min width and height */
-  0, 0,					/* Max width and height */
-  1, 1,					/* Width and height increments */
-  {0, 0}, {0, 0},			/* Aspect ratio - not used */
-  1, 1,					/* base size */
-  (NorthWestGravity)			/* gravity */
+  0, 0, 100, 100,	                /* x, y, width and height */
+  1, 1,		                        /* Min width and height */
+  0, 0,		                        /* Max width and height */
+  1, 1,	                         	/* Width and height increments */
+  {0, 0}, {0, 0},                       /* Aspect ratio - not used */
+  1, 1,                 		/* base size */
+  (NorthWestGravity)                    /* gravity */
 };
 
 void initialize_pager(void)
@@ -358,8 +301,6 @@ void initialize_pager(void)
   XClassHint class1;
 #ifdef I18N_MB
   char **ml;
-  int mc;
-  char *ds;
   XFontStruct **fs_list;
 #endif
 
@@ -386,29 +327,40 @@ void initialize_pager(void)
   wm_del_win = XInternAtom(dpy,"WM_DELETE_WINDOW",False);
 
   /* load the font */
+  /* Note: "font" is always created, whether labels are used or not
+     because a GC below is set to use a font. dje Dec 2001.
+     OK, I fixed the GC below, but now something else is blowing up.
+     Right now, I've got to do some Real Life stuff, so this kludge is
+     in place, its still better than I found it.
+  */
+  if (!uselabel) {
 #ifdef I18N_MB
-  if (!uselabel || ((fontset =
-		     XCreateFontSet(dpy, font_string, &ml, &mc, &ds)) == NULL))
+#ifdef STRICTLY_FIXED
+#define FALLBACK_FONT "fixed"
+#else
+#define FALLBACK_FONT "-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*"
+#endif
+#else
+#define FALLBACK_FONT "fixed"
+#endif
+    font_string = safestrdup(FALLBACK_FONT);
+  }    
+#ifdef I18N_MB
+  fontset = GetFontSetOrFixed(dpy, font_string);
+  if (fontset == NULL)
   {
-    if ((fontset =
-	 XCreateFontSet(dpy, "-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*",
-			&ml, &mc, &ds)) == NULL)
-    {
-      fprintf(stderr,"%s: No fonts available\n",MyName);
-      exit(1);
-    }
+    fprintf(stderr,"%s: No I18N fonts available, giving up\n",MyName);
+    exit(1);
   }
   XFontsOfFontSet(fontset, &fs_list, &ml);
   font = fs_list[0];
 #else
-  if (!uselabel || ((font = XLoadQueryFont(dpy, font_string)) == NULL))
+  font = GetFontOrFixed(dpy, font_string);
+  if (font == NULL)
   {
-    if ((font = XLoadQueryFont(dpy, "fixed")) == NULL)
-    {
-      fprintf(stderr,"%s: No fonts available\n",MyName);
-      exit(1);
-    }
-  };
+    fprintf(stderr,"%s: No fonts available, giving up\n",MyName);
+    exit(1);
+  }
 #endif
   label_h = (uselabel) ? font->ascent + font->descent + 2 : 0;
 
@@ -419,30 +371,22 @@ void initialize_pager(void)
   }
 
 #ifdef I18N_MB
+  windowFontset = NULL;
+  windowFont = NULL;
   if(smallFont != NULL)
   {
-    windowFontset = XCreateFontSet(dpy, smallFont, &ml, &mc, &ds);
+    windowFontset = GetFontSetOrFixed(dpy,smallFont);
     if (windowFontset != NULL)
     {
       XFontsOfFontSet(windowFontset, &fs_list, &ml);
       windowFont = fs_list[0];
-    } else {
-      windowFontset = NULL;
-      windowFont = NULL;
     }
   }
-  else
-  {
-    windowFontset = NULL;
-    windowFont = NULL;
-  }
 #else
   if(smallFont!= NULL)
   {
-    windowFont= XLoadQueryFont(dpy, smallFont);
+    windowFont= GetFontOrFixed(dpy, smallFont);
   }
-  else
-    windowFont= NULL;
 #endif
 
   /* Load the colors */
@@ -553,8 +497,6 @@ void initialize_pager(void)
   {
     window_h = Rows * (Scr.VHeight / Scr.VScale + m + label_h + 1) - 1;
   }
-  desk_w = (window_w - Columns + 1) / Columns;
-  desk_h = (window_h - Rows * label_h - Rows + 1) / Rows;
 
   if (is_transient)
   {
@@ -721,18 +663,22 @@ void initialize_pager(void)
 
   for(i=0;i<ndesks;i++)
   {
-    w = window_w / Columns;
-    h = window_h / Rows;
-    x = (w + 1) * (i % Columns);
-    y = (h + 1) * (i / Columns);
+    w = window_w/ndesks;
+    h = window_h;
+    x = w*i;
+    y = 0;
 
     /* create the GC for desk labels */
     gcv.foreground = (Desks[i].colorset < 0) ? fore_pix
       : Colorset[Desks[i].colorset].fg;
-    gcv.font = font->fid;
-    Desks[i].NormalGC =
-      fvwmlib_XCreateGC(dpy, Scr.Pager_w, GCForeground | GCFont, &gcv);
-
+    if (uselabel) {
+      gcv.font = font->fid;
+      Desks[i].NormalGC =
+        fvwmlib_XCreateGC(dpy, Scr.Pager_w, GCForeground | GCFont, &gcv);
+    } else {
+      Desks[i].NormalGC =
+        fvwmlib_XCreateGC(dpy, Scr.Pager_w, GCForeground, &gcv);
+    }
     /* create the active desk hilite GC */
     if(Pdepth < 2)
       gcv.foreground = fore_pix;
@@ -748,9 +694,13 @@ void initialize_pager(void)
     else
       gcv.foreground = (Desks[i].highcolorset < 0) ? fore_pix
 	: Colorset[Desks[i].highcolorset].fg;
-    Desks[i].rvGC =
-      fvwmlib_XCreateGC(dpy, Scr.Pager_w, GCForeground | GCFont, &gcv);
-
+    if (uselabel) {
+      Desks[i].rvGC =
+        fvwmlib_XCreateGC(dpy, Scr.Pager_w, GCForeground | GCFont, &gcv);
+    } else {
+      Desks[i].rvGC =
+        fvwmlib_XCreateGC(dpy, Scr.Pager_w, GCForeground, &gcv);
+    }
     /* create the virtual page boundary GC */
     gcv.foreground = (Desks[i].colorset < 0) ? fore_pix
       : Colorset[Desks[i].colorset].fg;
@@ -772,23 +722,23 @@ void initialize_pager(void)
     valuemask = (CWBorderPixel | CWColormap | CWEventMask);
     if (Desks[i].colorset >= 0 && Colorset[Desks[i].colorset].pixmap)
     {
-	valuemask |= CWBackPixmap;
-	attributes.background_pixmap = Colorset[Desks[i].colorset].pixmap;
+        valuemask |= CWBackPixmap;
+        attributes.background_pixmap = Colorset[Desks[i].colorset].pixmap;
     }
     else
     {
-	valuemask |= CWBackPixel;
-	attributes.background_pixel = (Desks[i].colorset < 0) ?
-	    (Desks[i].Dcolor ? GetColor(Desks[i].Dcolor) : back_pix)
-	    : Colorset[Desks[i].colorset].bg;
+        valuemask |= CWBackPixel;
+        attributes.background_pixel = (Desks[i].colorset < 0) ?
+            (Desks[i].Dcolor ? GetColor(Desks[i].Dcolor) : back_pix)
+            : Colorset[Desks[i].colorset].bg;
     }
 
     attributes.border_pixel = (Desks[i].colorset < 0) ? fore_pix
       : Colorset[Desks[i].colorset].fg;
     attributes.event_mask = (ExposureMask | ButtonReleaseMask);
-    Desks[i].title_w = XCreateWindow(
-      dpy, Scr.Pager_w, x - 1, y - 1, w, h, 1, CopyFromParent, InputOutput,
-      CopyFromParent, valuemask, &attributes);
+    Desks[i].title_w = XCreateWindow(dpy, Scr.Pager_w, x, y, w, h, 1,
+				     CopyFromParent, InputOutput,
+				     CopyFromParent, valuemask, &attributes);
     attributes.event_mask = (ExposureMask | ButtonReleaseMask |
 			     ButtonPressMask |ButtonMotionMask);
     desk_h = (window_h - Rows * label_h - Rows + 1) / Rows;
@@ -824,9 +774,17 @@ void initialize_pager(void)
 	: Colorset[Desks[i].colorset].bg;
     }
 
-    Desks[i].w = XCreateWindow(
-      dpy, Desks[i].title_w, x - 1, LabelsBelow ? -1 : label_h - 1, w, desk_h,
-      1, CopyFromParent, InputOutput, CopyFromParent, valuemask, &attributes);
+    Desks[i].w = XCreateWindow(dpy, Desks[i].title_w, x, y, w, desk_h, 1,
+			       CopyFromParent, InputOutput, CopyFromParent,
+			       valuemask, &attributes);
+    if (Desks[i].colorset > -1 &&
+	Colorset[Desks[i].colorset].pixmap)
+    {
+      SetWindowBackground(
+          dpy, Desks[i].w, w, desk_h, &Colorset[Desks[i].colorset], Pdepth,
+          Scr.NormalGC, True);
+    }
+
 
     if (HilightDesks)
     {
@@ -852,18 +810,20 @@ void initialize_pager(void)
 	  : Colorset[Desks[i].highcolorset].bg;
       }
 
-      w = w / (n + 1);
-      h = desk_h / (m + 1);
+      w = (window_w - n)/(n+1);
+      h = (window_h - label_h - m)/(m+1);
       Desks[i].CPagerWin=XCreateWindow(dpy, Desks[i].w, -32768, -32768, w, h, 0,
 				       CopyFromParent, InputOutput,
 				       CopyFromParent, valuemask, &attributes);
-      draw_desk_background(i, w, h);
+      if (Desks[i].highcolorset > -1 &&
+	  Colorset[Desks[i].highcolorset].pixmap)
+      {
+	SetWindowBackground(
+            dpy, Desks[i].CPagerWin, w, h, &Colorset[Desks[i].highcolorset],
+            Pdepth, Scr.NormalGC, True);
+      }
       XMapRaised(dpy,Desks[i].CPagerWin);
     }
-    else
-    {
-      draw_desk_background(i, 0, 0);
-    }
 
     XMapRaised(dpy,Desks[i].w);
     XMapRaised(dpy,Desks[i].title_w);
@@ -888,28 +848,21 @@ void initialize_pager(void)
 	? balloon_back_pix
 	: Colorset[Desks[i].ballooncolorset].bg;
       if (Desks[i].ballooncolorset > -1 &&
-	  Colorset[Desks[i].ballooncolorset].pixmap)
+          Colorset[Desks[i].ballooncolorset].pixmap)
       {
-	valuemask |= CWBackPixmap;
-	valuemask &= ~CWBackPixel;
-	attributes.background_pixmap = None; /* set later */
+        valuemask |= CWBackPixmap;
+        valuemask &= ~CWBackPixel;
+        attributes.background_pixmap = None; /* set later */
       }
 
       /* get font for balloon */
 #ifdef I18N_MB
-      if ( (Desks[i].balloon.fontset = XCreateFontSet(dpy, BalloonFont, &ml,
-						      &mc, &ds)) == NULL ) {
-	if ( (Desks[i].balloon.fontset =
-	      XCreateFontSet(dpy,
-			     "-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*",
-			     &ml, &mc, &ds)) == NULL )
+      if ( (Desks[i].balloon.fontset = GetFontSetOrFixed(dpy, BalloonFont))
+           == NULL ) {
 	{
-	  fprintf(stderr,"%s: No fonts available.\n", MyName);
+	  fprintf(stderr,"%s: No fonts available, giving up!.\n", MyName);
 	  exit(1);
 	}
-	fprintf(stderr, "%s: Can't find font '%s', using: \n"
-		"\t-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*\n",
-		MyName, BalloonFont);
       }
       XFontsOfFontSet(Desks[i].balloon.fontset, &fs_list, &ml);
       Desks[i].balloon.font = fs_list[0];
@@ -945,11 +898,6 @@ void initialize_pager(void)
 	Desks[i].balloon.yoffset = 3;
       }
 
-      /* tell WM to ignore this window */
-      attributes.override_redirect = True;
-      attributes.event_mask = ExposureMask;
-      attributes.colormap = Pcmap;
-
       /* now create the window */
       Desks[i].balloon.w = XCreateWindow(dpy, Scr.Root,
 					 0, 0, /* coords set later */
@@ -970,7 +918,7 @@ void initialize_pager(void)
 				     GCFont | GCForeground, &gcv);
 /* don't do this yet, wait for map since size will change
       if (Desks[i].ballooncolorset > -1 &&
-	  Colorset[Desks[i].ballooncolorset].pixmap)
+          Colorset[Desks[i].ballooncolorset].pixmap)
       {
 	SetWindowBackground(dpy, Desks[i].balloon.w, 100,
 			    Desks[i].balloon.height,
@@ -1075,8 +1023,8 @@ void DispatchEvent(XEvent *Event)
     if (w != icon_win)
     {
       /* icon_win is not handled here */
-      discard_events(Expose, w, NULL);
       ReConfigure();
+      discard_events(Expose, w, NULL);
     }
     break;
   case Expose:
@@ -1129,15 +1077,21 @@ void DispatchEvent(XEvent *Event)
       {
 	if(Event->xany.window == Desks[i].w)
 	{
-	  XQueryPointer(dpy, Desks[i].w, &JunkRoot, &JunkChild,
-			&JunkX, &JunkY,&x, &y, &JunkMask);
+	  if (XQueryPointer(dpy, Desks[i].w, &JunkRoot, &JunkChild,
+			    &JunkX, &JunkY,&x, &y, &JunkMask) == False)
+	  {
+	    /* pointer is on a different screen - that's okay here */
+	  }
 	  Scroll(desk_w, desk_h, x, y, i, False);
 	}
       }
       if(Event->xany.window == icon_win)
       {
-	XQueryPointer(dpy, icon_win, &JunkRoot, &JunkChild,
-		      &JunkX, &JunkY,&x, &y, &JunkMask);
+	if (XQueryPointer(dpy, icon_win, &JunkRoot, &JunkChild,
+			  &JunkX, &JunkY,&x, &y, &JunkMask) == False)
+	{
+	  /* pointer is on a different screen - that's okay here */
+	}
 	Scroll(icon_w, icon_h, x, y, 0, True);
       }
     }
@@ -1179,8 +1133,11 @@ void DispatchEvent(XEvent *Event)
       {
 	if(Event->xany.window == Desks[i].w)
 	{
-	  XQueryPointer(dpy, Desks[i].w, &JunkRoot, &JunkChild,
-			&JunkX, &JunkY,&x, &y, &JunkMask);
+	  if (XQueryPointer(dpy, Desks[i].w, &JunkRoot, &JunkChild,
+			    &JunkX, &JunkY,&x, &y, &JunkMask) == False)
+	  {
+	    /* pointer is on a different screen - that's okay here */
+	  }
 	  Scroll(desk_w, desk_h, x, y, Scr.CurrentDesk, False);
 	  if (Scr.CurrentDesk != i + desk1)
 	  {
@@ -1192,8 +1149,11 @@ void DispatchEvent(XEvent *Event)
       }
       if(Event->xany.window == icon_win)
       {
-	XQueryPointer(dpy, icon_win, &JunkRoot, &JunkChild,
-		      &JunkX, &JunkY,&x, &y, &JunkMask);
+	if (XQueryPointer(dpy, icon_win, &JunkRoot, &JunkChild,
+			  &JunkX, &JunkY,&x, &y, &JunkMask) == False)
+	{
+	  /* pointer is on a different screen - that's okay here */
+	}
 	Scroll(icon_w, icon_h, x, y, 0, True);
       }
     }
@@ -1209,15 +1169,21 @@ void DispatchEvent(XEvent *Event)
       {
 	if(Event->xany.window == Desks[i].w)
 	{
-	  XQueryPointer(dpy, Desks[i].w, &JunkRoot, &JunkChild,
-			&JunkX, &JunkY,&x, &y, &JunkMask);
+	  if (XQueryPointer(dpy, Desks[i].w, &JunkRoot, &JunkChild,
+			    &JunkX, &JunkY,&x, &y, &JunkMask) == False)
+	  {
+	    /* pointer is on a different screen - that's okay here */
+	  }
 	  Scroll(desk_w, desk_h, x, y, i, False);
 	}
       }
       if(Event->xany.window == icon_win)
       {
-	XQueryPointer(dpy, icon_win, &JunkRoot, &JunkChild,
-		      &JunkX, &JunkY,&x, &y, &JunkMask);
+	if (XQueryPointer(dpy, icon_win, &JunkRoot, &JunkChild,
+			  &JunkX, &JunkY,&x, &y, &JunkMask) == False)
+	{
+	  /* pointer is on a different screen - that's okay here */
+	}
 	Scroll(icon_w, icon_h, x, y, 0, True);
       }
 
@@ -1327,18 +1293,12 @@ void ReConfigure(void)
   Window root;
   unsigned border_width, depth;
   int n,m,w,h,n1,m1,x,y,i,j,k;
-  int old_ww;
-  int old_wh;
-  int is_size_changed;
 
-  old_ww = window_w;
-  old_wh = window_h;
   if (!XGetGeometry(dpy, Scr.Pager_w, &root, &x, &y, (unsigned *)&window_w,
 		    (unsigned *)&window_h, &border_width,&depth))
   {
     return;
   }
-  is_size_changed = (old_ww != window_w || old_wh != window_h);
 
   n1 = Scr.Vx / Scr.MyDisplayWidth;
   m1 = Scr.Vy / Scr.MyDisplayHeight;
@@ -1379,14 +1339,19 @@ void ReConfigure(void)
 
       if(i<ndesks)
       {
-	XMoveResizeWindow(
-	  dpy,Desks[i].title_w, (desk_w+1)*j-1,(desk_h+label_h+1)*k-1,
-	  desk_w,desk_h+label_h);
-	XMoveResizeWindow(
-	  dpy,Desks[i].w, -1, (LabelsBelow) ? -1 : label_h - 1, desk_w,desk_h);
-	if (!is_size_changed)
-	{
-	  continue;
+	int y_pos;
+	XMoveResizeWindow(dpy,Desks[i].title_w,
+			  (desk_w+1)*j-1,(desk_h+label_h+1)*k-1,
+			  desk_w,desk_h+label_h);
+	y_pos = (LabelsBelow ? 0 : label_h - 1);
+	XMoveResizeWindow(dpy,Desks[i].w,-1, y_pos,
+			  desk_w,desk_h);
+        if (Desks[i].colorset > -1 &&
+            Colorset[Desks[i].colorset].pixmap)
+        {
+          SetWindowBackground(
+              dpy, Desks[i].w, desk_w, desk_h, &Colorset[Desks[i].colorset],
+              Pdepth, Scr.NormalGC, True);
 	}
 	if (HilightDesks)
 	{
@@ -1394,8 +1359,18 @@ void ReConfigure(void)
 	    XMoveResizeWindow(dpy, Desks[i].CPagerWin, x,y,w,h);
 	  else
 	    XMoveResizeWindow(dpy, Desks[i].CPagerWin, -32768, -32768,w,h);
+	  if (Desks[i].highcolorset > -1 &&
+	      Colorset[Desks[i].highcolorset].pixmap)
+	  {
+	    SetWindowBackground(
+                dpy, Desks[i].CPagerWin, w, h, &Colorset[Desks[i].highcolorset],
+                Pdepth, Scr.NormalGC, True);
+	  }
 	}
-	draw_desk_background(i, w, h);
+
+	XClearArea(dpy,Desks[i].w, 0, 0, 0, 0,True);
+	if(uselabel)
+	  XClearArea(dpy,Desks[i].title_w, 0, 0, 0, 0,True);
       }
     }
   }
@@ -1429,6 +1404,9 @@ void MovePage(Bool is_new_desk)
       {
 	XMoveWindow(dpy, Desks[i].CPagerWin, x,y);
 	XLowerWindow(dpy,Desks[i].CPagerWin);
+	if (Desks[i].highcolorset > -1 &&
+	    Colorset[Desks[i].highcolorset].pixmap == ParentRelative)
+	  XClearWindow(dpy, Desks[i].CPagerWin);
       }
       else
       {
@@ -1453,7 +1431,7 @@ void MovePage(Bool is_new_desk)
     }
 #ifdef I18N_MB
     if ((ret = XmbTextListToTextProperty(dpy,&sptr,1,XStdICCTextStyle,&name))
-	== XNoMemory)
+        == XNoMemory)
     {
       fprintf(stderr,"%s: cannot allocate window name",MyName);
       return;
@@ -1494,8 +1472,6 @@ void ReConfigureIcons(Bool do_reconfigure_desk_only)
     if (do_reconfigure_desk_only && t->desk != Scr.CurrentDesk)
       continue;
     CalcGeom(t, icon_w, icon_h, &x, &y, &w, &h);
-    t->icon_view_x = x;
-    t->icon_view_y = y;
     t->icon_view_width = w;
     t->icon_view_height = h;
     if(Scr.CurrentDesk == t->desk)
@@ -1543,13 +1519,13 @@ void DrawGrid(int desk, int erase)
   {
     if(uselabel)
       XFillRectangle(dpy,Desks[desk].title_w,Desks[desk].HiliteGC,
-		     0,(LabelsBelow ? desk_h : 0),desk_w,label_h);
+		     0,(LabelsBelow ? desk_h : 0),desk_w,label_h -1);
   }
   else
   {
     if(uselabel && erase)
       XClearArea(dpy,Desks[desk].title_w,
-		 0,(LabelsBelow ? desk_h : 0),desk_w,label_h,False);
+		 0,(LabelsBelow ? desk_h : 0),desk_w,label_h - 1,False);
   }
 
   d = desk1+desk;
@@ -1609,7 +1585,7 @@ void DrawIconGrid(int erase)
     {
       if (Desks[tmp].bgPixmap)
 	XSetWindowBackgroundPixmap(
-	    dpy, icon_win,Desks[tmp].bgPixmap->picture);
+            dpy, icon_win,Desks[tmp].bgPixmap->picture);
       else if (Desks[tmp].Dcolor)
 	XSetWindowBackground(dpy, icon_win, GetColor(Desks[tmp].Dcolor));
       else if (PixmapBack)
@@ -1629,7 +1605,7 @@ void DrawIconGrid(int erase)
     x1 = x * icon_w / Scr.VWidth;
     if (!use_no_separators)
       for(i=0;i<ndesks;i++)
-	XDrawLine(dpy,icon_win,Desks[i].DashedGC,x1,y1,x1,y2);
+        XDrawLine(dpy,icon_win,Desks[i].DashedGC,x1,y1,x1,y2);
     x += Scr.MyDisplayWidth;
   }
 
@@ -1641,7 +1617,7 @@ void DrawIconGrid(int erase)
     y1 = y * icon_h / Scr.VHeight;
     if (!use_no_separators)
       for(i=0;i<ndesks;i++)
-	XDrawLine(dpy,icon_win,Desks[i].DashedGC,x1,y1,x2,y1);
+        XDrawLine(dpy,icon_win,Desks[i].DashedGC,x1,y1,x2,y1);
     y += Scr.MyDisplayHeight;
   }
   n1 = Scr.Vx / Scr.MyDisplayWidth;
@@ -1752,27 +1728,26 @@ void AddNewWindow(PagerWindow *t)
 
   if ((i >= 0) && (i < ndesks))
   {
-    t->PagerView = XCreateWindow(
-      dpy,Desks[i].w, x, y, w, h, 0, CopyFromParent, InputOutput,
-      CopyFromParent, valuemask, &attributes);
+    t->PagerView = XCreateWindow(dpy,Desks[i].w, x, y, w, h, 0,
+				 CopyFromParent, InputOutput, CopyFromParent,
+				 valuemask, &attributes);
     if (windowcolorset > -1)
     {
       SetWindowBackground(
-	      dpy, t->PagerView, w, h, &Colorset[windowcolorset], Pdepth,
-	      Scr.NormalGC, True);
+	dpy, t->PagerView, w, h, &Colorset[windowcolorset], Pdepth,
+	Scr.NormalGC, True);
     }
-    if (IS_ICONIFIED(t))
+    if (!UseSkipList || !DO_SKIP_WINDOW_LIST(t))
     {
-      XMoveResizeWindow(dpy, t->PagerView, -32768, -32768, 1, 1);
+      XMapRaised(dpy, t->PagerView);
     }
-    XMapRaised(dpy, t->PagerView);
   }
   else
-	  t->PagerView = None;
+  {
+    t->PagerView = None;
+  }
 
   CalcGeom(t, icon_w, icon_h, &x, &y, &w, &h);
-  t->icon_view_x = x;
-  t->icon_view_y = y;
   t->icon_view_width = w;
   t->icon_view_height = h;
   if(Scr.CurrentDesk != t->desk)
@@ -1786,8 +1761,8 @@ void AddNewWindow(PagerWindow *t)
   if (windowcolorset > -1)
   {
     SetWindowBackground(
-      dpy, t->IconView, w, h, &Colorset[windowcolorset],
-      Pdepth, Scr.NormalGC, True);
+      dpy, t->IconView, w, h, &Colorset[windowcolorset], Pdepth,
+      Scr.NormalGC, True);
   }
   if(Scr.CurrentDesk == t->desk)
   {
@@ -1796,11 +1771,15 @@ void AddNewWindow(PagerWindow *t)
       ButtonPressMask | ButtonReleaseMask|ButtonMotionMask,
       GrabModeAsync, GrabModeAsync, None, None);
   }
-  if (IS_ICONIFIED(t))
+  if (!UseSkipList || !DO_SKIP_WINDOW_LIST(t))
   {
-    XMoveResizeWindow(dpy, t->IconView, -32768, -32768, 1, 1);
+    XMapRaised(dpy,t->IconView);
+    t->myflags.is_mapped = 1;
+  }
+  else
+  {
+    t->myflags.is_mapped = 0;
   }
-  XMapRaised(dpy,t->IconView);
   Hilight(t,False);
 }
 
@@ -1851,8 +1830,6 @@ void ChangeDeskForWindow(PagerWindow *t,long newdesk)
 
   CalcGeom(t, icon_w, icon_h, &x, &y, &w, &h);
   size_changed = (t->icon_view_width != w || t->icon_view_height != h);
-  t->icon_view_x = x;
-  t->icon_view_y = y;
   t->icon_view_width = w;
   t->icon_view_height = h;
   if(Scr.CurrentDesk != t->desk)
@@ -1879,6 +1856,23 @@ void MoveResizePagerView(PagerWindow *t, Bool do_force_redraw)
   Bool size_changed;
   Bool position_changed;
 
+  if (UseSkipList && DO_SKIP_WINDOW_LIST(t) && t->myflags.is_mapped)
+  {
+    if (t->PagerView)
+      XUnmapWindow(dpy, t->PagerView);
+    if (t->IconView)
+      XUnmapWindow(dpy, t->IconView);
+    t->myflags.is_mapped = 0;
+  }
+  else if (UseSkipList && !DO_SKIP_WINDOW_LIST(t) && !t->myflags.is_mapped)
+  {
+    if (t->PagerView)
+      XMapRaised(dpy, t->PagerView);
+    if (t->IconView)
+      XMapRaised(dpy, t->IconView);
+    t->myflags.is_mapped = 1;
+  }
+
   CalcGeom(t, desk_w, desk_h, &x, &y, &w, &h);
   position_changed = (t->pager_view_x != x || t->pager_view_y != y);
   size_changed = (t->pager_view_width != w || t->pager_view_height != h);
@@ -1911,10 +1905,10 @@ void MoveResizePagerView(PagerWindow *t, Bool do_force_redraw)
   }
 
   CalcGeom(t, icon_w, icon_h, &x, &y, &w, &h);
-  position_changed = (t->icon_view_x != x || t->icon_view_y != y);
+  position_changed = (t->pager_view_x != x || t->pager_view_y != y);
   size_changed = (t->icon_view_width != w || t->icon_view_height != h);
-  t->icon_view_x = x;
-  t->icon_view_y = y;
+  t->pager_view_x = x;
+  t->pager_view_y = y;
   t->icon_view_width = w;
   t->icon_view_height = h;
   if (Scr.CurrentDesk == t->desk)
@@ -2199,8 +2193,13 @@ void MoveWindow(XEvent *Event)
       XDestroyWindow(dpy,t->PagerView);
       t->PagerView = None;
     }
-    XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
-		  &x, &y, &JunkX, &JunkY, &JunkMask);
+    if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
+		      &x, &y, &JunkX, &JunkY, &JunkMask) == False)
+    {
+      /* pointer is on a different screen */
+      x = 0;
+      y = 0;
+    }
     XUngrabPointer(dpy,CurrentTime);
     XSync(dpy,0);
     sprintf(command, "Silent Move %dp %dp", x, y);
@@ -2332,7 +2331,7 @@ void MoveWindow(XEvent *Event)
 /***********************************************************************
  *
  *  Procedure:
- *	FvwmErrorHandler - displays info on internal errors
+ *      FvwmErrorHandler - displays info on internal errors
  *
  ************************************************************************/
 int FvwmErrorHandler(Display *dpy, XErrorEvent *event)
@@ -2552,8 +2551,13 @@ void IconMoveWindow(XEvent *Event,PagerWindow *t)
   {
     char command[48];
 
-    XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
-		  &x, &y, &JunkX, &JunkY, &JunkMask);
+    if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
+		      &x, &y, &JunkX, &JunkY, &JunkMask) == False)
+    {
+      /* pointer is on a different screen */
+      x = 0;
+      y = 0;
+    }
     XUngrabPointer(dpy,CurrentTime);
     XSync(dpy,0);
     sprintf(command, "Silent Move %dp %dp", x, y);
@@ -2677,7 +2681,7 @@ void MapBalloonWindow(PagerWindow *t, Bool is_icon_view)
   /* balloon is a top-level window, therefore need to
      translate pager window coords to root window coords */
   XTranslateCoordinates(dpy, view, Scr.Root, x, y,
-			&window_changes.x, &window_changes.y, &dummy);
+                        &window_changes.x, &window_changes.y, &dummy);
 
 
   /* make sure balloon doesn't go off screen
@@ -2706,7 +2710,7 @@ void MapBalloonWindow(PagerWindow *t, Bool is_icon_view)
 
   /* too close to right */
   else if ( window_changes.x + window_changes.width >
-	    Scr.MyDisplayWidth - (2 * Desks[i].balloon.border) - 2 )
+            Scr.MyDisplayWidth - (2 * Desks[i].balloon.border) - 2 )
     window_changes.x = Scr.MyDisplayWidth - window_changes.width -
       (2 * Desks[i].balloon.border) - 2;
 
@@ -2763,21 +2767,21 @@ char *GetBalloonLabel(const PagerWindow *pw,const char *fmt)
       pos++;
       switch (*pos) {
       case 'i':
-	InsertExpand(&dest, pw->icon_name);
-	break;
+        InsertExpand(&dest, pw->icon_name);
+        break;
       case 't':
-	InsertExpand(&dest, pw->window_name);
-	break;
+        InsertExpand(&dest, pw->window_name);
+        break;
       case 'r':
-	InsertExpand(&dest, pw->res_name);
-	break;
+        InsertExpand(&dest, pw->res_name);
+        break;
       case 'c':
-	InsertExpand(&dest, pw->res_class);
-	break;
+        InsertExpand(&dest, pw->res_class);
+        break;
       case '%':
-	buffer[0] = '%';
-	InsertExpand(&dest, buffer);
-	break;
+        buffer[0] = '%';
+        InsertExpand(&dest, buffer);
+        break;
       default:;
       }
     } else {
@@ -2817,8 +2821,8 @@ void DrawInBalloonWindow (int i)
 #else
   XDrawString(dpy, Desks[i].balloon.w, Desks[i].BalloonGC,
 #endif
-	      2, Desks[i].balloon.font->ascent,
-	      Desks[i].balloon.label,strlen(Desks[i].balloon.label));
+              2, Desks[i].balloon.font->ascent,
+              Desks[i].balloon.label,strlen(Desks[i].balloon.label));
 }
 
 static void set_window_colorset_background(
@@ -2858,13 +2862,13 @@ void change_colorset(int colorset)
 	if (uselabel && (i == Scr.CurrentDesk))
 	{
 	  SetWindowBackground(
-	    dpy, Desks[i].title_w, desk_w, desk_h, &Colorset[colorset], Pdepth,
+	    dpy, Desks[i].title_w, 0, 0, &Colorset[colorset], Pdepth,
 	    Scr.NormalGC, True);
 	}
-	SetWindowBackground(
+        SetWindowBackground(
 	  dpy, Desks[i].CPagerWin, 0, 0, &Colorset[colorset], Pdepth,
 	  Scr.NormalGC, True);
-	XLowerWindow(dpy,Desks[i].CPagerWin);
+        XLowerWindow(dpy,Desks[i].CPagerWin);
       }
     }
 
@@ -2877,22 +2881,12 @@ void change_colorset(int colorset)
       if (uselabel)
       {
 	SetWindowBackground(
-	  dpy, Desks[i].title_w, desk_w, desk_h + label_h, &Colorset[colorset],
-	  Pdepth, Scr.NormalGC, True);
+	  dpy, Desks[i].title_w, 0, 0, &Colorset[colorset], Pdepth,
+	  Scr.NormalGC, True);
       }
-      if (label_h != 0 && uselabel && !LabelsBelow)
-      {
-	SetWindowBackgroundWithOffset(
-	  dpy, Desks[i].w, 0, -label_h, desk_w, desk_h + label_h,
-	  &Colorset[colorset], Pdepth, Scr.NormalGC, True);
-      }
-      else
-      {
-	SetWindowBackground(
-	  dpy, Desks[i].w, desk_w, desk_h + label_h,
-	  &Colorset[Desks[i].colorset], Pdepth, Scr.NormalGC, True);
-      }
-
+      SetWindowBackground(
+	dpy, Desks[i].w, 0, 0, &Colorset[colorset], Pdepth,
+	Scr.NormalGC, True);
     }
     else if (Desks[i].highcolorset == colorset && uselabel)
     {
