@@ -261,7 +261,7 @@ void ParseOptions(void)
         CopyString(&x11base->shadcolor,&tline[24]);
       }
       else if (strncasecmp(tline,"*FvwmScriptDefaultFont",22) == 0)
-	CopyStringWithQuotes(&x11base->font,&tline[22]);
+	CopyString(&x11base->font,&tline[22]);
       else if (strncasecmp(tline,"ColorLimit",10) == 0)
 	save_color_limit = atoi(&tline[10]);
       else if (strncasecmp(tline,"Colorset",8) == 0)
@@ -410,7 +410,7 @@ void OpenWindow (void)
   /* Choix des evts recus par la fenetre */
   XSelectInput(dpy,x11base->win, KeyPressMask|ButtonPressMask|
 	       ExposureMask|ButtonReleaseMask|EnterWindowMask|LeaveWindowMask|
-	       ButtonMotionMask);
+	       ButtonMotionMask|StructureNotifyMask);
   XSelectInput(dpy, x11base->root, PropertyChangeMask);
 
   /* Specification des parametres utilises par le gestionnaire de fenetre */
@@ -733,6 +733,28 @@ void SendMsgToScript(XEvent event)
   }
 }
 
+static
+void UpdateRootTransparency(void)
+{
+  int i;
+
+  if (x11base->colorset > 0 &&
+      Colorset[x11base->colorset].pixmap == ParentRelative)
+  {
+    XClearArea(dpy, x11base->win, 0, 0, 0, 0, True);
+    for (i=0; i<nbobj; i++)
+    {
+      if (Rectangle != tabxobj[i]->TypeWidget &&
+	  SwallowExec != tabxobj[i]->TypeWidget && 
+	  tabxobj[i]->colorset > 0 &&
+	  Colorset[tabxobj[i]->colorset].pixmap == ParentRelative)
+      {
+	XClearArea(dpy, tabxobj[i]->win, 0, 0, 0, 0, True);
+      }
+    }
+  }
+}
+
 /* read an X event */
 void ReadXServer (void)
 {
@@ -773,6 +795,23 @@ void ReadXServer (void)
 	}
       }
       break;
+    case ConfigureNotify:
+    {
+      Bool moved = False;
+
+      moved = event.xconfigure.send_event;
+      while (XCheckTypedEvent(dpy, ConfigureNotify, &event))
+      {
+	/* check for movement */
+	if (event.xconfigure.send_event)
+	  moved = True;
+      }
+      if (moved)
+      {
+	UpdateRootTransparency();
+      }
+    }
+    break;
     case KeyPress:
       /* Touche presse dans un objet / Key press in an object */
       isTab = 0;
@@ -1001,6 +1040,10 @@ void MainLoop (void)
 	  else if (StrEquals(token, XINERAMA_CONFIG_STRING)) {
 	    FScreenConfigureModule(line);
 	  }
+	  else if (StrEquals(token, ROOT_BG_CHANGE_STRING))
+	  {
+	    UpdateRootTransparency();
+	  }
 	  if (token)
 	    free(token);
 	}
