@@ -1249,27 +1249,21 @@ static void ChangeTitle (int NbArg,long *TabArg)
 /******* ChangeIcon *******/
 static void ChangeIcon (int NbArg,long *TabArg)
 {
-	int i=0;
-	char *arg[2];
-	int IdItem;
+  int i=0;
+  char *arg[2];
+  int IdItem;
 
-	arg[0] = CalcArg(TabArg,&i);
-	i++;
-	arg[1] = CalcArg(TabArg,&i);
-	IdItem = TabIdObj[atoi(arg[0])];
-	if (tabxobj[IdItem]->icon)
-	{
-		free(tabxobj[IdItem]->icon);
-	}
-	tabxobj[IdItem]->icon = safestrdup(arg[1]);
-	LoadIcon(tabxobj[IdItem]);
-	if (tabxobj[IdItem]->TypeWidget != SwallowExec)
-	{
-		XClearWindow(dpy, tabxobj[IdItem]->win);
-	}
-	tabxobj[IdItem]->DrawObj(tabxobj[IdItem]);
-	free(arg[0]);
-	free(arg[1]);
+  arg[0] = CalcArg(TabArg,&i);
+  i++;
+  arg[1] = CalcArg(TabArg,&i);
+  IdItem = TabIdObj[atoi(arg[0])];
+  tabxobj[IdItem]->icon = safestrdup(arg[1]);
+  LoadIcon(tabxobj[IdItem]);
+  if (tabxobj[IdItem]->TypeWidget != SwallowExec)
+    XClearWindow(dpy, tabxobj[IdItem]->win);
+  tabxobj[IdItem]->DrawObj(tabxobj[IdItem]);
+  free(arg[0]);
+  free(arg[1]);
 }
 
 /******* ChangeForeColor *******/
@@ -1288,11 +1282,8 @@ static void ChangeForeColor (int NbArg,long *TabArg)
   if (tabxobj[IdItem]->colorset < 0)
     XFreeColors(dpy,Pcmap,(void*)(&(tabxobj[IdItem])->TabColor[fore]),1,0);
 
-  if (tabxobj[IdItem]->forecolor)
-  {
-	  free(tabxobj[IdItem]->forecolor);
-  }
-  tabxobj[IdItem]->forecolor = safestrdup(arg[1]);
+  tabxobj[IdItem]->forecolor=(char*)safecalloc(100,sizeof(char));
+  sprintf(tabxobj[IdItem]->forecolor,"%s",arg[1]);
 
   tabxobj[IdItem]->TabColor[fore] = GetColor(tabxobj[IdItem]->forecolor);
   if (tabxobj[IdItem]->colorset >= 0) {
@@ -1327,11 +1318,8 @@ static void ChangeBackColor (int NbArg,long *TabArg)
   if (tabxobj[IdItem]->colorset < 0)
     XFreeColors(dpy,Pcmap,(void*)(&(tabxobj[IdItem])->TabColor[back]),1,0);
 
-  if (tabxobj[IdItem]->backcolor)
-  {
-	  free(tabxobj[IdItem]->backcolor);
-  }
-  tabxobj[IdItem]->backcolor= safestrdup(arg[1]);
+  tabxobj[IdItem]->backcolor = (char*)safecalloc(100,sizeof(char));
+  sprintf(tabxobj[IdItem]->backcolor,"%s",arg[1]);
 
   tabxobj[IdItem]->TabColor[back] = GetColor(tabxobj[IdItem]->backcolor);
   if (tabxobj[IdItem]->colorset >= 0) {
@@ -1456,8 +1444,8 @@ static void SendSign (int NbArg,long *TabArg)
   free(arg[1]);
 }
 
-/******* MyWarpPointer *******/
-static void MyWarpPointer(int NbArg,long *TabArg)
+/******* WarpPointer *******/
+static void WarpPointer(int NbArg,long *TabArg)
 {
   int i=0;
   char *arg;
@@ -1539,10 +1527,9 @@ static void Loop (int NbArg,long *TabArg)
   {
     if (TabArg[i] > 100000)	/* Cas du codage d'un nombre */
     {
-      int x;
-      x = (int)TabArg[i] - 200000;
+      i = (int)TabArg[i] - 200000;
       arg[CurrArg] = (char*)safecalloc(1,sizeof(char)*10);
-      sprintf(arg[CurrArg],"%d",x);
+      sprintf(arg[CurrArg],"%d",i);
     }
     else if (TabArg[i] < -100000)	/* Cas d'un id de fonction */
     {
@@ -1842,7 +1829,7 @@ void InitCom()
   TabCom[14]=IfThen;
   TabCom[15]=Loop;
   TabCom[16]=While;
-  TabCom[17]=MyWarpPointer;
+  TabCom[17]=WarpPointer;
   TabCom[18]=WriteToFile;
   TabCom[19]=ChangeBackColor;
   TabCom[21]=ChangeValueMax;
