@@ -54,6 +54,7 @@
 #include <sys/time.h>
 #include <sys/stat.h>
 #include <stdarg.h>
+#include <locale.h>
 
 #include <unistd.h>
 #include <ctype.h>
@@ -162,12 +163,9 @@ int  win_width    = 5,
 			 *  0 hidden -> not hidden (for the events loop) */
      FocusInWin = 0;    /* 1 if the Taskbar has the focus */
 
-static int auto_stick_y = 0;
-
 static volatile sig_atomic_t AlarmSet = NOT_SET;
 static volatile sig_atomic_t tip_window_alarm = False;
 static volatile sig_atomic_t hide_taskbar_alarm = False;
-static volatile sig_atomic_t stick_taskbar_alarm = False;
 
 
 int UpdateInterval = 30;
@@ -240,12 +238,11 @@ static void ParseConfigLine(char *tline);
 static void ShutMeDown(void);
 static RETSIGTYPE TerminateHandler(int sig);
 static RETSIGTYPE Alarm(int sig);
-static void SetAlarm(tb_alarm_t event);
-static void ClearAlarm(tb_alarm_t event);
+static void SetAlarm(int event);
+static void ClearAlarm(void);
 static void DoAlarmAction(void);
 static int ErrorHandler(Display*, XErrorEvent*);
 static Bool change_colorset(int cset, Bool force);
-static void WarpTaskBar(int y);
 int IsItemIndexIconSuppressed(List *list, int i);
 
 /******************************************************************************
@@ -259,6 +256,8 @@ int main(int argc, char **argv)
   char *s;
   int i;
 
+  setlocale(LC_ALL, "");
+
   for (i = 3; i < NUMBER_OF_MOUSE_BUTTONS; i++)
   {
     ClickAction[i] = DEFAULT_CLICK_N;
@@ -342,13 +341,15 @@ int main(int argc, char **argv)
 #endif
 
   SetMessageMask(Fvwm_fd,M_ADD_WINDOW | M_CONFIGURE_WINDOW | M_DESTROY_WINDOW |
-		 M_WINDOW_NAME | M_ICON_NAME | M_RES_NAME | M_DEICONIFY |
-		 M_ICONIFY | M_END_WINDOWLIST | M_FOCUS_CHANGE |
+		 M_VISIBLE_NAME | M_RES_NAME |
+		 M_DEICONIFY | M_ICONIFY | M_END_WINDOWLIST | M_FOCUS_CHANGE |
 		 M_CONFIG_INFO | M_END_CONFIG_INFO | M_NEW_DESK | M_SENDCONFIG
 #ifdef MINI_ICONS
 		 | M_MINI_ICON
 #endif
 		);
+  /* extended messages */
+  SetMessageMask(Fvwm_fd, MX_VISIBLE_ICON_NAME);
 
   /* Parse the config file */
   ParseConfig();
@@ -420,10 +421,9 @@ void EndLessLoop(void)
         LoopOnEvents();
 
       if (FD_ISSET(Fvwm_fd[1], &readset))
-      {
-	ReadFvwmPipe();
-      }
+        ReadFvwmPipe();
     }
+
     DoAlarmAction();
 
     DrawGoodies();
@@ -553,7 +553,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 
 	XSetWMNormalHints(dpy,win,&hints);
 
-	if (AutoStick)
+ 	if (AutoStick)
 	  XMoveResizeWindow(dpy, win, win_x, win_y, win_width, win_height);
 	else
 	  XResizeWindow(dpy, win, win_width, win_height);
@@ -628,7 +628,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	}
 	else if (!IsItemIndexSticky(&windows,i))
 	{
-	  is_on_desk = (DeskNumber == cfgpacket->desk);
+	  is_on_desk = (DeskNumber != cfgpacket->desk);
 	  is_changing_desk = (Desk != cfgpacket->desk);
 	}
 	UpdateItemIndexDesk(&windows, i, cfgpacket->desk);
@@ -692,7 +692,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	redraw = 1;
       }
     }
-    else if (!UseSkipList || !DO_SKIP_WINDOW_LIST(cfgpacket))
+    else
     {
       AddItem(&windows, cfgpacket->w,
 	      cfgpacket, cfgpacket->desk, Count++);
@@ -733,10 +733,10 @@ void ProcessMessage(unsigned long type,unsigned long *body)
     break;
 #endif
 
-  case M_WINDOW_NAME:
-  case M_ICON_NAME:
-    if ((type == M_ICON_NAME   && !UseIconNames) ||
-	(type == M_WINDOW_NAME &&  UseIconNames)) break;
+  case M_VISIBLE_NAME:
+  case MX_VISIBLE_ICON_NAME:
+    if ((type == MX_VISIBLE_ICON_NAME   && !UseIconNames) ||
+	(type == M_VISIBLE_NAME &&  UseIconNames)) break;
     string = (char *) &body[3];
     if ((i = UpdateItemName(&windows, body[0], (char *)&body[3])) == -1)
       break;
@@ -809,7 +809,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	  SendText(Fvwm_fd, buff, 0);
 	}
 	temp->needsupdate = 1;
-	temp->iconified = iconified;
+  	temp->iconified = iconified;
 	DrawButtonArray(&buttons, 0);
       }
       if (AnimCommand && AnimCommand[0] != 0)
@@ -862,6 +862,12 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	FScreenConfigureModule(
 	  tline + sizeof(XINERAMA_CONFIG_STRING) - 1);
       }
+      else if (strncasecmp(tline, ROOT_BG_CHANGE_STRING,
+			   sizeof(ROOT_BG_CHANGE_STRING) - 1) == 0)
+      {
+	if (colorset > -1 && Colorset[colorset].pixmap == ParentRelative)
+	  redraw = 1;
+      }
     }
     break;
   } /* switch */
@@ -878,8 +884,7 @@ void redraw_buttons()
 
   for (item=windows.head; item; item=item->next)
   {
-    if ((DeskNumber == item->Desk || IS_STICKY(item) ||
-	 (IS_ICONIFIED(item) && IS_ICON_STICKY(item))) &&
+    if ((DeskNumber == item->Desk || IS_STICKY(item)) &&
 	(!DO_SKIP_WINDOW_LIST(item) || !UseSkipList))
     {
       AddButton(&buttons, item->name, &(item->p), BUTTON_UP, item->count,
@@ -1058,10 +1063,10 @@ static void ParseConfigLine(char *tline)
     switch(index)
     {
     case 0: /* Font */
-      CopyStringWithQuotes(&font_string, rest);
+      CopyString(&font_string, rest);
       break;
-    case 1: /* SelFont */
-      CopyStringWithQuotes(&selfont_string, rest);
+    case 1: /* Selfont */
+      CopyString(&selfont_string, rest);
       break;
     case 2: /* Geometry */
       while (isspace((unsigned char)*rest) && *rest != '\n' && *rest != 0)
@@ -1183,17 +1188,16 @@ static void ParseConfigLine(char *tline)
 static RETSIGTYPE
 Alarm(int nonsense)
 {
-  if (AlarmSet & SHOW_TIP)
+
+  switch(AlarmSet)
   {
+  case SHOW_TIP:
     tip_window_alarm = True;
-  }
-  if (AlarmSet & HIDE_TASK_BAR)
-  {
-    hide_taskbar_alarm = True;
-  }
-  if (AlarmSet & STICK_TASK_BAR)
-  {
-    stick_taskbar_alarm = True;
+    break;
+
+  case HIDE_TASK_BAR:
+      hide_taskbar_alarm = True;
+    break;
   }
 
   AlarmSet = NOT_SET;
@@ -1214,12 +1218,7 @@ DoAlarmAction(void)
     hide_taskbar_alarm = False;
     HideTaskBar();
   }
-  if (stick_taskbar_alarm)
-  {
-    stick_taskbar_alarm = False;
-    WarpTaskBar(auto_stick_y);
-  }
-  if (tip_window_alarm)
+  else if (tip_window_alarm)
   {
     tip_window_alarm = False;
     if (AutoHide && WindowState == 0)
@@ -1232,8 +1231,13 @@ DoAlarmAction(void)
       /* We are now "sure" that the TaskBar is not hidden for the
 	 Event loop. We send a motion notify for activating tips */
       WindowState = 1;
-      XQueryPointer(dpy, win, &dummy_rt,&dummy_c, &abs_x, &abs_y,
-		    &pos_x, &pos_y, &dummy);
+      if (XQueryPointer(dpy, win, &dummy_rt,&dummy_c, &abs_x, &abs_y,
+			&pos_x, &pos_y, &dummy) == False)
+      {
+	/* pointer is on a different screen */
+	abs_x = 0;
+	abs_y = 0;
+      }
       sevent.xmotion.x = pos_x;
       sevent.xmotion.y = pos_y;
       sevent.xany.type = MotionNotify;
@@ -1270,8 +1274,8 @@ void CheckForTip(int x, int y)
     num = LocateButton(&buttons, x, y, &bx, &by, &name, &trunc);
     if (num != -1) {
       if ((Tip.type != num)  ||
-	  (Tip.text == NULL) || (!trunc && (strcmp(name, Tip.text) != 0)))
-	PopupTipWindow(bx+3, by, name);
+          (Tip.text == NULL) || (!trunc && (strcmp(name, Tip.text) != 0)))
+        PopupTipWindow(bx+3, by, name);
       Tip.type = num;
     } else {
       Tip.type = NO_TIP;
@@ -1283,7 +1287,7 @@ void CheckForTip(int x, int y)
       SetAlarm(SHOW_TIP);
   }
   else {
-    ClearAlarm(SHOW_TIP);
+    ClearAlarm();
     if (Tip.open) ShowTipWindow(0);
   }
 }
@@ -1426,7 +1430,7 @@ void LoopOnEvents(void)
 
     case LeaveNotify:
       NewTimestamp = Event.xcrossing.time;
-      ClearAlarm(SHOW_TIP);
+      ClearAlarm();
       if (Tip.open)
 	ShowTipWindow(0);
 
@@ -1494,14 +1498,16 @@ void LoopOnEvents(void)
       /* useful because of dynamic style change */
       if (!Event.xconfigure.send_event)
       {
+	PurgeConfigEvents();
 	break;
       }
       else if (Event.xconfigure.x != win_x || Event.xconfigure.y != win_y)
       {
+	if (colorset > -1 && Colorset[colorset].pixmap == ParentRelative)
+	  redraw = 1;
 	if (AutoStick)
 	{
-	  SetAlarm(STICK_TASK_BAR);
-	  auto_stick_y = Event.xconfigure.y;
+	  WarpTaskBar(Event.xconfigure.y, 0);
 	}
 	else
 	{
@@ -1737,7 +1743,7 @@ static void CreateOrUpdateGCs(void)
 
    /* brighting */
    gcmask = GCForeground | GCBackground | GCTile |
-	    GCFillStyle  | GCGraphicsExposures;
+            GCFillStyle  | GCGraphicsExposures;
    if (focuscolorset >= 0 || FocusBackColor != NULL)
      gcval.foreground = pfocusback;
    else
@@ -1767,7 +1773,7 @@ static Bool change_colorset(int cset, Bool force)
     {
       SetWindowBackground(
 	dpy, win, win_width, win_height, &Colorset[colorset],
-	Pdepth, graph, True);
+	Pdepth,	graph, True);
     }
     do_redraw = True;
   }
@@ -1789,6 +1795,8 @@ void StartMeUp(void)
    XSetWindowAttributes attr;
 #ifdef I18N_MB
   char **ml;
+  int mc;
+  char *ds;
   XFontStruct **fs_list;
 #endif
 
@@ -1829,19 +1837,27 @@ void StartMeUp(void)
      selfont_string = font_string;
 
 #ifdef I18N_MB
-   if ((ButtonFontset=GetFontSetOrFixed(dpy,font_string)) == NULL)
-   {
-	   fprintf(stderr,
-		   "%s: Couldn't load font. Exiting!\n",Module);
-	   exit(1);
+   if ((ButtonFontset=XCreateFontSet(dpy,font_string,&ml,&mc,&ds)) == NULL) {
+#ifdef STRICTLY_FIXED
+     if ((ButtonFontset=XCreateFontSet(dpy,"fixed",&ml,&mc,&ds)) == NULL) {
+#else
+     if ((ButtonFontset=XCreateFontSet(dpy,"-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*",&ml,&mc,&ds)) == NULL) {
+#endif
+       fprintf(stderr, "%s: Couldn't load fixed font. Exiting!\n",Module);
+       exit(1);
+     }
    }
    XFontsOfFontSet(ButtonFontset,&fs_list,&ml);
    ButtonFont = fs_list[0];
-   if ((SelButtonFontset=GetFontSetOrFixed(dpy,selfont_string)) == NULL)
-   {
-	   fprintf(stderr,
-		   "%s: Couldn't load selfont. Exiting!\n",Module);
-	   exit(1);
+   if ((SelButtonFontset = XCreateFontSet(dpy,selfont_string,&ml,&mc,&ds))
+       == NULL) {
+#ifdef STRICTLY_FIXED
+     if ((SelButtonFontset=XCreateFontSet(dpy,"fixed",&ml,&mc,&ds)) == NULL)
+#else
+     if ((SelButtonFontset=XCreateFontSet(dpy,"-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*",&ml,&mc,&ds)) == NULL)
+#endif
+       fprintf(stderr, "%s: Couldn't load fixed font. Exiting!\n",Module);
+       exit(1);
    }
    XFontsOfFontSet(SelButtonFontset,&fs_list,&ml);
    SelButtonFont = fs_list[0];
@@ -1997,8 +2013,8 @@ void StartMeUp(void)
    }
 
    XSelectInput(dpy,win,(ExposureMask | KeyPressMask | PointerMotionMask |
-			 EnterWindowMask | LeaveWindowMask |
-			 StructureNotifyMask));
+                         EnterWindowMask | LeaveWindowMask |
+                         StructureNotifyMask));
 
    InitGoodies();
    atexit(ShutMeDown);
@@ -2080,12 +2096,12 @@ PropMwmHints prop;
 /***********************************************************************
  WarpTaskBar -- Enforce AutoStick feature
  ***********************************************************************/
-static void WarpTaskBar(int y)
+void WarpTaskBar(int y, Bool force)
 {
   /* The tests on y are really useful ! */
   if (!AutoHide &&
       ((y != screen_g.y + screen_g.height - win_height - win_border &&
-	y !=  screen_g.y + win_border))) {
+	y !=  screen_g.y + win_border) || force)) {
     if (y > Midline)
     {
       win_y = screen_g.height - win_height - win_border;
@@ -2102,7 +2118,12 @@ static void WarpTaskBar(int y)
 
   if (AutoHide)
      SetAlarm(HIDE_TASK_BAR);
-  ClearAlarm(STICK_TASK_BAR);
+
+  /* Prevent oscillations caused by race with
+     time delayed TaskBarHide().  Is there any way
+     to prevent these Xevents from being sent
+     to the server in the first place? */
+  PurgeConfigEvents();
 }
 
 /***********************************************************************
@@ -2130,7 +2151,7 @@ void RevealTaskBar()
   int new_win_y;
   int inc_y = 2;
 
-  ClearAlarm(HIDE_TASK_BAR);
+  ClearAlarm();
    /* do not reveal if the taskbar is already revealed */
   if (WindowState >= 0)
     return;
@@ -2172,15 +2193,18 @@ void HideTaskBar()
   int d_x, d_y, wx, wy;
   unsigned int mask;
 
-  ClearAlarm(HIDE_TASK_BAR);
+  ClearAlarm();
   /* do not hide if the taskbar is already hiden */
   if (WindowState == -1 || win_is_shaded)
     return;
 
   if (FocusInWin)
   {
-    XQueryPointer(dpy, win, &d_rt,&d_ch, &d_x, &d_y,
-		  &wx, &wy, &mask);
+    if (XQueryPointer(
+		dpy, win, &d_rt,&d_ch, &d_x, &d_y, &wx, &wy, &mask) == False)
+    {
+      /* pointer is on a different screen - that's okay here */
+    }
     if (wy >= -(win_border + win_title_height*(1-win_has_bottom_title)) &&
 	wy < win_height + win_border +
 	win_title_height*win_has_bottom_title)
@@ -2223,10 +2247,10 @@ void HideTaskBar()
  SetAlarm -- Schedule a timeout event
  ************************************************************************/
 static void
-SetAlarm(tb_alarm_t event)
+SetAlarm(int event)
 {
   alarm(0);  /* remove a race-condition */
-  AlarmSet |= event;
+  AlarmSet = event;
   alarm(1);
 }
 
@@ -2234,13 +2258,22 @@ SetAlarm(tb_alarm_t event)
  ClearAlarm -- Disable timeout events
  ************************************************************************/
 static void
-ClearAlarm(tb_alarm_t event)
+ClearAlarm(void)
 {
-  AlarmSet &= ~event;
-  if (AlarmSet == 0)
-  {
-    alarm(0);
-  }
+  AlarmSet = NOT_SET;
+  alarm(0);
+}
+
+/***********************************************************************
+ PurgeConfigEvents -- Wait for and purge ConfigureNotify events.
+ ************************************************************************/
+void PurgeConfigEvents(void)
+{
+  XEvent Event;
+
+  XPeekEvent(dpy, &Event);
+  while (XCheckTypedWindowEvent(dpy, win, ConfigureNotify, &Event))
+    ;
 }
 
 /************************************************************************
