@@ -548,7 +548,7 @@ void Loop(void)
 	      }
 	      XTranslateCoordinates(
 		dpy,main_win,Root,LastX,LastY,&LastX,&LastY,&junk);
-              /* Note: maybe it will be better to use (LastX+W/2,LastY+H/2)
+              /*!!! Note: maybe it will be better to use (LastX+W/2,LastY+H/2)
                 in ScrRect query? That could help in pathological cases when
                 Wharf is not completely on a single screen. */
     	      fscr.xypos.x = LastX;
@@ -713,11 +713,12 @@ void Loop(void)
 			    )
 			    break; */
 
-	  XQueryPointer(dpy,main_win,
-			&dummy_rt,&dummy_c,
-			&dummy_x,&dummy_y,
-			&pos_x,&pos_y,
-			&dummy);
+	  if (XQueryPointer(
+		dpy, main_win, &dummy_rt, &dummy_c, &dummy_x, &dummy_y,
+		&pos_x, &pos_y, &dummy) == False)
+	  {
+	    /* pointer is on a different screen - that's okay here */
+	  }
 	  base = 0;
 	  dummy_y = (pos_y/BUTTONHEIGHT);
 	  dummy_x= (pos_x/BUTTONWIDTH);
@@ -1348,6 +1349,7 @@ void CreateShadowGC(void)
   }
   else if (global_colorset >= 0)
   {
+    /*!!! ??? right?*/
     fore_pix = Colorset[global_colorset].shadow;
     light_grey = Colorset[global_colorset].hilite;
   }
@@ -1585,6 +1587,10 @@ void ParseOptions(char *filename)
 
     if((strncasecmp(tline,CatString3("*", MyName, "Geometry"),Clength+9)==0))
     {
+      w= -1;
+      h= -1;
+      x= -100000;
+      y= -100000;
       tmp = &tline[Clength+9];
       while(((isspace(*tmp))&&(*tmp != '\n'))&&(*tmp != 0))
       {
