@@ -304,9 +304,11 @@ int main(int argc, char **argv)
 		 M_MINI_ICON |
 #endif
 		 M_CONFIGURE_WINDOW | M_ADD_WINDOW | M_DESTROY_WINDOW |
-		 M_WINDOW_NAME | M_ICON_NAME | M_DEICONIFY | M_ICONIFY |
-		 M_END_WINDOWLIST | M_NEW_DESK | M_FOCUS_CHANGE |
+		 M_VISIBLE_NAME | M_DEICONIFY |
+		 M_ICONIFY | M_END_WINDOWLIST | M_NEW_DESK | M_FOCUS_CHANGE |
 		 M_CONFIG_INFO | M_SENDCONFIG);
+  /* extended messages */
+  SetMessageMask(Fvwm_fd, MX_VISIBLE_ICON_NAME);
 
   SendFvwmPipe(Fvwm_fd, "Send_WindowList",0);
 
@@ -451,9 +453,9 @@ void ProcessMessage(unsigned long type,unsigned long *body)
     break;
 #endif
 
-    case M_WINDOW_NAME:
-    case M_ICON_NAME:
-      if ((type == M_ICON_NAME) ^ UseIconNames)
+    case M_VISIBLE_NAME:
+    case MX_VISIBLE_ICON_NAME:
+      if ((type == MX_VISIBLE_ICON_NAME) ^ UseIconNames)
 	break;
       string=(char *)&body[3];
       if ((i=UpdateItemName(&windows,body[0],string))==-1) break;
@@ -622,8 +624,8 @@ ParseConfigLine(char *tline)
     return;
 
   if (strlen(tline) > 1) {
-    if (strncasecmp(tline, CatString3(Module, "Font", ""), Clength + 4) == 0)
-      CopyStringWithQuotes(&font_string, &tline[Clength + 4]);
+    if (strncasecmp(tline, CatString3(Module, "Font", ""), Clength + 4)	== 0)
+      CopyString(&font_string, &tline[Clength + 4]);
     else if (strncasecmp(tline, CatString3(Module, "Fore", ""), Clength + 4)
 	     == 0) {
       CopyString(&ForeColor[0], &tline[Clength + 4]);
@@ -761,6 +763,12 @@ ParseConfigLine(char *tline)
       FScreenConfigureModule(
 	tline + sizeof(XINERAMA_CONFIG_STRING) - 1);
     }
+    else if (strncasecmp(tline, ROOT_BG_CHANGE_STRING,
+			 sizeof(ROOT_BG_CHANGE_STRING) - 1) == 0)
+    {
+      if (WindowState && win_bg == ParentRelative)
+	RedrawWindow(True, True);
+    }
   }
 }
 
@@ -775,14 +783,19 @@ void LoopOnEvents(void)
   XEvent Event;
 #if 1
   Window dummyroot,dummychild;
-  int x,x1,y,y1;
+  int x,xdummy,y,ydummy;
   unsigned int dummy1;
 
   if (Transient && !Checked)
   {
     if (Pressed)
     {
-      XQueryPointer(dpy,win,&dummyroot,&dummychild,&x1,&y1,&x,&y,&dummy1);
+      if (XQueryPointer(
+	    dpy, win, &dummyroot, &dummychild, &xdummy, &ydummy, &x, &y,
+	    &dummy1) == False)
+      {
+	/* pointer is on a different screen - that's okay here */
+      }
       num=WhichButton(&buttons,x,y);
       if (num!=-1)
       {
@@ -813,7 +826,7 @@ void LoopOnEvents(void)
 	      Event.xbutton.button <= NUMBER_OF_MOUSE_BUTTONS)
 	  {
             SendFvwmPipe(Fvwm_fd,
-			 ClickAction[Event.xbutton.button-1],
+			 ClickAction[(Transient) ? 0:Event.xbutton.button-1],
 			 ItemID(&windows,num));
             SwitchButton(&buttons,num);
           }
@@ -864,7 +877,7 @@ void LoopOnEvents(void)
 	  }
 	}
         if (Event.xconfigure.send_event && (win_x != Event.xconfigure.x
-            || win_y != Event.xconfigure.y)) {
+            || win_y != Event.xconfigure.y || win_bg == ParentRelative)) {
 	  win_x = Event.xconfigure.x;
 	  win_y = Event.xconfigure.y;
 	  if (win_bg == ParentRelative)
@@ -1125,8 +1138,14 @@ void MakeMeWindow(void)
   {
     fscreen_scr_arg fscr;
 
-    XQueryPointer(dpy,Root,&dummyroot,&dummychild,&hints.x,&hints.y,&x,&y,
-		  &dummy1);
+    if (XQueryPointer(
+	  dpy, Root, &dummyroot, &dummychild, &hints.x, &hints.y, &x, &y,
+	  &dummy1) == False)
+    {
+      /* pointer is on a different screen */
+      hints.x = 0;
+      hints.y = 0;
+    }
     fscr.xypos.x = hints.x;
     fscr.xypos.y = hints.y;
     FScreenGetScrRect(
@@ -1348,8 +1367,14 @@ void MakeMeWindow(void)
       fprintf(stderr,"failed to grab pointer\n");
       exit(1);
     }
-    XQueryPointer(
-      dpy,Root,&dummyroot,&dummychild,&hints.x,&hints.y,&x,&y,&dummy1);
+    if (XQueryPointer(
+	  dpy, Root, &dummyroot, &dummychild, &hints.x, &hints.y, &x, &y,
+	  &dummy1) == False)
+    {
+      /* pointer is on a different screen */
+      hints.x = 0;
+      hints.y = 0;
+    }
     Pressed = !!SomeButtonDown(dummy1);
   }
 }
